import { ActiveTrigger, CardEntry, CardInterface, CardModelInterface, CardValue } from '../types/card';
import { NodeInterface } from '../types/node';
import { RangeInterface } from '../types/range';
import { EditorInterface } from '../types/engine';
import './index.css';
declare class CardModel implements CardModelInterface {
    classes: {
        [k: string]: CardEntry;
    };
    private components;
    private editor;
    constructor(editor: EditorInterface);
    get active(): CardInterface | undefined;
    get length(): number;
    init(cards: Array<CardEntry>): void;
    add(clazz: CardEntry): void;
    each(callback: (card: CardInterface, index?: number) => boolean | void): void;
    closest(selector: Node | NodeInterface, ignoreEditable?: boolean): NodeInterface | undefined;
    find(selector: string | Node | NodeInterface, ignoreEditable?: boolean): CardInterface | undefined;
    findBlock(selector: Node | NodeInterface): CardInterface | undefined;
    getSingleCard(range: RangeInterface): CardInterface | undefined;
    getSingleSelectedCard(range: RangeInterface): CardInterface | undefined;
    insertNode(range: RangeInterface, card: CardInterface): CardInterface;
    removeNode(card: CardInterface): void;
    updateNode(card: CardInterface, value: CardValue): void;
    replaceNode(node: NodeInterface, name: string, value?: CardValue): NodeInterface;
    activate(node: NodeInterface, trigger?: ActiveTrigger, event?: MouseEvent): void;
    select(card: CardInterface): void;
    focus(card: CardInterface, toStart?: boolean): void;
    insert(name: string, value?: CardValue): CardInterface;
    update(selector: NodeInterface | Node | string, value: CardValue): void;
    remove(selector: NodeInterface | Node | string): void;
    removeRemote(selector: NodeInterface | Node | string): void;
    create(name: string, options?: {
        value?: CardValue;
        root?: NodeInterface;
    }): CardInterface;
    /**
     * 渲染
     * @param container 需要重新渲染包含卡片的节点，如果不传，则渲染全部待创建的卡片节点
     * @param asyncRender 是否异步渲染， 全部异步渲染完成后触发
     */
    render(container?: NodeInterface, asyncRender?: ((count: number) => void) | true): void;
    removeComponent(card: CardInterface): void;
    gc(): void;
}
export default CardModel;
