import { NodeInterface } from '../types/node';
import { ChangeInterface, ChangeOptions } from '../types/change';
import { EngineInterface } from '../types/engine';
import { RangeInterface } from '../types/range';
import ChangeEvent from './event';
import { Path } from 'sharedb';
declare class ChangeModel implements ChangeInterface {
    private engine;
    private options;
    private changeTimer;
    event: ChangeEvent;
    valueCached: string | null;
    onChange: (value: string, trigger: 'remote' | 'local' | 'both') => void;
    onSelect: () => void;
    onSetValue: () => void;
    rangePathBeforeCommand: Path[] | null;
    marks: Array<NodeInterface>;
    blocks: Array<NodeInterface>;
    inlines: Array<NodeInterface>;
    changeTrigger: Array<string>;
    constructor(engine: EngineInterface, options?: ChangeOptions);
    private _change;
    change(isRemote?: boolean): void;
    private clearChangeTimer;
    getRange(): RangeInterface;
    select(range: RangeInterface): this;
    /**
     * 聚焦编辑器
     * @param toStart true:开始位置,false:结束位置，默认为之前操作位置
     */
    focus(toStart?: boolean): this;
    blur(): this;
    isComposing(): boolean;
    isSelecting(): boolean;
    setValue(value: string, onParse?: (node: Node) => void): void;
    getOriginValue(): string;
    getValue(options?: {
        ignoreCursor?: boolean;
    }): string;
    cacheRangeBeforeCommand(): void;
    getRangePathBeforeCommand(): import("sharedb/lib/sharedb").Path[] | null;
    isEmpty(): boolean;
    private repairInput;
    /**
     * 获取安全可控的光标对象
     * @param range 默认当前光标
     */
    getSafeRange(range?: RangeInterface): RangeInterface;
    private focusCardRang;
    private initNativeEvents;
    paste(source: string): void;
    combinTextNode(): void;
    /**
     * 应用一个具有改变dom结构的操作
     * @param range 光标
     */
    apply(range?: RangeInterface): void;
    /**
     * 插入片段
     * @param fragment 片段
     * @param callback 插入后的回调函数
     */
    insertFragment(fragment: DocumentFragment, callback?: (range: RangeInterface) => void): void;
    /**
     * 删除内容
     * @param range 光标，默认获取当前光标
     * @param isDeepMerge 删除后是否合并
     */
    deleteContent(range?: RangeInterface, isDeepMerge?: boolean): void;
    /**
     * 删除节点，删除后如果是空段落，自动添加 BR
     * @param node 要删除的节点
     */
    addBrAfterDelete(node: NodeInterface): void;
    /**
     * 去除当前光标最接近的block节点或传入的节点外层包裹
     * @param node 节点
     */
    unwrapNode(node?: NodeInterface): void;
    /**
     * 删除当前光标最接近的block节点或传入的节点的前面一个节点后合并
     * @param node 节点
     */
    mergeAfterDeletePrevNode(node?: NodeInterface): void;
    /**
     * 焦点移动到当前光标最接近的block节点或传入的节点前一个 Block
     * @param block 节点
     * @param isRemoveEmptyBlock 如果前一个block为空是否删除，默认为否
     */
    focusPrevBlock(block?: NodeInterface, isRemoveEmptyBlock?: boolean): void;
    destroy(): void;
}
export default ChangeModel;
