import { Selector, NodeInterface, EventInterface, EventListener, NodeModelInterface } from '../types/node';
import { ChangeInterface } from '../types/change';
import { EngineInterface, EngineOptions } from '../types/engine';
import { HistoryInterface } from '../types/history';
import { OTInterface } from '../types/ot';
import { SchemaInterface } from '../types/schema';
import { ConversionInterface } from '../types/conversion';
import { CommandInterface } from '../types/command';
import { PluginModelInterface } from '../types/plugin';
import { HotkeyInterface } from '../types/hotkey';
import { CardModelInterface } from '../types/card';
import { ClipboardInterface } from '../types/clipboard';
import { LanguageInterface } from '../types/language';
import { MarkModelInterface } from '../types/mark';
import { ListModelInterface } from '../types/list';
import { InlineModelInterface } from '../types/inline';
import { BlockModelInterface } from '../types/block';
import { RequestInterface } from '../types/request';
import { TypingInterface } from '../types';
import './index.css';
declare class Engine implements EngineInterface {
    private _readonly;
    private _container;
    readonly kind = "engine";
    options: EngineOptions;
    language: LanguageInterface;
    root: NodeInterface;
    change: ChangeInterface;
    card: CardModelInterface;
    plugin: PluginModelInterface;
    node: NodeModelInterface;
    list: ListModelInterface;
    mark: MarkModelInterface;
    inline: InlineModelInterface;
    block: BlockModelInterface;
    event: EventInterface;
    typing: TypingInterface;
    ot: OTInterface;
    schema: SchemaInterface;
    conversion: ConversionInterface;
    history: HistoryInterface;
    scrollNode: NodeInterface | null;
    command: CommandInterface;
    hotkey: HotkeyInterface;
    clipboard: ClipboardInterface;
    request: RequestInterface;
    get container(): NodeInterface;
    get readonly(): boolean;
    set readonly(readonly: boolean);
    constructor(selector: Selector, options?: EngineOptions);
    isSub(): boolean;
    isFocus(): boolean;
    focus(): void;
    on(eventType: string, listener: EventListener, rewrite?: boolean): this;
    off(eventType: string, listener: EventListener): this;
    trigger(eventType: string, ...args: any): any;
    getValue(ignoreCursor?: boolean): string;
    getValueAsync(ignoreCursor?: boolean): Promise<string>;
    getHtml(): string;
    setValue(value: string): this;
    setJsonValue(value: Array<any>): this;
    destroy(): void;
    private normalizeTree;
    messageSuccess(message: string): void;
    messageError(error: string): void;
    messageConfirm(message: string): Promise<boolean>;
}
export default Engine;
