import { NodeInterface } from '../types/node';
import { EditorInterface } from '../types/engine';
import { SchemaInterface, ParserInterface, Callbacks, ConversionInterface } from '../types';
import TextParser from './text';
declare class Parser implements ParserInterface {
    private root;
    private editor;
    constructor(source: string | Node | NodeInterface, editor: EditorInterface, paserBefore?: (node: NodeInterface) => void);
    normalize(root: NodeInterface, schema: SchemaInterface, conversion: ConversionInterface | null): void;
    /**
     * data type:
     *
     * Value: <p>foo</p><p><br /><cursor /></p>
     * LowerValue: <p>foo</p><p><br /><span data-element="cursor"></span></p>
     * DOM: HTML DOM tree
     * Markdown: ### heading
     * Text: plain text
     *
     */
    walkTree(node: NodeInterface, schema: SchemaInterface | null | undefined, conversion: ConversionInterface | null, callbacks: Callbacks, includeCard?: boolean): void;
    /**
     * 遍历 DOM 树，生成符合标准的 XML 代码
     * @param schema 标签保留规则
     * @param conversion 标签转换规则
     * @param replaceSpaces 是否替换空格
     * @param customTags 是否将光标、卡片节点转换为标准代码
     */
    toValue(schema?: SchemaInterface | null, conversion?: ConversionInterface | null, replaceSpaces?: boolean, customTags?: boolean): string;
    /**
     * 转换为HTML代码
     * @param inner 内包裹节点
     * @param outter 外包裹节点
     */
    toHTML(inner?: Node, outter?: Node): {
        html: string;
        text: string;
    };
    /**
     * 返回DOM树
     */
    toDOM(schema: SchemaInterface | null | undefined, conversion: ConversionInterface | null): DocumentFragment;
    /**
     * 转换为文本
     * @param includeCard 是遍历卡片内部
     */
    toText(schema?: SchemaInterface | null, includeCard?: boolean): string;
}
export default Parser;
export { TextParser };
