import isHotkey from 'is-hotkey';
export { default as isHotkey } from 'is-hotkey';
import _regeneratorRuntime from '@babel/runtime/regenerator';
import _asyncToGenerator from '@babel/runtime/helpers/esm/asyncToGenerator';
import _objectSpread from '@babel/runtime/helpers/esm/objectSpread2';
import _classCallCheck from '@babel/runtime/helpers/esm/classCallCheck';
import _createClass from '@babel/runtime/helpers/esm/createClass';
import _objectWithoutProperties from '@babel/runtime/helpers/esm/objectWithoutProperties';
import _typeof from '@babel/runtime/helpers/esm/typeof';
import _defineProperty from '@babel/runtime/helpers/esm/defineProperty';
import md5 from 'blueimp-md5';
import tinycolor2 from 'tinycolor2';
import _slicedToArray from '@babel/runtime/helpers/esm/slicedToArray';
import _toConsumableArray from '@babel/runtime/helpers/esm/toConsumableArray';
import { cloneDeep, merge, omit, isEqual, debounce, findLastIndex, startsWith } from 'lodash-es';
import _inherits from '@babel/runtime/helpers/esm/inherits';
import _createSuper from '@babel/runtime/helpers/esm/createSuper';
import { EventEmitter2 } from 'eventemitter2';
import OTJSON from 'ot-json0';
import { diff_match_patch, DIFF_DELETE, DIFF_INSERT, DIFF_EQUAL } from 'diff-match-patch';
import _get from '@babel/runtime/helpers/esm/get';
import _getPrototypeOf from '@babel/runtime/helpers/esm/getPrototypeOf';
import copyTo from 'copy-to-clipboard';
import filesize from 'filesize';

var DATA_ELEMENT = 'data-element';
var ROOT = 'root';
var ROOT_SELECTOR = '['.concat(DATA_ELEMENT, '=').concat(ROOT, ']');
var UI = 'ui';
var UI_SELECTOR = '['.concat(DATA_ELEMENT, '=').concat(UI, ']');
var EDITABLE = 'editable';
var EDITABLE_SELECTOR = '['.concat(DATA_ELEMENT, '=').concat(EDITABLE, ']');
var DATA_TRANSIENT_ATTRIBUTES = 'data-transient-attributes';
var DATA_TRANSIENT_ELEMENT = 'data-transient-element';

var CARD_TAG = 'card';
var CARD_KEY = 'data-card-key';
var READY_CARD_KEY = 'data-ready-card';
var CARD_TYPE_KEY = 'data-card-type';
var CARD_VALUE_KEY = 'data-card-value';
var CARD_ELEMENT_KEY = 'data-card-element';
var CARD_SELECTOR = 'div['.concat(CARD_KEY, '],span[').concat(CARD_KEY, ']');
var READY_CARD_SELECTOR = 'div['.concat(READY_CARD_KEY, '],span[').concat(READY_CARD_KEY, ']');
var CARD_LEFT_SELECTOR = 'span['.concat(CARD_ELEMENT_KEY, '=left]');
var CARD_CENTER_SELECTOR = 'div['.concat(CARD_ELEMENT_KEY, '=center],span[').concat(CARD_ELEMENT_KEY, '=center]');
var CARD_RIGHT_SELECTOR = 'span['.concat(CARD_ELEMENT_KEY, '=right]');
var CARD_ASYNC_RENDER = 'data-card-async-render';

var ANCHOR = 'anchor';
var FOCUS = 'focus';
var CURSOR = 'cursor';
var ANCHOR_SELECTOR = 'span['.concat(DATA_ELEMENT, '=').concat(ANCHOR, ']');
var FOCUS_SELECTOR = 'span['.concat(DATA_ELEMENT, '=').concat(FOCUS, ']');
var CURSOR_SELECTOR = 'span['.concat(DATA_ELEMENT, '=').concat(CURSOR, ']');

/**
 * 事件
 */
var Event = /*#__PURE__*/function () {
  function Event() {
    _classCallCheck(this, Event);

    this.listeners = {};
  }

  _createClass(Event, [{
    key: "on",
    value:
    /**
     * 绑定事件
     * @param {string} eventType 事件名称
     * @param {Function} listener 事件处理方法
     * @param {boolean} rewrite 是否重写事件
     */
    function on(eventType, listener, rewrite) {
      if (!this.listeners[eventType] || rewrite) {
        this.listeners[eventType] = [];
      }

      this.listeners[eventType].push(listener);
    }
    /**
     * 解除绑定
     * @param {string} eventType
     * @param listener
     */

  }, {
    key: "off",
    value: function off(eventType, listener) {
      var listeners = this.listeners[eventType];

      if (!listeners) {
        return;
      }

      for (var i = 0; i < listeners.length; i++) {
        if (listeners[i] === listener) {
          listeners.splice(i, 1);
          break;
        }
      }
    }
    /**
     * 触发事件
     * @param eventType 事件类型
     * @param args 事件参数
     */

  }, {
    key: "trigger",
    value: function trigger(eventType) {
      var listeners = this.listeners[eventType];

      if (listeners) {
        var result;

        for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
          args[_key - 1] = arguments[_key];
        }

        for (var i = 0; i < listeners.length; i++) {
          result = listeners[i].apply(listeners, args);

          if (result === false) {
            break;
          }
        }

        return result;
      }

      return;
    }
  }]);

  return Event;
}();

var isNodeEntry = function isNodeEntry(selector) {
  return !!selector && selector.get !== undefined;
};
var isNodeList = function isNodeList(selector) {
  return !!selector && selector.entries !== undefined;
};
var isNode = function isNode(selector) {
  return !!selector && selector.nodeType !== undefined;
};

var getDocument = function getDocument(node) {
  if (typeof document === 'undefined' && typeof global['__amWindow'] === 'undefined') throw 'document is not defined,If you are using ssr, you can assign a value to the `__amWindow` global variable.';
  return node ? node.ownerDocument || node['document'] || node : typeof document === 'undefined' ? global['__amWindow'].document : document;
};
var getWindow = function getWindow(node) {
  if (typeof window === 'undefined' && typeof global['__amWindow'] === 'undefined') throw 'window is not defined,If you are using ssr, you can assign a value to the `__amWindow` global variable.';
  var win = typeof window === 'undefined' ? global['__amWindow'] : window;
  if (!node) return win;
  var document = getDocument(node);
  return document['parentWindow'] || document.defaultView || win;
};
/**
 * 移除空的文本节点，并连接相邻的文本节点
 * @param node 节点
 */

var combinTextNode = function combinTextNode(node) {
  if (isNodeEntry(node)) node = node[0];
  node.normalize();
};
/**
 * 获取一个 dom 元素内所有的 textnode 类型的元素
 * @param  {Node} node - dom节点
 * @param  {Function} filter - 过滤器
 * @return {Array} 获取的文本节点
 */

var getTextNodes = function getTextNodes(node, filter) {
  var textNodes = [];

  if (filter && !filter(node)) {
    return textNodes;
  }

  var nodes = node.childNodes;

  for (var i = 0; i < nodes.length; i++) {
    var _node = nodes[i];
    var nodeType = _node.nodeType;

    if (nodeType === 3) {
      textNodes.push(_node);
    } else if (nodeType === 1 || nodeType === 9 || nodeType === 11) {
      textNodes = textNodes.concat(getTextNodes(_node, filter));
    }
  }

  return textNodes;
};
var getParentInRoot = function getParentInRoot(node) {
  return node.nodeType === getDocument().ELEMENT_NODE && node.getAttribute(DATA_ELEMENT) === ROOT ? undefined : node.parentNode || undefined;
};
var inside = function inside(source, node) {
  var callback = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : function (node) {
    return node.parentNode || undefined;
  };
  var parentNode = (isNodeEntry(source) ? source[0] : source).parentNode;

  while (parentNode) {
    if (parentNode === (isNodeEntry(node) ? node[0] : node)) return true;
    parentNode = callback(parentNode) || null;
  }

  return false;
};

/**
 * 解析节点
 * @param selector 选择器
 * @param isSpecialText 是否为特殊文本比如 \u200b 0宽字符
 * @param context 上下文节点，默认使用 getDocument 获取document
 */

function domParser(selector, context) {
  if (!selector) return []; //文本字符串

  if (typeof selector === 'string') {
    //特殊字符，或者html代码
    if (!context || /<.+>/.test(selector)) {
      var isTr = selector.indexOf('<tr') === 0;
      var isTd = selector.indexOf('<td') === 0; //替换注释

      selector = selector.trim().replace(/<!--[^>]*-->/g, '');
      /**
       * 无法单独解析 tr、td 标签，如果有tr、td标签这里需要补充 table 节点的结构
       */

      if (isTr) {
        selector = '<table><tbody>'.concat(selector, '</tbody></table>');
      }

      if (isTd) {
        selector = '<table><tbody><tr>'.concat(selector, '</tr></tbody></table>');
      } //创建一个空节点，用来包裹需要生成的节点


      var container = getDocument().createElement('div');
      container.innerHTML = selector;

      if (isTr) {
        var tbody = container.querySelector('tbody');
        return tbody ? tbody.childNodes : [];
      }

      if (isTd) {
        var tr = container.querySelector('tr');
        return tr ? tr.childNodes : [];
      } // 返回解析后的所有子级


      return container.childNodes;
    } //默认根据选择器查询所有


    return context.querySelectorAll(selector);
  } //类型为 NodeList ，node数组 直接返回


  if (isNodeList(selector) || Array.isArray(selector)) {
    return selector;
  } //类型为 DOMNode 类型


  if (isNodeEntry(selector)) {
    var nodes = [];
    selector.each(function (node) {
      nodes.push(node);
    });
    return nodes;
  } // 片段


  if (isNode(selector) && selector.nodeType === getWindow().Node.DOCUMENT_FRAGMENT_NODE) {
    var _nodes = [];
    var node = selector.firstChild;

    while (node) {
      _nodes.push(node);

      node = node.nextSibling;
    }

    return _nodes;
  } // 其他


  return [selector];
}

var TinyCanvas = /*#__PURE__*/function () {
  function TinyCanvas(options) {
    _classCallCheck(this, TinyCanvas);

    this.options = {
      limitHeight: 5000,
      canvasCache: [],
      canvasCount: 0
    };
    if (!options.container) throw new Error('please set a dom cantainer!');
    this.options = _objectSpread(_objectSpread({}, this.options), options);
    options.container.style['line-height'] = '0px';
  }

  _createClass(TinyCanvas, [{
    key: "removeCanvas",
    value: function removeCanvas() {
      var canvasCache = this.options.canvasCache;
      canvasCache === null || canvasCache === void 0 ? void 0 : canvasCache.forEach(function (canvas) {
        var _canvas$parentElement;

        canvas === null || canvas === void 0 ? void 0 : (_canvas$parentElement = canvas.parentElement) === null || _canvas$parentElement === void 0 ? void 0 : _canvas$parentElement.removeChild(canvas);
      });
      this.options.canvasCache = [];
    }
  }, {
    key: "getCanvas",
    value: function getCanvas(key) {
      var canvasCache = this.options.canvasCache;
      key = key > 0 ? key - 1 : key;
      return canvasCache ? canvasCache[key] : undefined;
    }
  }, {
    key: "resize",
    value: function resize(width, height) {
      var _this$options = this.options,
          limitHeight = _this$options.limitHeight,
          canvasCount = _this$options.canvasCount,
          container = _this$options.container;
      var canvasCache = this.options.canvasCache;
      var index = Math.ceil(height / (limitHeight || 0));

      if (index !== canvasCount) {
        this.removeCanvas();
        canvasCache = [];

        for (var i = 0; i < index; i++) {
          var canvas = document.createElement('canvas');
          canvas.style['vertical-align'] = 'bottom';
          canvas.setAttribute('width', width.toString());

          if (i === index - 1) {
            canvas.setAttribute('height', (height % (limitHeight || 0)).toString());
          } else {
            canvas.setAttribute('height', (limitHeight || 0).toString());
          }

          container === null || container === void 0 ? void 0 : container.appendChild(canvas);
          canvasCache.push(canvas);
        }

        this.options.canvasCache = canvasCache;
      } else {
        var _canvasCache;

        var _canvas = this.getCanvas(index);

        (_canvasCache = canvasCache) === null || _canvasCache === void 0 ? void 0 : _canvasCache.forEach(function (can) {
          can.setAttribute('width', width.toString());
        });

        if (_canvas) {
          _canvas.setAttribute('height', (height % (limitHeight || 0)).toString());
        }
      }
    }
  }, {
    key: "handleSingleRect",
    value: function handleSingleRect(options) {
      var x = options.x,
          y = options.y,
          index = options.index,
          width = options.width,
          height = options.height,
          callback = options.callback;
      var limitHeight = this.options.limitHeight;
      var canvas = this.getCanvas(index);

      if (canvas) {
        var context = canvas.getContext('2d');
        var rect = new DOMRect(x, y - (limitHeight || 0) * (index - 1), width, height);
        callback(_objectSpread(_objectSpread({}, rect.toJSON()), {}, {
          context: context
        }));
      }
    }
  }, {
    key: "drawRect",
    value: function drawRect(options) {
      var x = options.x,
          y = options.y,
          width = options.width,
          height = options.height,
          fill = options.fill,
          stroke = options.stroke;

      var callback = function callback(opts) {
        var context = opts.context;
        if (!context) return;
        context.fillStyle = fill === undefined ? '#FFEC3D' : fill;
        context.strokeStyle = stroke === undefined ? '#FFEC3D' : stroke;
        context.fillRect(opts.x, opts.y, opts.width, opts.height);
      };

      var rect = new DOMRect(x, y, width, height);
      this.handleRect(_objectSpread(_objectSpread({}, rect.toJSON()), {}, {
        callback: callback
      }));
    }
  }, {
    key: "handleRect",
    value: function handleRect(options) {
      var x = options.x,
          y = options.y,
          width = options.width,
          height = options.height,
          callback = options.callback;
      var limitHeight = this.options.limitHeight;
      var last = {
        x: x + width,
        y: y + height
      };
      var dftIndex = Math.ceil(y / (limitHeight || 0));
      var lastIndex = Math.ceil(last.y / (limitHeight || 0));
      var rect = new DOMRect(x, y, width, height);
      this.handleSingleRect(_objectSpread(_objectSpread({}, rect.toJSON()), {}, {
        index: dftIndex,
        callback: callback
      }));

      if (dftIndex !== lastIndex) {
        this.handleSingleRect(_objectSpread(_objectSpread({}, rect.toJSON()), {}, {
          index: lastIndex,
          callback: callback
        }));
      }
    }
  }, {
    key: "getImageData",
    value: function getImageData(options) {
      var x = options.x,
          y = options.y,
          width = options.width,
          height = options.height;
      var limitHeight = this.options.limitHeight;
      var index = Math.ceil(y / (limitHeight || 0));
      var canvas = this.getCanvas(index);
      var context = canvas === null || canvas === void 0 ? void 0 : canvas.getContext('2d');
      return context === null || context === void 0 ? void 0 : context.getImageData(x, y, width, height);
    }
  }, {
    key: "clearRect",
    value: function clearRect(options) {
      var x = options.x,
          y = options.y,
          width = options.width,
          height = options.height;

      var callback = function callback(opts) {
        var context = opts.context,
            x = opts.x,
            y = opts.y,
            width = opts.width,
            height = opts.height;
        context === null || context === void 0 ? void 0 : context.clearRect(x, y, width, height);
      };

      var rect = new DOMRect(x, y, width, height);
      this.handleRect(_objectSpread(_objectSpread({}, rect.toJSON()), {}, {
        callback: callback
      }));
    }
  }, {
    key: "clear",
    value: function clear() {
      var canvasCache = this.options.canvasCache;
      canvasCache === null || canvasCache === void 0 ? void 0 : canvasCache.forEach(function (canvas) {
        var context = canvas.getContext('2d');
        var width = Number(canvas.getAttribute('width'));
        var height = Number(canvas.getAttribute('height'));
        context === null || context === void 0 ? void 0 : context.clearRect(0, 0, width, height);
      });
    }
  }, {
    key: "destroy",
    value: function destroy() {
      this.removeCanvas();
    }
  }]);

  return TinyCanvas;
}();

var userAgent = (typeof navigator !== 'undefined' ? navigator : getWindow().navigator).userAgent.toLowerCase();
var isServer = typeof navigator === 'undefined';
/**
 * 是否是 Edge 浏览器
 * Mozilla/5.0 (Windows NT 10.0 Win64 x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/64.0.3282.140 Safari/537.36 Edge/17.17134
 */

var isEdge = /edge/i.test(userAgent);
/**
 * 是否是 Chrome 浏览器
 * Mozilla/5.0 (Macintosh Intel Mac OS X 10_13_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/68.0.3440.106 Safari/537.36
 */

var isChrome = !isEdge && /chrome/i.test(userAgent);
/**
 * 是否是 Firefox 浏览器
 * Mozilla/5.0 (Macintosh Intel Mac OS X 10.13 rv:62.0) Gecko/20100101 Firefox/62.0
 */

var isFirefox = /firefox/i.test(userAgent);
/**
 * 是否是 Safari 浏览器
 * Mozilla/5.0 (Macintosh Intel Mac OS X 10_13_4) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/11.1 Safari/605.1.15
 */

var isSafari = !isEdge && !isChrome && /safari/i.test(userAgent);
/**
 * 是否是 手机浏览器
 */

var isMobile = /mobile/i.test(userAgent);
/**
 * 是否是iOS系统
 */

var isIos = /os [\.\_\d]+ like mac os/i.test(userAgent);
/**
 * 是否是 安卓系统
 */

var isAndroid = /android/i.test(userAgent);
/**
 * 是否是 Mac OS X 系统
 */

var isMacos = !isIos && /mac os x/i.test(userAgent);
/**
 * 是否是 Windows 系统
 */

var isWindows = /windows\s*(?:nt)?\s*[\.\_\d]+/i.test(userAgent);

/**
 * 随机字符串
 * @param length 长度
 */

var random = function random() {
  var length = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 5;
  if (length < 5) length = 5;
  var str = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
  var word = '';

  for (var index = 0; index < length; index++) {
    word += str.charAt(Math.floor(Math.random() * str.length));
  }

  return word;
};
/**
 * 驼峰命名转换枚举
 */

var CamelCaseType;

(function (CamelCaseType) {
  CamelCaseType["UPPER"] = "upper";
  CamelCaseType["LOWER"] = "lower";
})(CamelCaseType || (CamelCaseType = {}));
/**
 * 转换为驼峰命名法
 * @param {string} value 需要转换的字符串
 * @param {upper,lower} type 转换类型，upper 大驼峰命名法，lower，小驼峰命名法（默认）
 */


var toCamelCase = function toCamelCase(value) {
  var type = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : CamelCaseType.LOWER;
  return value.split('-').map(function (str, index) {
    if (type === 'upper' || type === 'lower' && index > 0) {
      return str.charAt(0).toUpperCase() + str.substr(1);
    }

    if (type === 'lower' && index === 0) {
      return str.charAt(0).toLowerCase() + str.substr(1);
    }

    return str;
  }).join('');
};
/**
 * RGB 颜色转换为16进制颜色代码
 * @param {string} rgb
 */

var toHex = function toHex(rgb) {
  var hex = function hex(num) {
    var char = parseInt(num, 10).toString(16).toUpperCase();
    return char.length > 1 ? char : '0' + char;
  };

  var reg = /rgb\s*\(\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*\)/gi;
  return rgb.replace(reg, function ($0, $1, $2, $3) {
    return '#' + hex($1) + hex($2) + hex($3);
  });
};
/**
 * 将节点属性转换为 map 数据类型
 * @param {string} value
 */

var getAttrMap = function getAttrMap(value) {
  var map = {};
  var reg = /\s+(?:([\w\-:]+)|(?:([\w\-:]+)=([^\s"'<>]+))|(?:([\w\-:"]+)="([^"]*)")|(?:([\w\-:"]+)='([^']*)'))(?=(?:\s|\/|>)+)/g;
  var match;

  while (match = reg.exec(value)) {
    var key = (match[1] || match[2] || match[4] || match[6]).toLowerCase();
    var val = (match[2] ? match[3] : match[4] ? match[5] : match[7]) || '';
    map[key] = val;
  }

  return map;
};
/**
 * 将 style 样式转换为 map 数据类型
 * @param {string} style
 */

var getStyleMap = function getStyleMap(style) {
  style = style.replace(/&quot;/g, '"');
  var map = {};
  var reg = /\s*([\w\-]+)\s*:([^;]*)(;|$)/g;
  var match;

  while (match = reg.exec(style)) {
    var key = match[1].toLowerCase().trim();
    var val = toHex(match[2]).trim();
    map[key] = val;
  }

  return map;
};
/**
 * 使用window内置函数getComputedStyle获取节点style
 * @param {Node} node
 * @param {string} attrName
 */

var getComputedStyle = function getComputedStyle(element, attrName) {
  var win = getWindow(element);
  var camelKey = toCamelCase(attrName);
  var style = win === null || win === void 0 ? void 0 : win.getComputedStyle(element, null);
  return style ? style[camelKey] : '';
};
/**
 * 字符串编码
 * @param value 需要编码的字符串
 */

var escape = function escape(value) {
  return value.replace(/&/g, '&amp;').replace(/</g, '&lt;').replace(/>/g, '&gt;').replace(/"/g, '&quot;');
};
/**
 * 字符串解码
 * @param value 需要解码的字符串
 */

var unescape = function unescape(value) {
  return value.replace(/&lt;/g, '<').replace(/&gt;/g, '>').replace(/&quot;/g, '"').replace(/&amp;/g, '&');
};
/**
 * 字符 `.` 编码
 * @param value 需要编码的字符串
 */

var escapeDots = function escapeDots(value) {
  return value.replace(/\./g, '&dot;');
};
/**
 * 字符 `.` 解码
 * @param value 需要解码的字符串
 */

var unescapeDots = function unescapeDots(value) {
  return value.replace(/&dot;/g, '.');
};
/**
 * 验证字符串是否是合法的URL
 * @param url 需要验证的字符串
 */

var isUrl = function isUrl(url) {
  url = url.toLowerCase(); // https://developer.mozilla.org/en-US/docs/Web/HTTP/Basics_of_HTTP/Data_URIs

  if (url.startsWith('data:text/html')) {
    return false;
  }

  if (!!!url.match(/^\S*$/)) {
    return false;
  }

  if (!!['http:', 'https:', 'data:', 'ftp:'].some(function (protocol) {
    return url.startsWith(protocol);
  })) {
    return true;
  }

  if (url.startsWith('./') || url.startsWith('/')) {
    return true;
  }

  if (url.indexOf(':') < 0) {
    return true;
  }

  return false;
};
/**
 * 给值增加单位
 * @param value 值
 * @param unit 单位
 */

var addUnit = function addUnit(value) {
  var unit = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'px';
  return value && /^-?\d+(?:\.\d+)?$/.test(value.toString()) ? value + unit : value;
};
/**
 * 移除值的单位
 * @param value 值
 */

var removeUnit = function removeUnit(value) {
  var match;
  return value && (match = /^(-?\d+)/.exec(value)) ? parseInt(match[1], 10) : 0;
};
/**
 * Card组件值编码
 * @param value 需要编码的字符串
 */

var encodeCardValue = function encodeCardValue(value) {
  try {
    value = encodeURIComponent(JSON.stringify(value || ''));
  } catch (e) {
    value = '';
  }

  return 'data:'.concat(value);
};
/**
 * Card组件值解码
 * @param value 需要解码的字符串
 */

var decodeCardValue = function decodeCardValue(value) {
  try {
    value = value.substr(5);
    return JSON.parse(decodeURIComponent(value));
  } catch (e) {
    return {};
  }
};
/**
 * 转换光标以及Card组件标签为html
 * @param value 需要转换的字符串
 */

var transformCustomTags = function transformCustomTags(value) {
  return value.replace(/<anchor\s*\/>/gi, '<span '.concat(DATA_ELEMENT, '="').concat(ANCHOR, '"></span>')).replace(/<focus\s*\/>/gi, '<span '.concat(DATA_ELEMENT, '="').concat(FOCUS, '"></span>')).replace(/<cursor\s*\/>/gi, '<span '.concat(DATA_ELEMENT, '="').concat(CURSOR, '"></span>')).replace(/(<card\s+[^>]+>).*?<\/card>/gi, function (_, tag) {
    //获取Card属性
    var attrs = getAttrMap(tag);
    var type = attrs.type,
        name = attrs.name,
        value = attrs.value;
    var isInline = type === 'inline';
    var tagName = isInline ? 'span' : 'div';
    var list = ['<'.concat(tagName)];
    list.push(' '.concat(CARD_TYPE_KEY, '="').concat(type || '', '"'));
    list.push(' '.concat(READY_CARD_KEY, '="').concat(name || '', '"'));
    Object.keys(attrs).forEach(function (attrsName) {
      if (attrsName.indexOf('data-') === 0 && attrsName.indexOf('data-card') !== 0) {
        list.push(' '.concat(attrsName, '="').concat(attrs[attrsName] || '', '"'));
      }
    });

    if (value !== undefined) {
      list.push(' '.concat(CARD_VALUE_KEY, '="').concat(value, '"'));
    }

    list.push('></'.concat(tagName, '>'));
    return list.join('');
  });
};
/**
 * 验证是否是合法的url地址
 * @param url URL地址
 */

var validUrl = function validUrl(url) {
  if (typeof url !== 'string') {
    return false;
  }

  url = url.toLowerCase(); // https://developer.mozilla.org/en-US/docs/Web/HTTP/Basics_of_HTTP/Data_URIs

  if (url.startsWith('data:text/html')) {
    return false;
  }

  if (!!!url.match(/^\S*$/)) {
    return false;
  }

  if (!!['http:', 'https:', 'data:', 'ftp:'].some(function (protocol) {
    return url.startsWith(protocol);
  })) {
    return true;
  }

  if (url.startsWith('./') || url.startsWith('/')) {
    return true;
  }

  if (url.indexOf(':') < 0) {
    return true;
  }

  return false;
};
var sanitizeUrl = function sanitizeUrl(url) {
  return validUrl(url) ? url : '';
};
/**
 * 格式化编辑器值，移除光标标记标签，以及标签无效属性
 * @param value
 * @returns
 */

var formatEngineValue = function formatEngineValue(value) {
  if (!value) return value;
  var newValue = value.replace(/<(anchor|focus|cursor)[^>]*?\/>/gi, '');
  return /^<p(\s[^>]*?)><br \/><\/p>$/i.test(newValue) ? value.replace(RegExp.$1, '') : value;
};
/**
 * 格式化热键
 * @param key 热键
 */

var formatHotkey = function formatHotkey(key) {
  var keys = key.toLowerCase().split('+');
  keys = keys.map(function (key) {
    if (key === 'mod') {
      return isMacos ? '⌘' : 'Ctrl';
    } else if (key === 'opt') {
      return isMacos ? 'Option' : 'Alt';
    } else if (key.length > 1) {
      return key.substr(0, 1).toUpperCase() + key.substr(1).toLowerCase();
    }

    return key.toUpperCase();
  });
  return keys.join('+');
};

/**
 * 获取列表样式
 * @param type 类型
 * @param code
 */
var getListStyle = function getListStyle(type) {
  var code = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
  if (!(code = +code)) return '•';

  switch (type === null || type === void 0 ? void 0 : type.toLowerCase()) {
    case 'disc':
      return '•';

    case 'circle':
      return '◦';

    case 'square':
      return '◼';

    case 'lower-alpha':
      return String.fromCharCode('a'.charCodeAt(0) + code);

    case 'lower-roman':
      return String.fromCharCode(8559 + code);

    case 'decimal':
    default:
      return code;
  }
};

var CardType;

(function (CardType) {
  CardType["INLINE"] = "inline";
  CardType["BLOCK"] = "block";
})(CardType || (CardType = {}));

var ActiveTrigger;

(function (ActiveTrigger) {
  ActiveTrigger["UPDATE_CARD"] = "update_card";
  ActiveTrigger["INSERT_CARD"] = "insert_card";
  ActiveTrigger["CUSTOM_SELECT"] = "custom_select";
  ActiveTrigger["CLICK"] = "click";
  ActiveTrigger["MOUSE_DOWN"] = "mouse_down";
  ActiveTrigger["MANUAL"] = "manual";
})(ActiveTrigger || (ActiveTrigger = {}));

/**
 * 是否是引擎
 * @param editor 编辑器
 */
var isEngine = function isEngine(editor) {
  return editor.kind === 'engine';
};

var isSelection = function isSelection(param) {
  return param.getRangeAt !== undefined;
};
var isRange = function isRange(param) {
  return param.collapsed !== undefined;
};
var isRangeInterface = function isRangeInterface(selector) {
  return !!selector && selector.base !== undefined;
};

var isSchemaRule = function isSchemaRule(rule) {
  return !!rule['name'];
};

var isMarkPlugin = function isMarkPlugin(plugin) {
  return plugin.kind === 'mark';
};

var isInlinePlugin = function isInlinePlugin(plugin) {
  return plugin.kind === 'inline';
};

var isBlockPlugin = function isBlockPlugin(plugin) {
  return plugin.kind === 'block';
};

/**
 * 扩展 Node 类
 * @class NodeEntry
 * @constructor
 * @param nodes 需要扩展的 NodeList
 * @param context 节点上下文，或根节点
 */

var NodeEntry = /*#__PURE__*/function () {
  function NodeEntry(nodes, context) {
    _classCallCheck(this, NodeEntry);

    this.length = 0;
    this.events = [];
    this.document = null;
    this.context = void 0;
    this.name = '';
    this.type = void 0;
    this.window = null;
    this.display = void 0;
    this.isFragment = false;

    if (isNode(nodes)) {
      if (nodes.nodeType === getWindow().Node.DOCUMENT_FRAGMENT_NODE) this.isFragment = true;
      nodes = [nodes];
    }

    for (var i = 0; i < nodes.length; i++) {
      this[i] = nodes[i];
      this.events[i] = new Event(); // 初始化事件对象
    }

    this.length = nodes.length;

    if (this.length > 0) {
      this.document = getDocument(context);
      this.context = context;
      this.name = this[0].nodeName ? this[0].nodeName.toLowerCase() : '';
      this.type = this[0].nodeType;
      this.window = getWindow(this[0]);
    }
  }
  /**
   * 如果元素被指定的选择器字符串选择，Element.matches()  方法返回true; 否则返回false。
   * @param element 节点
   * @param selector 选择器
   */


  _createClass(NodeEntry, [{
    key: "isMatchesSelector",
    value: function isMatchesSelector(element, selector) {
      if (element.nodeType !== getWindow().Node.ELEMENT_NODE) {
        return false;
      }

      var defaultMatches = function defaultMatches(element, selector) {
        var _domNode$document;

        var domNode = new NodeEntry(element);
        var matches = (_domNode$document = domNode.document) === null || _domNode$document === void 0 ? void 0 : _domNode$document.querySelectorAll(selector),
            i = matches ? matches.length : 0;

        while (--i >= 0 && (matches === null || matches === void 0 ? void 0 : matches.item(i)) !== domNode.get()) {}

        return i > -1;
      };

      var matchesSelector = element.matches || element.webkitMatchesSelector || element.mozMatchesSelector || element.msMatchesSelector || element.oMatchesSelector || element.matchesSelector || defaultMatches;
      return matchesSelector.call(element, selector);
    }
    /**
     * 遍历
     * @param {Function} callback 回调函数
     * @return 返回当前实例
     */

  }, {
    key: "each",
    value: function each(callback) {
      for (var i = 0; i < this.length; i++) {
        if (callback(this[i], i) === false) {
          break;
        }
      }

      return this;
    }
    /**
     * 将 NodeEntry 转换为 Array
     * @return {Array} 返回数组
     */

  }, {
    key: "toArray",
    value: function toArray() {
      var nodeArray = [];
      this.each(function (node) {
        nodeArray.push(new NodeEntry(node));
      });
      return nodeArray;
    }
    /**
     * 判断当前节点是否为 Node.ELEMENT_NODE 节点类型
     * @return {boolean}
     */

  }, {
    key: "isElement",
    value: function isElement() {
      return this.type === getWindow().Node.ELEMENT_NODE;
    }
    /**
     * 判断当前节点是否为 Node.TEXT_NODE 节点类型
     * @return {boolean}
     */

  }, {
    key: "isText",
    value: function isText() {
      return this.type === getWindow().Node.TEXT_NODE;
    }
    /**
     * 判断当前节点是否为Card组件
     */

  }, {
    key: "isCard",
    value: function isCard() {
      return this.name === CARD_TAG || !!this.attributes(CARD_TYPE_KEY);
    }
    /**
     * 判断当前节点是否为block类型的Card组件
     */

  }, {
    key: "isBlockCard",
    value: function isBlockCard() {
      return 'block' === this.attributes(CARD_TYPE_KEY);
    }
    /**
     * 判断当前节点是否为inline类型的Card组件
     * @returns
     */

  }, {
    key: "isInlineCard",
    value: function isInlineCard() {
      return 'inline' === this.attributes(CARD_TYPE_KEY);
    }
    /**
     * 是否是可编辑器卡片
     * @returns
     */

  }, {
    key: "isEditableCard",
    value: function isEditableCard() {
      return this.find(EDITABLE_SELECTOR).length > 0;
    }
    /**
     * 具有 display:block css 属性的inline card
     */

  }, {
    key: "isPseudoBlockCard",
    value: function isPseudoBlockCard() {
      return this.attributes(CARD_TYPE_KEY) === 'inline' && this.css('display') === 'block';
    }
    /**
     * 判断当前节点是否为根节点
     */

  }, {
    key: "isRoot",
    value: function isRoot() {
      return this.attributes(DATA_ELEMENT) === ROOT;
    }
  }, {
    key: "isEditable",
    value: function isEditable() {
      return this.isRoot() || this.attributes(DATA_ELEMENT) === EDITABLE;
    }
    /**
     * 判断当前是否在根节点内
     */

  }, {
    key: "inEditor",
    value: function inEditor() {
      if (this.isRoot()) {
        return false;
      }

      return this.closest(ROOT_SELECTOR).length > 0;
    }
    /**
     * 是否是光标标记节点
     * @returns
     */

  }, {
    key: "isCursor",
    value: function isCursor() {
      return [ANCHOR, FOCUS, CURSOR].indexOf(this.attributes(DATA_ELEMENT)) > -1;
    }
  }, {
    key: "get",
    value: function get() {
      var index = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
      return this.length === 0 ? null : this[index];
    }
    /**
     * 获取当前第 index 节点
     * @param {number} index
     * @return {NodeEntry|undefined} NodeEntry 类，或 undefined
     */

  }, {
    key: "eq",
    value: function eq(index) {
      return index > -1 && index < this.length ? new NodeEntry(this[index]) : undefined;
    }
    /**
     * 获取当前节点所在父节点中的索引
     * @return {number} 返回索引
     */

  }, {
    key: "index",
    value: function index() {
      var _this$get;

      var prev = (_this$get = this.get()) === null || _this$get === void 0 ? void 0 : _this$get.previousSibling;
      var index = 0;

      while (prev && prev.nodeType === getWindow().Node.ELEMENT_NODE) {
        index++;
        prev = prev.previousSibling;
      }

      return index;
    }
    /**
     * 获取当前节点父节点
     * @return 父节点
     */

  }, {
    key: "parent",
    value: function parent() {
      var _this$get2;

      var node = (_this$get2 = this.get()) === null || _this$get2 === void 0 ? void 0 : _this$get2.parentNode;
      return node ? new NodeEntry(node) : undefined;
    }
    /**
     * 查询当前节点的子节点
     * @param selector 查询器
     * @return 符合条件的子节点
     */

  }, {
    key: "children",
    value: function children(selector) {
      if (0 === this.length) return new NodeEntry([]);
      var childNodes = this.get().childNodes;

      if (selector) {
        var nodes = [];

        for (var i = 0; i < childNodes.length; i++) {
          var node = childNodes[i];

          if (this.isMatchesSelector(node, selector)) {
            nodes.push(node);
          }
        }

        return new NodeEntry(nodes);
      }

      return new NodeEntry(childNodes);
    }
    /**
     * 获取当前节点第一个子节点
     * @return NodeEntry 子节点
     */

  }, {
    key: "first",
    value: function first() {
      var _this$get3;

      if (this.isFragment) return this.eq(0) || null;
      var node = this.length === 0 ? null : (_this$get3 = this.get()) === null || _this$get3 === void 0 ? void 0 : _this$get3.firstChild;
      return node ? new NodeEntry(node) : null;
    }
    /**
     * 获取当前节点最后一个子节点
     * @return NodeEntry 子节点
     */

  }, {
    key: "last",
    value: function last() {
      var _this$get4;

      if (this.isFragment) return this.eq(this.length - 1) || null;
      var node = this.length === 0 ? null : (_this$get4 = this.get()) === null || _this$get4 === void 0 ? void 0 : _this$get4.lastChild;
      return node ? new NodeEntry(node) : null;
    }
    /**
     * 返回元素节点之前的兄弟节点（包括文本节点、注释节点）
     * @return NodeEntry 节点
     */

  }, {
    key: "prev",
    value: function prev() {
      var _this$get5;

      var node = this.length === 0 ? null : (_this$get5 = this.get()) === null || _this$get5 === void 0 ? void 0 : _this$get5.previousSibling;
      return node ? new NodeEntry(node) : null;
    }
    /**
     * 返回元素节点之后的兄弟节点（包括文本节点、注释节点）
     * @return NodeEntry 节点
     */

  }, {
    key: "next",
    value: function next() {
      var _this$get6;

      var node = this.length === 0 ? null : (_this$get6 = this.get()) === null || _this$get6 === void 0 ? void 0 : _this$get6.nextSibling;
      return node ? new NodeEntry(node) : null;
    }
    /**
     * 返回元素节点之前的兄弟元素节点（不包括文本节点、注释节点）
     * @return NodeEntry 节点
     */

  }, {
    key: "prevElement",
    value: function prevElement() {
      var node = this.length === 0 || !this.isElement() ? null : this.get().previousElementSibling;
      return node ? new NodeEntry(node) : null;
    }
    /**
     * 返回元素节点之后的兄弟元素节点（不包括文本节点、注释节点）
     * @return NodeEntry 节点
     */

  }, {
    key: "nextElement",
    value: function nextElement() {
      var node = this.length === 0 || !this.isElement() ? null : this.get().nextElementSibling;
      return node ? new NodeEntry(node) : null;
    }
    /**
     * 返回元素节点所在根节点路径，默认根节点为 document.body
     * @param {Node} context 根节点，默认为 document.body
     * @return {number} 路径
     */

  }, {
    key: "getPath",
    value: function getPath(context) {
      context = context || document.body;
      var path = [];

      if (this.length > 0) {
        path.unshift(this.offset());
      }

      var parent = this.parent();

      while (parent && !parent.equal(context)) {
        path.unshift(parent.offset());
        parent = parent.parent();
      }

      return path;
    }
    /**
     * 判断元素节点是否包含要查询的节点
     * @param {NodeInterface | Node} node 要查询的节点
     * @return {boolean} 是否包含
     */

  }, {
    key: "contains",
    value: function contains(node) {
      var _domNode;

      var domNode = isNode(node) ? node : node.get();

      if (this.length === 0) {
        return false;
      }

      if (this.get().nodeType === getWindow().Node.DOCUMENT_NODE && ((_domNode = domNode) === null || _domNode === void 0 ? void 0 : _domNode.nodeType) !== getWindow().Node.DOCUMENT_NODE) {
        return true;
      }

      while (domNode = ((_domNode2 = domNode) === null || _domNode2 === void 0 ? void 0 : _domNode2.parentNode) || null) {
        var _domNode2;

        if (domNode === this[0]) {
          return true;
        }
      }

      return false;
    }
    /**
     * 根据查询器查询当前元素节点
     * @param {string} selector 查询器
     * @return 返回一个 NodeEntry 实例
     */

  }, {
    key: "find",
    value: function find(selector) {
      if (this.length > 0 && this.isElement()) {
        var _this$get7;

        var nodeList = (_this$get7 = this.get()) === null || _this$get7 === void 0 ? void 0 : _this$get7.querySelectorAll(selector);
        return new NodeEntry(nodeList || []);
      }

      return new NodeEntry([]);
    }
    /**
     * 根据查询器查询符合条件的离当前元素节点最近的父节点
     * @param selector 查询器
     * @return 返回一个 NodeEntry 实例
     */

  }, {
    key: "closest",
    value: function closest(selector) {
      var callback = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : function (node) {
        return node.parentNode || undefined;
      };
      var nodeList = [];
      var node = this.get() || undefined;

      while (node) {
        if (this.isMatchesSelector(node, selector)) {
          nodeList.push(node);
          return new NodeEntry(nodeList);
        }

        node = callback(node);
      }

      return new NodeEntry(nodeList);
    }
    /**
     * 为当前元素节点绑定事件
     * @param {string} eventType 事件类型
     * @param {Function} listener 事件函数
     * @return 返回当前实例
     */

  }, {
    key: "on",
    value: function on(eventType, listener) {
      var _this = this;

      this.each(function (node, i) {
        node.addEventListener(eventType, listener, false);
        if (i < _this.events.length) _this.events[i].on(eventType, listener);
      });
      return this;
    }
    /**
     * 移除当前元素节点事件
     * @param {string} eventType 事件类型
     * @param {Function} listener 事件函数
     * @return 返回当前实例
     */

  }, {
    key: "off",
    value: function off(eventType, listener) {
      var _this2 = this;

      this.each(function (node, i) {
        node.removeEventListener(eventType, listener, false);
        if (i < _this2.events.length) _this2.events[i].off(eventType, listener);
      });
      return this;
    }
    /**
     * 获取当前元素节点相对于视口的位置
     * @param {Object} defaultValue 默认值
     * @return {Object}
     * {
     *  top,
     *  bottom,
     *  left,
     *  right
     * }
     */

  }, {
    key: "getBoundingClientRect",
    value: function getBoundingClientRect(defaultValue) {
      if (this.length === 0) return undefined;

      try {
        var element = this.get();
        var rect = element.getBoundingClientRect();
        var top = document.documentElement.clientTop;
        var left = document.documentElement.clientLeft;
        return {
          top: rect.top - top,
          bottom: rect.bottom - top,
          left: rect.left - left,
          right: rect.right - left
        };
      } catch (error) {
        console.error(error);
      }

      return defaultValue;
    }
    /**
     * 移除当前元素所有已绑定的事件
     * @return 当前 NodeEntry 实例
     */

  }, {
    key: "removeAllEvents",
    value: function removeAllEvents() {
      var _this3 = this;

      this.each(function (node, i) {
        if (!_this3.events[i]) {
          return;
        }

        Object.keys(_this3.events[i].listeners).forEach(function (eventType) {
          var listeners = _this3.events[i].listeners[eventType];

          for (var _i = 0; _i < listeners.length; _i++) {
            node.removeEventListener(eventType, listeners[_i], false);
          }
        });
      });
      this.events = [];
      return this;
    }
    /**
     * 获取当前元素节点相对于视口的位置
     * @return {Object}
     * {
     *  top,
     *  left
     * }
     */

  }, {
    key: "offset",
    value: function offset() {
      if (this.length === 0) return -1;
      var parent = this.parent();
      if (!parent) return -1;
      var children = parent.get().childNodes;
      return Array.from(children).indexOf(this.get());
    }
  }, {
    key: "attributes",
    value: function attributes(key, val) {
      var _this4 = this;

      if (key === undefined) {
        var element = this.clone(false).get();
        return getAttrMap((element === null || element === void 0 ? void 0 : element.outerHTML) || '');
      }

      if (_typeof(key) === 'object') {
        Object.keys(key).forEach(function (k) {
          var v = key[k];

          _this4.attributes(k, v);
        });
        return this;
      }

      if (val === undefined) {
        var _element = this.get();

        return this.length > 0 && this.isElement() ? (_element === null || _element === void 0 ? void 0 : _element.getAttribute(key)) || '' : '';
      }

      this.each(function (node) {
        var element = node;
        element.setAttribute(key, val.toString());
      });
      return this;
    }
    /**
     * 移除元素节点属性
     * @param {string} key 属性名称
     * @return 返当前实例
     */

  }, {
    key: "removeAttributes",
    value: function removeAttributes(key) {
      this.each(function (node) {
        var element = node;
        element.removeAttribute(key);
      });
      return this;
    }
    /**
     * 判断元素节点是否包含某个 class
     * @param {string} className 样式名称
     * @return {boolean} 是否包含
     */

  }, {
    key: "hasClass",
    value: function hasClass(className) {
      if (this.length === 0) return false;
      var element = this.get();

      if (element.classList) {
        for (var i = 0; i < element.classList.length; i++) {
          if (element.classList[i] === className) {
            return true;
          }
        }
      }

      return false;
    }
    /**
     * 为元素节点增加一个 class
     * @param {string} className
     * @return 返当前实例
     */

  }, {
    key: "addClass",
    value: function addClass(className) {
      this.each(function (node) {
        var element = node;
        element.classList.add(className);
      });
      return this;
    }
    /**
     * 移除元素节点 class
     * @param {string} className
     * @return 返当前实例
     */

  }, {
    key: "removeClass",
    value: function removeClass(className) {
      this.each(function (node) {
        var element = node;
        element.classList.remove(className);
      });
      return this;
    }
  }, {
    key: "css",
    value: function css(key, val) {
      var _this5 = this;

      if (key === undefined) {
        // 没有参数，返回style所有属性
        return getStyleMap(this.attributes('style') || '');
      }

      if (_typeof(key) === 'object') {
        Object.keys(key).forEach(function (attr) {
          var value = key[attr];

          _this5.css(attr, value);
        });
        return this;
      } // 获取style样式值


      if (val === undefined) {
        if (this.length === 0 || this.isText()) {
          return '';
        }

        var element = this.get();
        return element.style[toCamelCase(key)] || getComputedStyle(this[0], key) || '';
      }

      this.each(function (node) {
        var element = node;
        element.style[toCamelCase(key)] = val.toString();
      });
      return this;
    }
    /**
     * 获取元素节点宽度
     * @return {number} 宽度
     */

  }, {
    key: "width",
    value: function width() {
      var width = this.css('width');

      if (width === 'auto') {
        var element = this.get();
        width = element.offsetWidth.toString();
      }

      return width ? parseFloat(width) || 0 : 0;
    }
    /**
     * 获取元素节点高度
     * @return {number} 高度
     */

  }, {
    key: "height",
    value: function height() {
      var height = this.css('height');

      if (height === 'auto') {
        var element = this.get();
        height = element.offsetHeight.toString();
      }

      return height ? parseFloat(height) || 0 : 0;
    }
  }, {
    key: "html",
    value: function html(_html) {
      if (_html) {
        this.each(function (node) {
          node.innerHTML = _html;
        });
        return this;
      }

      return this.length > 0 ? this[0].innerHTML : '';
    }
  }, {
    key: "text",
    value: function text(_text) {
      var _this$get8;

      // 返回的数据包含 HTML 特殊字符，innerHTML 之前需要 escape
      // https://developer.mozilla.org/en-US/docs/Web/API/Node/textContent
      if (_text !== undefined) {
        this.each(function (node) {
          node.textContent = _text;
        });
        return this;
      }

      if (this.length === 0) return '';
      return ((_this$get8 = this.get()) === null || _this$get8 === void 0 ? void 0 : _this$get8.textContent) || '';
    }
    /**
     * 设置元素节点为显示状态
     * @param {string} display display值
     * @return 当前实例
     */

  }, {
    key: "show",
    value: function show(display) {
      if (display === undefined) {
        display = this.display || '';
      }

      if (display === 'none') {
        display = '';
      }

      if (this.css('display') !== 'none') {
        return this;
      }

      return this.css('display', display);
    }
    /**
     * 设置元素节点为隐藏状态
     * @return 当前实例
     */

  }, {
    key: "hide",
    value: function hide() {
      var _this$get9;

      if (this.length === 0) {
        return this;
      }

      this.display = (_this$get9 = this.get()) === null || _this$get9 === void 0 ? void 0 : _this$get9.style.display;
      return this.css('display', 'none');
    }
    /**
     * 移除当前实例所有元素节点
     * @return 当前实例
     */

  }, {
    key: "remove",
    value: function remove() {
      var _this6 = this;

      this.each(function (node, index) {
        if (!node.parentNode) {
          return;
        }

        node.parentNode.removeChild(node);
        delete _this6[index];
      });
      this.length = 0;
      return this;
    }
    /**
     * 清空元素节点下的所有子节点
     * @return 当前实例
     */

  }, {
    key: "empty",
    value: function empty() {
      this.each(function (node) {
        var child = node.firstChild;

        while (child) {
          var _child$parentNode;

          if (!node.parentNode) {
            return;
          }

          var next = child.nextSibling;
          (_child$parentNode = child.parentNode) === null || _child$parentNode === void 0 ? void 0 : _child$parentNode.removeChild(child);
          child = next;
        }
      });
      return this;
    }
    /**
     * 比较两个元素节点是否相同
     * @param {NodeEntry|Node} node 比较的节点
     * @return {boolean} 是否相同
     */

  }, {
    key: "equal",
    value: function equal(node) {
      if (isNode(node)) return this.get() === node;
      if (isNodeEntry(node)) return this.get() === node.get();
      return false;
    }
  }, {
    key: "clone",
    value: function clone(deep) {
      var nodes = [];
      this.each(function (node) {
        nodes.push(node.cloneNode(deep));
      });
      return new NodeEntry(nodes);
    }
    /**
     * 在元素节点的开头插入指定内容
     * @param selector 选择器或元素节点
     * @return 当前实例
     */

  }, {
    key: "prepend",
    value: function prepend(selector) {
      var _this7 = this;

      this.each(function (node) {
        var nodes = domParser(selector, _this7.context);

        if (node.firstChild) {
          node.insertBefore(nodes[0], node.firstChild);
        } else {
          node.appendChild(nodes[0]);
        }
      });
      return this;
    }
    /**
     * 在元素节点的结尾插入指定内容
     * @param selector 选择器或元素节点
     * @return 当前实例
     */

  }, {
    key: "append",
    value: function append(selector) {
      var _this8 = this;

      this.each(function (node) {
        var nodes = domParser(selector, _this8.context);

        for (var i = 0; i < nodes.length; i++) {
          var child = nodes[i];

          if (typeof selector === 'string') {
            node.appendChild(child.cloneNode(true));
          } else {
            node.appendChild(child);
          }
        }
      });
      return this;
    }
    /**
     * 在元素节点前插入新的节点
     * @param selector 选择器或元素节点
     * @return 当前实例
     */

  }, {
    key: "before",
    value: function before(selector) {
      var _this9 = this;

      this.each(function (node) {
        var _node$parentNode;

        var nodes = domParser(selector, _this9.context);
        (_node$parentNode = node.parentNode) === null || _node$parentNode === void 0 ? void 0 : _node$parentNode.insertBefore(nodes[0], node);
      });
      return this;
    }
    /**
     * 在元素节点后插入内容
     * @param selector 选择器或元素节点
     * @return 当前实例
     */

  }, {
    key: "after",
    value: function after(selector) {
      var _this10 = this;

      this.each(function (node) {
        var nodes = domParser(selector, _this10.context);

        if (node.nextSibling) {
          var _node$parentNode2;

          (_node$parentNode2 = node.parentNode) === null || _node$parentNode2 === void 0 ? void 0 : _node$parentNode2.insertBefore(nodes[0], node.nextSibling);
        } else {
          var _node$parentNode3;

          (_node$parentNode3 = node.parentNode) === null || _node$parentNode3 === void 0 ? void 0 : _node$parentNode3.appendChild(nodes[0]);
        }
      });
      return this;
    }
    /**
     * 将元素节点替换为新的内容
     * @param selector 选择器或元素节点
     * @return 当前实例
     */

  }, {
    key: "replaceWith",
    value: function replaceWith(selector) {
      var _this11 = this;

      var newNodes = [];
      this.each(function (node) {
        var _node$parentNode4;

        var nodes = domParser(selector, _this11.context);
        var newNode = nodes[0];
        (_node$parentNode4 = node.parentNode) === null || _node$parentNode4 === void 0 ? void 0 : _node$parentNode4.replaceChild(newNode, node);
        newNodes.push(newNode);
      });
      return new NodeEntry(newNodes);
    }
  }, {
    key: "getRoot",
    value: function getRoot() {
      return this.closest(ROOT_SELECTOR);
    }
  }, {
    key: "traverse",
    value: function traverse(callback) {
      var order = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

      var walk = function walk(node) {
        var child = order ? node.first() : node.last();

        while (child) {
          var next = order ? child.next() : child.prev();
          var result = callback(child);

          if (result === false) {
            return;
          }

          if (!child.isCard() && result !== true) {
            walk(child);
          }

          child = next;
        }
      };

      callback(this);
      walk(this);
    }
  }, {
    key: "getChildByPath",
    value: function getChildByPath(path, filter) {
      var node = this.get();

      var getChildNodes = function getChildNodes() {
        return filter ? Array.from(node.childNodes).filter(filter) : Array.from(node.childNodes);
      };

      var childNodes = getChildNodes();

      for (var i = 0; path[i] !== undefined && childNodes[path[i]];) {
        node = childNodes[path[i]];
        childNodes = getChildNodes();
        i++;
      }

      return node;
    }
  }, {
    key: "inside",
    value: function inside(node) {
      var callback = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : function (node) {
        return node.parentNode || undefined;
      };
      var parentNode = this[0].parentNode;

      while (parentNode) {
        if (parentNode === (isNodeEntry(node) ? node[0] : node)) return true;
        parentNode = callback(parentNode) || null;
      }

      return false;
    }
  }, {
    key: "getIndex",
    value: function getIndex(filter) {
      var index = 0;
      var parent = this[0].parentNode;
      if (!parent) return index;
      index = (filter ? Array.from(parent.childNodes).filter(filter) : Array.from(parent.childNodes)).indexOf(this.get());
      return index < 0 ? 0 : index;
    }
  }, {
    key: "findParent",
    value: function findParent() {
      var container = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.closest(ROOT_SELECTOR);
      if (isNode(container)) container = new NodeEntry(container);
      if (this.length === 0 || !this.parent()) return null;
      var node = this;

      while (!((_node$parent = node.parent()) === null || _node$parent === void 0 ? void 0 : _node$parent.equal(container))) {
        var _node$parent;

        if (!node.parent()) return null;
        node = node.parent();
      }

      return node;
    }
  }, {
    key: "allChildren",
    value: function allChildren() {
      var childNodes = [];
      this.traverse(function (node) {
        childNodes.push(node[0]);
      });
      childNodes.shift();
      return childNodes;
    }
  }, {
    key: "getViewport",
    value: function getViewport(node) {
      var _ref = this.window || {
        innerHeight: 0,
        innerWidth: 0
      },
          innerHeight = _ref.innerHeight,
          innerWidth = _ref.innerWidth;

      var top, left, bottom, right;

      if (node && node.length > 0) {
        var element = node.get();
        var rect = element.getBoundingClientRect();
        top = rect.top;
        left = rect.left;
        bottom = rect.bottom;
        right = rect.right;
      } else {
        var _element2 = this.get();

        var _rect = _element2.getBoundingClientRect();

        top = _rect.top;
        left = _rect.left;
        bottom = _rect.bottom;
        right = _rect.right;
      }

      return {
        top: top,
        left: left,
        bottom: Math.min(bottom, innerHeight),
        right: Math.min(right, innerWidth)
      };
    }
  }, {
    key: "inViewport",
    value: function inViewport(node, view) {
      var _viewNode$parentNode;

      var viewNode = null;

      if (view.type !== getWindow().Node.ELEMENT_NODE) {
        if (!view.document) return false;
        viewNode = view.document.createElement('span');

        if (view.next()) {
          var _view$0$parentNode;

          (_view$0$parentNode = view[0].parentNode) === null || _view$0$parentNode === void 0 ? void 0 : _view$0$parentNode.insertBefore(viewNode, view[0].nextSibling);
        } else {
          var _view$0$parentNode2;

          (_view$0$parentNode2 = view[0].parentNode) === null || _view$0$parentNode2 === void 0 ? void 0 : _view$0$parentNode2.appendChild(viewNode);
        }

        view = new NodeEntry(viewNode);
      }

      var viewElement = view[0];

      var _viewElement$getBound = viewElement.getBoundingClientRect(),
          top = _viewElement$getBound.top,
          left = _viewElement$getBound.left,
          right = _viewElement$getBound.right,
          bottom = _viewElement$getBound.bottom;

      var vp = this.getViewport(node);
      if (viewNode) (_viewNode$parentNode = viewNode.parentNode) === null || _viewNode$parentNode === void 0 ? void 0 : _viewNode$parentNode.removeChild(viewNode);
      return !(top < vp.top || bottom > vp.bottom || left < vp.left || right > vp.right);
    }
  }, {
    key: "scrollIntoView",
    value: function scrollIntoView(node, view) {
      var _view$document;

      var align = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'nearest';

      if (typeof ((_view$document = view.document) === null || _view$document === void 0 ? void 0 : _view$document.body.scrollIntoView) === 'function') {
        var _viewElement$parentNo;

        var viewElement = null;

        if (view.type !== getWindow().Node.ELEMENT_NODE || view.name.toLowerCase() === 'br') {
          var _view$0$parentNode3;

          viewElement = view.document.createElement('span');
          viewElement.innerHTML = '&nbsp;';
          (_view$0$parentNode3 = view[0].parentNode) === null || _view$0$parentNode3 === void 0 ? void 0 : _view$0$parentNode3.insertBefore(viewElement, view[0]);
          view = new NodeEntry(viewElement);
        }

        if (!this.inViewport(node, view)) {
          var _view$get;

          (_view$get = view.get()) === null || _view$get === void 0 ? void 0 : _view$get.scrollIntoView({
            block: align,
            inline: align
          });
        }

        if (viewElement) (_viewElement$parentNo = viewElement.parentNode) === null || _viewElement$parentNo === void 0 ? void 0 : _viewElement$parentNo.removeChild(viewElement);
      }
    }
  }]);

  return NodeEntry;
}();

var JSONML = {
  TAG_NAME_INDEX: 0,
  ATTRIBUTE_INDEX: 1,
  ELEMENT_LIST_OFFSET: 2
};

/**
 * 查询节点返回NodeInterface
 * @param selector 选择器
 * @param context 节点上下文，或根节点
 * @param nodeConstructor 需要使用的模型，默认 DOMNOde
 */

var $ = (function (selector, context, clazz) {
  if (context === undefined) context = getDocument();
  var nodes = domParser(selector, context);
  var entry = new (clazz || NodeEntry)(nodes, context ? context : undefined);
  if (isNode(selector) && selector.nodeType === getWindow().Node.DOCUMENT_FRAGMENT_NODE) entry.isFragment = true;
  return entry;
});

var defaultConversion = [{
  from: function from(_name, _styles, attributes) {
    return !!attributes[CARD_KEY] || !!attributes[READY_CARD_KEY];
  },
  to: function to(_, style, attributes) {
    var value = attributes[CARD_VALUE_KEY];

    var oldAttrs = _objectSpread({}, attributes);

    attributes = {
      type: attributes[CARD_TYPE_KEY],
      name: (attributes[CARD_KEY] || attributes[READY_CARD_KEY]).toLowerCase()
    }; //其它 data 属性

    Object.keys(oldAttrs).forEach(function (attrName) {
      if (attrName !== READY_CARD_KEY && attrName.indexOf('data-') === 0 && attrName.indexOf('data-card') !== 0) {
        attributes[attrName] = oldAttrs[attrName];
      }
    });

    if (value !== undefined) {
      attributes.value = value;
    }
    var card = $('<card />');
    Object.keys(attributes).forEach(function (name) {
      card.attributes(name, attributes[name]);
    });
    return card;
  }
}];

var _attributes, _attributes2, _attributes3, _attributes4;
var defualtSchema = [{
  name: 'p',
  type: 'block'
}, {
  name: 'br',
  type: 'inline',
  isVoid: true
}, {
  name: ANCHOR,
  type: 'inline',
  isVoid: true
}, {
  name: FOCUS,
  type: 'inline',
  isVoid: true
}, {
  name: CURSOR,
  type: 'inline',
  isVoid: true
}, {
  type: 'block',
  attributes: {
    'data-id': '*'
  }
}, {
  name: 'card',
  type: 'inline',
  attributes: {
    name: {
      required: true,
      value: /\w+/
    },
    type: {
      required: true,
      value: 'inline'
    },
    value: '*'
  }
}, {
  name: 'span',
  type: 'inline',
  attributes: (_attributes = {}, _defineProperty(_attributes, CARD_KEY, {
    required: true,
    value: /\w+/
  }), _defineProperty(_attributes, CARD_TYPE_KEY, {
    required: true,
    value: 'inline'
  }), _defineProperty(_attributes, CARD_VALUE_KEY, '*'), _defineProperty(_attributes, "class", '*'), _defineProperty(_attributes, "contenteditable", '*'), _attributes)
}, {
  name: 'span',
  type: 'inline',
  attributes: (_attributes2 = {}, _defineProperty(_attributes2, READY_CARD_KEY, {
    required: true,
    value: /\w+/
  }), _defineProperty(_attributes2, CARD_TYPE_KEY, {
    required: true,
    value: 'inline'
  }), _defineProperty(_attributes2, CARD_VALUE_KEY, '*'), _defineProperty(_attributes2, "class", '*'), _defineProperty(_attributes2, "contenteditable", '*'), _attributes2)
}, {
  name: 'card',
  type: 'block',
  attributes: {
    name: {
      required: true,
      value: /\w+/
    },
    type: {
      required: true,
      value: 'block'
    },
    value: '*'
  }
}, {
  name: 'div',
  type: 'block',
  attributes: (_attributes3 = {}, _defineProperty(_attributes3, CARD_KEY, {
    required: true,
    value: /\w+/
  }), _defineProperty(_attributes3, CARD_TYPE_KEY, {
    required: true,
    value: 'block'
  }), _defineProperty(_attributes3, CARD_VALUE_KEY, '*'), _defineProperty(_attributes3, "class", '*'), _defineProperty(_attributes3, "contenteditable", '*'), _attributes3)
}, {
  name: 'div',
  type: 'block',
  attributes: (_attributes4 = {}, _defineProperty(_attributes4, READY_CARD_KEY, {
    required: true,
    value: /\w+/
  }), _defineProperty(_attributes4, CARD_TYPE_KEY, {
    required: true,
    value: 'block'
  }), _defineProperty(_attributes4, CARD_VALUE_KEY, '*'), _defineProperty(_attributes4, "class", '*'), _defineProperty(_attributes4, "contenteditable", '*'), _attributes4)
}];

var _counters = {};
var getHashId = (function (value) {
  var unique = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
  var prefix = '';
  if (isNode(value)) value = $(value);

  if (isNodeEntry(value)) {
    var attributes = value.attributes();
    var styles = attributes['style'];
    delete attributes['style'];
    prefix = value.name.substring(0, 1);
    value = "".concat(value.name, "_").concat(Object.keys(attributes || {}).join(','), "_").concat(Object.values(attributes || {}).join(','), "_").concat(Object.keys(styles || {}).join(','), "_").concat(Object.values(styles || {}).join(','));
  }

  var hash = prefix + md5(value).substr(0, 8);

  if (unique) {
    var counter = _counters[hash] || 0;
    _counters[hash] = counter + 1;

    if (counter > 0) {
      hash = "".concat(hash, "-").concat(counter);
    }
  }

  return hash;
});

var _excluded = ["style"];

var NodeModel = /*#__PURE__*/function () {
  function NodeModel(editor) {
    _classCallCheck(this, NodeModel);

    this.editor = void 0;
    this.editor = editor;
  }

  _createClass(NodeModel, [{
    key: "isVoid",
    value: function isVoid(node, schema) {
      schema = schema || this.editor.schema;
      var name = typeof node === 'string' ? node : '';
      if (isNode(node)) name = node.nodeName.toLowerCase();else if (isNodeEntry(node)) name = node.name;
      return schema.find(function (rule) {
        return rule.name === name;
      }).some(function (rule) {
        return rule.isVoid;
      });
    }
  }, {
    key: "isMark",
    value: function isMark(node, schema) {
      schema = schema || this.editor.schema;
      if (isNode(node)) node = $(node);
      return schema.getType(node) === 'mark';
    }
    /**
     * 是否是inline标签
     * @param node 节点
     */

  }, {
    key: "isInline",
    value: function isInline(node, schema) {
      schema = schema || this.editor.schema;
      if (isNode(node)) node = $(node);
      return schema.getType(node) === 'inline';
    }
    /**
     * 是否是块级节点
     * @param node 节点
     */

  }, {
    key: "isBlock",
    value: function isBlock(node, schema) {
      schema = schema || this.editor.schema;
      if (isNode(node)) node = $(node);
      return schema.getType(node) === 'block';
    }
    /**
     * 判断当前节点是否为block类型的简单节点（子节点不包含blcok标签）
     */

  }, {
    key: "isSimpleBlock",
    value: function isSimpleBlock(node) {
      if (!this.isBlock(node)) return false;
      var child = node.first();

      while (child) {
        if (this.isBlock(child)) return false;
        child = child.next();
      }

      return true;
    }
    /**
     * 判断节点是否是顶级根节点，父级为编辑器根节点，且，子级节点没有block节点
     * @param node 节点
     * @returns
     */

  }, {
    key: "isRootBlock",
    value: function isRootBlock(node, schema) {
      var _node$parent;

      //父级不是根节点
      if (!((_node$parent = node.parent()) === null || _node$parent === void 0 ? void 0 : _node$parent.isEditable())) return false;
      if (!this.isSimpleBlock(node)) return false; //并且规则上不可以设置子节点

      return (schema || this.editor.schema).find(function (schema) {
        return schema.name === node.name;
      }).every(function (schema) {
        if (schema.type !== 'block') return false;
        var allowIn = schema.allowIn;
        if (!allowIn) return true;
        return allowIn.indexOf('$root') > -1;
      });
    }
    /**
     * 判断节点下的文本是否为空
     * @param withTrim 是否 trim
     */

  }, {
    key: "isEmpty",
    value: function isEmpty(node, withTrim) {
      var _value, _value2;

      if (node.isElement()) {
        //卡片不为空
        if (node.attributes(CARD_KEY) || node.find(CARD_SELECTOR).length > 0) {
          return false;
        } //只读卡片不为空


        if (node.attributes(READY_CARD_KEY) || node.find(READY_CARD_SELECTOR).length > 0) {
          return false;
        } //非br节点的空节点不为空


        if (node.name !== 'br' && this.isVoid(node)) {
          return false;
        } //多个br节点不为空


        if (node.find('br').length > 1) {
          return false;
        }
      }

      var value = node.isText() ? node[0].nodeValue || '' : node.text();
      value = (_value = value) === null || _value === void 0 ? void 0 : _value.replace(/\u200B/g, '');
      value = (_value2 = value) === null || _value2 === void 0 ? void 0 : _value2.replace(/\r\n|\n/, '');

      if (value && withTrim) {
        value = value.trim();
      }

      return value === '';
    }
    /**
     * 判断一个节点下的文本是否为空，或者只有空白字符
     */

  }, {
    key: "isEmptyWithTrim",
    value: function isEmptyWithTrim(node) {
      return this.isEmpty(node, true);
    }
  }, {
    key: "isLikeEmpty",
    value: function isLikeEmpty(node) {
      if (node.length === 0) return true;
      var childNodes = node[0].childNodes;
      if (childNodes.length === 0) return true;

      for (var i = 0; i < childNodes.length; i++) {
        var child = childNodes[i];

        if (child.nodeType === getWindow().Node.TEXT_NODE) {
          if (child['data'].replace(/\u200b/g, '') !== '') return false;
        } else if (child.nodeType === getWindow().Node.ELEMENT_NODE) {
          if (child.hasAttribute(CARD_KEY)) return false;

          if (!this.isLikeEmpty($(child))) {
            return false;
          }
        }
      }

      return true;
    }
    /**
     * 判断节点是否为列表节点
     * @param node 节点或者节点名称
     */

  }, {
    key: "isList",
    value: function isList(node) {
      var name = typeof node === 'string' ? node : '';
      if (isNode(node)) name = node.nodeName.toLowerCase();else if (isNodeEntry(node)) name = node.name;
      return ['ul', 'ol'].indexOf(name) > -1;
    }
    /**
     * 判断节点是否是自定义列表
     * @param node 节点
     */

  }, {
    key: "isCustomize",
    value: function isCustomize(node) {
      var list = this.editor.list;

      switch (node.name) {
        case 'li':
          return node.hasClass(list.CUSTOMZIE_LI_CLASS);

        case 'ul':
          return node.hasClass(list.CUSTOMZIE_UL_CLASS);

        default:
          return false;
      }
    }
    /**
     * 去除包裹
     * @param node 需要去除包裹的节点
     */

  }, {
    key: "unwrap",
    value: function unwrap(node) {
      var child = node.first();

      while (child) {
        var next = child.next();
        node.before(child);
        child = next;
      }

      node.remove();
    }
    /**
     * 包裹节点
     * @param source 需要包裹的节点
     * @param outer 包裹的外部节点
     * @param mergeSame 合并相同名称的节点样式和属性在同一个节点上
     */

  }, {
    key: "wrap",
    value: function wrap(source, outer) {
      var _this = this;

      var mergeSame = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
      var _this$editor = this.editor,
          node = _this$editor.node,
          mark = _this$editor.mark;
      if (isNode(source)) source = $(source);
      outer = node.clone(outer, false); // 文本节点

      if (source.isText()) {
        outer.append(node.clone(source, false));
        return source.replaceWith(outer);
      } // 包裹样式节点


      if (mergeSame && this.isMark(outer)) {
        //合并属性和样式值
        var outerClone = node.clone(outer, false);

        if (source.name === outer.name) {
          var attrs = source.attributes();
          delete attrs.style;
          Object.keys(attrs).forEach(function (key) {
            if (!outer.attributes(key)) outer.attributes(key, attrs[key]);else {
              var attributes = outer.attributes(key).split(',');
              if (attributes.indexOf(attrs[key]) < 0) attributes.push(attrs[key]);
              outer.attributes(key, attributes.join(','));
            }
          });
          var styles = source.css();
          Object.keys(styles).forEach(function (key) {
            if (!outer.css(key)) outer.css(key, styles[key]);
          });
          outer.append(node.clone(source, true).children());
        } else {
          outer.append(node.clone(source, true));
        }

        var children = outer.allChildren();
        children.forEach(function (node) {
          var child = $(node);

          if (!child.isText() && _this.isMark(child) && mark.compare(child, outerClone)) {
            _this.unwrap(child);
          }
        });
        return source.replaceWith(outer);
      } // 其它情况


      var shadowNode = node.clone(source, false);
      source.after(shadowNode);
      outer.append(source);
      return shadowNode.replaceWith(outer);
    }
    /**
     * 合并节点
     * @param source 合并的节点
     * @param target 需要合并的节点
     * @param remove 合并后是否移除
     */

  }, {
    key: "merge",
    value: function merge(source, target) {
      var remove = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;

      //要合并的节点是文本，就直接追加
      if (target.isText()) {
        source.append(target);
        return;
      }

      var _this$editor2 = this.editor,
          node = _this$editor2.node,
          block = _this$editor2.block,
          mark = _this$editor2.mark;
      var mergedNode = target;
      var toIsList = ['ul', 'ol'].includes(source.name);
      var fromIsList = ['ul', 'ol'].includes(target.name); // p 与列表合并时需要做特殊处理

      if (toIsList && !fromIsList) {
        var liBlocks = source.find('li'); //没有li标签

        if (liBlocks.length === 0) {
          return;
        } //设置被合并节点为最后一个li标签


        source = $(liBlocks[liBlocks.length - 1]);
      } //被合并的节点为列表


      if (!toIsList && fromIsList) {
        //查找li节点
        var _liBlocks = target.find('li');

        if (_liBlocks.length > 0) {
          //设置需要合并的节点为第一个li节点
          target = $(_liBlocks[0]);
        }

        if (_liBlocks[1]) {
          mergedNode = $(_liBlocks[0]);
        }
      } //被合并的节点最后一个子节点为br，则移除


      var toNodeLast = source.last();
      var child = target.first();
      var plugin = block.findPlugin(source); //循环追加

      while (child) {
        var next = child.next();
        var markPlugin = mark.findPlugin(child);

        if (plugin && markPlugin && plugin.disableMark && plugin.disableMark.indexOf(markPlugin.constructor.pluginName) > -1) {
          node.unwrap(child);
        } //追加到要合并的列表中


        source.append(child);
        child = next;
      } //移除需要合并的节点


      if (remove) mergedNode.remove();

      if (toNodeLast && toNodeLast.name === 'br') {
        var _next = toNodeLast.next();

        while (_next) {
          if ([CURSOR, ANCHOR, FOCUS].indexOf(_next.attributes(DATA_ELEMENT))) {
            toNodeLast.remove();
            break;
          }

          _next = _next.next();
        }
      }
    }
    /**
     * 将源节点的子节点追加到目标节点，并替换源节点
     * @param source 旧节点
     * @param target 新节点
     */

  }, {
    key: "replace",
    value: function replace(source, target) {
      var _source$first, _source$first2;

      var clone = this.editor.node.clone(target, false);
      var childNode = this.isCustomize(source) && source.name === 'li' && ((_source$first = source.first()) === null || _source$first === void 0 ? void 0 : _source$first.isCard()) ? (_source$first2 = source.first()) === null || _source$first2 === void 0 ? void 0 : _source$first2.next() : source.first();

      while (childNode) {
        var nextNode = childNode.next();
        clone.append(childNode);
        childNode = nextNode;
      }

      return source.replaceWith(clone);
    }
    /**
     * 光标位置插入文本
     * @param text 文本
     * @param range 光标
     */

  }, {
    key: "insertText",
    value: function insertText(text, range) {
      var _this$insert;

      if (!isEngine(this.editor)) return;
      var change = this.editor.change;
      var safeRange = range || change.getSafeRange();
      var doc = getDocument(safeRange.startContainer); // 范围为折叠状态时先删除内容

      if (!safeRange.collapsed) {
        change.deleteContent(range);
      }

      var node = doc.createTextNode(text);
      (_this$insert = this.insert(node, safeRange)) === null || _this$insert === void 0 ? void 0 : _this$insert.addOrRemoveBr();
      if (!range) change.apply(safeRange);
      return safeRange;
    }
    /**
     * 在光标位置插入一个节点
     * @param node 节点
     * @param range 光标
     */

  }, {
    key: "insert",
    value: function insert(node, range) {
      if (isNodeEntry(node)) {
        if (node.length === 0) throw 'Not found node';
        node = node[0];
      }

      var editor = this.editor;
      if (!isEngine(editor)) return;
      var change = editor.change;
      range = range || change.getRange();
      var nodeApi = editor.node;

      var _range$cloneRange$shr = range.cloneRange().shrinkToTextNode(),
          startNode = _range$cloneRange$shr.startNode,
          startOffset = _range$cloneRange$shr.startOffset;

      var prev = startNode.prev();
      var parent = startNode.parent();
      var text = startNode.text() || '';
      var leftText = text.substr(0, startOffset); //文本节点

      if (startNode.isText() && /\u200b$/.test(leftText)) {
        //零宽字符前面还有其它字符。或者节点前面还有节点，不能是inline节点。或者前面没有节点了，并且父级不是inline节点
        if (text.length > 1 || prev && !nodeApi.isInline(prev) || !prev && parent && !nodeApi.isInline(parent)) {
          startNode.get().splitText(text.length - 1).remove();
        }
      }

      range.insertNode(node);
      return range.select(node, true).shrinkToElementNode().collapse(false);
    }
    /**
     * 设置节点属性
     * @param node 节点
     * @param props 属性
     */

  }, {
    key: "setAttributes",
    value: function setAttributes(node, attributes) {
      var style = attributes.style,
          attrs = _objectWithoutProperties(attributes, _excluded);

      Object.keys(attrs).forEach(function (key) {
        if (key === 'className') {
          var value = attrs[key];

          if (Array.isArray(value)) {
            value.forEach(function (name) {
              return node.addClass(name);
            });
          } else node.addClass(value);
        } else node.attributes(key, attrs[key].toString());
      });
      if (typeof style === 'number') style = {};
      if (typeof style === 'string') style = getStyleMap(style);
      Object.keys(style || {}).forEach(function (key) {
        var val = style[key];

        if (/^0(px|em)?$/.test(val.toString())) {
          val = '';
        }

        node.css(key, val.toString());
      });
      return node;
    }
    /**
     * 移除值为负的样式
     * @param node 节点
     * @param style 样式名称
     */

  }, {
    key: "removeMinusStyle",
    value: function removeMinusStyle(node, style) {
      if (this.isBlock(node)) {
        var val = parseInt(node.css(style), 10) || 0;
        if (val < 0) node.css(style, '');
      }
    }
    /**
     * 合并节点下的子节点，两个相同的相邻节点的子节点
     * @param node 当前节点
     */

  }, {
    key: "mergeAdjacent",
    value: function mergeAdjacent(node) {
      var _this$editor3 = this.editor,
          schema = _this$editor3.schema,
          list = _this$editor3.list;
      var nodeApi = this.editor.node;
      var topTags = schema.getAllowInTags(); //获取第一个子节点

      var childDom = node.first(); //遍历全部子节点

      while (childDom) {
        //获取下一个子节点
        var nextNode = childDom.next();

        while ( //如果下一个子节点不为空，并且与上一个子节点名称一样
        nextNode && childDom.name === nextNode.name && ( //并且上一个节点是可拥有block子节点的节点 或者是 ul、li 并且list列表类型是一致的
        topTags.indexOf(childDom.name) > -1 && !nodeApi.isList(childDom) || nodeApi.isList(childDom) && list.isSame(childDom, nextNode))) {
          //获取下一个节点的下一个节点
          var nNextNode = nextNode.next(); //合并下一个节点

          var nextChildNode = nextNode.first(); //循环要合并节点的子节点

          while (nextChildNode) {
            var next = nextChildNode.next();
            childDom.append(nextChildNode);
            nextChildNode = next;
          }

          nextNode.remove(); //继续合并当前子节点的子节点

          this.mergeAdjacent(childDom);
          nextNode = nNextNode;
        }

        childDom = nextNode;
      }
    }
    /**
     * 删除节点两边标签
     * @param node 节点
     * @param tagName 标签名称，默认为br标签
     */

  }, {
    key: "removeSide",
    value: function removeSide(node) {
      var tagName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'br';
      // 删除第一个 BR
      var firstNode = node.first();

      if ((firstNode === null || firstNode === void 0 ? void 0 : firstNode.name) === tagName && node.children().length > 1) {
        firstNode.remove();
      } // 删除最后一个 BR


      var lastNode = node.last();

      if ((lastNode === null || lastNode === void 0 ? void 0 : lastNode.name) === tagName && node.children().length > 1) {
        lastNode.remove();
      }
    }
    /**
     * 整理节点
     * @param node 节点
     * @param root 根节点
     */

  }, {
    key: "flatten",
    value: function flatten(node) {
      var _root$get;

      var root = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : node;
      var block = this.editor.block;
      var nodeApi = this.editor.node; //第一个子节点

      var childNode = node.first();
      var rootElement = root.isFragment ? (_root$get = root.get()) === null || _root$get === void 0 ? void 0 : _root$get.parentNode : root.get();
      var tempNode = node.isFragment ? $('<p />') : nodeApi.clone(node, false);

      while (childNode) {
        //获取下一个兄弟节点
        var nextNode = childNode.next(); //如果当前子节点是块级的Card组件，或者是简单的block

        if (childNode.isBlockCard() || this.isSimpleBlock(childNode)) {
          block.flatten(childNode, $(rootElement || []));
        } //如果当前是块级标签，递归循环
        else if (this.isBlock(childNode)) {
            this.flatten(childNode, $(rootElement || []));
          } else {
            var cloneNode = nodeApi.clone(tempNode, false);
            var isLI = 'li' === cloneNode.name;
            childNode.before(cloneNode);

            while (childNode) {
              var _nextNode;

              nextNode = childNode.next();
              var isBR = 'br' === childNode.name && !isLI; //判断当前节点末尾是否是换行符，有换行符就跳出

              if (childNode.isText()) {
                var text = childNode.text(); //先移除开头的换行符

                var match = /^((\n|\r)+)/.exec(text);
                var isBegin = false;

                if (match) {
                  text = text.substring(match[1].length);
                  isBegin = true;

                  if (text.length === 0) {
                    childNode.remove();
                    break;
                  }
                } //移除末尾换行符


                match = /((\n|\r)+)$/.exec(text);

                if (match) {
                  childNode.text(text.substr(0, match.index));
                  cloneNode.append(childNode);
                  break;
                } else if (isBegin) {
                  childNode.text(text);
                }
              }

              cloneNode.append(childNode); //判断下一个节点的开头是换行符，有换行符就跳出

              if ((_nextNode = nextNode) === null || _nextNode === void 0 ? void 0 : _nextNode.isText()) {
                var _text = nextNode.text();

                var _match = /^(\n|\r)+/.exec(_text);

                if (_match) {
                  break;
                }
              }

              if (isBR || !nextNode || this.isBlock(nextNode) || nextNode.isBlockCard()) break;
              childNode = nextNode;
            }

            this.removeSide(cloneNode);
            block.flatten(cloneNode, $(rootElement || []));

            if (cloneNode.name === 'p' && cloneNode.children().length === 0) {
              cloneNode.append($('<br />'));
            }
          }

        childNode = nextNode;
      } //如果没有子节点了，就移除当前这个节点


      childNode = node.first();
      if (!childNode) node.remove();
    }
    /**
     * 标准化节点
     * @param node 节点
     */

  }, {
    key: "normalize",
    value: function normalize(node) {
      this.flatten(node);
      this.mergeAdjacent(node);
    }
  }, {
    key: "html",
    value: function html(node, val) {
      var _this2 = this;

      if (val === undefined) {
        var _node$get;

        return node.length > 0 ? ((_node$get = node.get()) === null || _node$get === void 0 ? void 0 : _node$get.innerHTML) || '' : '';
      }

      node.each(function (node) {
        var element = node;
        element.innerHTML = val;

        _this2.editor.block.generateRandomIDForDescendant(element);
      });
      return node;
    }
    /**
     * 复制元素节点
     * @param {boolean} deep 是否深度复制
     * @return 复制后的元素节点
     */

  }, {
    key: "clone",
    value: function clone(node, deep) {
      var block = this.editor.block;
      var nodes = [];
      node.each(function (node) {
        var cloneNode = node.cloneNode(deep);
        block.generateRandomIDForDescendant(cloneNode, true);

        if (block.needMarkDataID(cloneNode.nodeName)) {
          block.generateRandomID(cloneNode, true);
        }

        nodes.push(cloneNode);
      });
      return $(nodes);
    }
    /**
     * 获取批量追加子节点后的outerHTML
     * @param nodes 节点集合
     * @param appendExp 追加的节点
     */

  }, {
    key: "getBatchAppendHTML",
    value: function getBatchAppendHTML(nodes, appendExp) {
      var _appendNode$get;

      if (nodes.length === 0) return appendExp;
      var appendNode = $(appendExp);
      nodes.forEach(function (node) {
        node = node.clone(false);
        node.append(appendNode);
        appendNode = node;
      });
      return ((_appendNode$get = appendNode.get()) === null || _appendNode$get === void 0 ? void 0 : _appendNode$get.outerHTML) || '';
    }
  }, {
    key: "removeZeroWidthSpace",
    value: function removeZeroWidthSpace(node) {
      var nodeApi = this.editor.node;
      node.traverse(function (child) {
        var node = child[0];

        if (node.nodeType !== getWindow().Node.TEXT_NODE) {
          return;
        }

        var text = node.nodeValue;

        if ((text === null || text === void 0 ? void 0 : text.length) !== 2) {
          return;
        }

        var next = node.nextSibling;
        var prev = node.previousSibling;

        if (text.charCodeAt(1) === 0x200b && next && next.nodeType === getWindow().Node.ELEMENT_NODE && [ANCHOR, FOCUS, CURSOR].indexOf(next.getAttribute(DATA_ELEMENT) || '') >= 0) {
          return;
        }

        var parent = child.parent();

        if (text.charCodeAt(1) === 0x200b && (!next && parent && nodeApi.isInline(parent) || next && nodeApi.isInline(next))) {
          return;
        }

        if (text.charCodeAt(0) === 0x200b && (!prev && parent && nodeApi.isInline(parent) || prev && nodeApi.isInline(prev))) {
          return;
        }

        if (text.charCodeAt(0) === 0x200b) {
          var _newNode$parentNode;

          var newNode = node.splitText(1);
          if (newNode.previousSibling) (_newNode$parentNode = newNode.parentNode) === null || _newNode$parentNode === void 0 ? void 0 : _newNode$parentNode.removeChild(newNode.previousSibling);
        }
      });
    }
  }]);

  return NodeModel;
}();

var en = {
  dnd: {
    title: 'Drag to reposition'
  },
  copy: {
    title: 'Copy',
    success: 'Copied successfully',
    error: 'Copy error'
  },
  delete: {
    title: 'Delete'
  },
  copyAnchor: {
    title: 'Copy anchor link'
  },
  link: {
    placeholder: 'Please enter a link or anchor and press Enter to confirm',
    save: 'Apply',
    edit: 'Change',
    delete: 'Remove link',
    open: 'Open link',
    text: 'link'
  },
  copyContent: {
    title: 'copy content'
  },
  maximize: {
    title: 'Maximize',
    back: 'Back to document'
  },
  expand: {
    title: 'Embedded preview'
  },
  collapse: {
    title: 'Compact display'
  },
  card: {
    lockAlert: 'Please wait for the other user to finish editing'
  },
  preferences: {
    title: 'Preferences'
  },
  download: {
    title: 'Download'
  },
  more: {
    title: 'More'
  },
  checkMarkdown: {
    title: 'It is detected that the paste content conforms to the Markdown syntax. Do you need to do style conversion?'
  }
};

var cn = {
  dnd: {
    title: '拖动调整位置'
  },
  copy: {
    title: '复制',
    success: '复制成功',
    error: '复制失败'
  },
  delete: {
    title: '删除'
  },
  copyAnchor: {
    title: '复制锚点链接'
  },
  link: {
    placeholder: '请输入链接或锚点，回车确认',
    save: '保存',
    edit: '编辑',
    delete: '取消链接',
    open: '打开链接',
    text: '链接'
  },
  copyContent: {
    title: '复制内容'
  },
  maximize: {
    title: '最大化',
    back: '返回文档'
  },
  expand: {
    title: '嵌入预览'
  },
  collapse: {
    title: '紧凑展示'
  },
  card: {
    lockAlert: '请等待对方编辑完毕后，再进入编辑'
  },
  preferences: {
    title: '设置'
  },
  download: {
    title: '下载'
  },
  more: {
    title: '更多'
  },
  checkMarkdown: {
    title: '检测到粘贴内容符合 Markdown 语法，是否需要转换？'
  }
};

var language = {
  en: en,
  'zh-cn': cn
};

var Selection = /*#__PURE__*/function () {
  /**
   * 移除光标位置占位标签
   * @param value 需要移除的字符串
   */
  function Selection(editor, range) {
    _classCallCheck(this, Selection);

    this.range = void 0;
    this.editor = void 0;
    this.anchor = null;
    this.focus = null;
    this.editor = editor;
    this.range = range;
  }

  _createClass(Selection, [{
    key: "has",
    value: function has() {
      return !!this.focus && !!this.anchor;
    }
  }, {
    key: "create",
    value: function create() {
      var _this$range = this.range,
          commonAncestorNode = _this$range.commonAncestorNode,
          startNode = _this$range.startNode,
          endNode = _this$range.endNode; // 超出编辑区域

      if (!commonAncestorNode.isEditable() && !commonAncestorNode.inEditor()) {
        return;
      }

      var document = commonAncestorNode.document;
      if (!document) return; // 为了增加容错性，删除已有的标记

      var root = commonAncestorNode.closest(ROOT_SELECTOR);
      root.find(ANCHOR_SELECTOR).remove();
      root.find(FOCUS_SELECTOR).remove();
      root.find(FOCUS_SELECTOR).remove(); // card 组件

      var card = startNode.closest(CARD_SELECTOR);

      if (card.length > 0 && !card.isPseudoBlockCard()) {
        var cardLeft = startNode.closest(CARD_LEFT_SELECTOR);

        if (cardLeft.length > 0) {
          this.range.setStartBefore(card);
        }

        var cardRight = startNode.closest(CARD_RIGHT_SELECTOR);

        if (cardRight.length > 0) {
          this.range.setStartAfter(card);
        }
      }

      if (!startNode.equal(endNode)) {
        var _card = endNode.closest(CARD_SELECTOR); // 具有 block css 属性的行内Card，不调整光标位置


        if (_card.length > 0 && !_card.isPseudoBlockCard()) {
          var _cardLeft = endNode.closest(CARD_LEFT_SELECTOR);

          if (_cardLeft.length > 0) {
            this.range.setEndBefore(_card);
          }

          var _cardRight = endNode.closest(CARD_RIGHT_SELECTOR);

          if (_cardRight.length > 0) {
            this.range.setEndAfter(_card);
          }
        }
      } // cursor


      if (this.range.collapsed) {
        var cursor = $(document.createElement('span'));
        cursor.attributes(DATA_ELEMENT, CURSOR);
        this.range.insertNode(cursor);
        this.anchor = cursor;
        this.focus = cursor;
        return;
      } // anchor


      var startRange = this.range.cloneRange();
      startRange.collapse(true);
      var anchor = $(document.createElement('span'));
      anchor.attributes(DATA_ELEMENT, ANCHOR);
      startRange.insertNode(anchor);
      this.range.setStartAfter(anchor); // focus

      var endRange = this.range.cloneRange();
      endRange.collapse(false);
      var focus = $(document.createElement('span'));
      focus.attributes(DATA_ELEMENT, FOCUS);
      endRange.insertNode(focus);
      this.anchor = anchor;
      this.focus = focus;
    }
  }, {
    key: "move",
    value: function move() {
      if (!this.focus || !this.anchor) {
        return;
      }

      var node = this.editor.node;

      if (this.anchor === this.focus) {
        var cursor = this.anchor;

        var _parent = cursor.parent();

        if (!_parent) return;
        node.removeZeroWidthSpace(_parent);

        _parent[0].normalize();

        var isCardCursor = false;
        var prevNode = cursor.prev();
        var nextNode = cursor.next(); // 具有 block css 属性的行内Card，不调整光标位置

        if (prevNode && prevNode.isCard() && !prevNode.isPseudoBlockCard()) {
          var cardRight = prevNode.find(CARD_RIGHT_SELECTOR);

          if (cardRight.length > 0) {
            this.range.select(cardRight, true);
            this.range.collapse(false);
            isCardCursor = true;
          }
        } else if (nextNode && nextNode.isCard() && !nextNode.isPseudoBlockCard()) {
          var cardLeft = nextNode.find(CARD_LEFT_SELECTOR);

          if (cardLeft.length > 0) {
            this.range.select(cardLeft, true);
            this.range.collapse(false);
            isCardCursor = true;
          }
        }

        if (!isCardCursor) {
          this.range.setStartBefore(cursor[0]);
          this.range.collapse(true);
        }

        if (isEdge) {
          _parent[0].normalize();

          cursor.remove();
        } else {
          cursor.remove();

          _parent[0].normalize();
        }

        return;
      } // collapsed = false
      // range start


      var parent = this.anchor.parent();

      if (parent) {
        node.removeZeroWidthSpace(parent);
        this.range.setStartBefore(this.anchor);
        this.anchor.remove();
        parent[0].normalize();
      } // range end


      parent = this.focus.parent();

      if (parent) {
        node.removeZeroWidthSpace(parent);
        this.range.setEndBefore(this.focus);
        this.focus.remove();
        parent[0].normalize();

        if (isSafari) {
          var selection = window.getSelection();
          selection === null || selection === void 0 ? void 0 : selection.removeAllRanges();
          selection === null || selection === void 0 ? void 0 : selection.addRange(this.range.base);
        }
      }
    }
  }, {
    key: "getNode",
    value: function getNode(source) {
      var position = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'center';
      var isClone = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
      var callback = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : function () {
        return true;
      };
      var node = isClone ? source.clone(true) : source;

      if (!this.focus || !this.anchor) {
        return node;
      } // 删除右侧


      if (position === 'left' || position === 'center') {
        var selectionNode = position !== 'center' ? this.anchor : this.focus;
        var focus = $("[".concat(DATA_ELEMENT, "=").concat(selectionNode.attributes(DATA_ELEMENT), "]"), node.get());
        var isRemove = false;
        node.traverse(function (node) {
          if (node.equal(focus)) {
            focus.remove();
            isRemove = true;
            return;
          }

          if (isRemove && callback(node)) node.remove();
        }, true);
      } // 删除左侧


      if (position === 'right' || position === 'center') {
        var _selectionNode = position !== 'center' ? this.focus : this.anchor;

        var anchor = $("[".concat(DATA_ELEMENT, "=").concat(_selectionNode.attributes(DATA_ELEMENT), "]"), node.get());
        var _isRemove = false;
        node.traverse(function (node) {
          if (node.equal(anchor)) {
            anchor.remove();
            _isRemove = true;
            return;
          }

          if (_isRemove && callback(node)) node.remove();
        }, false);
      }

      return node;
    }
  }]);

  return Selection;
}();

Selection.removeTags = function (value) {
  return value.replace(/<anchor\s*\/>/gi, '').replace(/<focus\s*\/>/gi, '').replace(/<cursor\s*\/>/gi, '');
};

var Range = /*#__PURE__*/function () {
  function Range(editor, _range) {
    var _this = this;

    _classCallCheck(this, Range);

    this.editor = void 0;
    this.base = void 0;

    this.toRange = function () {
      return _this.base;
    };

    this.collapse = function (toStart) {
      _this.base.collapse(toStart);

      return _this;
    };

    this.cloneRange = function () {
      return Range.from(_this.editor, _this.base.cloneRange());
    };

    this.select = function (node, contents) {
      if (contents) {
        _this.base.selectNodeContents(isNodeEntry(node) ? node[0] : node);
      } else {
        _this.base.selectNode(isNodeEntry(node) ? node[0] : node);
      }

      return _this;
    };

    this.getText = function () {
      var contents = _this.cloneContents();

      return contents.textContent;
    };

    this.getClientRect = function () {
      var item = _this.getClientRects().item(0);

      if (!item) {
        item = _this.getBoundingClientRect();
      }

      return item;
    };

    this.enlargeFromTextNode = function () {
      var enlargePosition = function enlargePosition(node, offset, type) {
        var _node$nodeValue;

        if (node.nodeType !== getWindow().Node.TEXT_NODE) {
          return;
        }

        if (offset === 0) {
          switch (type) {
            case 'start':
              _this.setStartBefore(node);

              break;

            case 'end':
              _this.setEndBefore(node);

              break;
          }
        } else if (offset === ((_node$nodeValue = node.nodeValue) === null || _node$nodeValue === void 0 ? void 0 : _node$nodeValue.length)) {
          switch (type) {
            case 'start':
              _this.setStartAfter(node);

              break;

            case 'end':
              _this.setEndAfter(node);

              break;
          }
        }
      };

      enlargePosition(_this.startContainer, _this.startOffset, 'start');
      enlargePosition(_this.endContainer, _this.endOffset, 'end');
      return _this;
    };

    this.shrinkToTextNode = function () {
      var shrinkPosition = function shrinkPosition(node, offset, type) {
        if (node.nodeType !== getWindow().Node.ELEMENT_NODE) {
          return;
        }

        var childNodes = node.childNodes;

        if (childNodes.length === 0) {
          return;
        }

        var left;
        var right;
        var child;

        if (offset > 0) {
          left = childNodes[offset - 1];
        }

        if (offset < childNodes.length) {
          right = childNodes[offset];
        }

        if (left && left.nodeType === getWindow().Node.TEXT_NODE) {
          var _child$nodeValue;

          child = left;
          offset = ((_child$nodeValue = child.nodeValue) === null || _child$nodeValue === void 0 ? void 0 : _child$nodeValue.length) || 0;
        }

        if (right && right.nodeType === getWindow().Node.TEXT_NODE) {
          child = right;
          offset = 0;
        }

        if (!child) {
          return;
        }

        switch (type) {
          case 'start':
            _this.setStart(child, offset);

            break;

          case 'end':
            _this.setEnd(child, offset);

            break;
        }
      };

      shrinkPosition(_this.startContainer, _this.startOffset, 'start');
      shrinkPosition(_this.endContainer, _this.endOffset, 'end');
      return _this;
    };

    this.enlargeToElementNode = function (toBlock) {
      var range = _this.enlargeFromTextNode();

      var nodeApi = _this.editor.node;

      var enlargePosition = function enlargePosition(node, offset, isStart) {
        var domNode = $(node);

        if (domNode.type === getWindow().Node.TEXT_NODE || !toBlock && nodeApi.isBlock(domNode) || domNode.isEditable()) {
          return;
        }

        var parent;

        if (offset === 0) {
          while (!domNode.prev()) {
            parent = domNode.parent();

            if (!parent || !toBlock && nodeApi.isBlock(parent)) {
              break;
            }

            if (!parent.inEditor()) {
              break;
            }

            domNode = parent;
          }

          if (isStart) {
            range.setStartBefore(domNode[0]);
          } else {
            range.setEndBefore(domNode[0]);
          }
        } else if (offset === domNode.children().length) {
          while (!domNode.next()) {
            parent = domNode.parent();

            if (!parent || !toBlock && nodeApi.isBlock(parent)) {
              break;
            }

            if (!parent.inEditor()) {
              break;
            }

            domNode = parent;
          }

          if (isStart) {
            range.setStartAfter(domNode[0]);
          } else {
            range.setEndAfter(domNode[0]);
          }
        }
      };

      enlargePosition(range.startContainer, range.startOffset, true);
      enlargePosition(range.endContainer, range.endOffset, false);
      return _this;
    };

    this.shrinkToElementNode = function () {
      var node = _this.editor.node;
      var child;
      var childDom;

      while (_this.startContainer.nodeType === getWindow().Node.ELEMENT_NODE && (child = _this.startContainer.childNodes[_this.startOffset]) && (childDom = $(child)) && child.nodeType === getWindow().Node.ELEMENT_NODE && !childDom.isCursor() && !node.isVoid(child) && !childDom.isCard()) {
        _this.setStart(child, 0);
      }

      while (_this.endContainer.nodeType === getWindow().Node.ELEMENT_NODE && _this.endOffset > 0 && (child = _this.endContainer.childNodes[_this.endOffset - 1]) && (childDom = $(child)) && child.nodeType === getWindow().Node.ELEMENT_NODE && !node.isVoid(child) && !childDom.isCursor() && !childDom.isCard()) {
        _this.setEnd(child, child.childNodes.length);
      }

      return _this;
    };

    this.createSelection = function () {
      var selection = new Selection(_this.editor, _this);
      selection.create();
      return selection;
    };

    this.getSubRanges = function () {
      var includeCard = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
      var ranges = [];

      _this.commonAncestorNode.traverse(function (child) {
        if (child.isText()) {
          var _childNode$nodeValue;

          var offset = 0;
          var childNode = child.get();
          var valueLength = ((_childNode$nodeValue = childNode.nodeValue) === null || _childNode$nodeValue === void 0 ? void 0 : _childNode$nodeValue.length) || 0;

          var start = _this.comparePoint(childNode, offset);

          var end = _this.comparePoint(childNode, valueLength);

          var docRange = Range.create(_this.editor);

          if (start < 0) {
            if (end < 0) return;

            if (end === 0) {
              docRange.setOffset(childNode, _this.startOffset, valueLength);
            } else {
              docRange.setOffset(childNode, _this.startOffset, _this.endOffset);
            }
          } else {
            if (start !== 0) return;
            if (end < 0) return;

            if (end === 0) {
              docRange.setOffset(childNode, offset, valueLength);
            } else {
              docRange.setOffset(childNode, offset, _this.endOffset);
            }
          }

          ranges.push(docRange);
        } else if (includeCard && child.isCard() && !child.isEditableCard()) {
          var _center$get;

          var cardComponent = _this.editor.card.find(child);

          if (!cardComponent || cardComponent.constructor.singleSelectable === false) return;
          var center = cardComponent.getCenter();
          var body = (_center$get = center.get()) === null || _center$get === void 0 ? void 0 : _center$get.parentNode;
          if (!body) return;

          var _offset = center.index();

          var _childNode = child.get();

          var _start = _this.comparePoint(body, _offset);

          var _end = _this.comparePoint(body, _offset + 1);

          var _docRange = Range.create(_this.editor);

          if (_start < 0) {
            if (_end < 0) return;

            if (_end === 0) {
              _docRange.setOffset(_childNode, _this.startOffset, _offset + 1);
            } else {
              _docRange.setOffset(_childNode, _this.startOffset, _this.endOffset);
            }
          } else {
            if (_start !== 0) return;
            if (_end < 0) return;

            if (_end === 0) {
              _docRange.setOffset(body, _offset, _offset + 1);
            } else {
              _docRange.setOffset(body, _offset, _this.endOffset);
            }
          }

          ranges.push(_docRange);
        }
      });

      return ranges;
    };

    this.setOffset = function (node, start, end) {
      if (isNodeEntry(node)) node = node[0];

      _this.setStart(node, start);

      _this.setEnd(node, end);

      return _this;
    };

    this.findElementsInSimpleRange = function () {
      var startContainer = _this.startContainer,
          endContainer = _this.endContainer,
          startOffset = _this.startOffset,
          endOffset = _this.endOffset,
          collapsed = _this.collapsed;
      var elements = [];

      if (startContainer !== endContainer || collapsed === true || startContainer.nodeType === getWindow().Node.TEXT_NODE) {
        return elements;
      }

      var childNodes = startContainer.childNodes;

      for (var i = startOffset; i < endOffset; i++) {
        elements.push(childNodes[i]);
      }

      return elements;
    };

    this.inCard = function () {
      var card = _this.startNode.closest(CARD_SELECTOR);

      return card && card.length > 0;
    };

    this.getStartOffsetNode = function () {
      var startContainer = _this.startContainer,
          startOffset = _this.startOffset;

      if (startContainer.nodeType === getWindow().Node.ELEMENT_NODE) {
        return startContainer.childNodes[startOffset] || startContainer.childNodes[startOffset - 1] || startContainer;
      }

      return startContainer;
    };

    this.getEndOffsetNode = function () {
      var endContainer = _this.endContainer,
          endOffset = _this.endOffset;

      if (endContainer.nodeType === getWindow().Node.ELEMENT_NODE) {
        return endContainer.childNodes[endOffset] || endContainer.childNodes[endOffset - 1] || endContainer;
      }

      return endContainer;
    };

    this.scrollIntoView = function () {
      var endElement = _this.endNode.get();

      if (isMobile && endElement && endElement.scrollIntoView) {
        endElement.scrollIntoView({
          behavior: 'smooth',
          block: 'center',
          inline: 'center'
        });
      }
    };

    this.scrollRangeIntoView = function () {
      var node = _this.getEndOffsetNode();

      var root = node.nodeType === getWindow().Node.TEXT_NODE ? node.parentNode : node;
      var rect = _this.collapsed ? root.getBoundingClientRect() : _this.getClientRect();
      var innerHeight = window.innerHeight;

      if (rect.bottom >= innerHeight || rect.bottom <= 0) {
        root.scrollIntoView({
          block: 'center'
        });
      }
    };

    this.scrollIntoViewIfNeeded = function (node, view) {
      if (_this.collapsed) {
        node.scrollIntoView(view, $(_this.getEndOffsetNode()));
      } else {
        var startNode = _this.getStartOffsetNode();

        var endNode = _this.getEndOffsetNode();

        node.scrollIntoView(view, $(startNode));
        if (!node.inViewport(view, $(endNode))) node.scrollIntoView(view, $(endNode));
      }
    };

    this.containsCard = function () {
      var collapsed = _this.collapsed,
          commonAncestorNode = _this.commonAncestorNode;
      return !collapsed && (3 !== commonAncestorNode.type && commonAncestorNode.find(CARD_SELECTOR).length > 0 || commonAncestorNode.closest(CARD_SELECTOR).length > 0);
    };

    this.addOrRemoveBr = function (isLeft) {
      var _block$first, _block$first2, _block$last;

      var block = _this.editor.block.closest(_this.commonAncestorNode);

      block.find('br').each(function (br) {
        var _domBr$parent, _domBr$parent2, _domBr$parent2$first, _domBr$prev;

        var domBr = $(br);

        if ((!domBr.prev() || ((_domBr$parent = domBr.parent()) === null || _domBr$parent === void 0 ? void 0 : _domBr$parent.hasClass('data-list-item')) && ((_domBr$parent2 = domBr.parent()) === null || _domBr$parent2 === void 0 ? void 0 : (_domBr$parent2$first = _domBr$parent2.first()) === null || _domBr$parent2$first === void 0 ? void 0 : _domBr$parent2$first.equal(domBr.prev()))) && domBr.next() && domBr.next().name !== 'br' && ![CURSOR, ANCHOR, FOCUS].includes(domBr.next().attributes(DATA_ELEMENT)) || !domBr.next() && domBr.prev() && ((_domBr$prev = domBr.prev()) === null || _domBr$prev === void 0 ? void 0 : _domBr$prev.name) !== 'br') {
          var _domBr$parent3, _domBr$parent4, _domBr$parent4$first;

          if (isLeft && domBr.prev() && !(((_domBr$parent3 = domBr.parent()) === null || _domBr$parent3 === void 0 ? void 0 : _domBr$parent3.hasClass('data-list-item')) && ((_domBr$parent4 = domBr.parent()) === null || _domBr$parent4 === void 0 ? void 0 : (_domBr$parent4$first = _domBr$parent4.first()) === null || _domBr$parent4$first === void 0 ? void 0 : _domBr$parent4$first.equal(domBr.prev())))) return;
          domBr.remove();
        }
      });

      if (!block.first() || block.children().length === 1 && block.hasClass('data-list-item') && ((_block$first = block.first()) === null || _block$first === void 0 ? void 0 : _block$first.isCard())) {
        block.append($('<br />'));
        return _this;
      }

      if (block.children().length === 2 && block.hasClass('data-list-item') && ((_block$first2 = block.first()) === null || _block$first2 === void 0 ? void 0 : _block$first2.isCard()) && ['cursor', 'anchor', 'focus'].includes(((_block$last = block.last()) === null || _block$last === void 0 ? void 0 : _block$last.attributes(DATA_ELEMENT)) || '')) {
        var _block$first3;

        (_block$first3 = block.first()) === null || _block$first3 === void 0 ? void 0 : _block$first3.after('<br />');
      }

      return _this;
    };

    this.getPrevNode = function () {
      _this.enlargeFromTextNode();

      var startNode = _this.startNode,
          startOffset = _this.startOffset;

      if (startNode.isText()) {
        return;
      }

      var childNodes = startNode.children();

      if (childNodes.length === 0) {
        return;
      }

      return childNodes.eq(startOffset - 1);
    };

    this.getNextNode = function () {
      _this.enlargeFromTextNode();

      var endNode = _this.endNode,
          endOffset = _this.endOffset;

      if (endNode.isText()) {
        return;
      }

      var childNodes = endNode.children();

      if (childNodes.length === 0) {
        return;
      }

      return childNodes.eq(endOffset);
    };

    this.editor = editor;
    this.base = _range;
  }

  _createClass(Range, [{
    key: "collapsed",
    get: function get() {
      return this.base.collapsed;
    }
  }, {
    key: "endOffset",
    get: function get() {
      return this.base.endOffset;
    }
  }, {
    key: "startOffset",
    get: function get() {
      return this.base.startOffset;
    }
  }, {
    key: "startContainer",
    get: function get() {
      return this.base.startContainer;
    }
  }, {
    key: "endContainer",
    get: function get() {
      return this.base.endContainer;
    }
  }, {
    key: "commonAncestorContainer",
    get: function get() {
      return this.base.commonAncestorContainer;
    }
  }, {
    key: "cloneContents",
    value: function cloneContents() {
      return this.base.cloneContents();
    }
  }, {
    key: "deleteContents",
    value: function deleteContents() {
      return this.base.deleteContents();
    }
  }, {
    key: "extractContents",
    value: function extractContents() {
      return this.base.extractContents();
    }
  }, {
    key: "getBoundingClientRect",
    value: function getBoundingClientRect() {
      return this.base.getBoundingClientRect();
    }
  }, {
    key: "getClientRects",
    value: function getClientRects() {
      return this.base.getClientRects();
    }
  }, {
    key: "insertNode",
    value: function insertNode(node) {
      if (isNodeEntry(node)) node = node[0];
      return this.base.insertNode(node);
    }
  }, {
    key: "isPointInRange",
    value: function isPointInRange(node, offset) {
      if (isNodeEntry(node)) node = node[0];
      return this.base.isPointInRange(node, offset);
    }
  }, {
    key: "comparePoint",
    value: function comparePoint(node, offset) {
      if (isNodeEntry(node)) node = node[0];
      return this.base.comparePoint(node, offset);
    }
  }, {
    key: "setEnd",
    value: function setEnd(node, offset) {
      if (isNodeEntry(node)) node = node[0];
      return this.base.setEnd(node, offset);
    }
  }, {
    key: "setEndAfter",
    value: function setEndAfter(node) {
      if (isNodeEntry(node)) node = node[0];
      return this.base.setEndAfter(node);
    }
  }, {
    key: "setEndBefore",
    value: function setEndBefore(node) {
      if (isNodeEntry(node)) node = node[0];
      return this.base.setEndBefore(node);
    }
  }, {
    key: "setStart",
    value: function setStart(node, offset) {
      if (isNodeEntry(node)) node = node[0];
      return this.base.setStart(node, offset);
    }
  }, {
    key: "setStartAfter",
    value: function setStartAfter(node) {
      if (isNodeEntry(node)) node = node[0];
      return this.base.setStartAfter(node);
    }
  }, {
    key: "setStartBefore",
    value: function setStartBefore(node) {
      if (isNodeEntry(node)) node = node[0];
      return this.base.setStartBefore(node);
    }
  }, {
    key: "toString",
    value: function toString() {
      return this.base.toString();
    }
  }, {
    key: "startNode",
    get: function get() {
      return $(this.base.startContainer);
    }
  }, {
    key: "endNode",
    get: function get() {
      return $(this.base.endContainer);
    }
  }, {
    key: "commonAncestorNode",
    get: function get() {
      return $(this.base.commonAncestorContainer);
    }
  }, {
    key: "deepCut",
    value: function deepCut() {
      if (!this.collapsed) this.extractContents();
      var startNode = this.startNode;

      if (!startNode.isEditable()) {
        var node = startNode;

        if (node && !node.isEditable()) {
          var parentNode = node.parent();

          while (parentNode && !parentNode.isEditable()) {
            node = parentNode;
            parentNode = parentNode.parent();
          }

          this.setEndAfter(node[0]);
          var contents = this.extractContents();
          this.insertNode(contents);
          this.collapse(true);
        }
      }
    }
    /**
     * 对比两个范围是否相等
     * @param range 范围
     */

  }, {
    key: "equal",
    value: function equal(range) {
      return this.startContainer === range.startContainer && this.startOffset === range.startOffset && this.endContainer === range.endContainer && this.endOffset === range.endOffset;
    }
    /**
     * 获取当前选区最近的根节点
     */

  }, {
    key: "getRootBlock",
    value: function getRootBlock() {
      if (this.startNode.isEditable()) return this.startNode.children().eq(this.startOffset);
      var node = this.startNode;

      while (((_node = node) === null || _node === void 0 ? void 0 : _node.parent()) && !node.parent().isEditable()) {
        var _node;

        node = node.parent();
      }

      return node;
    }
  }, {
    key: "toPath",
    value: function toPath() {
      var range = this.cloneRange();
      var node = range.commonAncestorNode;
      if (!node.isRoot() && !node.inEditor()) return [];
      range.shrinkToTextNode();

      var getPath = function getPath(node, offset) {
        var domNode = node;
        var path = [];

        while (domNode && domNode.length > 0 && !domNode.isRoot()) {
          var prev = domNode.prev();
          var i = 0;

          while (prev && prev.length > 0) {
            if (!prev.attributes(DATA_TRANSIENT_ELEMENT) && prev.attributes(DATA_ELEMENT) !== UI) i++;
            prev = prev.prev();
          }

          path.unshift(i);
          domNode = domNode.parent();
        }

        path.push(offset);
        return path;
      };

      return [getPath(range.startNode, range.startOffset), getPath(range.endNode, range.endOffset)];
    }
  }]);

  return Range;
}();

Range.create = void 0;
Range.from = void 0;
Range.fromPath = void 0;

Range.create = function (editor) {
  var doc = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : document;
  var point = arguments.length > 2 ? arguments[2] : undefined;
  var range;
  if (point) range = doc.caretRangeFromPoint(point.x, point.y);else range = doc.createRange();
  return Range.from(editor, range);
};

Range.from = function (editor) {
  var win = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : window;

  if (!isRange(win)) {
    var selection = isSelection(win) ? win : win.getSelection();

    if (selection && selection.rangeCount > 0) {
      win = selection.getRangeAt(0);
    } else return null;
  }

  return new Range(editor, win);
};

Range.fromPath = function (editor, path) {
  var context = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : editor.container;
  var startPath = path[0].slice();
  var endPath = path[1].slice();
  var startOffset = startPath.pop();
  var endOffset = endPath.pop();

  var getNode = function getNode(path) {
    var domNode = context;

    for (var i = 0; i < path.length; i++) {
      var p = path[i];

      if (p < 0) {
        p = 0;
      }

      var needNode = undefined;
      var domChild = domNode.first();
      var offset = 0;

      while (domChild && domChild.length > 0) {
        if (!!domChild.attributes(DATA_TRANSIENT_ELEMENT) || domChild.attributes(DATA_ELEMENT) === UI) {
          domChild = domChild.next();
        } else {
          if (offset === p || !domChild.next()) {
            needNode = domChild;
            break;
          }

          offset++;
          domChild = domChild.next();
        }
      }

      if (!needNode) break;
      domNode = needNode;
    }

    return domNode;
  };

  var setRange = function setRange(method, range, node, offset) {
    if (node !== null) {
      var _node$nodeValue2;

      if (offset < 0) {
        offset = 0;
      }

      if (node.nodeType === getWindow().Node.ELEMENT_NODE && offset > node.childNodes.length) {
        offset = node.childNodes.length;
      }

      if (node.nodeType === getWindow().Node.TEXT_NODE && offset > (((_node$nodeValue2 = node.nodeValue) === null || _node$nodeValue2 === void 0 ? void 0 : _node$nodeValue2.length) || 0)) {
        var _node$nodeValue3;

        offset = ((_node$nodeValue3 = node.nodeValue) === null || _node$nodeValue3 === void 0 ? void 0 : _node$nodeValue3.length) || 0;
      }

      range[method](node, offset);
    }
  };

  var startNode = getNode(startPath);
  var endNode = getNode(endPath);
  var range = Range.create(editor, document);
  setRange('setStart', range, startNode.get(), startOffset ? parseInt(startOffset.toString()) : 0);
  setRange('setEnd', range, endNode.get(), endOffset ? parseInt(endOffset.toString()) : 0);
  return range;
};

function styleInject(css, ref) {
  if ( ref === void 0 ) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') { return; }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css_248z = ".data-drop-cursor {\r\n\tposition: absolute;\r\n\twidth: 2px;\r\n\tbackground-color: #347EFF;\r\n}\r\n\r\ndiv.data-drag-image {\r\n\tbackground-color: #f9f9f9;\r\n}";
styleInject(css_248z);

var DragoverHelper = /*#__PURE__*/function () {
  function DragoverHelper(engine, options) {
    _classCallCheck(this, DragoverHelper);

    this.x = 0;
    this.y = 0;
    this.doc = document;
    this.range = void 0;
    this.caretRange = void 0;
    this.targetCard = void 0;
    this.caretCard = void 0;
    this.isCardLeftRange = false;
    this.engine = void 0;
    this.options = {
      className: 'data-drop-cursor'
    };
    this.engine = engine;
    this.options = _objectSpread(_objectSpread({}, this.options), options);
  }
  /**
   * 获取逼近的插入区间
   */


  _createClass(DragoverHelper, [{
    key: "getCaretRange",
    value: function getCaretRange() {
      // https://developer.mozilla.org/zh-CN/docs/Web/API/DocumentOrShadowRoot/caretPositionFromPoint
      // https://developer.mozilla.org/en-US/docs/Web/API/Document/caretRangeFromPoint
      var doc = this.doc,
          x = this.x,
          y = this.y;

      if (doc.caretRangeFromPoint !== undefined) {
        var range = Range.create(this.engine, doc, {
          x: x,
          y: y
        });
        if (range) return range;
      }

      if (event && event['rangeParent'] !== undefined) {
        var _range = Range.create(this.engine, doc);

        _range.setStart(event['rangeParent'], event['rangeOffset']);

        _range.collapse(true);

        return _range;
      }

      return;
    }
    /**
     * 获取卡片
     */

  }, {
    key: "getCard",
    value: function getCard() {
      return this.targetCard || this.caretCard;
    }
    /**
     * 解析事件参数
     * @param e 事件
     */

  }, {
    key: "parseEvent",
    value: function parseEvent(e) {
      // 文件从 Finder 拖进来，不触发 dragstart 事件
      // Card拖动，禁止浏览器行为
      // 禁止拖图进浏览器，浏览器默认打开图片文件
      e.preventDefault();
      e.stopPropagation();

      if (e.dataTransfer) {
        e.dataTransfer.dropEffect = 'move';
      }

      var card = this.engine.card;
      this.x = e.clientX;
      this.y = e.clientY;
      var target = $(e.target || []);
      this.doc = target.document || document;
      this.targetCard = card.find(target); // 当前鼠标精确击中的Card

      this.caretRange = this.getCaretRange();
      this.caretCard = this.caretRange ? card.find(this.caretRange.commonAncestorContainer) : undefined;
    } // 有逼近选区获得的逼近Card

  }, {
    key: "getRange",
    value: function getRange() {
      var caretRange = this.caretRange,
          doc = this.doc,
          x = this.x;
      var card = this.getCard();
      var cardCaretRange;

      if (card && card.root.length > 0) {
        cardCaretRange = Range.create(this.engine, doc);

        var _ref = card.root.getBoundingClientRect() || {
          left: 0,
          right: 0
        },
            left = _ref.left,
            right = _ref.right;

        var centerX = (left + right) / 2;
        cardCaretRange.select(card.root.get()); // 以卡中点为中心为分割线，逼近两侧可插入的区间

        if (centerX < x) {
          cardCaretRange.collapse(false);
          this.isCardLeftRange = false;
        } else {
          cardCaretRange.collapse(true);
          this.isCardLeftRange = true;
        }
      }

      this.range = cardCaretRange || caretRange;
      return this.range;
    }
    /**
     * 获取光标位置
     * @param width 光标宽度，默认为2
     */

  }, {
    key: "getRect",
    value: function getRect() {
      var _this$range, _rect;

      var width = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 2;
      var isCardLeftRange = this.isCardLeftRange,
          range = this.range;
      var card = this.getCard();

      if (card && card.root.length > 0 && range) {
        if (isCardLeftRange) {
          // 如果选区在Card左侧，则向后选取一个元素，选中Card区域
          range.setEnd(range.commonAncestorContainer, range.endOffset + 1);

          var _range$getBoundingCli = range.getBoundingClientRect(),
              _left = _range$getBoundingCli.left,
              _bottom2 = _range$getBoundingCli.bottom,
              _top2 = _range$getBoundingCli.top;

          range.setEnd(range.commonAncestorContainer, range.endOffset - 1);
          return {
            x: _left - width,
            y: _top2,
            height: _bottom2 - _top2
          };
        } // 如果选区在Card右侧，则向前选取一个元素，选中Card区域


        range.setStart(range.commonAncestorContainer, range.startOffset - 1);

        var _range$getBoundingCli2 = range.getBoundingClientRect(),
            right = _range$getBoundingCli2.right,
            _top = _range$getBoundingCli2.top,
            _bottom = _range$getBoundingCli2.bottom;

        range.setStart(range.commonAncestorContainer, range.startOffset + 1);
        return {
          x: right - width,
          y: _top,
          height: _bottom - _top
        };
      } // 如果Card根节点不存在，则原逻辑不变


      var rect = (_this$range = this.range) === null || _this$range === void 0 ? void 0 : _this$range.getBoundingClientRect();

      if (((_rect = rect) === null || _rect === void 0 ? void 0 : _rect.height) === 0) {
        var _this$range2;

        var node = (_this$range2 = this.range) === null || _this$range2 === void 0 ? void 0 : _this$range2.startContainer;
        rect = node.getBoundingClientRect();
      }

      var _ref2 = rect || {},
          left = _ref2.left,
          top = _ref2.top,
          bottom = _ref2.bottom;

      return {
        x: left,
        y: top,
        height: (bottom || 0) - (top || 0)
      };
    }
  }, {
    key: "getCursor",
    value: function getCursor() {
      var className = this.options.className;
      return $("body > div.".concat(className));
    }
  }, {
    key: "removeCursor",
    value: function removeCursor() {
      this.getCursor().remove();
    }
  }, {
    key: "setCursor",
    value: function setCursor() {
      this.removeCursor();
      var className = this.options.className;
      var cursor = $("<div class=\"".concat(className, "\" />"));
      $(document.body).append(cursor);
    }
  }]);

  return DragoverHelper;
}();

var ChangeEvent = /*#__PURE__*/function () {
  function ChangeEvent(engine) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    _classCallCheck(this, ChangeEvent);

    this.events = void 0;
    this.engine = void 0;
    this.isComposing = void 0;
    this.isSelecting = void 0;
    this.dragoverHelper = void 0;
    this.options = void 0;
    this.keydownRange = null;
    this.events = [];
    this.engine = engine; // 中文输入状态

    this.isComposing = false; // 选择范围状态

    this.isSelecting = false;
    this.dragoverHelper = new DragoverHelper(engine);
    this.options = options;
  } // return true：焦点在Card里的其它输入框
  // return false：焦点在编辑区域，触发 change、select 事件


  _createClass(ChangeEvent, [{
    key: "isCardInput",
    value: function isCardInput(e) {
      var node = e.target ? $(e.target) : null;

      while (node) {
        if (node.isEditable()) {
          return false;
        }

        if (node.attributes(CARD_ELEMENT_KEY) === 'center') {
          return true;
        }

        if (node.attributes(DATA_ELEMENT) === UI) {
          return true;
        }

        var parent = node.parent();
        if (!parent) break;
        node = parent;
      }

      return false;
    }
  }, {
    key: "onInput",
    value: function onInput(callback) {
      var _this = this;

      var bindInput = this.options.bindInput;
      if (bindInput && !bindInput()) return; // 处理中文输入法状态
      // https://developer.mozilla.org/en-US/docs/Web/Events/compositionstart

      this.onContainer('compositionstart', function () {
        if (_this.engine.readonly) {
          return;
        }

        _this.isComposing = true;
      });
      this.onContainer('compositionend', function () {
        if (_this.engine.readonly) {
          return;
        }

        _this.isComposing = false;
      }); //对系统工具栏操作拦截，一般针对移动端的文本上下文工具栏
      //https://rawgit.com/w3c/input-events/v1/index.html#interface-InputEvent-Attributes

      this.onContainer('beforeinput', function (event) {
        var inputType = event.inputType;
        var commandTypes = ['format', 'history'];
        commandTypes.forEach(function (type) {
          if (inputType.indexOf(type) === 0) {
            event.preventDefault();
            var commandName = inputType.substring(type.length).toLowerCase();

            _this.engine.command.execute(commandName);
          }
        });
      });
      this.onContainer('input', function (e) {
        if (_this.engine.readonly) {
          return;
        }

        if (_this.isCardInput(e)) {
          return;
        }

        window.setTimeout(function () {
          if (!_this.isComposing) {
            callback(e);
          }
        }, 10);
      });
    }
  }, {
    key: "onSelect",
    value: function onSelect(callback) {
      var _this2 = this;

      var bindSelect = this.options.bindSelect;
      if (bindSelect && !bindSelect()) return; // 模拟 selection change 事件

      this.onContainer('mousedown', function (e) {
        if (_this2.isCardInput(e)) {
          return;
        }

        _this2.isSelecting = true;
      });
      this.onDocument('mouseup', function (e) {
        if (!_this2.isSelecting) {
          return;
        }

        _this2.isSelecting = false; // mouseup 瞬间选择状态不会马上被取消，需要延迟

        window.setTimeout(function () {
          return callback(e);
        }, 10);
      });
      this.onContainer('keydown', function () {
        var range = Range.from(_this2.engine);
        _this2.keydownRange = range;
      }); // 补齐通过键盘选中的情况

      this.onContainer('keyup', function (e) {
        if (_this2.engine.readonly) {
          return;
        }

        if (_this2.isCardInput(e)) {
          return;
        } // command + 方向键不会触发 keyup 事件，所以先用 e.key === 'Meta' 代替 isHotkey('mod+方向键',e)


        if (isHotkey('left', e) || isHotkey('right', e) || isHotkey('up', e) || isHotkey('down', e) || e.key === 'Meta' || isHotkey('shift+left', e) || isHotkey('shift+right', e) || isHotkey('shift+up', e) || isHotkey('shift+down', e) || isHotkey('ctrl+b', e) || isHotkey('ctrl+f', e) || isHotkey('ctrl+n', e) || isHotkey('ctrl+p', e) || isHotkey('ctrl+a', e) || isHotkey('ctrl+e', e) || isHotkey('home', e) || isHotkey('end', e)) {
          var range = Range.from(_this2.engine);
          if (_this2.keydownRange && range && range.equal(_this2.keydownRange)) return;

          if (!_this2.isComposing) {
            callback(e);
          }
        }
      });
    }
  }, {
    key: "onPaste",
    value: function onPaste(callback) {
      var _this3 = this;

      var bindPaste = this.options.bindPaste;
      if (bindPaste && !bindPaste()) return;
      var isPasteText = false;
      this.onContainer('keydown', function (e) {
        if (_this3.engine.readonly) {
          return;
        }

        if (!isHotkey('mod', e) || !isHotkey('shift', e) || !isHotkey('v', e)) {
          isPasteText = false;
        }

        if (isHotkey('mod+shift+v', e) || isHotkey('mod+alt+shift+v', e)) {
          isPasteText = true;
        }
      }); // https://developer.mozilla.org/en-US/docs/Web/Events/paste

      this.onContainer('paste', function (e) {
        if (_this3.engine.readonly) {
          return;
        }

        if (_this3.isCardInput(e)) {
          return;
        }

        e.preventDefault();

        var data = _this3.engine.clipboard.getData(e);

        var dataIsPasteText = isPasteText;
        isPasteText = false;
        callback(_objectSpread(_objectSpread({}, data), {}, {
          isPasteText: dataIsPasteText
        }));
      });
    }
  }, {
    key: "onDrop",
    value: function onDrop(callback) {
      var _this4 = this;

      var bindDrop = this.options.bindDrop;
      if (bindDrop && !bindDrop()) return;
      var cardComponet;
      var dragImage;
      var dropRange;

      var dragStart = function dragStart(e) {
        if (!e.target) return;
        e.stopPropagation();

        _this4.dragoverHelper.setCursor();

        var targetNode = $(e.target); // 拖动Card

        var dragCardTrigger = targetNode.attributes('drag-card-trigger');
        cardComponet = _this4.engine.card.find(!!dragCardTrigger ? dragCardTrigger : targetNode);

        if (cardComponet) {
          var _cardComponet$toolbar, _e$dataTransfer;

          (_cardComponet$toolbar = cardComponet.toolbarModel) === null || _cardComponet$toolbar === void 0 ? void 0 : _cardComponet$toolbar.hideCardToolbar(); // https://kryogenix.org/code/browser/custom-drag-image.html

          dragImage = cardComponet.find('img.data-drag-image');

          if (dragImage.length > 0) {
            dragImage = _this4.engine.node.clone(dragImage);
          } else {
            dragImage = $('<div class="data-drag-image" />');
            var cardRootElement = cardComponet.root.get();

            if (cardRootElement) {
              dragImage.css({
                width: cardRootElement.clientWidth + 'px',
                height: cardRootElement.clientHeight + 'px'
              });
            }
          }

          dragImage.css({
            position: 'absolute',
            top: '-10000px',
            right: '-10000px'
          });
          $(document.body).append(dragImage);
          (_e$dataTransfer = e.dataTransfer) === null || _e$dataTransfer === void 0 ? void 0 : _e$dataTransfer.setDragImage(dragImage[0], 0, 0);
        }
      };

      this.onRoot('dragstart', dragStart);
      this.onContainer('dragstart', dragStart);
      this.onContainer('dragover', function (e) {
        var dragoverHelper = _this4.dragoverHelper;
        var cursor = dragoverHelper.getCursor();

        if (cursor.length !== 0) {
          dragoverHelper.parseEvent(e);
          dropRange = dragoverHelper.getRange();
          var rect = dragoverHelper.getRect();
          cursor.css({
            height: rect.height + 'px',
            top: Math.round(window.pageYOffset + (rect.y || 0)) + 'px',
            left: Math.round(window.pageXOffset + (rect.x || 0)) + 'px'
          });
        } else _this4.dragoverHelper.setCursor();
      });
      this.onContainer('dragleave', function () {
        _this4.dragoverHelper.removeCursor();
      });
      this.onContainer('dragend', function () {
        _this4.dragoverHelper.removeCursor();

        if (dragImage) {
          dragImage.remove();
          dragImage = undefined;
        }
      });
      this.onContainer('drop', function (e) {
        // 禁止拖图进浏览器，浏览器默认打开图片文件
        e.preventDefault();

        _this4.dragoverHelper.removeCursor();

        if (dragImage) {
          dragImage.remove();
          dragImage = undefined;
        }

        var transfer = e.dataTransfer;
        var files = []; // Edge 兼容性处理

        try {
          if (transfer && transfer.items && transfer.items.length > 0) {
            Array.from(transfer.items).forEach(function (item) {
              if (item.kind === 'file') {
                var file = item.getAsFile();
                if (file) files.push(file);
              }
            });
          } else if (transfer && transfer.files && transfer.files.length > 0) {
            files = Array.from(transfer.files);
          }
        } catch (err) {
          if (transfer && transfer.files && transfer.files.length > 0) {
            files = Array.from(transfer.files);
          }
        }

        var data = {
          event: e,
          range: dropRange,
          card: cardComponet,
          files: files
        };
        callback(data);
        cardComponet = undefined;
      });
    }
  }, {
    key: "onDocument",
    value: function onDocument(eventType, listener, rewrite) {
      document.addEventListener(eventType, listener, rewrite);
      this.events.push({
        type: 'document',
        eventType: eventType,
        listener: listener,
        rewrite: rewrite
      });
    }
  }, {
    key: "onWindow",
    value: function onWindow(eventType, listener, rewrite) {
      window.addEventListener(eventType, listener, rewrite);
      this.events.push({
        type: 'window',
        eventType: eventType,
        listener: listener,
        rewrite: rewrite
      });
    }
  }, {
    key: "onContainer",
    value: function onContainer(eventType, listener) {
      this.engine.container.on(eventType, listener);
      this.events.push({
        type: 'container',
        eventType: eventType,
        listener: listener,
        rewrite: false
      });
    }
  }, {
    key: "onRoot",
    value: function onRoot(eventType, listener) {
      this.engine.root.on(eventType, listener);
      this.events.push({
        type: 'root',
        eventType: eventType,
        listener: listener,
        rewrite: false
      });
    }
  }, {
    key: "destroy",
    value: function destroy() {
      var _this5 = this;

      this.events.forEach(function (item) {
        if (item.type === 'window') {
          window.removeEventListener(item.eventType, item.listener, item.rewrite);
        } else if (item.type === 'document') {
          document.removeEventListener(item.eventType, item.listener, item.rewrite);
        } else if (item.type === 'container') {
          _this5.engine.container.off(item.eventType, item.listener);
        } else if (item.type === 'root') {
          _this5.engine.root.off(item.eventType, item.listener);
        }
      });
    }
  }]);

  return ChangeEvent;
}();

var TextParser = /*#__PURE__*/function () {
  function TextParser(source) {
    _classCallCheck(this, TextParser);

    this.source = void 0;
    this.source = source;
  }

  _createClass(TextParser, [{
    key: "toHTML",
    value: function toHTML() {
      var html = escape(this.source);
      html = html.replace(/\n/g, '</p><p>').replace(/<p><\/p>/g, '<p><br /></p>').replace(/^\s/, '&nbsp;').replace(/\s$/, '&nbsp;').replace(/\s\s/g, ' &nbsp;');

      if (html.indexOf('</p><p>') >= 0) {
        html = '<p>'.concat(html, '</p>');
      }

      return html;
    }
  }]);

  return TextParser;
}();

var style = {
  'font-size': '14px',
  color: '#262626',
  'line-height': '24px',
  'letter-spacing': '.05em',
  'outline-style': 'none',
  'overflow-wrap': 'break-word'
};

var escapeAttr = function escapeAttr(value) {
  return value.replace(/</g, '&lt;').replace(/>/g, '&gt;').replace(/"/g, '&quot;');
};

var attrsToString = function attrsToString(attrs) {
  var attrsString = '';
  Object.keys(attrs).forEach(function (key) {
    if (key === 'style') {
      return;
    }

    var val = escapeAttr(attrs[key]);
    attrsString += ' '.concat(key, '="').concat(val, '"');
  });
  return attrsString.trim();
};

var stylesToString = function stylesToString(styles) {
  var stylesString = '';
  Object.keys(styles).forEach(function (key) {
    var val = escape(styles[key]);

    if (/^(padding|margin|text-indent)/.test(key) && removeUnit(val) === 0) {
      return;
    }

    if (/[^a-z]color$/.test(key)) {
      val = toHex(val);
    }

    stylesString += ' '.concat(key, ': ').concat(val, ';');
  });
  return stylesString.trim();
};

var Parser = /*#__PURE__*/function () {
  function Parser(source, editor, paserBefore) {
    _classCallCheck(this, Parser);

    this.root = void 0;
    this.editor = void 0;
    this.editor = editor;
    var node = this.editor.node;

    if (typeof source === 'string') {
      source = source.replace(/<a\s{0,1000}\/>/gi, '<a></a>');
      source = source.replace(/<a(\s[^>]+?)\/>/gi, function (_, t) {
        return '<a'.concat(t, '></a>');
      }); // bugfix：在 p 里包含 div 标签时 DOMParser 解析错误
      // <p><div>foo</div></p>
      // 变成
      // <p></p><div>foo</div><p></p>

      source = source.replace(/<p(>|\s+[^>]*>)/gi, '<paragraph$1').replace(/<\/p>/gi, '</paragraph>');
      source = transformCustomTags(source);
      var doc = new (getWindow().DOMParser)().parseFromString(source, 'text/html');
      this.root = $(doc.body);
      var p = $('<p></p>');
      this.root.find('paragraph').each(function (child) {
        var cNode = $(child);
        var pNode = p.clone();
        var attributes = cNode.attributes();
        Object.keys(attributes).forEach(function (name) {
          pNode.attributes(name, attributes[name]);
        });
        node.replace(cNode, pNode);
      });
    } else if (isNodeEntry(source)) {
      this.root = source;
    } else {
      this.root = $(source);
    }

    if (paserBefore) paserBefore(this.root);
  }

  _createClass(Parser, [{
    key: "normalize",
    value: function normalize(root, schema, conversion) {
      var nodeApi = this.editor.node;
      var inlineApi = this.editor.inline;
      var markApi = this.editor.mark; //转换标签和分割 mark 和 inline 标签

      root.allChildren().forEach(function (child) {
        var node = $(child);

        if (node.isElement()) {
          //转换标签
          if (conversion) {
            (function () {
              var value = conversion.transform(node);
              var oldRules = [];

              while (value) {
                var _value = value,
                    rule = _value.rule;
                oldRules.push(rule);
                var _value$node = value.node,
                    name = _value$node.name,
                    attributes = _value$node.attributes,
                    _style = _value$node.style;
                var newNode = $("<".concat(name, " />"));
                nodeApi.setAttributes(newNode, _objectSpread(_objectSpread({}, attributes), {}, {
                  style: _style
                })); //把旧节点的子节点追加到新节点下

                newNode.append(node.children());

                if (node.attributes(CARD_KEY) || node.attributes(READY_CARD_KEY)) {
                  node.before(newNode);
                  node.remove();
                  value = undefined;
                  continue;
                } else {
                  //把包含旧子节点的新节点追加到旧节点下
                  node.append(newNode);
                } //排除之前的过滤规则后再次过滤


                value = conversion.transform(node, function (r) {
                  return oldRules.indexOf(r) < 0;
                });
              }
            })();
          }

          if (node.attributes(CARD_KEY) || node.attributes(READY_CARD_KEY)) return; //分割

          var filter = function filter(node) {
            //获取节点属性样式
            var attributes = node.attributes();
            var style = node.css();
            delete attributes.style; //过滤不符合当前节点规则的属性样式

            schema.filter(node, attributes, style); //复制一个节点

            var newNode = node.clone(); //移除 data-id，以免在下次判断类型的时候使用缓存

            newNode.removeAttributes('data-id'); //移除符合当前节点的属性样式，剩余的属性样式组成新的节点

            Object.keys(attributes).forEach(function (name) {
              if (attributes[name]) {
                newNode.removeAttributes(name);
              }
            });
            Object.keys(style).forEach(function (name) {
              if (style[name]) {
                newNode.css(name, '');
              }
            });
            if (newNode.attributes('style').trim() === '') newNode.removeAttributes('style');
            return newNode;
          }; //当前节点是 inline 节点，inline 节点不允许嵌套、不允许放入mark节点


          if (nodeApi.isInline(node) && node.name !== 'br') {
            var parentInline = inlineApi.closest(node); //不允许嵌套

            if (!parentInline.equal(node) && nodeApi.isInline(parentInline)) {
              nodeApi.unwrap(node);
            } //不允许放入mark
            else {
                var parentMark = markApi.closest(node);

                if (!parentMark.equal(node) && nodeApi.isMark(parentMark)) {
                  var cloneMark = parentMark.clone();
                  var inlineMark = node.clone();
                  parentMark.each(function (markChild) {
                    if (node.equal(markChild)) {
                      nodeApi.wrap(nodeApi.replace(node, cloneMark), inlineMark);
                    } else {
                      nodeApi.wrap(markChild, cloneMark);
                    }
                  });
                  nodeApi.unwrap(parentMark);
                }
              }
          } //当前节点是 mark 节点


          if (nodeApi.isMark(node)) {
            (function () {
              //过滤掉当前mark节点属性样式并使用剩下的属性样式组成新的节点
              var oldRules = [];
              var rule = schema.getRule(node);

              if (rule) {
                (function () {
                  oldRules.push(rule);
                  var newNode = filter(node); //获取这个新的节点所属类型，并且不能是之前节点一样的规则

                  var type = schema.getType(newNode, function (rule) {
                    return rule.name === newNode.name && rule.type === 'mark' && oldRules.indexOf(rule) < 0;
                  }); //如果是mark节点，使用新节点包裹旧节点子节点

                  while (type === 'mark') {
                    newNode.append(node.children());
                    node.append(newNode);
                    newNode = filter(newNode); //获取这个新的节点所属类型，并且不能是之前节点一样的规则

                    type = schema.getType(newNode, function (rule) {
                      return rule.name === newNode.name && rule.type === 'mark' && oldRules.indexOf(rule) < 0;
                    });
                    if (!type) break;
                    rule = schema.getRule(newNode);
                    if (!rule) break;
                    oldRules.push(rule);
                  }
                })();
              }
            })();
          }
        }
      });
    }
    /**
     * data type:
     *
     * Value: <p>foo</p><p><br /><cursor /></p>
     * LowerValue: <p>foo</p><p><br /><span data-element="cursor"></span></p>
     * DOM: HTML DOM tree
     * Markdown: ### heading
     * Text: plain text
     *
     */

  }, {
    key: "walkTree",
    value: function walkTree(node) {
      var schema = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
      var conversion = arguments.length > 2 ? arguments[2] : undefined;
      var callbacks = arguments.length > 3 ? arguments[3] : undefined;
      var includeCard = arguments.length > 4 ? arguments[4] : undefined;
      var nodeApi = this.editor.node;
      var child = node.first();

      while (child) {
        if (child.isElement()) {
          var name = child.name;
          var attrs = child.attributes();
          var styles = child.css(); //删除属性中的style属性

          delete attrs.style; // Card Combine 相关节点

          if (['left', 'right'].indexOf(attrs[CARD_ELEMENT_KEY]) >= 0) {
            child = child.next();
            continue;
          }

          var passed = true;
          var type = undefined;

          if (schema && attrs[DATA_ELEMENT] !== EDITABLE) {
            //不符合规则，跳过
            type = schema.getType(child);

            if (type === undefined) {
              passed = false;
            } else {
              //过滤不符合规则的属性和样式
              schema.filter(child, attrs, styles);
            }
          } // 执行回调函数


          if (attrs[CARD_ELEMENT_KEY] !== 'center' && callbacks.onOpen && passed) {
            var result = callbacks.onOpen(child, name, attrs, styles); //终止遍历当前节点

            if (result === false) {
              child = child.next();
              continue;
            }
          } // Card不遍历子节点


          if (name !== 'card' || includeCard) {
            this.walkTree(child, schema, conversion, callbacks, includeCard);
          } // 执行回调函数


          if (attrs[CARD_ELEMENT_KEY] !== 'center' && callbacks.onClose && passed) {
            callbacks.onClose(child, name, attrs, styles);
          }
        } else if (child.isText()) {
          var text = child[0].nodeValue ? escape(child[0].nodeValue) : ''; // 为了简化 DOM 操作复杂度，删除 block 两边的空白字符，不影响渲染展示

          if (text === '' && nodeApi.isBlock(child.parent())) {
            if (!child.prev()) {
              text = text.replace(/^[ \n]+/, '');
            }

            if (!child.next()) {
              text = text.replace(/[ \n]+$/, '');
            }
          } // 删除两个 block 中间的空白字符
          // <p>foo</p>\n<p>bar</p>


          var childPrev = child.prev();
          var childNext = child.next();

          if (childPrev && nodeApi.isBlock(childPrev) && childNext && nodeApi.isBlock(childNext) && text.trim() === '') {
            text = text.trim();
          } // 删除 zero width space


          text = text.replace(/\u200B/g, '');

          if (callbacks.onText) {
            callbacks.onText(child, text);
          }
        }

        child = child.next();
      }
    }
    /**
     * 遍历 DOM 树，生成符合标准的 XML 代码
     * @param schema 标签保留规则
     * @param conversion 标签转换规则
     * @param replaceSpaces 是否替换空格
     * @param customTags 是否将光标、卡片节点转换为标准代码
     */

  }, {
    key: "toValue",
    value: function toValue() {
      var _this = this;

      var schema = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
      var conversion = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
      var replaceSpaces = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
      var customTags = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
      var result = [];
      var nodeApi = this.editor.node;
      var root = this.root.clone(true);
      if (schema) this.normalize(root, schema, conversion);
      this.editor.trigger('paser:value-before', root);
      this.walkTree(root, schema, conversion, {
        onOpen: function onOpen(child, name, attrs, styles) {
          if (_this.editor.trigger('paser:value', child, attrs, styles, result) === false) return false;
          result.push('<');
          result.push(name);

          if (Object.keys(attrs).length > 0) {
            result.push(' ' + attrsToString(attrs));
          }

          if (Object.keys(styles).length > 0) {
            var stylesString = stylesToString(styles);

            if (stylesString !== '') {
              result.push(' style="');
              result.push(stylesString);
              result.push('"');
            }
          }

          if (nodeApi.isVoid(name, schema ? schema : undefined)) {
            result.push(' />');
          } else {
            result.push('>');
          }

          return;
        },
        onText: function onText(_, text) {
          if (replaceSpaces && text.length > 1) {
            text = text.replace(/[\u00a0 ]+/g, function (item) {
              var strArray = [];
              item = item.replace(/\u00a0/g, ' ');

              for (var n = 0; n < item.length; n++) {
                strArray[n] = n % 2 == 0 ? item[n] : ' ';
              }

              return strArray.join('');
            });
          }

          result.push(text);
        },
        onClose: function onClose(_, name) {
          if (nodeApi.isVoid(name, schema ? schema : undefined)) return;
          result.push('</'.concat(name, '>'));
        }
      });
      this.editor.trigger('paser:value-after', result); //移除前后的换行符

      result.some(function (value, index) {
        if (/^\n+/g.test(value)) {
          result[index] = value.replace(/^\n+/g, '');
          return;
        }

        return true;
      });

      for (var i = result.length - 1; i >= 0; i--) {
        var _value2 = result[i];

        if (/^\n+/g.test(_value2)) {
          result[i] = _value2.replace(/^\n+/g, '');
          continue;
        }

        break;
      }

      var value = result.join('');
      return customTags ? transformCustomTags(value) : value;
    }
    /**
     * 转换为HTML代码
     * @param inner 内包裹节点
     * @param outter 外包裹节点
     */

  }, {
    key: "toHTML",
    value: function toHTML(inner, outter) {
      var element = $('<div />');

      if (inner && outter) {
        $(inner).append(this.root).css(style);
        element.append(outter);
      } else {
        element.append(this.root);
      }

      this.editor.trigger('paser:html-before', this.root);
      element.traverse(function (domNode) {
        var node = domNode.get();

        if (node && node.nodeType === getWindow().Node.ELEMENT_NODE && 'none' === node.style['user-select'] && node.parentNode) {
          node.parentNode.removeChild(node);
        }
      });
      this.editor.trigger('paser:html', element);
      element.find('p').css(style);
      this.editor.trigger('paser:html-after', element);
      return {
        html: element.html(),
        text: new Parser(element, this.editor).toText(this.editor.schema, true)
      };
    }
    /**
     * 返回DOM树
     */

  }, {
    key: "toDOM",
    value: function toDOM() {
      var schema = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
      var conversion = arguments.length > 1 ? arguments[1] : undefined;
      var value = this.toValue(schema, conversion, false, true);
      var doc = new DOMParser().parseFromString(value, 'text/html');
      var fragment = doc.createDocumentFragment();
      var nodes = doc.body.childNodes;

      while (nodes.length > 0) {
        fragment.appendChild(nodes[0]);
      }

      return fragment;
    }
    /**
     * 转换为文本
     * @param includeCard 是遍历卡片内部
     */

  }, {
    key: "toText",
    value: function toText() {
      var _this2 = this;

      var schema = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
      var includeCard = arguments.length > 1 ? arguments[1] : undefined;
      var root = this.root.clone(true);
      var result = [];
      this.walkTree(root, null, null, {
        onOpen: function onOpen(node, name) {
          if (name === 'br') {
            result.push('\n');
          }

          var nodeElement = node[0];

          if (node.name === 'li') {
            if (node.hasClass('data-list-item')) {
              return;
            }

            var parent = node.parent();
            var styleType = parent === null || parent === void 0 ? void 0 : parent.css('listStyleType');

            if ((parent === null || parent === void 0 ? void 0 : parent.name) === 'ol') {
              var start = parent[0]['start'];
              var index = start ? start : 1;
              var childs = parent[0].childNodes;
              var liCount = -1;

              for (var i = 0; i < childs.length && childs[i].nodeName === 'LI' && liCount++ && childs[i] !== nodeElement; i++) {
                result.push(''.concat(getListStyle(styleType, index + liCount).toString(), '. '));
              }
            } else if ((parent === null || parent === void 0 ? void 0 : parent.name) === 'ul') {
              result.push(getListStyle(styleType) + ' ');
            }
          }
        },
        onText: function onText(_, text) {
          text = unescape(text);
          text = text.replace(/\u00a0/g, ' ');
          result.push(text);
        },
        onClose: function onClose(node, name) {
          if (name === 'p' || _this2.editor.node.isBlock(node, schema || _this2.editor.schema)) {
            result.push('\n');
          }
        }
      }, includeCard);
      return result.join('').replace(/\n{2,}/g, '\n').trim();
    }
  }]);

  return Parser;
}();

var Paste = /*#__PURE__*/function () {
  function Paste(source, engine) {
    _classCallCheck(this, Paste);

    this.source = void 0;
    this.engine = void 0;
    this.schema = void 0;
    this.source = source;
    this.engine = engine;
    this.schema = this.engine.schema.clone();
  }

  _createClass(Paste, [{
    key: "parser",
    value: function parser() {
      var _this = this;

      var conversion = this.engine.conversion.clone();
      this.engine.trigger('paste:schema', this.schema);
      var parser = new Parser(this.source, this.engine, function (root) {
        _this.engine.trigger('paste:origin', root);
      });
      return parser.toDOM(this.schema, conversion);
    }
  }, {
    key: "getDefaultStyle",
    value: function getDefaultStyle() {
      var defaultStyle = [{
        color: tinycolor2(this.engine.container.css('color')).toHex()
      }, {
        'background-color': tinycolor2('white').toHex()
      }, {
        'font-size': this.engine.container.css('font-size')
      }];
      return defaultStyle;
    }
  }, {
    key: "commonNormalize",
    value: function commonNormalize(fragment) {
      var _this2 = this;

      var defaultStyle = this.getDefaultStyle();
      var inline = this.engine.inline;
      var nodeApi = this.engine.node; // 第一轮预处理，主要处理 span 节点

      var nodes = $(fragment).allChildren();
      nodes.forEach(function (child) {
        var node = $(child); // 跳过Card

        if (node.isCard()) {
          return;
        } // 删除与默认样式一样的 inline 样式


        if (node.isElement()) {
          defaultStyle.forEach(function (style) {
            var _node$get;

            var key = Object.keys(style)[0];
            var defaultValue = style[key];
            var value = (_node$get = node.get()) === null || _node$get === void 0 ? void 0 : _node$get.style[key];

            if (value) {
              if (/color$/.test(key)) {
                value = tinycolor2(value).toHex();
              }

              if (value === defaultValue) {
                node.css(key, '');
              }
            }
          });
        }

        nodeApi.removeMinusStyle(node, 'text-indent');

        if (['ol', 'ul'].includes(node.name)) {
          node.css('padding-left', '');
        } // 删除空 style 属性


        if (node.isElement()) {
          if (!node.attributes('style')) {
            node.removeAttributes('style');
          }
        } // 删除空 span


        if (node.name === 'span' && Object.keys(node.attributes()).length === 0 && Object.keys(node.css()).length === 0 && (node.text().trim() === '' || node.first() && (nodeApi.isMark(node.first(), _this2.schema) || nodeApi.isBlock(node.first(), _this2.schema)))) {
          nodeApi.unwrap(node);
          return;
        } // br 换行改成正常段落


        if (nodeApi.isBlock(node, _this2.schema)) {
          _this2.engine.block.brToBlock(node);
        }

        if (node.isText()) {
          var text = node.text();

          if (/(\r|\n)+/.test(text)) {
            if (/^(\r|\n)+$/.test(text)) {
              node.text('\n');
              return;
            }

            text.split(/(\r|\n)+/).forEach(function (text) {
              if (text === '') return;
              node.before(document.createTextNode("".concat(text)));
            });
            node.remove();
          }
        }
      }); // 第二轮处理

      nodes = $(fragment).allChildren();
      nodes.forEach(function (child) {
        var _node$parent, _node$parent2, _node$parent3, _node$parent4, _node$parent5;

        var node = $(child); // 跳过已被删除的节点

        if (!node.parent()) {
          return;
        } // 删除 google docs 根节点
        // <b style="font-weight:normal;" id="docs-internal-guid-e0280780-7fff-85c2-f58a-6e615d93f1f2">


        if (/^docs-internal-guid-/.test(node.attributes('id'))) {
          nodeApi.unwrap(node);
          return;
        } // 跳过Card


        if (node.attributes(READY_CARD_KEY)) {
          return;
        } // 删除零高度的空行


        if (nodeApi.isBlock(node, _this2.schema) && node.attributes('data-type') !== 'p' && !nodeApi.isVoid(node, _this2.schema) && //!node.isSolid() &&
        nodeApi.html(node) === '') {
          node.remove();
          return;
        } // 段落


        if (node.attributes('data-type') === 'p') {
          node.removeAttributes('data-type');
        } // 补齐 ul 或 ol


        if (node.name === 'li' && ['ol', 'ul'].indexOf(((_node$parent = node.parent()) === null || _node$parent === void 0 ? void 0 : _node$parent.name) || '') < 0) {
          var ul = $('<ul />');
          node.before(ul);
          ul.append(node);
          return;
        } // 补齐 li


        if (['ol', 'ul'].indexOf(node.name) >= 0 && ['ol', 'ul'].indexOf(((_node$parent2 = node.parent()) === null || _node$parent2 === void 0 ? void 0 : _node$parent2.name) || '') >= 0) {
          var li = $('<li />');
          node.before(li);
          li.append(node);
          return;
        } // <li>two<ol><li>three</li></ol>four</li>


        if (['ol', 'ul'].indexOf(node.name) >= 0 && ((_node$parent3 = node.parent()) === null || _node$parent3 === void 0 ? void 0 : _node$parent3.name) === 'li' && (node.prev() || node.next())) {
          var _parent$parent;

          var parent = node.parent();

          var _li;

          var isCustomizeList = parent === null || parent === void 0 ? void 0 : (_parent$parent = parent.parent()) === null || _parent$parent === void 0 ? void 0 : _parent$parent.hasClass('data-list');
          parent === null || parent === void 0 ? void 0 : parent.children().each(function (child) {
            var node = $(child);

            if (nodeApi.isEmptyWithTrim(node)) {
              return;
            }

            var isList = ['ol', 'ul'].indexOf(node.name) >= 0;

            if (!_li || isList) {
              _li = isCustomizeList ? $('<li class="data-list-item" />') : $('<li />');
              parent.before(_li);
            }

            _li.append(child);

            if (isList) {
              _li = null;
            }
          });
          parent === null || parent === void 0 ? void 0 : parent.remove();
          return;
        } // p 改成 li


        if (node.name === 'p' && ['ol', 'ul'].indexOf(((_node$parent4 = node.parent()) === null || _node$parent4 === void 0 ? void 0 : _node$parent4.name) || '') >= 0) {
          nodeApi.replace(node, $('<li />'));
          return;
        } // 处理空 Block


        if (nodeApi.isBlock(node, _this2.schema) && !nodeApi.isVoid(node, _this2.schema) && nodeApi.html(node).trim() === '') {
          // <p></p> to <p><br /></p>
          if (nodeApi.isRootBlock(node, _this2.schema) || node.name === 'li') {
            nodeApi.html(node, '<br />');
          }
        } // <li><p>foo</p></li>


        if (nodeApi.isRootBlock(node, _this2.schema) && ((_node$parent5 = node.parent()) === null || _node$parent5 === void 0 ? void 0 : _node$parent5.name) === 'li') {
          var _node$first;

          // <li><p><br /></p></li>
          if (node.children().length === 1 && ((_node$first = node.first()) === null || _node$first === void 0 ? void 0 : _node$first.name) === 'br') ; else {
            node.after('<br />');
          }

          nodeApi.unwrap(node);
          return;
        }

        if (nodeApi.isInline(node)) {
          if (nodeApi.isEmptyWithTrim(node)) node.remove();else inline.repairCursor(node);
        } // 移除两边的 BR


        nodeApi.removeSide(node);
      });
    }
  }, {
    key: "normalize",
    value: function normalize() {
      var _this3 = this,
          _startNode$parent;

      var nodeApi = this.engine.node;
      var fragment = this.parser();
      this.commonNormalize(fragment);
      var range = this.engine.change.getRange();
      var root = range.commonAncestorNode;
      var inline = this.engine.inline.closest(root);

      if (nodeApi.isInline(inline)) {
        this.removeElementNodes($(fragment));
        return fragment;
      }

      if (root.isText() && range.startContainer === range.endContainer) {
        var text = root[0].nodeValue;
        var leftText = text === null || text === void 0 ? void 0 : text.substr(0, range.startOffset);
        var rightText = text === null || text === void 0 ? void 0 : text.substr(range.endOffset); // 光标在 [text](|) 里

        if (/\[.*?\]\($/.test(leftText || '') && /^\)/.test(rightText || '')) {
          this.removeElementNodes($(fragment));
          return fragment;
        }
      }

      var nodes = $(fragment).allChildren();
      nodes.forEach(function (child) {
        var node = $(child);
        if (node.parent()) _this3.engine.trigger('paste:each', node);
      });
      nodes = $(fragment).allChildren();
      nodes.forEach(function (child) {
        var node = $(child);

        _this3.engine.trigger('paste:each-after', node); // 删除包含Card的 pre 标签


        if (node.name === 'pre' && node.find(READY_CARD_SELECTOR).length > 0) {
          nodeApi.unwrap(node);
        }
      });
      nodeApi.normalize($(fragment));
      fragment.normalize();
      var fragmentNode = $(fragment);
      var first = fragmentNode.first(); //如果光标在文本节点，并且父级节点不是根节点，移除粘贴数据的第一个节点块级节点，让其内容接在光标所在行

      var _range$cloneRange$shr = range.cloneRange().shrinkToTextNode(),
          startNode = _range$cloneRange$shr.startNode;

      if (startNode.isText() && !((_startNode$parent = startNode.parent()) === null || _startNode$parent === void 0 ? void 0 : _startNode$parent.isEditable()) && first && first.name === 'p') {
        nodeApi.unwrap(first);
      }

      nodes = $(fragment).allChildren();
      nodes.forEach(function (child) {
        var node = $(child);

        if (['ol', 'ul'].includes(node.name)) {
          _this3.engine.list.addStart(node);
        }
      });
      this.engine.block.generateRandomIDForDescendant(fragment, true);
      return fragment;
    }
  }, {
    key: "removeElementNodes",
    value: function removeElementNodes(fragment) {
      var _this4 = this;

      var nodes = fragment.allChildren();
      nodes.forEach(function (child) {
        var node = $(child);

        if (node.isElement()) {
          _this4.engine.node.unwrap(node);
        }
      });
    }
  }]);

  return Paste;
}();

var ChangeModel = /*#__PURE__*/function () {
  function ChangeModel(engine) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    _classCallCheck(this, ChangeModel);

    this.engine = void 0;
    this.options = void 0;
    this.changeTimer = null;
    this.event = void 0;
    this.valueCached = null;
    this.onChange = void 0;
    this.onSelect = void 0;
    this.onSetValue = void 0;
    this.rangePathBeforeCommand = null;
    this.marks = [];
    this.blocks = [];
    this.inlines = [];
    this.changeTrigger = [];
    this.options = options;
    this.engine = engine;
    this.event = new ChangeEvent(engine, {});

    this.onChange = this.options.onChange || function () {};

    this.onSelect = this.options.onSelect || function () {};

    this.onSetValue = this.options.onSetValue || function () {};

    this.initNativeEvents();
  }

  _createClass(ChangeModel, [{
    key: "_change",
    value: function _change() {
      if (!this.isComposing()) {
        this.engine.card.gc();
        var value = this.getValue({
          ignoreCursor: true
        });

        if (!this.valueCached || value !== this.valueCached) {
          var trigger = this.changeTrigger.length === 2 ? 'both' : this.changeTrigger[0] === 'remote' ? 'remote' : 'local';
          this.onChange(value, trigger);
          this.changeTrigger = [];
          this.valueCached = value;
        }
      }
    }
  }, {
    key: "change",
    value: function change(isRemote) {
      var _this = this;

      var range = this.getRange();
      var editableElement = range.startNode.closest(EDITABLE_SELECTOR);

      if (editableElement.length > 0) {
        var card = this.engine.card.find(editableElement, true);
        if (card === null || card === void 0 ? void 0 : card.onChange) card === null || card === void 0 ? void 0 : card.onChange(editableElement);
      }

      var trigger = isRemote ? 'remote' : 'local';
      if (this.changeTrigger.indexOf(trigger) < 0) this.changeTrigger.push(trigger);
      this.clearChangeTimer();
      this.changeTimer = setTimeout(function () {
        _this._change();
      }, 200);
    }
  }, {
    key: "clearChangeTimer",
    value: function clearChangeTimer() {
      if (this.changeTimer) clearTimeout(this.changeTimer);
    }
  }, {
    key: "getRange",
    value: function getRange() {
      var container = this.engine.container;
      var window = container.window;
      var range = Range.from(this.engine, window, false);

      if (!range) {
        range = Range.create(this.engine, window.document).select(container, true).shrinkToElementNode().collapse(false);
      }

      return range;
    }
  }, {
    key: "select",
    value: function select(range) {
      var _this$engine = this.engine,
          container = _this$engine.container,
          mark = _this$engine.mark,
          block = _this$engine.block,
          inline = _this$engine.inline,
          node = _this$engine.node;
      var window = container.window;
      var selection = window === null || window === void 0 ? void 0 : window.getSelection(); //折叠状态

      if (range.collapsed) {
        var _startNode$first, _startNode$last;

        var _startNode = range.startNode,
            _startOffset = range.startOffset; //如果节点下只要一个br标签，并且是<p><br /><cursor /></p>,那么选择让光标选择在 <p><cursor /><br /></p>

        if ((_startNode.isElement() && 1 === _startOffset && 1 === _startNode.children().length || 2 === _startOffset && 2 === _startNode.children().length && ((_startNode$first = _startNode.first()) === null || _startNode$first === void 0 ? void 0 : _startNode$first.isCard())) && 'br' === ((_startNode$last = _startNode.last()) === null || _startNode$last === void 0 ? void 0 : _startNode$last.name)) {
          range.setStart(_startNode, _startOffset - 1);
          range.collapse(true);
        }
      } //修复inline光标


      var _range$cloneRange$shr = range.cloneRange().shrinkToTextNode(),
          startNode = _range$cloneRange$shr.startNode,
          endNode = _range$cloneRange$shr.endNode,
          startOffset = _range$cloneRange$shr.startOffset,
          endOffset = _range$cloneRange$shr.endOffset;

      var prev = startNode.prev();
      var next = endNode.next(); //光标上一个节点是inline节点，让其选择在inline节点后的零宽字符后面

      if (prev && !prev.isCard() && !node.isVoid(prev) && node.isInline(prev)) {
        var text = startNode.text(); //前面是inline节点，后面是零宽字符

        if (/^\u200B/g.test(text) && startOffset === 0) {
          range.setStart(endNode, startOffset + 1);
          if (range.collapsed) range.collapse(true);
        }
      } //光标下一个节点是inline节点，让其选择在inline节点前面的零宽字符前面


      if (next && !next.isCard() && !node.isVoid(next) && node.isInline(next)) {
        var _text = endNode.text();

        if (/\u200B$/g.test(_text) && endOffset === _text.length) {
          range.setEnd(endNode, endOffset - 1);
          if (range.collapsed) range.collapse(false);
        }
      } //光标内侧位置


      var inlineNode = inline.closest(startNode);

      if (!inlineNode.isCard() && node.isInline(inlineNode) && !node.isVoid(inlineNode)) {
        var _startNode$parent, _endNode$parent;

        //左侧
        if (startNode.isText() && !startNode.prev() && ((_startNode$parent = startNode.parent()) === null || _startNode$parent === void 0 ? void 0 : _startNode$parent.equal(inlineNode)) && startOffset === 0) {
          range.setStart(startNode, startOffset + 1);
          if (range.collapsed) range.collapse(true);
        } //右侧


        if (endNode.isText() && !endNode.next() && ((_endNode$parent = endNode.parent()) === null || _endNode$parent === void 0 ? void 0 : _endNode$parent.equal(inlineNode)) && endOffset === endNode.text().length) {
          range.setEnd(endNode, endOffset - 1);
          if (range.collapsed) range.collapse(false);
        }
      } //在非折叠，或者当前range对象和selection中的对象不一致的时候重新设置range


      if (selection && (range.collapsed || selection.rangeCount > 0 && !range.equal(selection.getRangeAt(0)))) {
        selection.removeAllRanges();
        selection.addRange(range.toRange());
      }

      this.marks = mark.findMarks(range);
      this.blocks = block.findBlocks(range);
      this.inlines = inline.findInlines(range);
      return this;
    }
    /**
     * 聚焦编辑器
     * @param toStart true:开始位置,false:结束位置，默认为之前操作位置
     */

  }, {
    key: "focus",
    value: function focus(toStart) {
      var _this$engine$containe;

      var range = this.getRange();

      if (toStart !== undefined) {
        range.select(this.engine.container, true).shrinkToElementNode().collapse(toStart);
      }

      this.select(range);
      (_this$engine$containe = this.engine.container.get()) === null || _this$engine$containe === void 0 ? void 0 : _this$engine$containe.focus();
      return this;
    }
  }, {
    key: "blur",
    value: function blur() {
      var _this$engine$containe2;

      (_this$engine$containe2 = this.engine.container.get()) === null || _this$engine$containe2 === void 0 ? void 0 : _this$engine$containe2.blur();
      return this;
    }
  }, {
    key: "isComposing",
    value: function isComposing() {
      return this.event.isComposing;
    }
  }, {
    key: "isSelecting",
    value: function isSelecting() {
      return this.event.isSelecting;
    }
  }, {
    key: "setValue",
    value: function setValue(value, onParse) {
      var range = this.getRange();
      var _this$engine2 = this.engine,
          schema = _this$engine2.schema,
          conversion = _this$engine2.conversion,
          container = _this$engine2.container,
          history = _this$engine2.history,
          mark = _this$engine2.mark,
          node = _this$engine2.node,
          inline = _this$engine2.inline,
          block = _this$engine2.block,
          card = _this$engine2.card;

      if (value === '') {
        range.setStart(container[0], 0);
        range.collapse(true);
        this.select(range);
      } else {
        var parser = new Parser(value, this.engine, function (root) {
          mark.removeEmptyMarks(root);
          root.allChildren().forEach(function (child) {
            if (node.isInline(child)) {
              inline.repairCursor(child);
            }

            if (onParse) {
              onParse(child);
            }
          });
        });
        container.html(parser.toValue(schema, conversion, false, true));
        container.allChildren().forEach(function (child) {
          if (node.isInline(child)) {
            inline.repairCursor(child);
          }
        });
        block.generateDataIDForDescendant(container.get());
        card.render(undefined, true);
        var cursor = container.find(CURSOR_SELECTOR);
        var selection = new Selection(this.engine, range);

        if (cursor.length > 0) {
          selection.anchor = cursor;
          selection.focus = cursor;
        }

        var anchor = container.find(ANCHOR_SELECTOR);
        var focus = container.find(FOCUS_SELECTOR);

        if (anchor.length > 0 && focus.length > 0) {
          selection.anchor = anchor;
          selection.focus = focus;
        }

        if (selection.anchor && selection.focus) {
          selection.move();
          this.select(range);
          this.onSelect();
        }

        this.onSetValue();

        if (history) {
          history.clear();
        }
      }
    }
  }, {
    key: "getOriginValue",
    value: function getOriginValue() {
      var _this$engine3 = this.engine,
          container = _this$engine3.container,
          schema = _this$engine3.schema,
          conversion = _this$engine3.conversion;
      return new Parser(container.clone(true), this.engine).toValue(schema, conversion);
    }
  }, {
    key: "getValue",
    value: function getValue() {
      var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var value;

      if (options.ignoreCursor || this.isComposing()) {
        value = this.getOriginValue();
      } else {
        var _selection;

        var range = this.getRange();
        var selection;

        if (!range.inCard()) {
          selection = range.createSelection();
        }

        value = this.getOriginValue();
        (_selection = selection) === null || _selection === void 0 ? void 0 : _selection.move();
      }

      return formatEngineValue(value);
    }
  }, {
    key: "cacheRangeBeforeCommand",
    value: function cacheRangeBeforeCommand() {
      this.rangePathBeforeCommand = this.getRange().toPath();
    }
  }, {
    key: "getRangePathBeforeCommand",
    value: function getRangePathBeforeCommand() {
      var rangePath = this.rangePathBeforeCommand;
      this.rangePathBeforeCommand = null;
      return rangePath;
    }
  }, {
    key: "isEmpty",
    value: function isEmpty() {
      var _this$engine4 = this.engine,
          container = _this$engine4.container,
          node = _this$engine4.node;
      return node.isEmptyWithTrim(container);
    }
  }, {
    key: "repairInput",
    value: function repairInput(event, range) {
      var _range = range,
          commonAncestorNode = _range.commonAncestorNode;
      var card = this.engine.card.find(commonAncestorNode);
      var _this$engine5 = this.engine,
          node = _this$engine5.node,
          mark = _this$engine5.mark,
          change = _this$engine5.change;

      if (card && card.constructor.cardType === CardType.INLINE) {
        if (card.isLeftCursor(commonAncestorNode)) {
          var cardLeft = commonAncestorNode.closest(CARD_LEFT_SELECTOR);
          var cardLeftText = cardLeft.text().replace(/\u200B/g, '');

          if (cardLeftText) {
            cardLeftText = escape(cardLeftText);
            range.setStartBefore(card.root);
            range.collapse(true);
            node.html(cardLeft, '&#8203;');
            node.insertText(cardLeftText, range);
            this.select(range);
          }
        } else if (card.isRightCursor(commonAncestorNode)) {
          var cardRight = commonAncestorNode.closest(CARD_RIGHT_SELECTOR);
          var cardRightText = cardRight.text().replace(/\u200B/g, '');

          if (cardRightText) {
            cardRightText = escape(cardRightText);
            range.setEndAfter(card.root);
            range.collapse(false);
            node.html(cardRight, '&#8203;');
            node.insertText(cardRightText, range);
            this.select(range);
          }
        } else this.getSafeRange(range);
      }

      var _range$cloneRange$shr2 = range.cloneRange().shrinkToTextNode(),
          startNode = _range$cloneRange$shr2.startNode,
          startOffset = _range$cloneRange$shr2.startOffset;

      var parent = startNode.parent(); //输入时删除mark标签内零宽字符。

      if (startNode.isText() && parent && node.isMark(parent)) {
        var textNode = startNode.get();
        var text = startNode.text(); //mark 插件禁止跟随样式时，将输入字符设置到mark标签外
        //输入光标在mark节点末尾

        if (startOffset === text.length && event.data && event.inputType.indexOf('insert') === 0) {
          var markParent = parent;
          var markTops = []; //循环查找

          while (markParent && node.isMark(markParent)) {
            var _markParent, _markParent2;

            var markPlugin = mark.findPlugin(markParent); //插件禁止跟随

            if (markPlugin && !markPlugin.followStyle) {
              markTops.push(markParent);
            }

            markParent = markParent.parent(); //如果还有位于下方的同级节点，并且父级节点也是mark节点，说明当前光标不在末尾了

            var markParentP = (_markParent = markParent) === null || _markParent === void 0 ? void 0 : _markParent.parent();

            if (((_markParent2 = markParent) === null || _markParent2 === void 0 ? void 0 : _markParent2.next()) && markParentP && node.isMark(markParentP)) {
              break;
            }
          } //查看下一个节点是否是紧紧挨着的相同样式如果有，那就继续跟随样式


          var startNext = startNode.next();
          markTops.forEach(function (markTop, index) {
            //第一种：<em>abc<cursor /></em><em>123</em> 或者 <em>abc<cursor /></em><strong><em>123</em></strong> 继续跟随
            //第二种：<span><strong><em>abc<cursor /></em></strong><em>123</em><span> 或者 <strong><em>abc<cursor /></em></strong><strong><em>123</em></strong> 继续跟随
            //第三种: <span><strong>abc<cursor /><em>123</em></strong></span> 继续跟随
            //是开始节点所在的mark节点，如果开始节点后面有节点就继续跟随
            if (parent.equal(markTop) && startNext) {
              markTops.splice(index, 1);
              return;
            }

            var next = markTop.next();
            var curNode = markTop; //循环找到下一个节点，如果没有下一级节点，从父级节点查找父级的下一级。如果有下一级节点，并且父节点

            while (!next && curNode) {
              //找到父节点
              var _parent = curNode.parent(); //如果父节点是块级节点，就不找了


              if (_parent && node.isBlock(_parent)) break; //找到父级节点的下一级

              next = (_parent === null || _parent === void 0 ? void 0 : _parent.next()) || null;
              curNode = _parent;
            }

            var first = next;

            while (first && !first.isText()) {
              if (node.isMark(first) && mark.compare(first, markTop)) {
                markTops.splice(index, 1);
                break;
              }

              first = first.first();
            }
          });

          if (markTops.length > 0) {
            var lastText = textNode.splitText(text.length - event.data.length);
            lastText.remove();
            if (node.isEmpty(parent)) parent.remove();
            mark.unwrap(markTops.map(function (mark) {
              return mark.clone();
            }));
            node.insertText(text.substr(text.length - event.data.length));
            mark.merge();
            range = change.getRange().cloneRange().shrinkToTextNode();
            startNode = range.startNode;
            startOffset = range.startOffset;
            textNode = startNode.get();
            text = startNode.text();
          }
        } //输入光标在mark节点开始位置
        else if (event.data && startOffset === event.data.length && event.inputType.indexOf('insert') === 0) {
            var _markParent3 = parent;
            var _markTops = []; //循环查找

            while (_markParent3 && node.isMark(_markParent3)) {
              var _markParent4, _markParent5;

              var _markPlugin = mark.findPlugin(_markParent3); //插件禁止跟随


              if (_markPlugin && !_markPlugin.followStyle) {
                _markTops.push(_markParent3);
              }

              _markParent3 = _markParent3.parent(); //如果还有位于下方的同级节点，并且父级节点也是mark节点，说明当前光标不在末尾了

              var _markParentP = (_markParent4 = _markParent3) === null || _markParent4 === void 0 ? void 0 : _markParent4.parent();

              if (((_markParent5 = _markParent3) === null || _markParent5 === void 0 ? void 0 : _markParent5.prev()) && _markParentP && node.isMark(_markParentP)) {
                break;
              }
            } //查看上一个节点是否是紧紧挨着的相同样式如果有，那就继续跟随样式


            var startPrev = startNode.prev();

            _markTops.forEach(function (markTop, index) {
              //第一种：<em>abc</em><em><cursor />123</em> 或者 <em>abc</em><strong><em><cursor />123</em></strong> 继续跟随
              //第二种：<span><strong><em>abc</em></strong><em><cursor />123</em><span> 或者 <strong><em>abc</em></strong><strong><em><cursor />123</em></strong> 继续跟随
              //第三种: <span><strong><em>123</em><cursor />abc</strong></span> 继续跟随
              //是开始节点所在的mark节点，如果开始节点后面有节点就继续跟随
              if (parent.equal(markTop) && startPrev) {
                _markTops.splice(index, 1);

                return;
              }

              var prev = markTop.prev();
              var curNode = markTop; //循环找到上一个节点，如果没有上一级节点，从父级节点查找父级的上一级。如果有上一级节点，并且父节点

              while (!prev && curNode) {
                //找到父节点
                var _parent2 = curNode.parent(); //如果父节点是块级节点，就不找了


                if (_parent2 && node.isBlock(_parent2)) break; //找到父级节点的下一级

                prev = (_parent2 === null || _parent2 === void 0 ? void 0 : _parent2.prev()) || null;
                curNode = _parent2;
              }

              var last = prev;

              while (last && !last.isText()) {
                if (node.isMark(last) && mark.compare(last, markTop)) {
                  _markTops.splice(index, 1);

                  break;
                }

                last = last.last();
              }
            });

            if (_markTops.length > 0) {
              textNode.splitText(event.data.length);
              textNode.remove();
              if (node.isEmpty(parent)) parent.remove();
              mark.unwrap(_markTops.map(function (mark) {
                return mark.clone();
              }));
              node.insertText(event.data);
              mark.merge();
              range = change.getRange().cloneRange().shrinkToTextNode();
              startNode = range.startNode;
              startOffset = range.startOffset;
              textNode = startNode.get();
              text = startNode.text();
            }
          } //输入时删除mark标签内零宽字符。


        if (text.length > 0 && /^\u200B$/g.test(text.substr(0, 1))) {
          textNode.splitText(1);
          textNode.remove();
        }
      } //输入时删除mark标签外最后的零宽字符


      var prev = startNode.prev();

      if (startNode.isText() && prev && node.isMark(prev)) {
        var _textNode = startNode.get();

        var _text2 = startNode.text();

        if (_text2.length > 0 && /^\u200B$/g.test(_text2.substr(0, 1))) {
          _textNode.splitText(1);

          _textNode.remove();
        }
      }
    }
    /**
     * 获取安全可控的光标对象
     * @param range 默认当前光标
     */

  }, {
    key: "getSafeRange",
    value: function getSafeRange() {
      var range = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.getRange();
      // 如果不在编辑器内，聚焦到编辑器
      var commonAncestorNode = range.commonAncestorNode;

      if (!commonAncestorNode.isEditable() && !commonAncestorNode.inEditor()) {
        range.select(this.engine.container, true).shrinkToElementNode().collapse(false);
      } //卡片


      var rangeClone = range.cloneRange();
      rangeClone.collapse(true);
      this.focusCardRang(rangeClone);
      if (!range.startNode.equal(rangeClone.startNode) || range.startOffset !== rangeClone.startOffset) range.setStart(rangeClone.startContainer, rangeClone.startOffset);
      rangeClone = range.cloneRange();
      rangeClone.collapse(false);
      this.focusCardRang(rangeClone);
      if (!range.endNode.equal(rangeClone.endNode) || range.endOffset !== rangeClone.endOffset) range.setEnd(rangeClone.endContainer, rangeClone.endOffset);

      if (range.collapsed) {
        rangeClone = range.cloneRange();
        rangeClone.enlargeFromTextNode();
        var startNode = $(rangeClone.startContainer);
        var startOffset = rangeClone.startOffset;

        if (this.engine.node.isInline(startNode) && startOffset === 0) {
          range.setStartBefore(startNode[0]);
        }

        if (this.engine.node.isInline(startNode) && startOffset === startNode[0].childNodes.length) {
          range.setStartAfter(startNode[0]);
        }

        range.collapse(true);
      }

      return range;
    }
  }, {
    key: "focusCardRang",
    value: function focusCardRang(range) {
      var _this2 = this;

      var startNode = range.startNode,
          startOffset = range.startOffset;
      var card = this.engine.card.find(startNode);

      if (card) {
        var cardCenter = card.getCenter().get();

        if (cardCenter && (!startNode.isElement() || startNode[0].parentNode !== card.root[0] || startNode.attributes(CARD_ELEMENT_KEY))) {
          var comparePoint = function comparePoint() {
            var doc_rang = Range.create(_this2.engine);
            doc_rang.select(cardCenter, true);
            return doc_rang.comparePoint(startNode, startOffset) < 0;
          };

          if ('inline' === card.constructor.cardType) {
            range.select(card.root);
            range.collapse(comparePoint());
            return;
          }

          if (comparePoint()) {
            card.focusPrevBlock(range, true);
          } else {
            card.focusNextBlock(range, true);
          }
        }
      }
    }
  }, {
    key: "initNativeEvents",
    value: function initNativeEvents() {
      var _this3 = this;

      var _this$engine6 = this.engine,
          container = _this$engine6.container,
          card = _this$engine6.card,
          clipboard = _this$engine6.clipboard;
      this.event.onInput(function (event) {
        var range = _this3.getRange();

        _this3.repairInput(event, range);

        _this3.select(range);

        _this3.onSelect();

        _this3.change();
      });
      this.event.onDocument('selectionchange', function () {
        var window = container.window;
        var selection = window === null || window === void 0 ? void 0 : window.getSelection();

        if (selection && selection.anchorNode) {
          var range = Range.from(_this3.engine, selection);
          card.each(function (card) {
            var center = card.getCenter();

            if (center && center.length > 0) {
              var isSelect = selection.containsNode(center[0]);

              if (!isSelect && !range.collapsed && selection.focusNode) {
                var focusCard = _this3.engine.card.find(selection.focusNode, true);

                if (focusCard && card.root.equal(focusCard.root) && (!selection.anchorNode || !focusCard.root.contains(selection.anchorNode))) {
                  isSelect = true;
                }
              }

              var singleSelectable = card.constructor.singleSelectable;
              if (singleSelectable !== false) card.select(isSelect);
            }
          });
          var cardComponent = card.getSingleSelectedCard(range);

          if (cardComponent) {
            cardComponent.select(true);
          }
        } else {
          card.each(function (card) {
            return card.select(false);
          });
        }
      });
      this.event.onSelect(function () {
        var range = _this3.getRange();

        if (range.collapsed && range.containsCard()) {
          _this3.getSafeRange(range);
        }

        _this3.select(range);

        card.activate(range.commonAncestorNode, ActiveTrigger.CUSTOM_SELECT);

        _this3.onSelect();
      });
      this.event.onDocument('click', function (e) {
        if (!e.target) return;
        var cardComponent = card.find($(e.target));

        if (cardComponent) {
          var cardEntry = cardComponent.constructor;

          if (cardEntry.cardType === CardType.INLINE) {
            card.activate(cardComponent.root, ActiveTrigger.CLICK, cardEntry.toolbarFollowMouse ? e : undefined);
          }

          if (cardComponent.onFocus) {
            cardComponent.onFocus();
          }
        }
      });
      this.event.onDocument('mousedown', function (e) {
        if (!e.target) return;
        var targetNode = $(e.target);
        var cardComponent = card.find(targetNode);

        if (cardComponent && cardComponent.constructor.cardType === CardType.INLINE) {
          return;
        } // 点击元素已被移除


        if (targetNode.closest('body').length === 0) {
          return;
        } // 阅读模式节点


        if (targetNode.closest('.am-view').length > 0) {
          return;
        } // 工具栏、侧边栏、内嵌工具栏的点击


        var node = targetNode;

        while (node) {
          var attrValue = node.attributes(DATA_ELEMENT);

          if (attrValue && [ROOT, EDITABLE].indexOf(attrValue) < 0) {
            return;
          }

          node = node.parent();
        } //如果当前target是卡片，但是光标不在卡片上，让其选中


        var _this3$getRange = _this3.getRange(),
            startNode = _this3$getRange.startNode;

        if (cardComponent && !card.find(startNode, true)) {
          card.select(cardComponent);
        }

        card.activate(targetNode, ActiveTrigger.MOUSE_DOWN);
      });
      this.event.onDocument('copy', function (event) {
        clipboard.write(event);
      });
      this.event.onContainer('cut', function (event) {
        event.stopPropagation();
        clipboard.write(event, undefined, function () {
          clipboard.cut();

          _this3.change();
        });
      });

      var pasteMarkdown = /*#__PURE__*/function () {
        var _ref = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(source) {
          var parser, schema, text, textNode, newText;
          return _regeneratorRuntime.wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  parser = new Parser(source, _this3.engine);
                  schema = _this3.engine.schema.clone(); //转换Text，没那么严格，加入以下规则，以免被过滤掉，并且 div后面会加换行符

                  schema.add([{
                    name: 'span',
                    type: 'mark'
                  }, {
                    name: 'div',
                    type: 'block'
                  }]);
                  text = parser.toText(schema);
                  textNode = $(document.createTextNode(text));

                  _this3.engine.trigger('paste:markdown-before', textNode);

                  _this3.engine.trigger('paste:markdown', textNode);

                  _this3.engine.trigger('paste:markdown-after', textNode);

                  newText = textNode.text();

                  if (newText !== text) {
                    _this3.engine.messageConfirm(_this3.engine.language.get('checkMarkdown', 'title')).then(function () {
                      var _textNode$get;

                      (_textNode$get = textNode.get()) === null || _textNode$get === void 0 ? void 0 : _textNode$get.normalize();

                      _this3.paste(textNode.text());
                    }).catch(function () {});
                  }

                case 10:
                case "end":
                  return _context.stop();
              }
            }
          }, _callee);
        }));

        return function pasteMarkdown(_x) {
          return _ref.apply(this, arguments);
        };
      }();

      this.event.onPaste(function (data) {
        var html = data.html,
            text = data.text,
            files = data.files,
            isPasteText = data.isPasteText;
        var source = '';

        if (files.length === 0) {
          // 纯文本粘贴
          if (isPasteText) {
            var value = '';
            if (text) value = text;else if (html) value = new Parser(html, _this3.engine).toText();
            source = new TextParser(value).toHTML();
          } else {
            // 富文本粘贴
            if (html && html.indexOf('<meta name="source" content="aomao" />') > -1) {
              source = html;
            } else if (text && /^https?:\/\/\S+$/.test(text)) {
              var _value = escape(text);

              source = "<a href=\"".concat(_value, "\" target=\"_blank\">").concat(_value, "</a>");
            } else if (html) {
              source = html;
            } else if (text) {
              source = new TextParser(text).toHTML();
            }
          }
        }

        if (_this3.engine.trigger('paste:event', data, source) === false) return;

        if (files.length === 0) {
          setTimeout(function () {
            pasteMarkdown(source);
          }, 200);

          _this3.paste(source);
        }
      });

      var insertCardAble = function insertCardAble(range) {
        // 找不到目标位置
        // TODO: 临时解决，如果 drop Range 在Card里则不触发
        return !range || card.closest(range.commonAncestorContainer);
      };

      this.event.onDrop(function (_ref2) {
        var event = _ref2.event,
            range = _ref2.range,
            card = _ref2.card,
            files = _ref2.files;

        if (card) {
          event.preventDefault();
          if (insertCardAble(range)) return;
          var cardEntry = card.constructor;
          var cardName = cardEntry.cardName;
          var cardValue = card.getValue();

          _this3.engine.card.remove(card.root);

          _this3.select(range);

          _this3.engine.card.insert(cardName, cardValue);
        }

        if (files.length > 0) {
          event.preventDefault();
          if (insertCardAble(range)) return;

          _this3.select(range);

          _this3.engine.trigger('drop:files', files);
        }
      });
    }
  }, {
    key: "paste",
    value: function paste(source) {
      var _this4 = this;

      var fragment = new Paste(source, this.engine).normalize();
      this.engine.trigger('paste:before', fragment);
      this.insertFragment(fragment, function (range) {
        _this4.engine.trigger('paste:insert', range);

        var selection = range.createSelection();

        _this4.engine.card.render(undefined, function () {
          selection.move();
          range.scrollRangeIntoView();

          _this4.apply(range);
        });
      });
      this.engine.trigger('paste:after');
    }
  }, {
    key: "combinTextNode",
    value: function combinTextNode$1() {
      combinTextNode(this.engine.container);
    }
    /**
     * 应用一个具有改变dom结构的操作
     * @param range 光标
     */

  }, {
    key: "apply",
    value: function apply(range) {
      this.combinTextNode();
      if (range) this.select(range);
      this.change();
      this.engine.block.generateRandomIDForDescendant(this.engine.container.get());
    }
    /**
     * 插入片段
     * @param fragment 片段
     * @param callback 插入后的回调函数
     */

  }, {
    key: "insertFragment",
    value: function insertFragment(fragment) {
      var callback = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : function () {};
      var _this$engine7 = this.engine,
          block = _this$engine7.block,
          list = _this$engine7.list,
          card = _this$engine7.card,
          schema = _this$engine7.schema;
      var nodeApi = this.engine.node;
      var range = this.getSafeRange();
      var firstBlock = block.closest(range.startNode);
      var lastBlock = block.closest(range.endNode);
      var onlyOne = lastBlock[0] === firstBlock[0];
      var isBlockLast = block.isLastOffset(range, 'end');
      var mergeTags = schema.getCanMergeTags();
      var allowInTags = schema.getAllowInTags();
      var mergeNode = firstBlock.closest(mergeTags.join(','));
      var isCollapsed = range.collapsed;
      var childNodes = fragment.childNodes;
      var firstNode = $(fragment.firstChild || []);

      if (!isCollapsed) {
        this.deleteContent(range, onlyOne || !isBlockLast);
      }

      if (!firstNode[0]) {
        if (callback) callback(range);
        this.apply(range);
        return;
      }

      if (!nodeApi.isBlock(firstNode) && !firstNode.isCard()) {
        range.shrinkToElementNode().insertNode(fragment);
        if (callback) callback(range);
        this.apply(range);
        return;
      }

      range.deepCut();
      var startNode = range.startContainer.childNodes[range.startOffset - 1];
      var endNode = range.startContainer.childNodes[range.startOffset];

      if (mergeNode[0]) {
        childNodes.forEach(function (node) {
          if (mergeTags.indexOf($(node).name) < 0) {
            nodeApi.wrap($(node), nodeApi.clone(mergeNode, false));
          }
        });
      }

      if (childNodes.length !== 0) {
        var doc = getDocument(range.startContainer);
        var lastNode = $(childNodes[childNodes.length - 1]);

        if ('br' === lastNode.name) {
          lastNode.remove();
          lastNode = $(childNodes[childNodes.length - 1]);
        }

        var _fragment = doc.createDocumentFragment();

        var node = $(childNodes[0]);

        while (node && node.length > 0) {
          nodeApi.removeSide(node);
          var next = node.next();

          if (!next) {
            lastNode = node;
          }

          _fragment.appendChild(node[0]);

          node = next;
        }

        range.insertNode(_fragment); //range.shrinkToElementNode().collapse(false);

        var component = card.find(range.startNode);
        if (component) component.focus(range, false);
      }

      var getFirstChild = function getFirstChild(node) {
        var child = node.first();
        if (!child || !nodeApi.isBlock(child)) return node;

        while (allowInTags.indexOf(child ? child.name : '') > -1) {
          child = child.first();
        }

        return child;
      };

      var getLastChild = function getLastChild(node) {
        var child = node.last();
        if (!child || !nodeApi.isBlock(child)) return node;

        while (allowInTags.indexOf(child ? child.name : '') > -1) {
          child = child.last();
        }

        return child;
      };

      var isSameListChild = function isSameListChild(_lastNode, _firstNode) {
        var _firstNode$parent, _lastNode$parent;

        var isSameParent = ((_firstNode$parent = _firstNode.parent()) === null || _firstNode$parent === void 0 ? void 0 : _firstNode$parent.name) === ((_lastNode$parent = _lastNode.parent()) === null || _lastNode$parent === void 0 ? void 0 : _lastNode$parent.name);
        return 'p' === _firstNode.name && isSameParent || _lastNode.name === _firstNode.name && isSameParent && !('li' === _lastNode.name && !list.isSame(_lastNode.parent(), _firstNode.parent()));
      };

      var removeEmptyNode = function removeEmptyNode(node) {
        while (!node.isEditable()) {
          var parent = node.parent();
          node.remove();
          if (!parent || !nodeApi.isEmpty(parent)) break;
          node = parent;
        }
      };

      var clearList = function clearList(lastNode, nextNode) {
        if (lastNode.name === nextNode.name && 'p' === lastNode.name) {
          var attr = nextNode.attributes();
          if (attr['data-id']) delete attr['data-id'];
          lastNode.attributes(attr);
        }

        if (nodeApi.isLikeEmpty(lastNode) && !nodeApi.isLikeEmpty(nextNode)) {
          lastNode.get().innerHTML = '';
        }

        if (nodeApi.isCustomize(lastNode) === nodeApi.isCustomize(nextNode)) list.unwrapCustomize(nextNode);
      };

      if (startNode) {
        var _firstNode = getFirstChild($(startNode.nextSibling || []));

        var _lastNode = getLastChild($(startNode));

        if (_lastNode.name === 'p' && _firstNode.name !== _lastNode.name && isSameListChild(_lastNode, _firstNode)) {
          clearList(_lastNode, _firstNode);
          nodeApi.merge(_lastNode, _firstNode, false);
          removeEmptyNode(_firstNode);
        } else {
          if (nodeApi.isEmpty(_lastNode) || list.isEmptyItem(_lastNode)) {
            removeEmptyNode(_lastNode);
          }
        }
      }

      if (endNode) {
        var prevNode = getLastChild($(endNode.previousSibling || []));
        var nextNode = getFirstChild($(endNode));
        /**range
            .select(prevNode, true)
            .shrinkToElementNode()
            .collapse(false);**/

        if (prevNode && nodeApi.isEmpty(prevNode)) {
          removeEmptyNode(prevNode);
        }

        if (nextNode && nodeApi.isEmpty(nextNode)) {
          removeEmptyNode(nextNode);
        } else if (prevNode && isSameListChild(prevNode, nextNode)) {
          nodeApi.merge(prevNode, nextNode, false);
          removeEmptyNode(nextNode);
        }
      }

      block.merge(range);
      list.merge(undefined, range);
      if (callback) callback(range);
      this.apply(range);
    }
    /**
     * 删除内容
     * @param range 光标，默认获取当前光标
     * @param isDeepMerge 删除后是否合并
     */

  }, {
    key: "deleteContent",
    value: function deleteContent(range, isDeepMerge) {
      var safeRange = range || this.getSafeRange();

      if (safeRange.collapsed) {
        return;
      }

      var _this$engine8 = this.engine,
          mark = _this$engine8.mark,
          inline = _this$engine8.inline,
          list = _this$engine8.list;
      var nodeApi = this.engine.node;
      var blockApi = this.engine.block;
      var cloneRange = safeRange.cloneRange();
      cloneRange.collapse(true);
      var activeMarks = mark.findMarks(cloneRange);
      safeRange.enlargeToElementNode(); // 获取上面第一个 Block

      var block = blockApi.closest(safeRange.startNode); // 获取的 block 超出编辑范围

      if (!block.isEditable() && !block.inEditor()) {
        if (!range) this.apply(safeRange);
        return;
      } // 先删除范围内的所有内容


      safeRange.extractContents();
      safeRange.collapse(true); // 后续处理

      var startNode = safeRange.startNode,
          startOffset = safeRange.startOffset; // 只删除了文本，不做处理

      if (startNode.isText()) {
        if (!range) this.apply(safeRange);
        return;
      }

      var prevNode = startNode[0].childNodes[startOffset - 1];
      var nextNode = startNode[0].childNodes[startOffset];
      var isEmptyNode = startNode[0].childNodes.length === 0;

      if (!isEmptyNode) {
        var _startNode$first2;

        var firstChild = startNode[0].firstChild;
        if (startNode[0].childNodes.length === 1 && firstChild.nodeType === getWindow().Node.ELEMENT_NODE && nodeApi.isCustomize(startNode) && ((_startNode$first2 = startNode.first()) === null || _startNode$first2 === void 0 ? void 0 : _startNode$first2.isCard())) isEmptyNode = true;
      }

      if (isEmptyNode && nodeApi.isBlock(startNode)) {
        var html = nodeApi.getBatchAppendHTML(activeMarks, '<br />');

        if (startNode.isEditable()) {
          html = '<p>'.concat(html, '</p>');
        }

        startNode.append($(html));
        var br = startNode.find('br');
        var parent = br.parent();

        if (parent && nodeApi.isMark(parent)) {
          nodeApi.replace(br, $("\u200B", null));
        }

        safeRange.select(startNode, true).shrinkToElementNode().shrinkToTextNode();
        safeRange.collapse(false);
        if (!range) this.apply(safeRange);
        return;
      } //深度合并


      var deepMergeNode = function deepMergeNode(range, prevNode, nextNode, marks) {
        if (nodeApi.isBlock(prevNode) && !nodeApi.isVoid(prevNode) && !prevNode.isCard()) {
          range.select(prevNode, true);
          range.collapse(false);
          var selection = range.createSelection();
          nodeApi.merge(prevNode, nextNode);
          selection.move();
          var prev = range.getPrevNode();
          var next = range.getNextNode(); // 合并之后变成空 Block

          var _startNode2 = range.startNode;

          if (!prev && !next && nodeApi.isBlock(_startNode2)) {
            _startNode2.append($(nodeApi.getBatchAppendHTML(marks, '<br />')));

            range.select(_startNode2.find('br'), true);
            range.collapse(false);
          }

          if (prev && next && !prev.isCard() && !next.isCard()) {
            deepMergeNode(range, prev, next, marks);
          }
        }
      };

      if (prevNode && nextNode && nodeApi.isBlock(prevNode) && nodeApi.isBlock(nextNode) && isDeepMerge) {
        deepMergeNode(safeRange, $(prevNode), $(nextNode), activeMarks);
      }

      startNode.children().each(function (node) {
        var domNode = $(node);
        if (!nodeApi.isVoid(domNode) && domNode.isElement() && '' === nodeApi.html(domNode)) domNode.remove(); //给inline节点添加零宽字符，用于光标选择

        if (nodeApi.isInline(domNode)) {
          inline.repairCursor(domNode);
        }
      }); //移除空列表

      if (nodeApi.isList(startNode) && nodeApi.isEmpty(startNode)) {
        startNode.remove();
      } //修复inline节点光标选择在最后的零宽字符上时，将光标位置移到inline节点末尾


      cloneRange = safeRange.cloneRange().shrinkToTextNode();

      if (cloneRange.startNode.isText() && /^\u200B/g.test(cloneRange.startNode.text()) && cloneRange.startOffset === 0) {
        var prev = cloneRange.startNode.prev();

        if (prev && this.engine.node.isInline(prev)) {
          safeRange.select(prev, true);
          safeRange.collapse(false);
        }
      }

      if (!range) this.apply(safeRange);
    }
    /**
     * 删除节点，删除后如果是空段落，自动添加 BR
     * @param node 要删除的节点
     */

  }, {
    key: "addBrAfterDelete",
    value: function addBrAfterDelete(node) {
      var range = this.getRange();
      var parent = node.parent();
      node.remove();

      if (parent && this.engine.node.isEmpty(parent)) {
        if (parent.isEditable()) {
          this.engine.node.html(parent, '<p><br /></p>');
          range.select(parent, true).shrinkToElementNode().collapse(false);
        } else {
          this.engine.node.html(parent, '<br />');
          range.select(parent, true).collapse(false);
        }

        this.select(range);
      }
    }
    /**
     * 去除当前光标最接近的block节点或传入的节点外层包裹
     * @param node 节点
     */

  }, {
    key: "unwrapNode",
    value: function unwrapNode(node) {
      var block = this.engine.block;
      var range = this.getRange();
      node = node || block.closest(range.startNode);

      if (!node.inEditor()) {
        return;
      }

      var selection = range.createSelection();
      this.engine.node.unwrap(node);
      selection.move();
      this.select(range);
    }
    /**
     * 删除当前光标最接近的block节点或传入的节点的前面一个节点后合并
     * @param node 节点
     */

  }, {
    key: "mergeAfterDeletePrevNode",
    value: function mergeAfterDeletePrevNode(node) {
      var _node$first, _node$first4, _prevBlock$first;

      var block = this.engine.block;
      var range = this.getRange();
      node = node || block.closest(range.startNode); // <p><br />foo</p>，先删除 BR

      if (node.children().length > 1 && ((_node$first = node.first()) === null || _node$first === void 0 ? void 0 : _node$first.name) === 'br') {
        var _node$first2;

        (_node$first2 = node.first()) === null || _node$first2 === void 0 ? void 0 : _node$first2.remove();
        return;
      }

      var prevBlock = node.prev(); // 前面没有 DOM 节点

      if (!prevBlock) {
        var _node$parent, _node$parent2;

        if (((_node$parent = node.parent()) === null || _node$parent === void 0 ? void 0 : _node$parent.inEditor()) && !((_node$parent2 = node.parent()) === null || _node$parent2 === void 0 ? void 0 : _node$parent2.isEditable())) {
          this.unwrapNode(node);
        }

        return;
      } // 前面是Card


      if (prevBlock.isCard()) {
        var _node$first3;

        if (node.children().length === 1 && ((_node$first3 = node.first()) === null || _node$first3 === void 0 ? void 0 : _node$first3.name) === 'br' || this.engine.node.isEmpty(node)) {
          node.remove();
        }

        var card = this.engine.card.find(prevBlock);

        if (card) {
          this.engine.card.focus(card);
          return;
        }
      } // 前面是 void 节点


      if (this.engine.node.isVoid(prevBlock)) {
        prevBlock.remove();
        return;
      } // 前面是空段落


      if (this.engine.node.isRootBlock(prevBlock) && this.engine.node.isEmpty(prevBlock)) {
        prevBlock.remove();
        return;
      } // 前面是文本节点


      if (prevBlock.isText()) {
        var paragraph = $('<p />');
        prevBlock.before(paragraph);
        paragraph.append(prevBlock);
        prevBlock = paragraph;
      }

      if (['ol', 'ul'].indexOf(prevBlock.name) >= 0) {
        prevBlock = prevBlock.last();
      } // 只有一个 <br /> 时先删除


      if (node.children().length === 1 && ((_node$first4 = node.first()) === null || _node$first4 === void 0 ? void 0 : _node$first4.name) === 'br') {
        var _node$first5;

        (_node$first5 = node.first()) === null || _node$first5 === void 0 ? void 0 : _node$first5.remove();
      } else if (prevBlock && prevBlock.children().length === 1 && ((_prevBlock$first = prevBlock.first()) === null || _prevBlock$first === void 0 ? void 0 : _prevBlock$first.name) === 'br') {
        var _prevBlock$first2;

        (_prevBlock$first2 = prevBlock.first()) === null || _prevBlock$first2 === void 0 ? void 0 : _prevBlock$first2.remove();
      }

      if (!prevBlock || prevBlock.isText()) {
        this.unwrapNode(node);
      } else {
        var selection = range.createSelection();
        this.engine.node.merge(prevBlock, node);
        selection.move();
        this.select(range);
        this.engine.mark.merge();
        this.engine.list.merge();
      }
    }
    /**
     * 焦点移动到当前光标最接近的block节点或传入的节点前一个 Block
     * @param block 节点
     * @param isRemoveEmptyBlock 如果前一个block为空是否删除，默认为否
     */

  }, {
    key: "focusPrevBlock",
    value: function focusPrevBlock(block) {
      var isRemoveEmptyBlock = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      var range = this.getRange();
      block = block || this.engine.block.closest(range.startNode);
      var prevBlock = block.prev();

      if (!prevBlock) {
        return;
      } // 前面是Card


      if (prevBlock.isCard()) {
        var card = this.engine.card.find(prevBlock);
        if (card) card.focus(range);
        return;
      } // 前面是列表


      if (['ol', 'ul'].indexOf(prevBlock.name) >= 0) {
        prevBlock = prevBlock.last();
      }

      if (!prevBlock) {
        return;
      }

      if (isRemoveEmptyBlock && this.engine.node.isEmptyWithTrim(prevBlock)) {
        prevBlock.remove();
        return;
      }

      range.select(prevBlock, true);
      range.collapse(false);
      this.select(range);
    }
  }, {
    key: "destroy",
    value: function destroy() {
      this.event.destroy();
      this.clearChangeTimer();
    }
  }]);

  return ChangeModel;
}();

/**
 * 标签规则
 */

var Schema = /*#__PURE__*/function () {
  function Schema() {
    _classCallCheck(this, Schema);

    this._all = [];
    this._typeMap = {};
    this.data = {
      blocks: [],
      inlines: [],
      marks: [],
      globals: {}
    };
  }

  _createClass(Schema, [{
    key: "add",
    value:
    /**
     * 增加规则
     * 只有 type 和 attributes 时，将作为此类型全局属性，与其它所有同类型标签属性将合并
     * @param rules 规则
     */
    function add(rules) {
      var _this = this;

      rules = cloneDeep(rules);

      if (!Array.isArray(rules)) {
        rules = [rules];
      }

      rules.forEach(function (rule) {
        if (isSchemaRule(rule)) {
          //删除全局属性已有的规则
          if (rule.attributes) {
            Object.keys(rule.attributes).forEach(function (key) {
              if (!_this.data.globals[rule.type]) return;

              if (key === 'style') {
                Object.keys(rule.attributes.style).forEach(function (styleName) {
                  if (_this.data.globals[rule.type][key] && _this.data.globals[rule.type][key][styleName] === rule.attributes.style[styleName]) {
                    delete rule.attributes.style[styleName];
                  }
                });
              } else if (_this.data.globals[rule.type][key] === rule.attributes[key]) {
                delete rule.attributes[key];
              }
            });
          }

          if (rule.type === 'block') {
            _this.data.blocks.push(rule);
          } else if (rule.type === 'inline') {
            _this.data.inlines.push(rule);
          } else if (rule.type === 'mark') {
            _this.data.marks.push(rule);
          }
        } else if (!!_this.data["".concat(rule.type, "s")]) {
          _this.data.globals[rule.type] = merge(_objectSpread({}, _this.data.globals[rule.type]), rule.attributes);
        }
      }); //按照必要属性个数排序

      var getCount = function getCount(rule) {
        var aAttributes = rule.attributes || {};
        var aStyles = aAttributes.style || {};
        var aCount = 0;
        var sCount = 0;
        Object.keys(aAttributes).forEach(function (attributesName) {
          var attributesValue = aAttributes[attributesName];
          if (isSchemaValueObject(attributesValue) && attributesValue.required) aCount++;
        });
        Object.keys(aStyles).forEach(function (stylesName) {
          var stylesValue = aStyles[stylesName];
          if (isSchemaValueObject(stylesValue) && stylesValue.required) sCount++;
        });
        return [aCount, sCount];
      };

      var _this$data = this.data,
          blocks = _this$data.blocks,
          marks = _this$data.marks,
          inlines = _this$data.inlines;
      this._all = [].concat(_toConsumableArray(blocks), _toConsumableArray(marks), _toConsumableArray(inlines)).sort(function (a, b) {
        var _getCount = getCount(a),
            _getCount2 = _slicedToArray(_getCount, 2),
            aACount = _getCount2[0],
            aSCount = _getCount2[1];

        var _getCount3 = getCount(b),
            _getCount4 = _slicedToArray(_getCount3, 2),
            bACount = _getCount4[0],
            bSCount = _getCount4[1];

        if (aACount > bACount) return -1;
        if (aACount === bACount) return aSCount === bSCount ? 0 : aSCount > bSCount ? -1 : 1;
        return 1;
      });
    }
    /**
     * 克隆当前schema对象
     */

  }, {
    key: "clone",
    value: function clone() {
      var schema = new Schema();
      schema.data = cloneDeep(this.data);
      return schema;
    }
    /**
     * 查找规则
     * @param callback 查找条件
     */

  }, {
    key: "find",
    value: function find(callback) {
      var _this2 = this;

      var schemas = [];
      Object.keys(this.data).some(function (key) {
        if (key !== 'globals') {
          var rules = _this2.data[key].filter(function (rule) {
            return callback(rule);
          });

          if (rules && rules.length > 0) {
            schemas = schemas.concat(rules);
          }
        }

        return;
      });
      return schemas;
    }
  }, {
    key: "getType",
    value: function getType(node, filter) {
      if (node.type !== getWindow().Node.ELEMENT_NODE) return undefined;
      var id = node.attributes('data-id');
      if (!id) id = getHashId(node, false);else id = id.split('-')[0];
      if (!!this._typeMap[id] && (!filter || filter(this._typeMap[id]))) return this._typeMap[id].type;
      var reuslt = this.getRule(node, filter);
      if (reuslt) this._typeMap[id] = reuslt;
      return reuslt === null || reuslt === void 0 ? void 0 : reuslt.type;
    }
    /**
     * 根据节点获取符合的规则
     * @param node 节点
     * @param filter 过滤
     * @returns
     */

  }, {
    key: "getRule",
    value: function getRule(node, filter) {
      var _this3 = this;

      filter = filter || function (rule) {
        return rule.name === node.name;
      };

      if (node.type !== getWindow().Node.ELEMENT_NODE) return undefined;
      return this._all.find(function (rule) {
        return filter(rule) && _this3.checkNode(node, rule.attributes);
      });
    }
    /**
     * 检测节点是否符合某一属性规则
     * @param node 节点
     * @param attributes 属性规则
     */

  }, {
    key: "checkNode",
    value: function checkNode(node, attributes) {
      var _this4 = this;

      //获取节点属性
      var nodeAttributes = node.attributes();
      var nodeStyles = node.css();
      delete nodeAttributes['style'];
      var styles = (attributes || {}).style;
      attributes = omit(_objectSpread({}, attributes), 'style'); //需要属性每一项都能效验通过

      var attrResult = Object.keys(attributes).every(function (attributesName) {
        return _this4.checkValue(attributes, attributesName, nodeAttributes[attributesName]);
      });
      if (!attrResult) return false;
      return Object.keys(styles || {}).every(function (styleName) {
        return _this4.checkValue(styles, styleName, nodeStyles[styleName]);
      });
    }
    /**
     * 检测值是否符合规则
     * @param rule 规则
     * @param attributesName 属性名称
     * @param attributesValue 属性值
     */

  }, {
    key: "checkValue",
    value: function checkValue(schema, attributesName, attributesValue, force) {
      if (!schema[attributesName]) return false;
      var rule = schema[attributesName];

      if (isSchemaValueObject(rule)) {
        //如果没有值，强制状态就返回 false，非强制就返回 true
        if (attributesValue === undefined) return !rule.required;
        rule = rule.value;
      } //默认都不为强制的
      else if (!force || attributesValue === undefined) return true;
      /**
       * 自定义规则解析
       */


      if (typeof rule === 'string' && rule.charAt(0) === '@') {
        switch (rule) {
          case '@number':
            rule = /^-?\d+(\.\d+)?$/;
            break;

          case '@length':
            rule = /^-?\d+(\.\d+)?(\w*|%)$/;
            break;

          case '@color':
            rule = /^(rgb(.+?)|#\w{3,6}|\w+)$/i;
            break;

          case '@url':
            rule = validUrl;
            break;
        }
      }
      /**
       * 字符串解析
       */


      if (typeof rule === 'string') {
        if (rule === '*') {
          return true;
        }

        if (attributesName === 'class') {
          return (attributesValue || '').split(/\s+/).some(function (value) {
            return value.trim() === rule;
          });
        }

        return rule === attributesValue;
      }
      /**
       * 数组解析
       */


      if (Array.isArray(rule)) {
        if (attributesName === 'class') {
          return (attributesValue || '').split(/\s+/).every(function (value) {
            return value.trim() === '' ? true : rule.indexOf(value.trim()) > -1;
          });
        }

        return rule.indexOf(attributesValue) > -1;
      }
      /**
       * 解析正则表达式
       */


      if (_typeof(rule) === 'object' && typeof rule.test === 'function') {
        if (attributesName === 'class') {
          return (attributesValue || '').split(/\s+/).every(function (value) {
            return value.trim() === '' ? true : rule.test(value.trim());
          });
        }

        return rule.test(attributesValue || '');
      }
      /**
       * 自定义函数解析
       */


      if (typeof rule === 'function') {
        return rule(attributesValue);
      }

      return true;
    }
    /**
     * 过滤节点样式
     * @param styles 样式
     * @param rule 规则
     */

  }, {
    key: "filterStyles",
    value: function filterStyles(styles, rule) {
      var _this5 = this;

      Object.keys(styles).forEach(function (styleName) {
        var _rule$attributes, _rule$attributes2;

        //没有设置规则，全部删除
        if (!((_rule$attributes = rule.attributes) === null || _rule$attributes === void 0 ? void 0 : _rule$attributes.style)) {
          delete styles[styleName];
          return;
        }

        if (!_this5.checkValue(rule === null || rule === void 0 ? void 0 : (_rule$attributes2 = rule.attributes) === null || _rule$attributes2 === void 0 ? void 0 : _rule$attributes2.style, styleName, styles[styleName], true)) delete styles[styleName];
      });
    }
    /**
     * 过滤节点属性
     * @param attributes 属性
     * @param rule 规则
     */

  }, {
    key: "filterAttributes",
    value: function filterAttributes(attributes, rule) {
      var _this6 = this;

      Object.keys(attributes).forEach(function (attributesName) {
        //没有设置规则，全部删除
        if (!rule.attributes) {
          delete attributes[attributesName];
          return;
        }

        if (!_this6.checkValue(rule === null || rule === void 0 ? void 0 : rule.attributes, attributesName, attributes[attributesName], true)) delete attributes[attributesName];
      });
    }
    /**
     * 过滤满足node节点规则的属性和样式
     * @param node 节点，用于获取规则
     * @param attributes 属性
     * @param styles 样式
     * @returns
     */

  }, {
    key: "filter",
    value: function filter(node, attributes, styles) {
      var rule = this.getRule(node);
      if (!rule) return;

      var allRule = _objectSpread({}, rule);

      var globalRule = Object.keys(this.data.globals).find(function (dataType) {
        return rule.type === dataType;
      });

      if (globalRule) {
        allRule.attributes = _objectSpread(_objectSpread({}, allRule.attributes), this.data.globals[globalRule]);
      }

      this.filterAttributes(attributes, allRule);
      this.filterStyles(styles, allRule);
    }
    /**
     * 查找节点符合规则的最顶层的节点名称
     * @param name 节点名称
     * @param callback 回调函数，判断是否继续向上查找，返回false继续查找
     * @returns 最顶级的block节点名称
     */

  }, {
    key: "closest",
    value: function closest(name) {
      var _this7 = this;

      var topName = name;
      this.data.blocks.filter(function (rule) {
        return rule.name === name;
      }).forEach(function (block) {
        var schema = block;

        if (schema.allowIn) {
          schema.allowIn.forEach(function (parentName) {
            if (_this7.isAllowIn(parentName, topName)) {
              topName = parentName;
            }
          });
          topName = _this7.closest(topName);
        }
      });
      return topName;
    }
    /**
     * 判断子节点名称是否允许放入指定的父节点中
     * @param source 父节点名称
     * @param target 子节点名称
     * @returns true | false
     */

  }, {
    key: "isAllowIn",
    value: function isAllowIn(source, target) {
      //p节点下不允许放其它block节点
      if (source === 'p') return false; //目标节点是p标签

      if (target === 'p' && source !== 'p') return true;
      return this.data.blocks.filter(function (rule) {
        return rule.name === target;
      }).some(function (block) {
        var schema = block;

        if (schema.allowIn && schema.allowIn.indexOf(source) > -1) {
          return true;
        }

        return;
      });
    }
    /**
     * 获取允许有子block节点的标签集合
     * @returns
     */

  }, {
    key: "getAllowInTags",
    value: function getAllowInTags() {
      var tags = [];
      this.data.blocks.forEach(function (rule) {
        var schema = rule;

        if (schema.allowIn) {
          schema.allowIn.forEach(function (name) {
            if (tags.indexOf(name) < 0) tags.push(name);
          });
        }
      });
      return tags;
    }
    /**
     * 获取能够合并的block节点的标签集合
     * @returns
     */

  }, {
    key: "getCanMergeTags",
    value: function getCanMergeTags() {
      var tags = [];
      this.data.blocks.forEach(function (rule) {
        var schema = rule;

        if (schema.canMerge === true) {
          if (tags.indexOf(schema.name) < 0) tags.push(schema.name);
        }
      });
      return tags;
    }
  }]);

  return Schema;
}();
var isSchemaValueObject = function isSchemaValueObject(value) {
  return value.required !== undefined;
};

/**
 * 随机一个数字
 * @param start 开始
 * @param max 最大
 */

var random$1 = function random(start, max) {
  return Math.floor(start + Math.random() * (max - start));
};
/**
 * 随机一个字符串不包含，0、o、O、l字符
 * @param length 长度，默认8
 */

var randomString = function randomString() {
  var length = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 8;
  var str = '23456789abcdefghijkmnpqrstuvwxyzABCDEFGHJKLMNPQRSTUVWXYZ';
  var word = '';

  while (length--) {
    word += str[random$1(0, str.length)];
  }

  return word;
};
var isTransientElement = function isTransientElement(node, transientElements) {
  if (node.isElement()) {
    //范围标记
    if ([CURSOR, ANCHOR, FOCUS].indexOf(node.attributes(DATA_ELEMENT)) > -1) {
      return true;
    } //data-element=ui 属性


    if (!!node.attributes(DATA_TRANSIENT_ELEMENT) || node.attributes(DATA_ELEMENT) === UI) {
      return true;
    }

    var parent = node.parent();
    if (node.isRoot() || (parent === null || parent === void 0 ? void 0 : parent.isRoot())) return false;
    var isCard = node.isCard(); //父级是卡片，并且没有可编辑区域

    if (!isCard && (parent === null || parent === void 0 ? void 0 : parent.isCard()) && !parent.isEditableCard()) {
      return true;
    }

    if (transientElements) {
      if (!isCard && transientElements.find(function (element) {
        return element === node[0];
      })) return true;
    } else {
      var closestNode = node.closest("".concat(CARD_SELECTOR, ",").concat(UI_SELECTOR), getParentInRoot);

      if (closestNode.length > 0 && closestNode.attributes(DATA_ELEMENT) === UI) {
        return true;
      } //在卡片里面，并且卡片不是可编辑卡片 或者是标记为正在异步渲染时的卡片


      if (!isCard && closestNode.length > 0 && closestNode.isCard() && (!closestNode.isEditableCard() || !!closestNode.attributes(CARD_ASYNC_RENDER))) {
        return true;
      }

      if (closestNode.length === 0) return false;
    }

    if (!isCard || node.isEditableCard()) return false; //当前是卡片，父级也是卡片

    var parentCard = parent === null || parent === void 0 ? void 0 : parent.closest(CARD_SELECTOR, getParentInRoot);

    if (parentCard && parentCard.isCard() && !parentCard.isEditableCard()) {
      return true;
    }
  }

  return false;
};
var isTransientAttribute = function isTransientAttribute(node, attr) {
  if (attr === CARD_ASYNC_RENDER) return true;
  if (node.isRoot() && !/^data-selection-/.test(attr)) return true;
  if (node.isCard() && ['id', 'class', 'style'].includes(attr)) return true;
  var transient = node.attributes(DATA_TRANSIENT_ATTRIBUTES);
  if (transient === '*' || transient.split(',').some(function (value) {
    return value.trim().toLowerCase() === attr.toLowerCase();
  })) return true;
  return false;
};
var reduceOperations = function reduceOperations(ops) {
  var data = [];

  for (var i = 0; i < ops.length; i++) {
    var op = ops[i];
    var next = ops[i + 1];
    isReverseOp(op, next) ? i++ : data.push(op);
  }

  return data;
};
var isCursorOp = function isCursorOp(op) {
  var insertOp = op;
  var deleteOp = op;
  return (insertOp.oi || deleteOp.od) && op.p && op.p.length === 2 && op.p[0] === 1 && op.p[1].toString().startsWith('data-selection-');
};
var isReverseOp = function isReverseOp(op, next) {
  var insertOp = op;
  var deleteOp = op;
  var insertNext = next;
  var deleteNext = next;
  var insertStringOp = op;
  var deleteStringOp = op;
  var insertStringNext = next;
  var deleteStringNext = next;
  return !!(op && next && (insertOp.li && deleteNext.ld && isEqual(insertOp.li, deleteNext.ld) && (isEqual(op.p, next.p) || isPathEqual(op.p, next.p) || isPathEqual(next.p, op.p)) || deleteOp.ld && insertNext.li && isEqual(deleteOp.ld, insertNext.li) && isEqual(op.p, next.p) || insertStringOp.si && deleteStringNext.sd && isEqual(insertStringOp.si, deleteStringNext.sd) && (isEqual(op.p, next.p) || isPathEqual(op.p, next.p, insertStringOp.si.length) || isPathEqual(next.p, op.p, insertStringOp.si.length)) || deleteStringOp.sd && insertStringNext.si && isEqual(deleteStringOp.sd, insertStringNext.si) && isEqual(op.p, next.p)));
};

var isPathEqual = function isPathEqual(op, next) {
  var length = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 1;
  if (op.length !== next.length) return false;
  var nextClone = next.slice();
  nextClone[nextClone.length - 1] = nextClone[nextClone.length - 1] - length;
  return isEqual(op.slice(), nextClone);
};

var transformOp = function transformOp(op, otherOp) {
  var ops = [];
  otherOp.forEach(function (op) {
    if (op.ops) ops.push(op.ops);
  });
  ops.forEach(function (o) {
    op = OTJSON.type.transform(op, o, 'left');
  });
  return op;
};
var transformPath = function transformPath() {
  var path = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  var operation = arguments.length > 1 ? arguments[1] : undefined;

  var _path = _slicedToArray(path, 2),
      startPath = _path[0],
      endPath = _path[1];

  if (!startPath || !endPath) return path;
  var start = startPath.map(function (p) {
    return parseInt(p.toString()) + 2;
  });
  var end = endPath.map(function (p) {
    return parseInt(p.toString()) + 2;
  });
  var ops = [];
  operation === null || operation === void 0 ? void 0 : operation.forEach(function (op) {
    if (op.ops) ops = ops.concat(op.ops);
  });
  ops.forEach(function (op) {
    start = handlePath(start, op);
    end = handlePath(end, op);
  });
  return [start.map(function (p) {
    return parseInt(p.toString()) - 2;
  }), end.map(function (p) {
    return parseInt(p.toString()) - 2;
  })];
};

var handlePath = function handlePath(path, op) {
  var writePath = path.slice();

  if (affectPath(op.p, path)) {
    if ('li' in op) {
      writePath[op.p.length - 1] = parseInt(writePath[op.p.length - 1].toString()) + 1;
    } else if ('ld' in op) writePath[op.p.length - 1] = parseInt(writePath[op.p.length - 1].toString()) - 1;else if ('si' in op) writePath[op.p.length - 1] = parseInt(writePath[op.p.length - 1].toString()) + op.si.length;else if ('sd' in op) {
      var length = op.p.length;
      writePath[length - 1] = parseInt(writePath[length - 1].toString()) - op.sd.length;
      writePath[length - 1] = Math.max(parseInt(op.p[length - 1].toString()), parseInt(writePath[length - 1].toString()));
    }
  }

  return writePath;
};

var affectPath = function affectPath(path, otherPath) {
  var index = 0;

  while (path[index]) {
    if (!otherPath[index]) return false;
    if (path[index] === otherPath[index]) index++;else {
      if (path[index] > otherPath[index]) return false;
      if (path[index] < otherPath[index]) return path[index + 1] === undefined;
    }
  }

  return true;
};
var getOldIndex = function getOldIndex(index, ops) {
  var i = index;
  ops.forEach(function (op) {
    if (parseInt(op.p[op.p.length - 1].toString()) - 2 <= index) {
      if ('li' in op) i -= 1;else if ('ld' in op) i += 1;
    }
  });
  return i;
};

var SelectionData = /*#__PURE__*/function () {
  function SelectionData(engine) {
    _classCallCheck(this, SelectionData);

    this.engine = void 0;
    this.currentRangePath = [];
    this.engine = engine;
  }

  _createClass(SelectionData, [{
    key: "getAll",
    value: function getAll() {
      var _container$get;

      var container = this.engine.container;
      var data = [];
      var attributes = (_container$get = container.get()) === null || _container$get === void 0 ? void 0 : _container$get.attributes;
      if (!attributes) return data;

      for (var i = 0; i < attributes.length; i++) {
        var item = attributes.item(i);
        if (!item) continue;
        var nodeName = item.nodeName,
            nodeValue = item.nodeValue;

        if (/^data-selection-/.test(nodeName) && nodeValue) {
          var value = JSON.parse(decodeURIComponent(nodeValue));

          if (value) {
            data.push(value);
          }
        }
      }

      return data;
    }
  }, {
    key: "setAll",
    value: function setAll(data) {
      var _container$get2;

      var container = this.engine.container;
      var dataState = {};
      data.forEach(function (item) {
        if (item) {
          var name = 'data-selection-'.concat(item.uuid);
          dataState[name] = true;
          var value = container.attributes(name);
          var value_str = encodeURIComponent(JSON.stringify(item));

          if (value !== value_str) {
            container.attributes(name, value_str);
          }
        }
      });
      var attributes = (_container$get2 = container.get()) === null || _container$get2 === void 0 ? void 0 : _container$get2.attributes;
      if (!attributes) return;

      for (var i = 0; i < attributes.length; i++) {
        var item = attributes.item(i);
        if (!item) continue;
        var nodeName = item.nodeName;

        if (/^data-selection-/.test(nodeName) && !dataState[nodeName]) {
          container.removeAttributes(nodeName);
        }
      }
    }
  }, {
    key: "remove",
    value: function remove(name) {
      var container = this.engine.container;
      container.removeAttributes('data-selection-'.concat(name));
    }
  }, {
    key: "updateAll",
    value: function updateAll(currentMember, members) {
      var _this$engine = this.engine,
          change = _this$engine.change,
          card = _this$engine.card;
      var range = change.getRange().cloneRange();
      var activeCard = card.active;

      if (activeCard && !activeCard.isEditable) {
        var center = activeCard.getCenter();

        if (center && center.length > 0) {
          range.select(center.get(), true);
        }
      } else if ((activeCard === null || activeCard === void 0 ? void 0 : activeCard.isEditable) && activeCard.updateBackgroundSelection) {
        activeCard.updateBackgroundSelection(range);
      }

      if (!activeCard && !range.collapsed) {
        var startCard = this.engine.card.find(range.startNode, true);

        if (startCard && startCard.constructor.cardType === CardType.BLOCK) {
          range.setStart(startCard.getCenter().parent(), 1);
        }

        var endCard = this.engine.card.find(range.endNode, true);

        if (endCard && endCard.constructor.cardType === CardType.BLOCK) {
          range.setEnd(endCard.getCenter().parent(), 1);
        }
      }

      var path = range.toPath();
      this.currentRangePath = path;
      var pathString = JSON.stringify(path);
      var data = this.getAll();
      var isMember = false;
      var isUpdate = false;
      data = data.map(function (attr) {
        if (!attr) {
          isUpdate = true;
          return null;
        }

        if (attr.uuid === currentMember.uuid) {
          isMember = true;

          if (pathString !== JSON.stringify(attr.path)) {
            isUpdate = true;
            attr.path = path;
            attr.active = true;
          }

          return attr;
        } else {
          if (members.find(function (member) {
            return member.uuid === attr.uuid;
          })) {
            attr.active = false;
            return attr;
          } else {
            isUpdate = true;
            return null;
          }
        }
      });
      var newData = [];
      data.forEach(function (attr) {
        if (!!attr) newData.push(attr);
      });

      if (!isMember) {
        isUpdate = true;
        newData.push({
          path: path,
          uuid: currentMember.uuid,
          active: true
        });
      }

      if (isUpdate) {
        this.setAll(newData);
      }

      return {
        data: newData,
        range: range
      };
    }
  }]);

  return SelectionData;
}();

var css_248z$1 = ".data-tooltip {\r\n\tfont-size: 14px;\r\n\tfont-variant: tabular-nums;\r\n\tline-height: 1.5;\r\n\tcolor: rgba(0, 0, 0, 0.65);\r\n\t-webkit-box-sizing: border-box;\r\n\tbox-sizing: border-box;\r\n\tmargin: 0;\r\n\tpadding: 0;\r\n\tlist-style: none;\r\n\tposition: absolute;\r\n\tz-index: 1060;\r\n\tdisplay: block;\r\n\tvisibility: visible;\r\n\tmax-width: 250px;\r\n\twhite-space: nowrap;\r\n    top: 0;\r\n}\r\n\r\n.data-tooltip-hidden {\r\n\topacity: 0;\r\n\tvisibility: hidden;\r\n\ttransition: opacity 0.3s ease-in-out;\r\n}\r\n\r\n.data-tooltip-active {\r\n\topacity: 1;\r\n\tvisibility: visible;\r\n}\r\n\r\n.data-tooltip-placement-top,.data-tooltip-placement-topLeft,.data-tooltip-placement-topRight {\r\n\tpadding-bottom: 8px;\r\n}\r\n\r\n.data-tooltip-placement-right,.data-tooltip-placement-rightTop,.data-tooltip-placement-rightBottom {\r\n\tpadding-left: 8px;\r\n}\r\n\r\n.data-tooltip-placement-bottom,.data-tooltip-placement-bottomLeft,.data-tooltip-placement-bottomRight {\r\n\tpadding-top: 8px;\r\n}\r\n\r\n.data-tooltip-placement-left,.data-tooltip-placement-leftTop,.data-tooltip-placement-leftBottom {\r\n\tpadding-right: 8px;\r\n}\r\n\r\n.data-tooltip-inner {\r\n\tpadding: 6px 8px;\r\n\tcolor: #fff;\r\n\ttext-align: left;\r\n\ttext-decoration: none;\r\n\tbackground-color: rgba(0, 0, 0, 0.75);\r\n\tborder-radius: 4px;\r\n\t-webkit-box-shadow: 0 2px 8px rgba(0, 0, 0, 0.15);\r\n\tbox-shadow: 0 2px 8px rgba(0, 0, 0, 0.15);\r\n\tword-wrap: break-word;\r\n}\r\n\r\n.data-tooltip-arrow {\r\n\tposition: absolute;\r\n\twidth: 0;\r\n\theight: 0;\r\n\tborder-color: transparent;\r\n\tborder-style: solid;\r\n}\r\n\r\n.data-tooltip-placement-top .data-tooltip-arrow,.data-tooltip-placement-topLeft .data-tooltip-arrow,.data-tooltip-placement-topRight .data-tooltip-arrow {\r\n\tbottom: 3px;\r\n\tborder-width: 5px 5px 0;\r\n\tborder-top-color: rgba(0, 0, 0, 0.75);\r\n}\r\n\r\n.data-tooltip-placement-top .data-tooltip-arrow {\r\n\tleft: 50%;\r\n\tmargin-left: -5px;\r\n}\r\n\r\n.data-tooltip-placement-topLeft .data-tooltip-arrow {\r\n\tleft: 16px;\r\n}\r\n\r\n.data-tooltip-placement-topRight .data-tooltip-arrow {\r\n\tright: 16px;\r\n}\r\n\r\n.data-tooltip-placement-right .data-tooltip-arrow,.data-tooltip-placement-rightTop .data-tooltip-arrow,.data-tooltip-placement-rightBottom .data-tooltip-arrow {\r\n\tleft: 3px;\r\n\tborder-width: 5px 5px 5px 0;\r\n\tborder-right-color: rgba(0, 0, 0, 0.75);\r\n}\r\n\r\n.data-tooltip-placement-right .data-tooltip-arrow {\r\n\ttop: 50%;\r\n\tmargin-top: -5px;\r\n}\r\n\r\n.data-tooltip-placement-rightTop .data-tooltip-arrow {\r\n\ttop: 8px;\r\n}\r\n\r\n.data-tooltip-placement-rightBottom .data-tooltip-arrow {\r\n\tbottom: 8px;\r\n}\r\n\r\n.data-tooltip-placement-left .data-tooltip-arrow,.data-tooltip-placement-leftTop .data-tooltip-arrow,.data-tooltip-placement-leftBottom .data-tooltip-arrow {\r\n\tright: 3px;\r\n\tborder-width: 5px 0 5px 5px;\r\n\tborder-left-color: rgba(0, 0, 0, 0.75);\r\n}\r\n\r\n.data-tooltip-placement-left .data-tooltip-arrow {\r\n\ttop: 50%;\r\n\tmargin-top: -5px;\r\n}\r\n\r\n.data-tooltip-placement-leftTop .data-tooltip-arrow {\r\n\ttop: 8px;\r\n}\r\n\r\n.data-tooltip-placement-leftBottom .data-tooltip-arrow {\r\n\tbottom: 8px;\r\n}\r\n\r\n.data-tooltip-placement-bottom .data-tooltip-arrow,.data-tooltip-placement-bottomLeft .data-tooltip-arrow,.data-tooltip-placement-bottomRight .data-tooltip-arrow {\r\n\ttop: 3px;\r\n\tborder-width: 0 5px 5px;\r\n\tborder-bottom-color: rgba(0, 0, 0, 0.75);\r\n}\r\n\r\n.data-tooltip-placement-bottom .data-tooltip-arrow {\r\n\tleft: 50%;\r\n\tmargin-left: -5px;\r\n}\r\n\r\n.data-tooltip-placement-bottomLeft .data-tooltip-arrow {\r\n\tleft: 16px;\r\n}\r\n\r\n.data-tooltip-placement-bottomRight .data-tooltip-arrow {\r\n\tright: 16px;\r\n}";
styleInject(css_248z$1);

var template = function template(options) {
  return "\n    <div ".concat(DATA_ELEMENT, "=\"tooltip\" class=\"data-tooltip data-tooltip-placement-").concat(options.placement, " data-tooltip-hidden\" style=\"transform-origin: 50% 45px 0px;\">\n        <div class=\"data-tooltip-content\">\n            <div class=\"data-tooltip-arrow\"></div>\n            <div class=\"data-tooltip-inner\" data-role=\"tooltip\"></div>\n        </div>\n    </div>");
};

var Tooltip = /*#__PURE__*/function () {
  function Tooltip() {
    _classCallCheck(this, Tooltip);
  }

  _createClass(Tooltip, null, [{
    key: "show",
    value: function show(node, title) {
      var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {
        placement: 'top'
      };
      Tooltip.hide();
      var root = $(template(options)); // 设置提示文字

      if (typeof title === 'string') root.find('[data-role=tooltip]').html(title);else root.find('[data-role=tooltip]').append(title); // 计算定位

      var body = $(document.body);
      body.append(root);
      var element = root.get();
      var width = (element === null || element === void 0 ? void 0 : element.clientWidth) || 0;
      var height = (element === null || element === void 0 ? void 0 : element.clientHeight) || 0;
      var nodeElement = node.get();
      var nodeWidth = nodeElement.clientWidth;
      var nodeRect = nodeElement.getBoundingClientRect();
      var left = Math.round(window.pageXOffset + nodeRect.left + nodeWidth / 2 - width / 2);
      var top = Math.round(window.pageYOffset + nodeRect.top - height - 2);
      root.css({
        left: left + 'px',
        top: top + 'px'
      });
      root.addClass('data-tooltip-active');
    }
  }, {
    key: "hide",
    value: function hide() {
      $("div[".concat(DATA_ELEMENT, "=tooltip]")).remove();
    }
  }]);

  return Tooltip;
}();

var template$1 = function template(options) {
  return "\n    <span class=\"data-toolbar-item\">\n        <a class=\"data-toolbar-btn\"".concat(options.disabled ? ' disabled="disabled"' : '', " ").concat(options.link ? ' href="' + options.link + '" target="_blank"' : '', ">\n            ").concat(options.content, "\n        </a>\n    </span>");
};

var Button = /*#__PURE__*/function () {
  function Button(options) {
    _classCallCheck(this, Button);

    this.options = void 0;
    this.root = void 0;
    this.options = options;
    this.root = $(template$1(options));

    if (options.style) {
      this.root.attributes('style', options.style);
    }

    if (options.class) {
      this.root.addClass(options.class);
    }
  }

  _createClass(Button, [{
    key: "render",
    value: function render(container) {
      var _this = this;

      var _this$options = this.options,
          title = _this$options.title,
          didMount = _this$options.didMount,
          onClick = _this$options.onClick;
      container.append(this.root);

      if (title) {
        this.root.on('mouseenter', function () {
          Tooltip.show(_this.root, typeof title === 'function' ? title() : title);
        });
        this.root.on('mouseleave', function () {
          Tooltip.hide();
        });
        this.root.on('mousedown', function () {
          Tooltip.hide();
        });
      }

      this.root.find('a').on('click', function (e) {
        e.preventDefault();
        e.stopPropagation();
        if (onClick) onClick();
      });

      if (didMount) {
        didMount(this.root);
      }
    }
  }]);

  return Button;
}();

var template$2 = function template(options) {
  var checked = !!options.checked;
  if (options.getState) checked = options.getState();
  return "\n    <div class=\"data-toolbar-item data-toolbar-dropdown-item data-toolbar-dropdown-switch\">\n        <span class=\"data-toolbar-dropdown-item-content\"".concat(options.disabled ? ' disabled="disabled"' : '', ">").concat(options.content, "</span>\n        <button type=\"button\" role=\"switch\" aria-checked=\"true\" class=\"switch-btn ").concat(checked ? ' switch-checked' : '', "\">\n            <div class=\"switch-handle\"></div>\n            <span class=\"switch-inner\"></span>\n        </button>\n    </div>");
};

var _default = /*#__PURE__*/function () {
  function _default(options) {
    _classCallCheck(this, _default);

    this.options = void 0;
    this.root = void 0;
    this.switch = void 0;
    this.options = options;
  }

  _createClass(_default, [{
    key: "renderTo",
    value: function renderTo(container) {
      var _this = this;

      this.root = $(template$2(this.options));
      this.switch = this.root.find('.ant-switch');
      container.append(this.root);
      this.root.on('mousedown', function (e) {
        return e.preventDefault();
      });
      this.root.on('click', function (e) {
        e.stopPropagation();

        if (_this.options.onClick) {
          _this.options.onClick();

          _this.updateSwitch();
        }
      });
    }
  }, {
    key: "updateSwitch",
    value: function updateSwitch() {
      if (this.options.getState) {
        if (this.options.getState()) {
          var _this$switch;

          (_this$switch = this.switch) === null || _this$switch === void 0 ? void 0 : _this$switch.addClass('ant-switch-checked');
        } else {
          var _this$switch2;

          (_this$switch2 = this.switch) === null || _this$switch2 === void 0 ? void 0 : _this$switch2.removeClass('ant-switch-checked');
        }
      }
    }
  }]);

  return _default;
}();

var template$3 = function template(options) {
  return "\n    <div class=\"data-toolbar-item data-toolbar-dropdown-item data-toolbar-dropdown-btn\">\n        <span class=\"data-toolbar-dropdown-item-content\"".concat(options.disabled ? ' disabled="disabled"' : '', ">\n            ").concat(options.content, "\n        </span>\n    </div>");
};

var _default$1 = /*#__PURE__*/function () {
  function _default(options) {
    _classCallCheck(this, _default);

    this.options = void 0;
    this.root = void 0;
    this.options = options;
  }

  _createClass(_default, [{
    key: "renderTo",
    value: function renderTo(container) {
      this.root = $(template$3(this.options));
      container.append(this.root);
      var onClick = this.options.onClick;

      if (onClick) {
        this.root.on('click', function () {
          return onClick();
        });
      }
    }
  }]);

  return _default;
}();

var template$4 = function template(options) {
  return "\n    <span class=\"data-toolbar-item data-toolbar-item-dropdown\">\n        <a class=\"data-toolbar-btn data-toolbar-dropdown\"".concat(options.disabled ? ' disabled="disabled"' : '', ">").concat(options.content, "</a>\n        <div class=\"dropdown-container\"></div>\n    </span>");
};

var Dropdown = /*#__PURE__*/function () {
  function Dropdown(options) {
    var _this = this;

    _classCallCheck(this, Dropdown);

    this.options = void 0;
    this.root = void 0;
    this.dropdown = void 0;

    this.documentMouseDown = function (e) {
      var _this$dropdown;

      if (!_this.root) return;

      if (!_this.root[0].contains(e.target) && ((_this$dropdown = _this.dropdown) === null || _this$dropdown === void 0 ? void 0 : _this$dropdown.hasClass('show'))) {
        _this.hideDropdown();
      }
    };

    this.options = options;
  }

  _createClass(Dropdown, [{
    key: "initToggleEvent",
    value: function initToggleEvent() {
      var _this2 = this;

      var dropdownBtn = this.root.find('.data-toolbar-dropdown');
      dropdownBtn.on('mousedown', function (e) {
        e.preventDefault();
        e.stopPropagation();
      });
      dropdownBtn.on('click', function (e) {
        e.stopPropagation();

        _this2.toggleDropdown();
      });
      document.addEventListener('mousedown', this.documentMouseDown, true);
    }
  }, {
    key: "toggleDropdown",
    value: function toggleDropdown() {
      var _this$dropdown2;

      if ((_this$dropdown2 = this.dropdown) === null || _this$dropdown2 === void 0 ? void 0 : _this$dropdown2.hasClass('show')) {
        this.hideDropdown();
      } else {
        this.showDropdown();
      }
    }
  }, {
    key: "showDropdown",
    value: function showDropdown() {
      var _this$dropdown3;

      (_this$dropdown3 = this.dropdown) === null || _this$dropdown3 === void 0 ? void 0 : _this$dropdown3.addClass('show');
    }
  }, {
    key: "hideDropdown",
    value: function hideDropdown() {
      var _this$dropdown4;

      (_this$dropdown4 = this.dropdown) === null || _this$dropdown4 === void 0 ? void 0 : _this$dropdown4.removeClass('show');
    }
  }, {
    key: "renderTooltip",
    value: function renderTooltip() {
      var _this3 = this;

      var title = this.options.title;

      if (title) {
        this.root.on('mouseenter', function () {
          Tooltip.show(_this3.root, typeof title === 'function' ? title() : title);
        });
        this.root.on('mouseleave', function () {
          Tooltip.hide();
        });
        this.root.on('mousedown', function () {
          Tooltip.hide();
        });
      }
    }
  }, {
    key: "renderDropdown",
    value: function renderDropdown(container) {
      var _this4 = this;

      this.dropdown = container.find('.dropdown-container');
      var items = this.options.items;
      items.forEach(function (item) {
        switch (item.type) {
          case 'switch':
            return new _default(item).renderTo(_this4.dropdown);

          case 'button':
            return new _default$1(item).renderTo(_this4.dropdown);
        }
      });
      this.dropdown.on('click', function (e) {
        e.stopPropagation();

        _this4.hideDropdown();
      });
    }
  }, {
    key: "render",
    value: function render(container) {
      this.root = $(template$4(this.options));
      container.append(this.root);
      this.initToggleEvent();
      this.renderTooltip();
      this.renderDropdown(container);
    }
  }, {
    key: "destroy",
    value: function destroy() {
      document.removeEventListener('mousedown', this.documentMouseDown, true);
    }
  }]);

  return Dropdown;
}();

var template$5 = function template(options) {
  return "\n    <span class=\"data-toolbar-item data-toolbar-item-input\">\n        ".concat(options.prefix ? "<span class='data-toolbar-input-prefix'>" + escape(options.prefix) + '</span>' : '', "<input data-role=\"input\" placeholder=\"").concat(escape(options.placeholder), "\" class=\"data-toolbar-input\" type=\"input\" value=\"").concat(escape(options.value.toString()), "\" />").concat(options.suffix ? "<span class='data-toolbar-input-suffix'>" + escape(options.suffix) + '</span>' : '', "\n    </span>");
};

var Input = /*#__PURE__*/function () {
  function Input(options) {
    _classCallCheck(this, Input);

    this.options = void 0;
    this.root = void 0;
    this.onEnter = void 0;
    this.onInput = void 0;
    this.onChange = void 0;
    this.options = options;
    this.root = $(template$5(options));

    this.onEnter = options.onEnter || function () {};

    this.onInput = options.onInput || function () {};

    this.onChange = options.onChange || function () {};
  }

  _createClass(Input, [{
    key: "find",
    value: function find(role) {
      var expr = '[data-role='.concat(role, ']');
      return this.root.find(expr);
    }
  }, {
    key: "render",
    value: function render(container) {
      var _this = this;

      var _this$options = this.options,
          value = _this$options.value,
          didMount = _this$options.didMount;
      var input = this.find('input');
      var inputElement = input.get();
      if (!inputElement) return;
      inputElement.value = (value !== undefined ? value : '').toString();
      input.on('keydown', function (e) {
        e.stopPropagation();

        if (isHotkey('enter', e)) {
          e.preventDefault();
          inputElement.blur();

          _this.onEnter(inputElement.value);
        }
      });
      input.on('input', function () {
        _this.onInput(inputElement.value);
      });
      input.on('change', function () {
        setTimeout(function () {
          _this.onChange(inputElement.value);
        }, 10);
      });
      container.append(this.root);
      if (didMount) didMount(this.root);
    }
  }]);

  return Input;
}();

var css_248z$2 = ".data-toolbar {\r\n\tposition: absolute;\r\n\topacity: 0;\r\n\tvisibility: hidden;\r\n\twidth: auto;\r\n\tline-height: 26px;\r\n\tdisplay: flex;\r\n\tflex-direction: row;\r\n\tfont-size: 14px;\r\n\tfont-weight: normal;\r\n\ttext-indent: 0;\r\n\t-webkit-user-select: none;\r\n\t-moz-user-select: none;\r\n\t -ms-user-select: none;\r\n\t     user-select: none;\r\n}\r\n\r\n.data-toolbar-active {\r\n\topacity: 1;\r\n\tvisibility: visible;\r\n}\r\n\r\n.data-toolbar-block {\r\n\ttop: auto;\r\n\tbottom: -46px;\r\n\tleft: -1px;\r\n\tright: auto;\r\n\theight: 40px;\r\n\tz-index: 125;\r\n}\r\n\r\n.data-toolbar-btn {\r\n\tline-height: 26px;\r\n\tmin-width: 28px;\r\n\tdisplay: inline-block;\r\n\ttext-align: center;\r\n\tcolor: #595959;\r\n\ttransition: background-color 0.3s ease-in-out;\r\n\tcursor: pointer;\r\n}\r\n\r\n.data-toolbar-btn-disabled,.data-toolbar-btn-disabled:hover {\r\n\tbackground-color: transparent;\r\n\tbox-shadow: none;\r\n\tcursor: not-allowed;\r\n}\r\n\r\n.data-toolbar-group {\r\n\tborder: 1px solid rgba(226, 226, 226, 0.84);\r\n    border-radius: 4px;\r\n    box-shadow: 0px 2px 4px 0px rgb(225 225 225 / 50%);\r\n    background: #fff;\r\n    position: relative;\r\n    display: inline-flex;\r\n    padding: 5px;\r\n    align-items: center;\r\n}\r\n\r\n.data-toolbar-item {\r\n\tposition: relative;\r\n\tdisplay: inline-block;\r\n\tline-height: 26px;\r\n\ttext-align: left;\r\n\tcolor: #595959;\r\n\tflex: 0 0 auto;\r\n    font-size: 12px;\r\n    cursor: pointer;\r\n}\r\n\r\n.data-toolbar-item:not(.data-toolbar-item-input):hover{\r\n    background-color: #f4f4f4;\r\n    border-radius: 2px;\r\n}\r\n\r\n.data-toolbar-item > * {\r\n    font-size: 12px !important;\r\n}\r\n\r\n.data-toolbar-item[disabled] {\r\n\topacity: 0.5;\r\n\tcursor: not-allowed;\r\n}\r\n\r\n.data-toolbar-item-split {\r\n\twidth: 1px;\r\n\theight: 16px;\r\n\tline-height: 16px;\r\n\tmargin: 6px 4px;\r\n\tborder-left: 1px solid #e8e8e8;\r\n\tdisplay: inline-block;\r\n}\r\n\r\n.data-toolbar-item-dropdown-active {\r\n\topacity: 1;\r\n\tvisibility: visible;\r\n\ttransform: translateY(0px);\r\n}\r\n\r\n.data-toolbar-item-input {\r\n\tdisplay: flex;\r\n    margin: 0 4px;\r\n}\r\n\r\n.data-toolbar-item-input .data-toolbar-input {\r\n    width: 46px;\r\n    line-height: 12px;\r\n    font-size: 12px;\r\n    outline: none;\r\n    border: 1px solid #dadada;\r\n    border-radius: 4px;\r\n}\r\n\r\n.data-toolbar-item-input .data-toolbar-input::-moz-selection {\r\n    color: inherit;\r\n    background:transparent\r\n}\r\n\r\n.data-toolbar-item-input .data-toolbar-input::selection {\r\n    color: inherit;\r\n    background:transparent\r\n}\r\n\r\n.data-toolbar-item-input .data-toolbar-input:focus::-moz-selection\r\n{\r\n    color: #fff;\r\n    background: #1890ff;\r\n}\r\n\r\n.data-toolbar-item-input .data-toolbar-input:focus::selection\r\n{\r\n    color: #fff;\r\n    background: #1890ff;\r\n}\r\n\r\n.data-toolbar-item-dropdown .dropdown-container {\r\n\tdisplay: none;\r\n\tposition: absolute;\r\n\tpadding: 8px 0;\r\n\ttop: 100%;\r\n\tmargin-top: 9px;\r\n\tborder-radius: 4px;\r\n\tbackground-color: #fff;\r\n\tbox-shadow: 0 1px 4px -2px rgba(0, 0, 0, 0.13), 0 2px 8px 0 rgba(0, 0, 0, 0.08), 0 8px 16px 4px rgba(0, 0, 0, 0.04);\r\n}\r\n\r\n.data-toolbar-item-dropdown .dropdown-container.show {\r\n\tdisplay: block;\r\n}\r\n\r\n.data-toolbar-dropdown-item {\r\n\twidth: 218px;\r\n\tpadding: 8px 16px;\r\n\tmargin: 0;\r\n\twhite-space: nowrap;\r\n\tfont-size: 15px;\r\n\tline-height: 26px;\r\n\tcolor: #404040;\r\n\tcursor: pointer;\r\n}\r\n\r\n.data-toolbar-dropdown-item:hover {\r\n\tbackground-color: #f5f5f5;\r\n}\r\n\r\n.data-toolbar-dropdown-switch {\r\n\tdisplay: flex;\r\n\talign-items: center;\r\n}\r\n\r\n.data-toolbar-dropdown-switch .data-toolbar-dropdown-item-content {\r\n\tflex: 1;\r\n}\r\n\r\n.data-toolbar-dropdown-switch .switch-btn {\r\n    margin: 0;\r\n    padding: 0;\r\n    color: #595959;\r\n    font-size: 14px;\r\n    font-variant: tabular-nums;\r\n    line-height: 1.5;\r\n    list-style: none;\r\n    -webkit-font-feature-settings: \"tnum\";\r\n    font-feature-settings: \"tnum\";\r\n    position: relative;\r\n    display: inline-block;\r\n    -webkit-box-sizing: border-box;\r\n    box-sizing: border-box;\r\n    vertical-align: middle;\r\n    background-color: rgba(0,0,0,.25);\r\n    border: 0;\r\n    border-radius: 100px;\r\n    cursor: pointer;\r\n    -webkit-transition: all .2s;\r\n    transition: all .2s;\r\n    -webkit-user-select: none;\r\n    -moz-user-select: none;\r\n    -ms-user-select: none;\r\n    user-select: none;\r\n    min-width: 28px;\r\n    height: 16px;\r\n    line-height: 16px;\r\n}\r\n\r\n.data-toolbar-dropdown-switch .switch-btn.switch-checked {\r\n    background-color: #347EFF\r\n}\r\n\r\n.data-toolbar-dropdown-switch .switch-btn .switch-handle {\r\n    top: 2px;\r\n    left: 2px;\r\n    width: 12px;\r\n    height: 12px;\r\n}\r\n\r\n.data-toolbar-dropdown-switch .switch-btn .switch-handle, .data-toolbar-dropdown-switch .switch-btn .switch-handle:before {\r\n    position: absolute;\r\n    -webkit-transition: all .2s ease-in-out;\r\n    transition: all .2s ease-in-out;\r\n}\r\n\r\n.data-toolbar-dropdown-switch .switch-btn.switch-checked .switch-handle {\r\n    left: calc(100% - 14px);\r\n}\r\n\r\n.data-toolbar-dropdown-switch .switch-btn .switch-handle:before {\r\n    top: 0;\r\n    right: 0;\r\n    bottom: 0;\r\n    left: 0;\r\n    background-color: #fff;\r\n    border-radius: 9px;\r\n    -webkit-box-shadow: 0 2px 4px 0 rgb(0 35 11 / 20%);\r\n    box-shadow: 0 2px 4px 0 rgb(0 35 11 / 20%);\r\n    content: \"\";\r\n}\r\n\r\n.data-toolbar-dropdown-switch .switch-btn .switch-inner {\r\n    display: block;\r\n    margin: 0 5px 0 18px;\r\n    font-size: 12px;\r\n    color: #fff;\r\n    -webkit-transition: margin .2s;\r\n    transition: margin .2s;\r\n}\r\n\r\n.data-toolbar-dropdown-switch .switch-btn.switch-checked .switch-inner {\r\n    margin: 0 18px 0 5px;\r\n}";
styleInject(css_248z$2);

var template$6 = function template() {
  return "<div ".concat(DATA_ELEMENT, "=\"ui\" class=\"data-toolbar data-toolbar-active\" contenteditable=\"false\"></div>");
};

var Toolbar = /*#__PURE__*/function () {
  function Toolbar(options) {
    _classCallCheck(this, Toolbar);

    this.options = void 0;
    this.root = void 0;
    this.items = [];
    this.options = _objectSpread({}, options);
    this.root = $(template$6());
  }

  _createClass(Toolbar, [{
    key: "addItems",
    value: function addItems(node) {
      var _this = this;

      this.options.items.forEach(function (options) {
        var item;

        if (options.type === 'button') {
          item = new Button(options);
          item.render(node);
        }

        if (options.type === 'input') {
          var inputOptions = options;
          item = new Input(inputOptions);
          item.render(node);
        }

        if (options.type === 'dropdown') {
          item = new Dropdown(options);
          item.render(node);
        }

        if (options.type === 'node') {
          var nodeOptions = options;
          var nodeItem = nodeOptions.node;
          nodeItem.addClass('data-toolbar-item');
          var title = nodeOptions.title;

          if (title) {
            nodeItem.on('mouseenter', function () {
              Tooltip.show(nodeItem, typeof title === 'function' ? title() : title);
            });
            nodeItem.on('mouseleave', function () {
              Tooltip.hide();
            });
            nodeItem.on('mousedown', function () {
              Tooltip.hide();
            });
          }

          node.append(nodeItem);
          if (options.didMount) options.didMount(nodeItem);
        }

        if (item) _this.items.push(item);
      });
    }
  }, {
    key: "find",
    value: function find(role) {
      var expr = '[data-role='.concat(role, ']');
      return this.root.find(expr);
    }
  }, {
    key: "destroy",
    value: function destroy() {
      this.root.remove();
    }
  }, {
    key: "hide",
    value: function hide() {
      this.root.removeClass('data-toolbar-active');
    }
  }, {
    key: "show",
    value: function show() {
      this.root.addClass('data-toolbar-active');
    }
  }, {
    key: "render",
    value: function render(container) {
      var group = $('<div class="data-toolbar-group"></div>');
      this.root.append(group);
      this.addItems(group);

      if (container) {
        container.append(this.root);
      }

      this.root.addClass('data-toolbar-block');
      return this.root;
    }
  }]);

  return Toolbar;
}();

var USER_BACKGROUND_CLASS = 'ot-user-background';
var USER_CURSOR_CLASS = 'ot-user-cursor';
var USER_CURSOR_CARD_CLASS = 'ot-user-cursor-card';
var USER_MASK_CLASS = 'ot-card-mask';
var USER_CURSOR_TRIGGER_CLASS = 'ot-user-cursor-trigger';
var USER_CURSOR_TRIGGER_ACTIVE_CLASS = 'ot-user-cursor-trigger-active';

var RangeColoring = /*#__PURE__*/function () {
  function RangeColoring(engine) {
    _classCallCheck(this, RangeColoring);

    this.engine = void 0;
    this.root = void 0;
    this.hideCursorInfoTimeoutIdMap = void 0;
    this.engine = engine;
    this.root = engine.root;
    this.hideCursorInfoTimeoutIdMap = {};
  }

  _createClass(RangeColoring, [{
    key: "destroy",
    value: function destroy() {
      this.root.children(".".concat(USER_BACKGROUND_CLASS)).remove();
      this.root.children(".".concat(USER_CURSOR_CLASS)).remove();
      this.root.children(".".concat(USER_MASK_CLASS)).remove();
    }
  }, {
    key: "getRelativeRect",
    value: function getRelativeRect(node, range) {
      var _node$get;

      var clientReact = range.getClientRect();
      var react = (_node$get = node.get()) === null || _node$get === void 0 ? void 0 : _node$get.getBoundingClientRect();
      return new DOMRect(clientReact.left - (react ? react.left : 0), clientReact.top - (react ? react.top : 0), clientReact.right - clientReact.left, clientReact.bottom - clientReact.top);
    }
  }, {
    key: "cacheRange",
    value: function cacheRange(range) {
      var startOffset = range.startOffset,
          endOffset = range.endOffset,
          startContainer = range.startContainer,
          endContainer = range.endContainer,
          commonAncestorContainer = range.commonAncestorContainer;
      return {
        startOffset: startOffset,
        endOffset: endOffset,
        startContainer: startContainer,
        endContainer: endContainer,
        commonAncestorContainer: commonAncestorContainer
      };
    }
  }, {
    key: "isRangeWrap",
    value: function isRangeWrap(range) {
      var cloneRange = range.cloneRange();
      var clientReact = cloneRange.collapse(true).getClientRect();
      var cloneRange1 = range.cloneRange();
      var clientReact1 = cloneRange1.collapse(false).getClientRect();
      return clientReact.bottom !== clientReact1.bottom;
    }
  }, {
    key: "drawOneByOne",
    value: function drawOneByOne(node, canvas, range, style) {
      var cacheRange = this.cacheRange(range);
      var startOffset = cacheRange.startOffset;

      while (startOffset < cacheRange.endOffset) {
        range.setStart(cacheRange.commonAncestorContainer, startOffset);
        range.setEnd(cacheRange.commonAncestorContainer, startOffset + 1);
        var rect = this.getRelativeRect(node, range);
        canvas.clearRect(rect);
        canvas.drawRect(_objectSpread(_objectSpread({}, rect.toJSON()), style));
        startOffset++;
      }

      range.setStart(cacheRange.startContainer, cacheRange.startOffset);
      range.setEnd(cacheRange.endContainer, cacheRange.endOffset);
    }
  }, {
    key: "drawBackground",
    value: function drawBackground(range, options) {
      var _cardInfo,
          _this = this;

      var card = this.engine.card;
      var uuid = options.uuid,
          color = options.color;
      var tinyColor = tinycolor2(color);
      tinyColor.setAlpha(0.3);
      var targetCanvas;
      var rgb = tinyColor.toRgbString();
      var child = this.root.children(".".concat(USER_BACKGROUND_CLASS, "[data-uuid=\"").concat(uuid, "\"]"));

      if (child && child.length > 0) {
        child.attributes('data-color', color.toString());
        targetCanvas = child[0]['__targetCanvas'];
        targetCanvas.clear();
      } else {
        child = $("<div class=\"".concat(USER_BACKGROUND_CLASS, "\" data-uuid=\"").concat(uuid, "\" data-color=\"").concat(color, "\" />"));
        child.css({
          position: 'absolute',
          top: 0,
          left: 0,
          'pointer-events': 'none'
        });
        this.root.append(child);
        targetCanvas = new TinyCanvas({
          container: child.get()
        });
        child[0]['__targetCanvas'] = targetCanvas;
      }

      child[0]['__targetRange'] = range;
      var parentWidth = this.root.width();
      var parentHeight = this.root.height();
      targetCanvas.resize(parentWidth, parentHeight);
      var cardInfo = card.find(range.commonAncestorNode, true); //如果是卡片，并且选区不在内容模块中，而是在卡片两侧的光标位置处，就不算作卡片

      if (cardInfo && !cardInfo.isCenter(range.commonAncestorNode)) {
        cardInfo = undefined;
      }

      var fill = {
        fill: rgb
      };
      var subRanges = range.getSubRanges();

      if (((_cardInfo = cardInfo) === null || _cardInfo === void 0 ? void 0 : _cardInfo.isEditable) && cardInfo.drawBackground) {
        var result = cardInfo.drawBackground(child, range, targetCanvas);
        if (result === false) return [range];

        if (!!result) {
          if (Array.isArray(result)) subRanges = result;else {
            targetCanvas.clearRect(result);
            targetCanvas.drawRect(_objectSpread(_objectSpread({}, result.toJSON()), fill));
            return [range];
          }
        }
      }

      subRanges.forEach(function (subRange) {
        if (_this.isRangeWrap(subRange)) {
          _this.drawOneByOne(child, targetCanvas, subRange, fill);
        } else {
          var rect = _this.getRelativeRect(child, subRange);

          targetCanvas.clearRect(rect);
          targetCanvas.drawRect(_objectSpread(_objectSpread({}, rect.toJSON()), fill));
        }
      });
      return subRanges;
    }
  }, {
    key: "getNodeRect",
    value: function getNodeRect(node, rect) {
      var _node$parent, _node$parent2, _node$parent2$first;

      //自定义列表项的第一个card跳过
      if (node.isCard() && ((_node$parent = node.parent()) === null || _node$parent === void 0 ? void 0 : _node$parent.hasClass('data-list-item')) && ((_node$parent2 = node.parent()) === null || _node$parent2 === void 0 ? void 0 : (_node$parent2$first = _node$parent2.first()) === null || _node$parent2$first === void 0 ? void 0 : _node$parent2$first.equal(node)) && node.next()) {
        node = node.next();
      }

      if (node.isElement()) {
        rect = node.get().getBoundingClientRect();
      }

      if (node.isText()) {
        var range = Range.create(this.engine);
        range.select(node, true);
        rect = range.getClientRect();
      }

      return rect;
    }
  }, {
    key: "getCursorRect",
    value: function getCursorRect(selector) {
      var _this$root$get, _node$get2;

      var leftSpace = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 2;
      var parentRect = ((_this$root$get = this.root.get()) === null || _this$root$get === void 0 ? void 0 : _this$root$get.getBoundingClientRect()) || {
        top: 0,
        left: 0
      };

      if (isRangeInterface(selector)) {
        var range = selector;
        var startNode = range.startNode;
        range.shrinkToElementNode();

        var _rect = range.getClientRect();

        if (startNode.isElement() && _rect.height === 0) {
          var childNode = $(startNode[0].childNodes[range.startOffset]);

          if (childNode && childNode.length > 0) {
            _rect = this.getNodeRect(childNode, _rect);
          } else {
            childNode = startNode.first();

            if (childNode && childNode.length > 0) {
              _rect = this.getNodeRect(childNode, _rect);
            }
          }
        }

        var _top = _rect.top - (parentRect.top || 0);

        var _left = _rect.left - (parentRect.left || 0) - leftSpace;

        var height = _rect.height;
        return {
          top: _top + 'px',
          left: _left + 'px',
          height: height > 0 ? height + 'px' : -1
        };
      }

      var node = selector;
      var outlineWidth = removeUnit(node.css('outline-width'));
      var rect = ((_node$get2 = node.get()) === null || _node$get2 === void 0 ? void 0 : _node$get2.getBoundingClientRect()) || {
        top: 0,
        left: 0
      };
      var top = rect.top - parentRect.top - 1;
      var left = rect.left - parentRect.left;

      if (outlineWidth) {
        top -= outlineWidth + 1;
        left -= 2;
      }

      return {
        left: left + 'px',
        top: top + 'px',
        height: 0
      };
    }
  }, {
    key: "setCursorRect",
    value: function setCursorRect(node, rect) {
      if (-1 !== rect.height) {
        if (0 === rect.height) {
          node.css(rect);
          node.addClass(USER_CURSOR_CARD_CLASS);
          return;
        }

        node.css(rect);
        node.removeClass(USER_CURSOR_CARD_CLASS);
      } else node.remove();
    }
  }, {
    key: "showCursorInfo",
    value: function showCursorInfo(node, member) {
      var uuid = member.uuid,
          color = member.color;

      if (this.hideCursorInfoTimeoutIdMap[uuid]) {
        clearTimeout(this.hideCursorInfoTimeoutIdMap[uuid]);
      }

      var trigger = node.find(".".concat(USER_CURSOR_TRIGGER_CLASS));
      var bgColor = node.css('background-color');
      node.attributes('data-old-background-color', bgColor);
      trigger.addClass("".concat(USER_CURSOR_TRIGGER_ACTIVE_CLASS));
      node.css('background-color', color);
      trigger.css('background-color', color);
    }
  }, {
    key: "hideCursorInfo",
    value: function hideCursorInfo(node) {
      var trigger = node.find(".".concat(USER_CURSOR_TRIGGER_CLASS));
      var bgColor = node.attributes('data-old-background-color');
      trigger.removeClass("".concat(USER_CURSOR_TRIGGER_ACTIVE_CLASS));
      node.css('background-color', bgColor);
      trigger.css('background-color', bgColor);
    }
  }, {
    key: "drawCursor",
    value: function drawCursor(selector, member) {
      var _this2 = this;

      var uuid = member.uuid,
          name = member.name,
          color = member.color;
      var cursorRect = this.getCursorRect(selector);
      var childCursor = this.root.children(".".concat(USER_CURSOR_CLASS, "[data-uuid=\"").concat(uuid, "\"]"));

      if (childCursor && childCursor.length > 0) {
        this.setCursorRect(childCursor, cursorRect);
      } else {
        var userCursor = "\n            <div class=\"".concat(USER_CURSOR_CLASS, "\" data-uuid=\"").concat(uuid, "\">\n                <div class=\"").concat(USER_CURSOR_TRIGGER_CLASS, "\">").concat(escape(name || ''), "</div>\n            </div>");
        childCursor = $(userCursor);
        var trigger = childCursor.find(".".concat(USER_CURSOR_TRIGGER_CLASS));
        this.setCursorRect(childCursor, cursorRect);
        childCursor.on('mouseenter', function () {
          return _this2.showCursorInfo(childCursor, member);
        });
        var transitionState = true;
        childCursor.on('transitionstart', function () {
          transitionState = false;
        });
        childCursor.on('transitionend', function () {
          transitionState = true;
        });
        childCursor.on('mouseleave', function () {
          if (transitionState) {
            _this2.hideCursorInfo(childCursor);
          }
        });
        childCursor.css('background-color', color);
        trigger.css('background-color', color);
        this.root.append(childCursor);
      }

      if (childCursor && childCursor[0]) {
        childCursor[0]['__target'] = selector;
        this.showCursorInfo(childCursor, member);

        if (this.hideCursorInfoTimeoutIdMap[uuid]) {
          clearTimeout(this.hideCursorInfoTimeoutIdMap[uuid]);
        }

        this.hideCursorInfoTimeoutIdMap[uuid] = setTimeout(function () {
          _this2.hideCursorInfo(childCursor);
        }, 2000);
        return childCursor;
      }

      return;
    }
  }, {
    key: "drawCardMask",
    value: function drawCardMask(node, cursor, member) {
      var _this$root$get2,
          _node$get3,
          _this3 = this;

      var language = this.engine.language;
      var parentRect = ((_this$root$get2 = this.root.get()) === null || _this$root$get2 === void 0 ? void 0 : _this$root$get2.getBoundingClientRect()) || {
        left: 0,
        top: 0,
        width: 0,
        height: 0
      };
      var nodeRect = ((_node$get3 = node.get()) === null || _node$get3 === void 0 ? void 0 : _node$get3.getBoundingClientRect()) || {
        left: 0,
        top: 0,
        width: 0,
        height: 0
      };
      var mask = this.root.children(".".concat(USER_MASK_CLASS, "[data-uuid=\"").concat(member.uuid, "\"]"));

      if (mask && mask.length > 0) {
        mask[0]['__targetNode'] = node[0];
        mask.css({
          left: nodeRect.left - parentRect.left + 'px',
          top: nodeRect.top - parentRect.top + 'px'
        });
        return;
      }

      mask = $("<div class=\"".concat(USER_MASK_CLASS, "\" data-uuid=\"").concat(member.uuid, "\" />"));
      mask[0]['__targetNode'] = node[0];
      mask.css({
        left: nodeRect.left - parentRect.left + 'px',
        top: nodeRect.top - parentRect.top + 'px',
        width: nodeRect.width + 'px',
        height: nodeRect.height + 'px'
      });
      mask.on('mouseenter', function () {
        _this3.showCursorInfo(cursor, member);

        Tooltip.show(mask, language.get('card', 'lockAlert').toString(), {
          placement: 'bottomLeft'
        });
      });
      mask.on('mousemove', function (event) {
        var tooltipElement = $("div[".concat(DATA_ELEMENT, "=tooltip]"));
        tooltipElement.css({
          left: event.pageX - 16 + 'px',
          top: event.pageY + 32 + 'px'
        });
      });
      mask.on('mouseleave', function () {
        _this3.hideCursorInfo(cursor);

        Tooltip.hide();
      });
      mask.on('click', function (event) {
        event.preventDefault();
        event.stopPropagation();
      });
      mask.on('mousedown', function (event) {
        event.preventDefault();
        event.stopPropagation();
      });
      this.root.append(mask);
    }
  }, {
    key: "setCardSelectedByOther",
    value: function setCardSelectedByOther(card, member) {
      var _ref = member || {},
          uuid = _ref.uuid,
          color = _ref.color;

      if (color) {
        var tinyColor = tinycolor2(color);
        tinyColor.setAlpha(0.3);
        var rgb = tinyColor.toRgbString();
        var customNode;

        if (!card.selectedByOther) {
          customNode = card.onSelectByOther(true, {
            color: color,
            rgb: rgb
          });
        }

        card.selectedByOther = uuid;
        return customNode;
      }

      if (card.selectedByOther) {
        card.onSelectByOther(false);
      }

      card.selectedByOther = false;
    }
  }, {
    key: "setCardActivatedByOther",
    value: function setCardActivatedByOther(card, member) {
      if (card.isEditable) return;

      var _ref2 = member || {},
          uuid = _ref2.uuid,
          color = _ref2.color;

      if (color) {
        var tinyColor = tinycolor2(color);
        tinyColor.setAlpha(0.3);
        var rgb = tinyColor.toRgbString();
        var customNode;

        if (!card.activatedByOther) {
          customNode = card.onActivateByOther(true, {
            color: color,
            rgb: rgb
          });
        }

        card.activatedByOther = uuid;
        return customNode;
      }

      if (card.activatedByOther) {
        card.onActivateByOther(false);
      }

      card.activatedByOther = false;
    }
  }, {
    key: "drawRange",
    value: function drawRange(range, member) {
      var _this4 = this;

      var card = this.engine.card;
      var uuid = member.uuid;
      var cardInfo = card.find(range.commonAncestorNode); //如果是卡片，并且选区不在内容模块中，而是在卡片两侧的光标位置处，就不算作卡片

      if (cardInfo && !cardInfo.isCenter(range.commonAncestorNode)) {
        cardInfo = undefined;
      }

      card.each(function (cardComponent) {
        if (cardComponent.isEditable) return;

        if (!cardInfo || !cardComponent.root.equal(cardInfo.root)) {
          if (cardComponent.activatedByOther === uuid) {
            _this4.setCardActivatedByOther(cardComponent);
          }

          _this4.root.children(".".concat(USER_MASK_CLASS, "[data-uuid=\"").concat(uuid, "\"]")).remove();

          Tooltip.hide();
        }
      });

      if (cardInfo && !cardInfo.isEditable) {
        var root = this.setCardActivatedByOther(cardInfo, member) || cardInfo.root;
        var cursor = this.drawCursor(root, member);
        var collab = cardInfo.constructor.collab;

        if ((collab === undefined || collab === true) && cursor) {
          this.drawCardMask(root, cursor, member);
        }

        this.drawBackground(range, member);
      } else {
        //可编辑卡片
        if (cardInfo) {
          this.drawBackground(range, member);
          return;
        }

        card.each(function (cardComponent) {
          var centerNode = cardComponent.getCenter();

          if (centerNode && centerNode.length > 0) {
            if (cardComponent.isEditable) {
              if (centerNode.contains(range.startNode) && centerNode.contains(range.endNode) && (range.startNode.closest(EDITABLE_SELECTOR).length > 0 || range.endNode.closest(EDITABLE_SELECTOR).length > 0)) {
                _this4.setCardSelectedByOther(cardComponent);

                return;
              }
            }

            if (range.isPointInRange(centerNode.get(), 0)) {
              _this4.setCardSelectedByOther(cardComponent, member);
            } else if (cardComponent.selectedByOther === uuid) {
              _this4.setCardSelectedByOther(cardComponent);
            }
          }
        });
        var singleCard = card.getSingleSelectedCard(range);

        if (singleCard) {
          if (singleCard.isEditable) {
            var center = singleCard.getCenter();

            if (center.contains(range.startNode) && center.contains(range.endNode) && (range.startNode.closest(EDITABLE_SELECTOR).length > 0 || range.endNode.closest(EDITABLE_SELECTOR).length > 0)) {
              return;
            }
          }

          var _root = this.setCardSelectedByOther(singleCard, member) || singleCard.root;

          this.drawCursor(_root, member);
        } else {
          range.shrinkToElementNode();
          var ranges = this.drawBackground(range, member);

          if (!range.collapsed) {
            ranges.forEach(function (sub) {
              if (!sub.collapsed) {
                range = sub;
              }
            });
            range.shrinkToElementNode();
            range.collapse(false);
          }

          this.drawCursor(range, member);
        }
      }
    }
  }, {
    key: "updateBackgroundPosition",
    value: function updateBackgroundPosition() {
      var _this5 = this;

      this.root.children(".".concat(USER_BACKGROUND_CLASS)).each(function (child) {
        var node = $(child);
        var range = child['__targetRange'];
        var uuid = node.attributes('data-uuid');
        var color = node.attributes('data-color');

        _this5.drawBackground(range, {
          uuid: uuid,
          color: color
        });
      });
    }
  }, {
    key: "updateCursorPosition",
    value: function updateCursorPosition() {
      var _this6 = this;

      this.root.children(".".concat(USER_CURSOR_CLASS)).each(function (child) {
        var node = $(child);
        var target = child['__target'];

        if (target.startContainer || 0 !== $(target).closest('body').length) {
          var rect = _this6.getCursorRect(target);

          _this6.setCursorRect(node, rect);
        } else node.remove();
      });
    }
  }, {
    key: "updateCardMaskPosition",
    value: function updateCardMaskPosition() {
      var _this$root$get3;

      var parentRect = ((_this$root$get3 = this.root.get()) === null || _this$root$get3 === void 0 ? void 0 : _this$root$get3.getBoundingClientRect()) || {
        left: 0,
        top: 0
      };
      this.root.children(".".concat(USER_MASK_CLASS)).each(function (child) {
        var node = $(child);
        var target = child['__targetNode'];

        if (0 !== $(target).closest('body').length) {
          var rect = target.getBoundingClientRect();
          node.css({
            left: rect.left - parentRect.left + 'px',
            top: rect.top - parentRect.top + 'px'
          });
        } else node.remove();
      });
    }
  }, {
    key: "updatePosition",
    value: function updatePosition() {
      this.updateBackgroundPosition();
      this.updateCursorPosition();
      this.updateCardMaskPosition();
    }
  }, {
    key: "updateBackgroundAlpha",
    value: function updateBackgroundAlpha(range) {
      var cursorRect = this.getCursorRect(range);
      this.root.children(".".concat(USER_CURSOR_CLASS)).each(function (child) {
        var node = $(child);
        var trigger = node.find(".".concat(USER_CURSOR_TRIGGER_CLASS));
        var left = node.css('left');
        var top = node.css('top');
        var bgColor = tinycolor2(node.css('background-color'));

        if (cursorRect.left === left && cursorRect.top === top) {
          bgColor.setAlpha(0.3);
        } else {
          bgColor.setAlpha(1);
        }

        node.css('background-color', bgColor.toRgbString());
        trigger.css('background-color', bgColor.toRgbString());
      });
    }
  }, {
    key: "render",
    value: function render(data, members, idDraw) {
      var _this7 = this;

      var root = this.root,
          engine = this.engine;
      var info = {};
      data.forEach(function (item) {
        var path = item.path,
            uuid = item.uuid,
            active = item.active;
        var member = members.find(function (m) {
          return m.uuid === uuid;
        });

        if (member && (idDraw || active)) {
          if (path.length !== 0) {
            var range = Range.fromPath(engine, path);

            _this7.drawRange(range, member);
          } else {
            info[uuid] = true;
          }
        }
      });
      this.root.children('[data-uuid]').each(function (child) {
        var domChild = $(child);
        var uuid = domChild.attributes('data-uuid');
        var member = members.find(function (m) {
          return m.uuid === uuid;
        });

        if (!member || info[uuid]) {
          if (domChild.hasClass(USER_MASK_CLASS)) {
            var target = $(domChild[0]['__targetNode']);
            var component = engine.card.find(target);

            if (component && !component.isEditable && component.activatedByOther === uuid) {
              _this7.setCardActivatedByOther(component);
            }
          }

          domChild.remove();
        }
      });
      engine.card.each(function (component) {
        if (component.isEditable) return;
        var member = members.find(function (m) {
          return m.uuid === component.selectedByOther;
        });

        if (!member || info[member.uuid]) {
          _this7.setCardSelectedByOther(component);
        }
      });
    }
  }]);

  return RangeColoring;
}();

var toDOM = function toDOM(ops) {
  var fragment = document.createDocumentFragment();
  var elementName = null;
  var i = 0;
  var element = null;

  if (typeof ops[0] === 'string') {
    elementName = ops[0];
    i = 1;
  }

  for (; i < ops.length; i++) {
    if (Array.isArray(ops[i])) {
      fragment.appendChild(toDOM(ops[i]));
    } else if ('[object Object]' === ops[i].toString()) {
      if (elementName) {
        element = document.createElement(elementName);

        for (var attr in ops[i]) {
          var _element;

          (_element = element) === null || _element === void 0 ? void 0 : _element.setAttribute(unescapeDots(attr), unescape(ops[i][attr]));
        } //不是待渲染Card，转换为待渲染Card


        if (Object.keys(ops[i]).indexOf(CARD_KEY) > -1 && Object.keys(ops[i]).indexOf(READY_CARD_KEY) < 0) {
          var _element2;

          (_element2 = element) === null || _element2 === void 0 ? void 0 : _element2.setAttribute(READY_CARD_KEY, ops[i][CARD_KEY]);
        }
      }
    } else if ('number' === typeof ops[i] || 'string' === typeof ops[i]) {
      var textNode = document.createTextNode(ops[i].toString());
      fragment.appendChild(textNode);
    }
  }

  if (!element && elementName) {
    element = document.createElement(elementName);
  }

  if (element) {
    element.appendChild(fragment);
    return element;
  } else {
    return fragment.childNodes[0];
  }
};

function fromChildDom(node, values) {
  var childNodes = node.children();

  if (0 !== childNodes.length) {
    for (var i = 0; i < childNodes.length; i++) {
      var child = childNodes.eq(i);
      if (!child) continue;
      var data = fromDOM(child);

      if (data) {
        values.push(data);
      }
    }
  }
}

var fromDOM = function fromDOM(node) {
  var values;

  if (!isTransientElement(node)) {
    var _node$get = node.get(),
        nodeName = _node$get.nodeName,
        nodeType = _node$get.nodeType,
        attributes = _node$get.attributes,
        nodeValue = _node$get.nodeValue;

    if (nodeType === getWindow().Node.ELEMENT_NODE) {
      values = [nodeName.toLowerCase()];
      var data = {};

      for (var i = 0; attributes && i < attributes.length; i++) {
        var _attributes$i = attributes[i],
            name = _attributes$i.name,
            specified = _attributes$i.specified,
            value = _attributes$i.value;

        if (specified && !isTransientAttribute(node, name)) {
          if (name === 'style') {
            var _node$get2;

            data['style'] = toHex(((_node$get2 = node.get()) === null || _node$get2 === void 0 ? void 0 : _node$get2.style.cssText) || value);
          } else if ('string' === typeof value) {
            data[name] = value;
          }
        }
      }

      values.push(data);
      fromChildDom(node, values);
      return values;
    }

    return nodeType === getWindow().Node.TEXT_NODE ? String(nodeValue) : undefined;
  }

  return;
};
var getPathValue = function getPathValue(data, path) {
  if (path.length === 0) return data;
  var value = data;

  for (var i = 0; i < path.length && value !== undefined; i++) {
    value = value[path[i]];
  }

  return value;
};
var pushAndRepair = function pushAndRepair(ops, op) {
  var _op$oldPath;

  var oldPath = ((_op$oldPath = op.oldPath) === null || _op$oldPath === void 0 ? void 0 : _op$oldPath.slice()) || [];
  ops.forEach(function (opItem) {
    var _opItem$oldPath;

    var opOffset = op.newPath[op.newPath.length - 1];
    var itemOffset = opItem.newPath[opItem.newPath.length - 1];
    var itemOldPath = ((_opItem$oldPath = opItem.oldPath) === null || _opItem$oldPath === void 0 ? void 0 : _opItem$oldPath.slice()) || [];
    var itemP = opItem.p.slice();

    if ('ld' in op) {
      if ('ld' in opItem) {
        if (itemOffset <= opOffset) {
          oldPath[oldPath.length - 1] = parseInt(oldPath[oldPath.length - 1].toString()) + 1;
        } else {
          itemOldPath[itemOldPath.length - 1] = parseInt(itemOldPath[itemOldPath.length - 1].toString()) + 1;
          itemP[itemP.length - 1] = parseInt(itemP[itemP.length - 1].toString()) + 1;
        }
      }

      if ('li' in opItem) {
        if (itemOffset >= opOffset) {
          itemP[itemP.length - 1] = parseInt(itemP[itemP.length - 1].toString()) + 1;
        } else {
          oldPath[oldPath.length - 1] = parseInt(oldPath[oldPath.length - 1].toString()) - 1;
        }
      }
    }

    if ('li' in op) {
      if ('ld' in opItem) {
        if (itemOffset > opOffset) {
          itemOldPath[itemOldPath.length - 1] = parseInt(itemOldPath[itemOldPath.length - 1].toString()) - 1;
          itemP[itemP.length - 1] = parseInt(itemP[itemP.length - 1].toString()) - 1;
        }
      }

      if ('li' in opItem) {
        if (itemOffset > opOffset) {
          itemP[itemP.length - 1] = parseInt(itemP[itemP.length - 1].toString()) - 1;
        }
      }
    }

    opItem.oldPath = itemOldPath;
    opItem.p = itemP;
  });
  op.oldPath = oldPath;
  ops.push(op);
};

var Doc = /*#__PURE__*/function (_EventEmitter) {
  _inherits(Doc, _EventEmitter);

  var _super = _createSuper(Doc);

  function Doc(engine) {
    var _this;

    _classCallCheck(this, Doc);

    _this = _super.call(this);
    _this.engine = void 0;
    _this.mode = 'lock';
    _this.version = 0;
    _this.data = void 0;
    _this.engine = engine;

    _this.create();

    return _this;
  }

  _createClass(Doc, [{
    key: "destroy",
    value: function destroy() {
      delete this.data;
    }
  }, {
    key: "create",
    value: function create() {
      this.data = fromDOM(this.engine.container);
    }
  }, {
    key: "apply",
    value: function apply(ops) {
      if (ops.length) {
        try {
          this.data = OTJSON.type.apply(this.data, ops);
        } catch (error) {
          console.log(error);
        }
      }
    }
  }, {
    key: "submitOp",
    value: function submitOp(ops) {
      this.apply(ops);
    }
  }]);

  return Doc;
}(EventEmitter2);

var Applier = /*#__PURE__*/function () {
  function Applier(engine) {
    var _this = this;

    _classCallCheck(this, Applier);

    this.engine = void 0;

    this.elementAtPath = function (node, path) {
      if (isNodeEntry(node)) node = node[0];
      var index = path[0];
      if (index === JSONML.ATTRIBUTE_INDEX) return [node, undefined, node, index];
      var offset = index - JSONML.ELEMENT_LIST_OFFSET;
      var childNode = Array.from(node.childNodes).filter(function (node) {
        var childNode = $(node);
        return !isTransientElement(childNode);
      })[offset];
      var pathOffset = path[1];

      if (1 === path.length || pathOffset === JSONML.TAG_NAME_INDEX || pathOffset === JSONML.ATTRIBUTE_INDEX || childNode.nodeType === getWindow().Node.TEXT_NODE) {
        return [childNode, offset, node, pathOffset];
      }

      return _this.elementAtPath(childNode, path.slice(1));
    };

    this.engine = engine;
  }

  _createClass(Applier, [{
    key: "fromRemoteAttr",
    value: function fromRemoteAttr(attr) {
      if (!attr) return;
      var id = attr.id,
          leftText = attr.leftText,
          rightText = attr.rightText;
      var idNode = $("[data-id=\"".concat(id, "\"]"));
      if (idNode.length === 0) return;
      var node = idNode.get();
      var text = node.textContent || '';
      if (text === '') return {
        container: node,
        offset: 0
      };

      if (text === null || text === void 0 ? void 0 : text.startsWith(leftText)) {
        var _nextChild = node.firstChild;
        var _offset = leftText.length;

        while (_nextChild && (_nextChild.nodeType !== 3 || (((_nextChild$textConten = _nextChild.textContent) === null || _nextChild$textConten === void 0 ? void 0 : _nextChild$textConten.length) || 0) < _offset)) {
          var _nextChild$textConten, _nextChild$textConten2;

          if ((((_nextChild$textConten2 = _nextChild.textContent) === null || _nextChild$textConten2 === void 0 ? void 0 : _nextChild$textConten2.length) || 0) < _offset) {
            var _nextChild$textConten3;

            _offset -= ((_nextChild$textConten3 = _nextChild.textContent) === null || _nextChild$textConten3 === void 0 ? void 0 : _nextChild$textConten3.length) || 0;
            _nextChild = _nextChild.nextSibling;
          } else {
            _nextChild = _nextChild.firstChild;
          }
        }

        return {
          container: _nextChild,
          offset: _offset
        };
      }

      if (text === null || text === void 0 ? void 0 : text.endsWith(rightText)) {
        var _prevChild2, _prevChild2$textConte;

        var _offset2 = rightText.length;
        var prevChild = node.lastChild;

        while (prevChild && (prevChild.nodeType !== 3 || (((_prevChild$textConten = prevChild.textContent) === null || _prevChild$textConten === void 0 ? void 0 : _prevChild$textConten.length) || 0) < _offset2)) {
          var _prevChild$textConten, _prevChild$textConten2;

          if (((_prevChild$textConten2 = prevChild.textContent) === null || _prevChild$textConten2 === void 0 ? void 0 : _prevChild$textConten2.length) || 0 < _offset2) {
            var _prevChild, _prevChild$textConten3;

            _offset2 -= ((_prevChild = prevChild) === null || _prevChild === void 0 ? void 0 : (_prevChild$textConten3 = _prevChild.textContent) === null || _prevChild$textConten3 === void 0 ? void 0 : _prevChild$textConten3.length) || 0;
            prevChild = prevChild.previousSibling;
          } else {
            prevChild = prevChild.lastChild;
          }
        }

        return {
          container: prevChild,
          offset: ((_prevChild2 = prevChild) === null || _prevChild2 === void 0 ? void 0 : (_prevChild2$textConte = _prevChild2.textContent) === null || _prevChild2$textConte === void 0 ? void 0 : _prevChild2$textConte.length) || 0 - _offset2
        };
      }

      var offset = 0;

      while (text[offset] === leftText[offset]) {
        offset++;
      }

      var nextChild = node.firstChild;

      while (((_nextChild2 = nextChild) === null || _nextChild2 === void 0 ? void 0 : _nextChild2.nodeType) !== 3 || (((_nextChild$textConten4 = nextChild.textContent) === null || _nextChild$textConten4 === void 0 ? void 0 : _nextChild$textConten4.length) || 0) < offset) {
        var _nextChild2, _nextChild$textConten4, _nextChild3, _nextChild3$textConte;

        if (((_nextChild3 = nextChild) === null || _nextChild3 === void 0 ? void 0 : (_nextChild3$textConte = _nextChild3.textContent) === null || _nextChild3$textConte === void 0 ? void 0 : _nextChild3$textConte.length) || 0 < offset) {
          var _nextChild4, _nextChild4$textConte, _nextChild5;

          offset -= ((_nextChild4 = nextChild) === null || _nextChild4 === void 0 ? void 0 : (_nextChild4$textConte = _nextChild4.textContent) === null || _nextChild4$textConte === void 0 ? void 0 : _nextChild4$textConte.length) || 0;
          nextChild = (_nextChild5 = nextChild) === null || _nextChild5 === void 0 ? void 0 : _nextChild5.nextSibling;
        } else {
          var _nextChild6;

          nextChild = (_nextChild6 = nextChild) === null || _nextChild6 === void 0 ? void 0 : _nextChild6.firstChild;
        }
      }

      return {
        container: nextChild,
        offset: offset
      };
    }
  }, {
    key: "getSideText",
    value: function getSideText(node, offset) {
      var idNode = node.closest('[data-id]');

      if (idNode.length > 0) {
        var id = idNode.attributes('data-id');
        var leftRange = Range.create(this.engine);
        var rightRange = Range.create(this.engine);
        leftRange.setStart(idNode[0], 0);
        leftRange.setEnd(node[0], offset);
        rightRange.setStart(node[0], offset);
        rightRange.setEnd(idNode[0], idNode[0].childNodes.length);
        return {
          id: id,
          leftText: leftRange.toString(),
          rightText: rightRange.toString()
        };
      }

      return;
    }
  }, {
    key: "setAttribute",
    value: function setAttribute(path, attr, value) {
      var engine = this.engine;
      var card = this.engine.card;

      var _this$elementAtPath = this.elementAtPath(engine.container[0], path),
          _this$elementAtPath2 = _slicedToArray(_this$elementAtPath, 1),
          node = _this$elementAtPath2[0];

      var domNode = $(node);

      if (domNode && domNode.length > 0 && !domNode.isRoot() || /^data-selection-/.test(attr)) {
        var _domNode$get;

        attr = unescapeDots(attr);
        value = unescape(value);
        (_domNode$get = domNode.get()) === null || _domNode$get === void 0 ? void 0 : _domNode$get.setAttribute(attr, value);

        if (domNode.isCard()) {
          var component = card.find(domNode);
          if (!(component === null || component === void 0 ? void 0 : component.isEditable)) card.render($(node));
        }
      }
    }
  }, {
    key: "removeAttribute",
    value: function removeAttribute(path, attr) {
      var engine = this.engine;

      var _this$elementAtPath3 = this.elementAtPath(engine.container[0], path),
          _this$elementAtPath4 = _slicedToArray(_this$elementAtPath3, 1),
          node = _this$elementAtPath4[0];

      var domNode = $(node);

      if (domNode.length > 0 && !domNode.isRoot() || /^data-selection-/.test(attr)) {
        var _domNode$get2;

        (_domNode$get2 = domNode.get()) === null || _domNode$get2 === void 0 ? void 0 : _domNode$get2.removeAttribute(attr);
      }
    }
  }, {
    key: "insertNode",
    value: function insertNode(path, value) {
      var engine = this.engine;

      var _this$elementAtPath5 = this.elementAtPath(engine.container[0], path),
          _this$elementAtPath6 = _slicedToArray(_this$elementAtPath5, 3),
          begine = _this$elementAtPath6[0],
          beginOffset = _this$elementAtPath6[1],
          end = _this$elementAtPath6[2];

      var domBegine = $(begine);
      var domEnd = $(end);

      if (domEnd.length > 0 && !domBegine.isRoot()) {
        var element = typeof value === 'string' ? document.createTextNode(value) : toDOM(value);

        if (domBegine && domBegine.parent()) {
          var _domEnd$get;

          (_domEnd$get = domEnd.get()) === null || _domEnd$get === void 0 ? void 0 : _domEnd$get.insertBefore(element, domBegine.get());
        } else {
          var _domEnd$get2;

          (_domEnd$get2 = domEnd.get()) === null || _domEnd$get2 === void 0 ? void 0 : _domEnd$get2.insertBefore(element, null);
        }

        engine.card.render($(element));
      }
    }
  }, {
    key: "deleteNode",
    value: function deleteNode(path, isRemote) {
      var engine = this.engine;

      var _this$elementAtPath7 = this.elementAtPath(engine.container[0], path),
          _this$elementAtPath8 = _slicedToArray(_this$elementAtPath7, 1),
          begine = _this$elementAtPath8[0];

      var domBegine = $(begine);

      if (domBegine.length > 0 && !domBegine.isRoot()) {
        if (domBegine.isCard()) {
          engine.readonly = false;
          if (isRemote) engine.card.removeRemote(domBegine);else {
            engine.card.remove(domBegine);
            var card = engine.card.find(domBegine);

            if (card && card.activated && engine.readonly) {
              engine.readonly = false;
              if (!engine.isFocus()) engine.focus();
            }
          }
        } else domBegine.remove();
      }
    }
  }, {
    key: "insertInText",
    value: function insertInText(path, offset, text) {
      var engine = this.engine;

      var _this$elementAtPath9 = this.elementAtPath(engine.container[0], path),
          _this$elementAtPath10 = _slicedToArray(_this$elementAtPath9, 4),
          begine = _this$elementAtPath10[0],
          beginOffset = _this$elementAtPath10[1],
          end = _this$elementAtPath10[2],
          endOffset = _this$elementAtPath10[3];

      switch (endOffset) {
        case JSONML.TAG_NAME_INDEX:
          throw Error('Unsupported indexType JSONML.TAG_NAME_INDEX (0)');

        case JSONML.ATTRIBUTE_INDEX:
          throw Error('Unsupported indexType JSONML.ATTRIBUTE_INDEX (1)');

        default:
          if (begine && !$(begine).isText()) return;
          var nodeValue = begine && begine.nodeValue ? begine.nodeValue : '';
          var value = nodeValue.substring(0, offset) + text + nodeValue.substring(offset);
          if (begine && begine.parentNode === end) begine.nodeValue = value;else {
            var textNode = document.createTextNode(value);
            end.insertBefore(textNode, end.firstChild);
          }
      }
    }
  }, {
    key: "deleteInText",
    value: function deleteInText(path, offset, text) {
      var engine = this.engine;

      var _this$elementAtPath11 = this.elementAtPath(engine.container[0], path),
          _this$elementAtPath12 = _slicedToArray(_this$elementAtPath11, 4),
          begine = _this$elementAtPath12[0],
          beginOffset = _this$elementAtPath12[1],
          end = _this$elementAtPath12[2],
          endOffset = _this$elementAtPath12[3];

      switch (endOffset) {
        case JSONML.TAG_NAME_INDEX:
          throw Error('Unsupported indexType JSONML.TAG_NAME_INDEX (0)');

        case JSONML.ATTRIBUTE_INDEX:
          throw Error('Unsupported indexType JSONML.ATTRIBUTE_INDEX (1)');

        default:
          end = begine;
          if (!$(end).isText()) return;
          var nodeValue = end && end.nodeValue ? end.nodeValue : '';
          var value = nodeValue.substring(0, offset) + nodeValue.substring(offset + text.length);
          end.nodeValue = value;
      }
    }
  }, {
    key: "applyOperation",
    value: function applyOperation(op, isRemote) {
      var path = op.p;
      var attr, offset;

      if (path.length !== 0) {
        if ('si' in op || 'sd' in op) {
          offset = path[path.length - 1];
          path = path.slice(0, -1);
        }

        if ('oi' in op || 'od' in op) {
          attr = path[path.length - 1].toString();
          path = path.slice(0, -1);
        }

        if ('oi' in op) {
          return this.setAttribute(path, attr, op.oi);
        } else if ('od' in op) {
          return this.removeAttribute(path, attr);
        } else if ('sd' in op) {
          return this.deleteInText(path, offset, op.sd);
        } else if ('si' in op) {
          return this.insertInText(path, offset, op.si);
        } else if ('ld' in op) {
          return this.deleteNode(path, isRemote);
        } else if ('li' in op) {
          return this.insertNode(path, op.li);
        }

        return;
      }
    }
  }, {
    key: "applyRemoteOperations",
    value: function applyRemoteOperations(ops) {
      var _this2 = this;

      try {
        var path = this.getRangeRemotePath();
        ops.forEach(function (op) {
          return _this2.applyOperation(op, true);
        });
        if (path) this.setRangeByRemotePath(path);
        this.engine.change.change(true);
      } catch (error) {
        console.log(error);
      }
    }
  }, {
    key: "applySelfOperations",
    value: function applySelfOperations(ops) {
      var _this3 = this;

      ops.forEach(function (op) {
        return _this3.applyOperation(op);
      });
      this.engine.change.change();
    }
  }, {
    key: "setRangeAfterOp",
    value: function setRangeAfterOp(op) {
      var engine = this.engine;
      var offset;
      var path = op.p;

      if ('si' in op || 'sd' in op) {
        offset = path[path.length - 1];
        path = path.slice(0, -1);
      }

      if ('oi' in op || 'od' in op) {
        path = path.slice(0, -1);
      }

      var _this$elementAtPath13 = this.elementAtPath(engine.container[0], path),
          _this$elementAtPath14 = _slicedToArray(_this$elementAtPath13, 3),
          begine = _this$elementAtPath14[0],
          beginOffset = _this$elementAtPath14[1],
          end = _this$elementAtPath14[2];

      var range = Range.create(this.engine);

      if ('si' in op || 'sd' in op) {
        var node = begine['data'] === '' ? end : begine;
        var stringInsertOp = op;
        var rangeOffset = offset + (stringInsertOp.si ? stringInsertOp.si.length : 0);
        range.setOffset(node, rangeOffset, rangeOffset);
        engine.change.select(range);
        return;
      }

      range.select(begine || end.lastChild || end, true).shrinkToElementNode().collapse(false);
      engine.change.select(range);
    }
  }, {
    key: "getRangeRemotePath",
    value: function getRangeRemotePath() {
      try {
        var _window$getSelection;

        if (((_window$getSelection = window.getSelection()) === null || _window$getSelection === void 0 ? void 0 : _window$getSelection.rangeCount) === 0) return;
        var range = Range.from(this.engine);
        if (!range || range.inCard()) return;
        var startNode = range.startNode,
            startOffset = range.startOffset,
            endNode = range.endNode,
            endOffset = range.endOffset;
        return {
          start: this.getSideText(startNode, startOffset),
          end: this.getSideText(endNode, endOffset)
        };
      } catch (error) {
        console.log(error);
        return;
      }
    }
  }, {
    key: "setRangeByRemotePath",
    value: function setRangeByRemotePath(path) {
      try {
        var _Range$from;

        var selection = window.getSelection();
        var range = selection ? (_Range$from = Range.from(this.engine, selection)) === null || _Range$from === void 0 ? void 0 : _Range$from.cloneRange() : undefined;
        if (!range) return;
        var start = path.start,
            end = path.end;
        var startInfo;
        var endInfo;
        if (start) startInfo = this.fromRemoteAttr(start);
        if (end) endInfo = this.fromRemoteAttr(end);

        if (startInfo && startInfo.container) {
          range.setStart(startInfo.container, startInfo.offset);
        }

        if (endInfo && endInfo.container) {
          range.setStart(endInfo.container, endInfo.offset);
        }

        selection.removeAllRanges();
        this.engine.change.select(range);
      } catch (error) {
        console.log(error);
      }
    }
  }, {
    key: "setRangeByPath",
    value: function setRangeByPath(path) {
      if (path) {
        var _path = _slicedToArray(path, 2),
            start = _path[0],
            end = _path[1];

        if (start && end) {
          var beginOffset = start[start.length - 1];
          var endOffset = end[end.length - 1];
          var startClone = start.slice();
          var endClone = end.slice();
          startClone.pop();
          endClone.pop();
          var _this$engine = this.engine,
              container = _this$engine.container,
              change = _this$engine.change;
          var startChild = container.getChildByPath(startClone, function (child) {
            return !isTransientElement($(child));
          });
          var endChild = container.getChildByPath(endClone, function (child) {
            return !isTransientElement($(child));
          });

          try {
            var range = change.getRange();
            range.setStart(startChild, beginOffset);
            range.setEnd(endChild, endOffset);
            change.select(range);
            range.scrollRangeIntoView();
          } catch (error) {
            console.log(error);
          }
        }
      }
    }
  }]);

  return Applier;
}();

var Creator = /*#__PURE__*/function (_EventEmitter) {
  _inherits(Creator, _EventEmitter);

  var _super = _createSuper(Creator);

  function Creator(engine, options) {
    var _this;

    _classCallCheck(this, Creator);

    _this = _super.call(this);
    _this.engine = void 0;
    _this.doc = void 0;
    _this.dmp = void 0;
    _this.addedNodes = [];
    _this.cacheTransientElements = void 0;
    _this.timer = null;
    _this.lineStart = false;
    _this.laterOps = null;
    _this.engine = engine;
    _this.doc = options.doc;
    _this.dmp = new diff_match_patch();
    return _this;
  }

  _createClass(Creator, [{
    key: "patchesToOps",
    value: function patchesToOps(path, text1, text2) {
      var ops = [];
      var patches = this.dmp.patch_make(text1, text2);
      Object.keys(patches).forEach(function (key) {
        var patch = patches[key];
        var start1 = patch.start1;
        patch.diffs.forEach(function (diff) {
          var _diff = _slicedToArray(diff, 2),
              type = _diff[0],
              data = _diff[1];

          if (type !== DIFF_DELETE) {
            if (type !== DIFF_INSERT) {
              if (type === DIFF_EQUAL) {
                start1 += data.length;
              }
            } else {
              var p = [];
              ops.push({
                si: data,
                p: p.concat(_toConsumableArray(path), [start1])
              });
            }
          } else {
            var _p = [];
            ops.push({
              sd: data,
              p: _p.concat(_toConsumableArray(path), [start1])
            });
          }
        });
      });
      return ops;
    }
  }, {
    key: "cacheAddedNode",
    value: function cacheAddedNode(node) {
      var _this2 = this;

      this.addedNodes.push(node);
      node.childNodes.forEach(function (child) {
        _this2.addedNodes.push(child);

        _this2.cacheAddedNode(child);
      });
    }
  }, {
    key: "clearAddedNodeCache",
    value: function clearAddedNodeCache() {
      this.addedNodes = [];
    }
  }, {
    key: "inAddedCache",
    value: function inAddedCache(node) {
      return this.addedNodes.find(function (n) {
        return n === node;
      });
    }
  }, {
    key: "isTransientMutation",
    value: function isTransientMutation(record, transientElements) {
      var addedNodes = record.addedNodes,
          removedNodes = record.removedNodes,
          target = record.target,
          type = record.type,
          attributeName = record.attributeName;
      var targetNode = $(target);

      if (type === 'childList') {
        if (addedNodes[0] && isTransientElement($(addedNodes[0]), transientElements)) return true;
        if (removedNodes[0] && isTransientElement($(removedNodes[0]), transientElements)) return true;
        if (isTransientElement(targetNode, transientElements)) return true;
      }

      return type === 'attributes' && (isTransientAttribute(targetNode, attributeName || '') || isTransientElement(targetNode, transientElements)) || type === 'characterData' && isTransientElement(targetNode, transientElements);
    }
  }, {
    key: "makeOpsFromMutations",
    value: function makeOpsFromMutations(records) {
      var _this3 = this;

      var path = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
      var oldPath = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];
      var node = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : this.engine.container;
      var addNodes = [];
      var ops = [];
      var attrOps = [];
      var cacheNodes = [];
      var mutationsNodes = [];
      var isDataString = false;

      var setNodeMutations = function setNodeMutations(root, record) {
        if (!cacheNodes.includes(root)) {
          if (!mutationsNodes.includes(root)) mutationsNodes.push(root);
          root['mutations'] = root['mutations'] || [];
          root['mutations'].push(record);
        }
      };

      var _loop = function _loop(_i) {
        var record = records[_i];
        var target = record.target,
            addedNodes = record.addedNodes,
            removedNodes = record.removedNodes,
            type = record.type;

        var inCache = _this3.inAddedCache(target);

        if (inCache) {
          _i++;
          i = _i;
          return "continue";
        }

        if (type === 'childList') {
          if (node.equal(target)) {
            if (removedNodes[0]) {
              var removeIndex = _this3.getRemoveNodeIndex(record, records);

              Array.from(removedNodes).forEach(function (removedNode) {
                if (!addNodes.find(function (n) {
                  return n === removedNode;
                })) {
                  var p = [];
                  p = p.concat(_toConsumableArray(path), [removeIndex + 2]);
                  var op = [];
                  op = op.concat(_toConsumableArray(oldPath), [removeIndex + 2]);
                  pushAndRepair(ops, {
                    ld: true,
                    p: p,
                    newPath: p.slice(),
                    oldPath: op
                  });
                }
              });
            }

            if (addedNodes[0]) {
              Array.from(addedNodes).forEach(function (addedNode) {
                var domAddedNode = $(addedNode);
                var data = fromDOM(domAddedNode);

                if (addedNode.parentNode === node.get()) {
                  var _domAddedNode$parent;

                  //父节点就是编辑器根节点，就不需要过滤
                  var index = domAddedNode.getIndex(((_domAddedNode$parent = domAddedNode.parent()) === null || _domAddedNode$parent === void 0 ? void 0 : _domAddedNode$parent.isRoot()) ? undefined : function (node) {
                    return !isTransientElement($(node), _this3.cacheTransientElements);
                  }) + 2;
                  var p = [];
                  p = p.concat(_toConsumableArray(path), [index]);
                  pushAndRepair(ops, {
                    li: data,
                    p: p,
                    newPath: p.slice()
                  });
                  cacheNodes.push(addedNode);

                  _this3.cacheAddedNode(addedNode);
                } else if (addedNode.parentNode !== null) {
                  var parent = $(addedNode).findParent(node);
                  if (parent && parent.length > 0) setNodeMutations(parent.get(), record);
                  addNodes.push(addedNode);
                } else {
                  addNodes.push(addedNode);
                }
              });
            }
          } else {
            var parent = $(target).findParent(node);
            if (parent && parent.length > 0) setNodeMutations(parent.get(), record);
          }
        } else if (type === 'characterData') {
          if (node.equal(target)) {
            if (!isDataString) {
              var _this3$doc2;

              if (typeof getPathValue((_this3$doc2 = _this3.doc) === null || _this3$doc2 === void 0 ? void 0 : _this3$doc2.data, oldPath) === 'string') {
                attrOps.push({
                  path: path,
                  oldPath: oldPath,
                  newValue: target['data']
                });
                isDataString = true;
              }
            }
          } else {
            var _parent = $(target).findParent(node);

            if (_parent && _parent.length > 0) setNodeMutations(_parent.get(), record);
          }
        } else if (type === 'attributes') {
          if (node.equal(target)) {
            var oldValue = record.oldValue,
                attributeName = record.attributeName;
            var attrValue = attributeName ? target.getAttribute(attributeName) : '';
            if (!oldValue) oldValue = '';
            attrValue = attrValue ? escape(attrValue) : '';

            if (oldValue !== attrValue) {
              var p = [];
              var newOp = {};
              newOp.p = p.concat(_toConsumableArray(path), [1, escapeDots(attributeName || '')]);
              if (oldValue) newOp.od = oldValue;
              if (attrValue) newOp.oi = attrValue;
              attrOps.push(newOp);
            }
          } else {
            var _parent2 = $(target).findParent(node);

            if (_parent2 && _parent2.length > 0) setNodeMutations(_parent2.get(), record);
          }
        }

        _i++;
        i = _i;
      };

      for (var i = 0; records[i];) {
        var _ret = _loop(i);

        if (_ret === "continue") continue;
      }

      var allOps = [];
      ops.forEach(function (op) {
        if ('ld' in op) {
          var _this3$doc;

          var pathValue = getPathValue((_this3$doc = _this3.doc) === null || _this3$doc === void 0 ? void 0 : _this3$doc.data, op.oldPath || []);

          if (pathValue !== undefined) {
            allOps.push({
              ld: pathValue,
              p: op.p
            });
          }
        }

        if ('li' in op) {
          allOps.push({
            li: op.li,
            p: op.p
          });
        }
      });
      allOps = allOps.concat(attrOps);
      mutationsNodes.forEach(function (node) {
        var mutations = node['mutations'];
        delete node['mutations'];

        if (node !== null) {
          var _element$parent;

          var element = $(node);
          var index = element.getIndex(((_element$parent = element.parent()) === null || _element$parent === void 0 ? void 0 : _element$parent.isRoot()) ? undefined : function (node) {
            return !isTransientElement($(node), _this3.cacheTransientElements);
          });
          var oldIndex = getOldIndex(index, ops);
          var p = [];
          allOps = allOps.concat(_this3.makeOpsFromMutations(mutations, p.concat.apply(p, _toConsumableArray(path).concat([[index + 2]])), p.concat(_toConsumableArray(oldPath), [oldIndex + 2]), element));
        }
      });
      return allOps;
    }
  }, {
    key: "getRemoveNodeIndex",
    value: function getRemoveNodeIndex(record, records) {
      var _this4 = this;

      var target = record.target,
          nextSibling = record.nextSibling,
          previousSibling = record.previousSibling,
          addedNodes = record.addedNodes;
      var targetElement = target;
      var childNodes = target.nodeType === getDocument().ELEMENT_NODE && targetElement.getAttribute(DATA_ELEMENT) === ROOT ? Array.from(target.childNodes) : Array.from(target.childNodes).filter(function (node) {
        return !isTransientElement($(node), _this4.cacheTransientElements);
      });
      var addedIndex = childNodes.indexOf(addedNodes[0]);
      var prevIndex = childNodes.indexOf(previousSibling);
      var nextIndex = childNodes.indexOf(nextSibling);
      var index;

      if (prevIndex !== -1) {
        index = prevIndex + 1;
      } else if (nextIndex !== -1) {
        index = nextIndex;
      } else if (addedIndex !== -1) {
        index = addedIndex;
      } else if (previousSibling) {
        if (nextSibling) {
          if (previousSibling) {
            index = this.getRemoveNodeIndexFromMutation(previousSibling, target, records);
          }
        } else {
          index = target.childNodes.length;
        }
      } else index = 0;

      return index !== undefined ? index : 0;
    }
  }, {
    key: "getRemoveNodeIndexFromMutation",
    value: function getRemoveNodeIndexFromMutation(node, target, records) {
      var record = records.find(function (record) {
        return record.target === target && record.removedNodes[0] === node;
      });

      if (record) {
        return this.getRemoveNodeIndex(record, records);
      }

      return 0;
    }
  }, {
    key: "handleMutations",
    value: function handleMutations(records) {
      var _this5 = this;

      //记录大于300的时候，先获取所有的不需要参与协同交互的节点，以提高效率
      if (records.length > 299) {
        this.cacheTransientElements = []; //非可编辑卡片的子节点

        var _this$engine = this.engine,
            card = _this$engine.card,
            container = _this$engine.container;
        card.each(function (card) {
          if (!card.isEditable || !!card.root.attributes(CARD_ASYNC_RENDER)) {
            card.root.allChildren().forEach(function (child) {
              var _this5$cacheTransient;

              if (child.nodeType === getDocument().ELEMENT_NODE) (_this5$cacheTransient = _this5.cacheTransientElements) === null || _this5$cacheTransient === void 0 ? void 0 : _this5$cacheTransient.push(child);
            });
          }
        }); //所有的UI子节点

        container.find("".concat(UI_SELECTOR)).allChildren().forEach(function (child) {
          var _this5$cacheTransient2;

          if (child.nodeType === getDocument().ELEMENT_NODE) (_this5$cacheTransient2 = _this5.cacheTransientElements) === null || _this5$cacheTransient2 === void 0 ? void 0 : _this5$cacheTransient2.push(child);
        });
      }

      records = records.filter(function (record) {
        return !_this5.isTransientMutation(record, _this5.cacheTransientElements);
      });
      this.clearAddedNodeCache();
      var ops = this.makeOpsFromMutations(records); //重置缓存

      this.cacheTransientElements = undefined;
      ops = reduceOperations(ops);

      if (ops.length !== 0) {
        this.normalizeOps(ops);
      }
    }
  }, {
    key: "normalizeOps",
    value: function normalizeOps(ops) {
      var _this6 = this;

      if (this.engine.change.isComposing()) {
        if (ops.length === 2 && 'ld' in ops[1] && ops[1].ld[0] && 'li' in ops[0] && typeof ops[0].li === 'string') {
          this.lineStart = true;
          ops[0].li = '';
          this.readyToEmitOps(ops);
          return;
        }

        if (ops.length === 1 && isCursorOp(ops[0])) return;
        if (ops.length === 2 && 'si' in ops[0] && isEqual(ops[0], ops[1])) ops.splice(1, 1);
        this.laterOps = ops;
        if (this.timer) clearTimeout(this.timer);
        this.timer = setTimeout(function () {
          if (!_this6.engine.change.isComposing()) {
            if (_this6.lineStart) {
              _this6.engine.history.hold();

              _this6.lineStart = false;
            }

            if (_this6.laterOps) {
              _this6.readyToEmitOps(_this6.laterOps);

              _this6.laterOps = null;
            }

            _this6.timer = null;
          }
        }, 0);
      }

      if (this.laterOps) {
        var equal = isEqual(this.laterOps, ops);
        this.readyToEmitOps(this.laterOps);
        this.laterOps = null;
        if (equal) return;
      }

      this.readyToEmitOps(ops);
    }
  }, {
    key: "setDoc",
    value: function setDoc(doc) {
      this.doc = doc;
    }
  }, {
    key: "readyToEmitOps",
    value: function readyToEmitOps(ops) {
      var _this7 = this;

      var emitOps = [];
      ops.forEach(function (op) {
        if ('path' in op && op.newValue !== undefined) {
          var _this7$doc;

          var pathValue = getPathValue((_this7$doc = _this7.doc) === null || _this7$doc === void 0 ? void 0 : _this7$doc.data, op.oldPath);
          emitOps = emitOps.concat(_this7.patchesToOps(_toConsumableArray(op.path), pathValue, op.newValue));
        } else if (op.p.length !== 0) {
          emitOps.push(op);
        }
      });
      if (emitOps.length !== 0) this.emit('ops', emitOps);
    }
  }]);

  return Creator;
}(EventEmitter2);

var config = {
  childList: true,
  subtree: true,
  attributes: true,
  characterData: true,
  attributeOldValue: true,
  characterDataOldValue: true
};

var Mutation = /*#__PURE__*/function (_EventEmitter) {
  _inherits(Mutation, _EventEmitter);

  var _super = _createSuper(Mutation);

  function Mutation(node, options) {
    var _this;

    _classCallCheck(this, Mutation);

    _this = _super.call(this);
    _this.node = void 0;
    _this.engine = void 0;
    _this.doc = void 0;
    _this.isStopped = void 0;
    _this.observer = void 0;
    _this.creator = void 0;
    _this.isCache = false;
    _this.cache = [];
    _this.node = node;
    _this.isStopped = true;
    _this.engine = options.engine;
    _this.doc = options.doc;
    _this.creator = new Creator(_this.engine, {
      doc: _this.doc
    }); //https://dom.spec.whatwg.org/#mutationobserver

    _this.observer = new MutationObserver(function (records) {
      if (_this.isCache) {
        var _this$cache;

        (_this$cache = _this.cache).push.apply(_this$cache, _toConsumableArray(records));
      }

      if (!_this.isStopped && !_this.isCache) {
        _this.creator.handleMutations(records);
      }
    });

    _this.creator.on('ops', function (ops) {
      _this.onOpsReady(ops);
    });

    return _this;
  }

  _createClass(Mutation, [{
    key: "setDoc",
    value: function setDoc(doc) {
      this.doc = doc;
      this.creator.setDoc(doc);
    }
  }, {
    key: "start",
    value: function start() {
      if (this.isStopped) {
        this.observer.observe(this.node[0], config);
        this.isStopped = false;
      }
    }
  }, {
    key: "stop",
    value: function stop() {
      if (!this.isStopped) {
        this.observer.disconnect();
        this.isStopped = true;
      }
    }
    /**
     * 开始缓存操作
     */

  }, {
    key: "startCache",
    value: function startCache() {
      if (!this.isCache) {
        this.cache = [];
        this.isCache = true;
      }
    }
    /**
     * 将缓存提交处理，最后停止缓存
     */

  }, {
    key: "submitCache",
    value: function submitCache() {
      var _this2 = this;

      if (this.isCache) {
        setTimeout(function () {
          _this2.isCache = false;
          if (_this2.cache.length > 0) _this2.creator.handleMutations(_this2.cache);
          _this2.cache = [];
        }, 20);
      }
    }
    /**
     * 将缓存情况注销掉
     */

  }, {
    key: "destroyCache",
    value: function destroyCache() {
      var _this3 = this;

      if (this.isCache) {
        setTimeout(function () {
          _this3.isCache = false;
          _this3.cache = [];
        }, 20);
      }
    }
  }, {
    key: "onOpsReady",
    value: function onOpsReady(ops) {
      if (!this.isStopped) this.emit('ops', ops);
    }
  }]);

  return Mutation;
}(EventEmitter2);

var css_248z$3 = ".ot-user-cursor {\r\n\tposition: absolute;\r\n\tz-index: 10;\r\n\twidth: 2px;\r\n}\r\n\r\n.ot-user-cursor-trigger {\r\n\tposition: absolute;\r\n\ttop: -5px;\r\n\tleft: -2px;\r\n\tborder-radius: 100%;\r\n\tcolor: #ffffff;\r\n\twidth: 6px;\r\n\theight: 6px;\r\n\tfont-size: 0;\r\n\toverflow: hidden;\r\n\ttransition: all 0.1s linear;\r\n}\r\n\r\n.ot-user-cursor-trigger-active {\r\n\ttop: -17px;\r\n\tborder-radius: 2px;\r\n\tcolor: #ffffff;\r\n\tfont-size: 10px;\r\n\tline-height: 18px;\r\n\theight: 18px;\r\n\twidth: auto;\r\n\tpadding: 0 3px;\r\n\twhite-space: nowrap;\r\n}\r\n\r\n.ot-card-mask {\r\n\tposition: absolute;\r\n\tz-index: 10;\r\n\tbackground: transparent;\r\n\tcursor: not-allowed;\r\n}\r\n\r\n.ot-user-cursor-card {\r\n\tposition: absolute;\r\n}\r\n\r\n.ot-user-cursor-card .ot-user-cursor-trigger {\r\n\tdisplay: none;\r\n}\r\n\r\n.ot-user-cursor-card .ot-user-cursor-trigger-active {\r\n\tposition: absolute;\r\n\tdisplay: block;\r\n\ttop: -19px;\r\n\tleft: 0;\r\n\tborder-radius: 2px;\r\n\tcolor: #ffffff;\r\n\tfont-size: 10px;\r\n\tline-height: 18px;\r\n\theight: 18px;\r\n\twidth: auto;\r\n\tpadding: 0 3px;\r\n\twhite-space: nowrap;\r\n}";
styleInject(css_248z$3);

var colors = ['#597EF7', '#73D13D', '#FF4D4F', '#9254DE', '#36CFC9', '#FFA940', '#F759AB', '#40A9FF'];

var OTModel = /*#__PURE__*/function (_EventEmitter) {
  _inherits(OTModel, _EventEmitter);

  var _super = _createSuper(OTModel);

  function OTModel(engine) {
    var _this;

    _classCallCheck(this, OTModel);

    _this = _super.call(this);
    _this.engine = void 0;
    _this.members = void 0;
    _this.currentMember = {
      uuid: '',
      name: '',
      color: ''
    };
    _this.waitingOps = [];
    _this.clientId = void 0;
    _this.selectionData = void 0;
    _this.rangeColoring = void 0;
    _this.applier = void 0;
    _this.mutation = void 0;
    _this.doc = null;
    _this.updateRangeColoringPosition = debounce(function () {
      _this.updateSelectionData();

      _this.rangeColoring.updatePosition();
    }, 100);
    _this.applyWaitingOps = debounce(function () {
      var operations = reduceOperations(_this.waitingOps);

      if (operations.length > 0) {
        _this.waitingOps = [];

        _this.applyAll(operations);

        _this.engine.history.collectRemoteOps(operations);

        var allData = _this.selectionData.getAll();

        _this.doRangeColoring(allData);
      }
    }, 0);
    _this.engine = engine;
    _this.members = [];
    _this.selectionData = new SelectionData(engine);
    _this.rangeColoring = new RangeColoring(engine);
    _this.applier = new Applier(engine);
    _this.mutation = new Mutation(engine.container, {
      engine: engine
    });

    _this.mutation.on('ops', function (ops) {
      return _this.handleOps(ops);
    });

    _this.clientId = randomString();
    _this.waitingOps = [];

    _this.engine.on('select', function () {
      _this.updateSelectionData();
    });

    return _this;
  }

  _createClass(OTModel, [{
    key: "destroy",
    value: function destroy() {
      if (this.doc) this.doc.destroy();
      this.stopMutation();
      this.rangeColoring.destroy();
      this.mutation = null;
    }
  }, {
    key: "initLockMode",
    value: function initLockMode() {
      var _this$mutation;

      this.stopMutation();
      this.doc = new Doc(this.engine);
      (_this$mutation = this.mutation) === null || _this$mutation === void 0 ? void 0 : _this$mutation.setDoc(this.doc);
      this.startMutation();
    }
  }, {
    key: "init",
    value: function init(doc) {
      var _this$mutation2,
          _this2 = this;

      var isDestroy = !this.doc || this.doc['mode'] === 'lock';
      this.stopMutation();

      if (!isDestroy) {
        this.doc.destroy();
      }

      this.doc = doc;
      (_this$mutation2 = this.mutation) === null || _this$mutation2 === void 0 ? void 0 : _this$mutation2.setDoc(doc);
      this.syncData();
      doc.on('op', function (op, clientId) {
        if (_this2.clientId !== clientId.toString()) {
          _this2.waitingOps = _this2.waitingOps.concat(op);

          _this2.applyWaitingOps();
        }
      });
      this.initSelection();
      this.startMutation();

      if (isDestroy) {
        this.emit('load');
      }
    }
  }, {
    key: "handleOps",
    value: function handleOps(ops) {
      this.submitOps(ops);
      this.engine.history.collectSelfOps(ops);

      if (this.doc['mode'] !== 'lock') {
        this.updateRangeColoringPosition();
      }

      this.engine.trigger('ops', ops);
    }
  }, {
    key: "submitOps",
    value: function submitOps(ops) {
      if (!this.doc) return;
      this.doc.submitOp(ops, {
        source: this.clientId
      });
    }
  }, {
    key: "applyAll",
    value: function applyAll(ops) {
      this.stopMutation();
      this.applier.applyRemoteOperations(ops);
      this.startMutation();
    }
  }, {
    key: "syncData",
    value: function syncData() {
      var doc = this.doc,
          engine = this.engine;
      if (!doc) return;

      if (Array.isArray(doc.data) && doc.data.length > 0 && doc['mode'] !== 'lock') {
        this.setData(doc.data);
        return;
      }

      var data = fromDOM(engine.container);

      if (doc.data) {
        doc.submitOp([{
          p: [],
          oi: data
        }], {
          source: this.clientId
        });
      } else {
        doc.create(data, 'ot-json0', {
          source: this.clientId
        });
      }
    }
  }, {
    key: "setData",
    value: function setData(data) {
      this.engine.setJsonValue(data);
    }
  }, {
    key: "startMutation",
    value: function startMutation() {
      if (this.mutation) this.mutation.start();
    }
  }, {
    key: "stopMutation",
    value: function stopMutation() {
      if (this.mutation) this.mutation.stop();
    }
  }, {
    key: "startMutationCache",
    value: function startMutationCache() {
      if (this.mutation) this.mutation.startCache();
    }
  }, {
    key: "submitMutationCache",
    value: function submitMutationCache() {
      if (this.mutation) this.mutation.submitCache();
    }
  }, {
    key: "destroyMutationCache",
    value: function destroyMutationCache() {
      if (this.mutation) this.mutation.destroyCache();
    }
  }, {
    key: "setMemberIdToUuid",
    value: function setMemberIdToUuid(member) {
      member.__uuid = member.uuid;
      member.uuid = String(member.id).toLowerCase();
    }
  }, {
    key: "setMemberUuidToId",
    value: function setMemberUuidToId(member) {
      member.uuid = member.__uuid;
      delete member.__uuid;
    }
  }, {
    key: "setMemberColor",
    value: function setMemberColor(member) {
      var iid = member.iid || 1;
      var index = (iid - 1) % colors.length;
      member.color = colors[index];
    }
  }, {
    key: "getMembers",
    value: function getMembers() {
      var _this3 = this;

      var members = cloneDeep(this.members);
      members.forEach(function (member) {
        _this3.setMemberUuidToId(member);
      });
      return members;
    }
  }, {
    key: "setMembers",
    value: function setMembers(members) {
      var _this4 = this;

      members = cloneDeep(members);
      members.forEach(function (member) {
        _this4.setMemberIdToUuid(member);

        _this4.setMemberColor(member);
      });
      this.members = members;
    }
  }, {
    key: "addMember",
    value: function addMember(member) {
      member = cloneDeep(member);
      this.setMemberIdToUuid(member);
      this.setMemberColor(member);

      if (!this.members.find(function (m) {
        return m.uuid === member.uuid;
      })) {
        this.members.push(member);
      }
    }
  }, {
    key: "removeMember",
    value: function removeMember(member) {
      member = cloneDeep(member);
      if (!member.uuid) return;
      this.setMemberIdToUuid(member);
      this.members = this.members.filter(function (m) {
        return m.uuid !== member.uuid;
      });
      this.selectionData.remove(member.uuid);
      var attrs = this.selectionData.getAll();
      this.doRangeColoring(attrs);
    }
  }, {
    key: "setCurrentMember",
    value: function setCurrentMember(member) {
      member = cloneDeep(member);
      this.setMemberIdToUuid(member);
      this.setMemberColor(member);
      var findMember = this.members.find(function (m) {
        return m.uuid === member.uuid;
      });
      if (!findMember) return;
      this.currentMember = findMember;
    }
  }, {
    key: "getCurrentMember",
    value: function getCurrentMember() {
      return this.currentMember && !!this.currentMember.id ? this.currentMember : undefined;
    }
  }, {
    key: "doRangeColoring",
    value: function doRangeColoring(attrs) {
      var isDraw = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      var members = this.members,
          currentMember = this.currentMember;
      attrs = attrs.filter(function (item) {
        return item.uuid !== (currentMember === null || currentMember === void 0 ? void 0 : currentMember.uuid);
      });
      this.rangeColoring.render(attrs, members, isDraw);
      this.rangeColoring.updatePosition();
    }
  }, {
    key: "updateSelectionData",
    value: function updateSelectionData() {
      if (!this.engine.change.isComposing()) {
        var range = this.selectionData.updateAll(this.currentMember, this.members).range;
        this.rangeColoring.updateBackgroundAlpha(range);
      }
    }
  }, {
    key: "initSelection",
    value: function initSelection() {
      var data = this.selectionData.updateAll(this.currentMember, this.members).data;
      this.doRangeColoring(data, true);
    }
  }]);

  return OTModel;
}(EventEmitter2);

var Conversion = /*#__PURE__*/function () {
  function Conversion(editor) {
    _classCallCheck(this, Conversion);

    this.editor = void 0;
    this.data = [];
    this.editor = editor;
  }

  _createClass(Conversion, [{
    key: "getData",
    value: function getData() {
      return this.data;
    }
  }, {
    key: "clone",
    value: function clone() {
      var dupData = cloneDeep(this.data);
      var dupConversion = new Conversion(this.editor);
      dupConversion.data = dupData;
      return dupConversion;
    }
  }, {
    key: "add",
    value: function add(from, to) {
      this.data.push({
        from: from,
        to: to
      });
    }
  }, {
    key: "transform",
    value: function transform(node, filter) {
      var name = node.name;
      var attributes = node.attributes();
      var style = node.css(); //删除属性中的style属性

      delete attributes.style; // 光标相关节点

      if (node.isCursor()) {
        name = attributes[DATA_ELEMENT].toLowerCase();
        attributes = {};
        style = {};
      }

      var rule = this.data.find(function (item) {
        if (filter && !filter(item)) return;
        var from = item.from,
            to = item.to;
        var result = false;

        if (typeof from === 'string') {
          result = from === name;
        } else if (typeof from === 'function') {
          result = from(name, style, attributes);
        } else {
          var elementNames = Object.keys(from);
          result = elementNames.indexOf(name) >= 0 && elementNames.some(function (elementName) {
            var elementRules = from[elementName];
            return Object.keys(elementRules.style || {}).every(function (styleName) {
              var styleValue = elementRules.style[styleName];
              return !!style[styleName] && Array.isArray(styleValue) ? styleValue.indexOf(style[styleName]) > -1 : styleValue === style[styleName];
            }) && Object.keys(elementRules.attributes || {}).every(function (attributesName) {
              var attributesValue = elementRules.attributes[attributesName];
              return !!style[attributesName] && Array.isArray(attributesValue) ? attributesValue.indexOf(style[attributesName]) > -1 : attributesValue === style[attributesName];
            });
          });
        }

        if (result) {
          if (typeof to === 'string') {
            name = to;
            style = {};
            attributes = {};
          } else {
            var _node = typeof to === 'function' ? to(name, style, attributes) : to;

            name = _node.name;
            style = _node.css();
            attributes = _node.attributes();
          }
        }

        return result;
      });
      return rule ? {
        rule: rule,
        node: {
          name: name,
          style: style,
          attributes: attributes
        }
      } : undefined;
    }
  }]);

  return Conversion;
}();

var HistoryModel = /*#__PURE__*/function () {
  function HistoryModel(engine) {
    var _this = this;

    _classCallCheck(this, HistoryModel);

    this.totalOps = [];
    this.actionOps = [];
    this.engine = void 0;
    this.currentAction = {};
    this.currentActionIndex = 0;
    this.holded = false;
    this.locked = false;
    this.holdTimer = void 0;
    this.lockTimer = void 0;
    this.lazySaveOp = debounce(function () {
      _this.saveOp();
    }, 200);
    this.engine = engine;
  }

  _createClass(HistoryModel, [{
    key: "reset",
    value: function reset() {
      this.totalOps = [];
      this.actionOps = [];
      this.currentAction = {};
      this.currentActionIndex = 0;
    }
  }, {
    key: "hasUndo",
    value: function hasUndo() {
      return !!this.getUndoOp();
    }
  }, {
    key: "hasRedo",
    value: function hasRedo() {
      return !!this.getRedoOp();
    }
  }, {
    key: "undo",
    value: function undo() {
      this.saveOp();
      var undoOp = this.getUndoOp();

      if (undoOp) {
        var isUndo = false;
        this.engine.ot.stopMutation();

        try {
          this.engine.ot.submitOps(undoOp.ops);
          this.engine.ot.applier.applySelfOperations(undoOp.ops);
          var lastOp = this.totalOps[this.totalOps.length - 1];

          if (lastOp && lastOp.uid === undoOp.uid && lastOp.status !== 'undo') {
            this.totalOps.pop();
          } else {
            this.totalOps.push(undoOp);
          }

          this.currentActionIndex--;
          isUndo = true;
        } catch (error) {
          this.reset();
          console.log(error);
        }

        this.engine.ot.startMutation();

        if (isUndo) {
          this.engine.ot.applier.setRangeByPath(undoOp.startRangePath);
          this.engine.change.change();
        }
      }
    }
  }, {
    key: "redo",
    value: function redo() {
      this.saveOp();
      var redoOp = this.getRedoOp();

      if (redoOp) {
        var isRedo = false;
        this.engine.ot.stopMutation();

        try {
          this.engine.ot.submitOps(redoOp.ops);
          this.engine.ot.applier.applySelfOperations(redoOp.ops);
          var lastOp = this.totalOps[this.totalOps.length - 1];

          if (lastOp && lastOp.uid === redoOp.uid && lastOp.status === 'undo') {
            this.totalOps.pop();
          } else {
            this.totalOps.push(redoOp);
          }

          this.currentActionIndex++;
          isRedo = true;
        } catch (error) {
          this.reset();
          console.log(error);
        }

        this.engine.ot.startMutation();

        if (isRedo) {
          this.engine.ot.applier.setRangeByPath(redoOp.rangePath);
          this.engine.change.change();
        }
      }
    }
    /**
     * 多少毫秒内的动作保持为一个历史片段
     * @param time 毫秒
     */

  }, {
    key: "hold",
    value: function hold() {
      var _this2 = this;

      var time = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 10;
      this.holded = true;
      if (this.holdTimer) clearTimeout(this.holdTimer);
      this.holdTimer = setTimeout(function () {
        return _this2.releaseHold();
      }, time);
    }
    /**
     * 多少毫秒内的动作将不作为历史记录
     * @param time 默认10毫秒
     */

  }, {
    key: "lock",
    value: function lock() {
      var _this3 = this;

      var time = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 10;
      this.locked = true;
      if (this.lockTimer) clearTimeout(this.lockTimer);
      this.lockTimer = setTimeout(function () {
        return _this3.releaseLock();
      }, time);
    }
  }, {
    key: "releaseHold",
    value: function releaseHold() {
      this.holded = false;
    }
  }, {
    key: "releaseLock",
    value: function releaseLock() {
      this.locked = false;
    }
  }, {
    key: "clear",
    value: function clear() {
      var _this4 = this;

      setTimeout(function () {
        _this4.reset();
      }, 10);
    }
    /**
     * 将后续操作暂时缓存，不会同步到协同服务端，不写入历史记录
     */

  }, {
    key: "startCache",
    value: function startCache() {
      this.engine.ot.startMutationCache();
    }
    /**
     * 将暂时缓存的操作提交，同步到协同服务端，写入历史记录
     */

  }, {
    key: "submitCache",
    value: function submitCache() {
      this.engine.ot.submitMutationCache();
    }
    /**
     * 将暂时缓存的操作遗弃
     */

  }, {
    key: "destroyCache",
    value: function destroyCache() {
      this.engine.ot.destroyMutationCache();
    }
  }, {
    key: "saveOp",
    value: function saveOp() {
      if (!this.locked && this.currentAction && this.currentAction.ops && this.currentAction.ops.length > 0) {
        this.totalOps.push(this.currentAction);

        if (this.currentAction.self) {
          this.currentAction.rangePath = this.getCurrentRangePath();
          this.currentAction.uid = random(8);
          this.actionOps.splice(this.currentActionIndex);
          this.actionOps.push(this.currentAction);
          this.currentActionIndex = this.actionOps.length;
          this.engine.change.change();
        }

        this.currentAction = {};
      }
    }
  }, {
    key: "collectSelfOps",
    value: function collectSelfOps(ops) {
      var _this$currentAction,
          _this5 = this;

      if (this.locked) return;
      if (!((_this$currentAction = this.currentAction) === null || _this$currentAction === void 0 ? void 0 : _this$currentAction.self)) this.saveOp();
      var isSave = false;
      ops.forEach(function (op) {
        if (!isCursorOp(op)) {
          var _this5$actionOps$ops;

          isSave = true;
          if (_this5.holded && _this5.actionOps.length > 0) (_this5$actionOps$ops = _this5.actionOps[_this5.actionOps.length - 1].ops) === null || _this5$actionOps$ops === void 0 ? void 0 : _this5$actionOps$ops.push(op);else {
            _this5.currentAction.self = true;
            if (!_this5.currentAction.ops) _this5.currentAction.ops = [];

            if (!_this5.currentAction.startRangePath) {
              _this5.currentAction.startRangePath = _this5.getRangePathBeforeCommand();
            }

            var lastOp = _this5.currentAction.ops[_this5.currentAction.ops.length - 1];

            if (lastOp && isReverseOp(op, lastOp)) {
              _this5.currentAction.ops.pop();
            } else {
              _this5.currentAction.ops.push(op);
            }
          }
        }
      });
      if (isSave) this.lazySaveOp();
    }
  }, {
    key: "collectRemoteOps",
    value: function collectRemoteOps(ops) {
      var _this6 = this;

      if (this.currentAction.self) this.saveOp();
      ops.forEach(function (op) {
        if (!isCursorOp(op)) {
          if (!_this6.currentAction.ops) {
            _this6.currentAction.ops = [];
          }

          var lastOp = _this6.currentAction.ops[_this6.currentAction.ops.length - 1];

          if (lastOp && isReverseOp(op, lastOp)) {
            _this6.currentAction.ops.pop();
          } else {
            _this6.currentAction.ops.push(op);
          }
        }
      });
    }
  }, {
    key: "getUndoOp",
    value: function getUndoOp() {
      var prevIndex = this.currentActionIndex - 1;

      if (this.actionOps[prevIndex]) {
        var prevOp = cloneDeep(this.actionOps[prevIndex]);
        var opIndex = findLastIndex(this.totalOps, function (op) {
          return op.uid == prevOp.uid;
        });
        if (opIndex !== -1) prevOp = this.totalOps[opIndex];else opIndex = prevIndex;
        var invertOps = OTJSON.type.invert(prevOp.ops || []);
        var ops = this.totalOps.slice(opIndex + 1);

        try {
          var transOps = transformOp(invertOps, ops);
          if (transOps.length === 0) return;
          return {
            self: true,
            ops: transOps,
            uid: prevOp.uid,
            status: 'undo',
            rangePath: transformPath(prevOp.rangePath, ops),
            startRangePath: transformPath(prevOp.startRangePath, ops)
          };
        } catch (error) {
          console.log(error);
        }
      }

      return;
    }
  }, {
    key: "getRedoOp",
    value: function getRedoOp() {
      var currentIndex = this.currentActionIndex;

      if (this.actionOps[currentIndex]) {
        var currentOp = cloneDeep(this.actionOps[currentIndex]);
        var opIndex = findLastIndex(this.totalOps, function (op) {
          return op.uid === currentOp.uid;
        });
        if (opIndex !== -1) currentOp = this.totalOps[opIndex];else opIndex = currentIndex;
        var invertOps = [];
        var ops = [];

        if (currentOp.status === 'undo') {
          invertOps = OTJSON.type.invert(currentOp.ops || []);
        } else {
          invertOps = currentOp.ops;
        }

        ops = this.totalOps.slice(opIndex + 1);

        try {
          var transOps = transformOp(invertOps || [], ops);
          if (transOps.length === 0) return;
          return {
            self: true,
            ops: transOps,
            uid: currentOp.uid,
            status: 'redo',
            rangePath: transformPath(currentOp.rangePath, ops),
            startRangePath: transformPath(currentOp.startRangePath, ops)
          };
        } catch (error) {
          console.log(error);
        }
      }

      return;
    }
  }, {
    key: "getCurrentRangePath",
    value: function getCurrentRangePath() {
      var _this$engine = this.engine,
          ot = _this$engine.ot,
          change = _this$engine.change;
      var currentPath = ot.selectionData.currentRangePath;
      return currentPath.length !== 0 ? currentPath : change.getRange().toPath();
    }
  }, {
    key: "getRangePathBeforeCommand",
    value: function getRangePathBeforeCommand() {
      return this.engine.change.getRangePathBeforeCommand() || this.getCurrentRangePath();
    }
  }]);

  return HistoryModel;
}();

var Command = /*#__PURE__*/function () {
  function Command(editor) {
    _classCallCheck(this, Command);

    this.editor = void 0;
    this.editor = editor;
  }

  _createClass(Command, [{
    key: "queryEnabled",
    value: function queryEnabled(name) {
      return !!this.editor.plugin.components[name];
    }
  }, {
    key: "queryState",
    value: function queryState(name) {
      var plugin = this.editor.plugin.components[name];

      if (plugin && plugin.queryState) {
        try {
          for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
            args[_key - 1] = arguments[_key];
          }

          return plugin.queryState(args);
        } catch (error) {
          console.log(error);
        }
      }
    }
  }, {
    key: "execute",
    value: function execute(name) {
      var plugin = this.editor.plugin.components[name];

      if (plugin && plugin.execute) {
        var _this$editor;

        var change;

        if (isEngine(this.editor)) {
          change = this.editor.change;
          change.cacheRangeBeforeCommand();
        }

        for (var _len2 = arguments.length, args = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
          args[_key2 - 1] = arguments[_key2];
        }

        (_this$editor = this.editor).trigger.apply(_this$editor, ['beforeCommandExecute', name].concat(args));

        try {
          var _change, _change2, _this$editor2;

          var result = plugin.execute.apply(plugin, args);
          (_change = change) === null || _change === void 0 ? void 0 : _change.combinTextNode();
          (_change2 = change) === null || _change2 === void 0 ? void 0 : _change2.onSelect();

          (_this$editor2 = this.editor).trigger.apply(_this$editor2, ['afterCommandExecute', name].concat(args));

          return result;
        } catch (error) {
          console.log(error);
        }
      }
    }
  }]);

  return Command;
}();

var Hotkey = /*#__PURE__*/function () {
  function Hotkey(engine) {
    var _this = this;

    _classCallCheck(this, Hotkey);

    this.engine = void 0;
    this.disabled = false;
    this.engine = engine; //绑定事件

    this.engine.container.on('keydown', function (event) {
      return _this.trigger(event);
    });
  }
  /**
   * 处理按键按下事件
   * @param e 事件
   */


  _createClass(Hotkey, [{
    key: "trigger",
    value: function trigger(e) {
      var _this2 = this;

      //禁用快捷键不处理
      if (this.disabled) {
        return;
      } //遍历插件


      Object.keys(this.engine.plugin.components).every(function (name) {
        var plugin = _this2.engine.plugin.components[name]; //插件实现了热键方法

        if (plugin.hotkey) {
          var result = plugin.hotkey(e);
          var isCommand = false;
          var commandArgs = []; //返回热键字符串，并且匹配当前按下的键

          if (typeof result === 'string' && isHotkey(result, e)) {
            isCommand = true;
          } //返回多个热键
          else if (Array.isArray(result)) {
              //遍历热键
              result.some(function (item) {
                if (typeof item === 'string') {
                  if (isHotkey(item, e)) {
                    isCommand = true;
                    commandArgs = [];
                    return true;
                  }
                } else {
                  var key = item.key,
                      args = item.args;

                  if (isHotkey(key, e)) {
                    isCommand = true;
                    commandArgs = Array.isArray(args) ? args : [args];
                    return true;
                  }
                }

                return false;
              });
            } //返回类型是对象，带执行命令参数
            else if (_typeof(result) === 'object' && isHotkey(result.key, e)) {
                isCommand = true; //参数以数组传递

                commandArgs = Array.isArray(result.args) ? result.args : [result.args];
              } //有匹配到热键，执行命令


          if (isCommand) {
            var _this2$engine$command;

            e.preventDefault();

            (_this2$engine$command = _this2.engine.command).execute.apply(_this2$engine$command, [name].concat(_toConsumableArray(commandArgs)));

            return false;
          }
        }

        return true;
      });
    }
  }, {
    key: "enable",
    value: function enable() {
      this.disabled = false;
    }
  }, {
    key: "disable",
    value: function disable() {
      this.disabled = true;
    }
  }, {
    key: "destroy",
    value: function destroy() {
      this.engine.container.off('keydown', this.trigger);
    }
  }]);

  return Hotkey;
}();

var PluginEntry = function PluginEntry(editor, options) {
  _classCallCheck(this, PluginEntry);

  this.editor = void 0;
  this.options = void 0;
  this.kind = 'plugin';
  this.editor = editor;
  this.options = options || {};
};

PluginEntry.pluginName = void 0;

var ElementPluginEntry = /*#__PURE__*/function (_PluginEntry) {
  _inherits(ElementPluginEntry, _PluginEntry);

  var _super = _createSuper(ElementPluginEntry);

  function ElementPluginEntry() {
    var _this;

    _classCallCheck(this, ElementPluginEntry);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));
    _this.sechamCache = void 0;
    _this.tagName = void 0;
    _this.style = void 0;
    _this.attributes = void 0;
    _this.variable = void 0;
    return _this;
  }

  _createClass(ElementPluginEntry, [{
    key: "init",
    value:
    /**
     * 初始化
     */
    function init() {
      var _this$editor = this.editor,
          schema = _this$editor.schema,
          conversion = _this$editor.conversion;
      schema.add(this.schema());

      if (this.conversion) {
        this.conversion().forEach(function (_ref) {
          var from = _ref.from,
              to = _ref.to;
          conversion.add(from, to);
        });
      }
    }
    /**
     * 将当前插件style属性应用到节点
     * @param node 节点
     * @param args style 对应 variable 中的变量参数
     */

  }, {
    key: "setStyle",
    value: function setStyle(node) {
      var _this2 = this;

      for (var _len2 = arguments.length, args = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
        args[_key2 - 1] = arguments[_key2];
      }

      if (isNode(node)) node = $(node);

      if (this.style) {
        Object.keys(this.style).forEach(function (styleName) {
          var _styleValue$match;

          var styleValue = _this2.style[styleName];
          if (_typeof(styleValue) === 'object') styleValue = styleValue.value; //替换变量

          (_styleValue$match = styleValue.match(/@var\d/g)) === null || _styleValue$match === void 0 ? void 0 : _styleValue$match.forEach(function (regMatch) {
            var index = parseInt(regMatch.replace('@var', ''), 10);
            styleValue = styleValue.replace(new RegExp(regMatch, 'gm'), args[index] || '');
          });
          node.css(styleName, styleValue);
        });
      }
    }
    /**
     * 将当前插件attributes属性应用到节点
     * @param node 节点
     * @param args attributes 对应 variable 中的变量参数
     */

  }, {
    key: "setAttributes",
    value: function setAttributes(node) {
      var _this3 = this;

      for (var _len3 = arguments.length, args = new Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
        args[_key3 - 1] = arguments[_key3];
      }

      if (isNode(node)) node = $(node);

      if (this.attributes) {
        Object.keys(this.attributes).forEach(function (attributesName) {
          var _attributesValue$matc;

          var attributesValue = _this3.attributes[attributesName];
          if (_typeof(attributesValue) === 'object') attributesValue = attributesValue.value; //替换变量

          (_attributesValue$matc = attributesValue.match(/@var\d/g)) === null || _attributesValue$matc === void 0 ? void 0 : _attributesValue$matc.forEach(function (regMatch) {
            var index = parseInt(regMatch.replace('@var', ''), 10);
            attributesValue = attributesValue.replace(new RegExp(regMatch, 'gm'), args[index] || '');
          });
          node.attributes(attributesName, attributesValue);
        });
      }
    }
    /**
     * 获取节点符合当前插件规则的样式
     * @param node 节点
     * @returns 样式名称和样式值键值对
     */

  }, {
    key: "getStyle",
    value: function getStyle(node) {
      var _this4 = this;

      if (isNode(node)) node = $(node);
      var values = {};

      if (this.style && this.isSelf(node)) {
        Object.keys(this.style).forEach(function (styleName) {
          node = node;
          var value = styleName.toLowerCase().indexOf('color') > -1 ? toHex(node.css(styleName) || '') : node.css(styleName);
          var styleValue = _this4.style[styleName];

          if (_typeof(styleValue) === 'object') {
            value = styleValue.format(value);
          }

          if (!!value) values[styleName] = value;
        });
      }

      return values;
    }
    /**
     * 获取节点符合当前插件规则的属性
     * @param node 节点
     * @returns 属性名称和属性值键值对
     */

  }, {
    key: "getAttributes",
    value: function getAttributes(node) {
      var _this5 = this;

      if (isNode(node)) node = $(node);
      var values = {};

      if (this.attributes && this.isSelf(node)) {
        Object.keys(this.attributes).forEach(function (attributesName) {
          var value = node.attributes(attributesName);
          var attributesValue = _this5.attributes[attributesName];

          if (_typeof(attributesValue) === 'object') {
            value = attributesValue.format(value);
          }

          if (!!value) values[attributesName] = value;
        });
      }

      return values;
    }
    /**
     * 检测当前节点是否符合当前插件设置的规则
     * @param node 节点
     * @returns true | false
     */

  }, {
    key: "isSelf",
    value: function isSelf(node) {
      if (isNode(node)) node = $(node);
      var schema = this.schema();
      if (Array.isArray(schema)) schema = schema.find(function (_ref2) {
        var name = _ref2.name;
        return name === node.name;
      });
      if (!schema) return false;
      return (Array.isArray(this.tagName) ? this.tagName.indexOf(node.name) > -1 : node.name === this.tagName) && this.editor.schema.checkNode(node, schema.attributes);
    }
    /**
     * 获取插件设置的属性和样式所生成的规则
     */

  }, {
    key: "schema",
    value: function schema() {
      var _this6 = this;

      if (this.sechamCache) return this.sechamCache;
      var attributes = {};

      if (this.attributes) {
        //替换变量
        Object.keys(this.attributes).forEach(function (attributesName) {
          var _attributesValue$matc2;

          var attributesValue = _this6.attributes[attributesName];
          if (_typeof(attributesValue) === 'object') attributesValue = attributesValue.value;
          attributes[attributesName] = attributesValue;
          (_attributesValue$matc2 = attributesValue.match(/@var\d/g)) === null || _attributesValue$matc2 === void 0 ? void 0 : _attributesValue$matc2.forEach(function (regMatch) {
            if (!_this6.variable) throw 'Please specify the variable type';
            attributes[attributesName] = _this6.variable[regMatch];
          });
        });
      }

      if (this.style) {
        //替换变量
        var style = {};
        Object.keys(this.style).forEach(function (styleName) {
          var _styleValue$match2;

          var styleValue = _this6.style[styleName];
          if (_typeof(styleValue) === 'object') styleValue = styleValue.value;
          (_styleValue$match2 = styleValue.match(/@var\d/g)) === null || _styleValue$match2 === void 0 ? void 0 : _styleValue$match2.forEach(function (regMatch) {
            if (!_this6.variable) throw 'Please specify the variable type';
            style[styleName] = _this6.variable[regMatch];
          });
        });
        attributes = _objectSpread(_objectSpread({}, attributes), {}, {
          style: style
        });
      }

      this.sechamCache = {
        type: this.kind,
        attributes: attributes
      };
      if (typeof this.tagName === 'string') this.sechamCache.name = this.tagName.toLowerCase();else if (Array.isArray(this.tagName)) {
        var sechamValue = [];
        this.tagName.forEach(function (name) {
          sechamValue.push(_objectSpread(_objectSpread({}, _this6.sechamCache), {}, {
            name: name
          }));
        });
        this.sechamCache = sechamValue;
      }
      return this.sechamCache;
    }
  }]);

  return ElementPluginEntry;
}(PluginEntry);

var BlockEntry = /*#__PURE__*/function (_ElementPluginEntry) {
  _inherits(BlockEntry, _ElementPluginEntry);

  var _super = _createSuper(BlockEntry);

  function BlockEntry() {
    var _this;

    _classCallCheck(this, BlockEntry);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));
    _this.kind = 'block';
    _this.allowIn = void 0;
    _this.disableMark = void 0;
    _this.canMerge = void 0;
    return _this;
  }

  _createClass(BlockEntry, [{
    key: "schema",
    value: function schema() {
      var _this2 = this;

      var schema = _get(_getPrototypeOf(BlockEntry.prototype), "schema", this).call(this);

      if (Array.isArray(schema)) {
        return schema.map(function (schema) {
          return _objectSpread(_objectSpread({}, schema), {}, {
            allowIn: _this2.allowIn
          });
        });
      }

      return _objectSpread(_objectSpread({}, schema), {}, {
        allowIn: this.allowIn,
        canMerge: this.canMerge
      });
    }
  }]);

  return BlockEntry;
}(ElementPluginEntry);

var InlineEntry = /*#__PURE__*/function (_ElementPluginEntry) {
  _inherits(InlineEntry, _ElementPluginEntry);

  var _super = _createSuper(InlineEntry);

  function InlineEntry() {
    var _this;

    _classCallCheck(this, InlineEntry);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));
    _this.kind = 'inline';
    _this.markdown = void 0;
    return _this;
  }

  _createClass(InlineEntry, [{
    key: "init",
    value: function init() {
      var _this2 = this;

      _get(_getPrototypeOf(InlineEntry.prototype), "init", this).call(this);

      var editor = this.editor;

      if (isEngine(editor) && this.markdown) {
        editor.on('paste:markdown', function (child) {
          return _this2.pasteMarkdown(child);
        });
      }
    }
  }, {
    key: "execute",
    value: function execute() {
      var editor = this.editor;
      if (!isEngine(editor)) return;
      var inlineNode = $("<".concat(this.tagName, " />"));

      for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        args[_key2] = arguments[_key2];
      }

      this.setStyle.apply(this, [inlineNode].concat(args));
      this.setAttributes.apply(this, [inlineNode].concat(args));
      var inline = editor.inline;
      var trigger = this.isTrigger ? this.isTrigger.apply(this, args) : !this.queryState();

      if (trigger) {
        inline.wrap(inlineNode);
      } else {
        inline.unwrap();
      }
    }
  }, {
    key: "queryState",
    value: function queryState() {
      var _this3 = this;

      var editor = this.editor;
      if (!isEngine(editor)) return;
      var change = editor.change; //如果没有属性和样式限制，直接查询是否包含当前标签名称

      if (!this.style && !this.attributes) return change.inlines.some(function (node) {
        return node.name === _this3.tagName;
      }); //获取属性和样式限制内的值集合

      var values = [];
      change.inlines.forEach(function (node) {
        values.push.apply(values, _toConsumableArray(Object.values(_this3.getStyle(node))));
        values.push.apply(values, _toConsumableArray(Object.values(_this3.getAttributes(node))));
      });
      return values.length === 0 ? undefined : values;
    }
    /**
     * 解析markdown
     * @param event 事件
     * @param text markdown文本
     * @param node 触发节点
     */

  }, {
    key: "triggerMarkdown",
    value: function triggerMarkdown(event, text, node) {
      var editor = this.editor;
      if (!isEngine(editor) || !this.markdown) return;
      var change = editor.change,
          command = editor.command;
      var key = this.markdown.replace(/(\*|\^|\$)/g, '\\$1');
      var match = new RegExp("^(.*)".concat(key, "(.+?)").concat(key, "$")).exec(text);

      if (match) {
        var range = change.getRange();
        var visibleChar = match[1] && /\S$/.test(match[1]);
        var codeChar = match[2];
        event.preventDefault();
        var leftText = text.substr(0, text.length - codeChar.length - 2 * this.markdown.length);
        node.get().splitText((leftText + codeChar).length + 2 * this.markdown.length);

        if (visibleChar) {
          leftText += ' ';
        }

        node[0].nodeValue = leftText + codeChar;
        range.setStart(node[0], leftText.length);
        range.setEnd(node[0], (leftText + codeChar).length);
        change.select(range);
        command.execute(this.constructor.pluginName);
        range = change.getRange();
        range.collapse(false);
        var inline = editor.inline.closest(range.startNode);
        var inlineNext = inline.next();

        if (inline && inlineNext && inlineNext.isText() && /^\u200B/g.test(inlineNext.text())) {
          range.setStart(inlineNext, 1);
          range.setEnd(inlineNext, 1);
        }

        change.select(range);
        editor.node.insertText('\xa0');
        return false;
      }

      return;
    }
  }, {
    key: "pasteMarkdown",
    value: function pasteMarkdown(node) {
      if (!isEngine(this.editor) || !this.markdown) return;
      if (!node.isText()) return;
      var text = node.text();
      if (!text) return;
      var key = this.markdown.replace(/(\*|\^|\$)/g, '\\$1');
      var reg = new RegExp("(".concat(key, "([^").concat(key, "\r\n]+)").concat(key, ")"));
      var match = reg.exec(text);
      if (!match) return;
      var newText = '';
      var textNode = node.clone(true).get();

      while (textNode.textContent && (match = reg.exec(textNode.textContent))) {
        var _inlineNode$get;

        //从匹配到的位置切断
        var regNode = textNode.splitText(match.index);
        newText += textNode.textContent; //从匹配结束位置分割

        textNode = regNode.splitText(match[0].length); //获取中间字符

        var inlineNode = $("<".concat(this.tagName, ">").concat(match[2], "</").concat(this.tagName, ">"));
        this.setStyle(inlineNode);
        this.setAttributes(inlineNode);
        newText += (_inlineNode$get = inlineNode.get()) === null || _inlineNode$get === void 0 ? void 0 : _inlineNode$get.outerHTML;
      }

      newText += textNode.textContent;
      node.text(newText);
    }
  }]);

  return InlineEntry;
}(ElementPluginEntry);

var css_248z$4 = ".am-engine ol, .am-engine-view ol, .am-engine ul, .am-engine-view ul {\r\n    margin: 0 0 0 3px;\r\n    padding: 0;\r\n    list-style: none;\r\n}\r\n\r\n.am-engine ol ul,.am-engine-view ol ul,.am-engine ul ul,.am-engine-view ul ul,.am-engine ol ol,.am-engine-view ol ol,.am-engine ul ol,.am-engine-view ul ol {\r\n\tmargin-left: 0;\r\n}\r\n\r\n.am-engine ol ul li,.am-engine-view ol ul li,.am-engine ul ul li,.am-engine-view ul ul li,.am-engine ol ol li,.am-engine-view ol ol li,.am-engine ul ol li,.am-engine-view ul ol li {\r\n\tmargin-left: 2em;\r\n}\r\n\r\n.am-engine ol ol[data-indent-new=\"0\"],.am-engine-view ol ol[data-indent-new=\"0\"],.am-engine ul ol[data-indent-new=\"0\"],.am-engine-view ul ol[data-indent-new=\"0\"],.am-engine ol ol[data-indent-new=\"3\"],.am-engine-view ol ol[data-indent-new=\"3\"],.am-engine ul ol[data-indent-new=\"3\"],.am-engine-view ul ol[data-indent-new=\"3\"],.am-engine ol ol[data-indent-new=\"6\"],.am-engine-view ol ol[data-indent-new=\"6\"],.am-engine ul ol[data-indent-new=\"6\"],.am-engine-view ul ol[data-indent-new=\"6\"] {\r\n\tlist-style-type: decimal;\r\n}\r\n\r\n.am-engine ol ol[data-indent-new=\"1\"],.am-engine-view ol ol[data-indent-new=\"1\"],.am-engine ul ol[data-indent-new=\"1\"],.am-engine-view ul ol[data-indent-new=\"1\"],.am-engine ol ol[data-indent-new=\"4\"],.am-engine-view ol ol[data-indent-new=\"4\"],.am-engine ul ol[data-indent-new=\"4\"],.am-engine-view ul ol[data-indent-new=\"4\"],.am-engine ol ol[data-indent-new=\"7\"],.am-engine-view ol ol[data-indent-new=\"7\"],.am-engine ul ol[data-indent-new=\"7\"],.am-engine-view ul ol[data-indent-new=\"7\"] {\r\n\tlist-style-type: lower-alpha;\r\n}\r\n\r\n.am-engine ol ol[data-indent-new=\"2\"],.am-engine-view ol ol[data-indent-new=\"2\"],.am-engine ul ol[data-indent-new=\"2\"],.am-engine-view ul ol[data-indent-new=\"2\"],.am-engine ol ol[data-indent-new=\"5\"],.am-engine-view ol ol[data-indent-new=\"5\"],.am-engine ul ol[data-indent-new=\"5\"],.am-engine-view ul ol[data-indent-new=\"5\"],.am-engine ol ol[data-indent-new=\"8\"],.am-engine-view ol ol[data-indent-new=\"8\"],.am-engine ul ol[data-indent-new=\"8\"],.am-engine-view ul ol[data-indent-new=\"8\"] {\r\n\tlist-style-type: lower-roman;\r\n}\r\n\r\n.am-engine ol ul[data-indent-new=\"3\"],.am-engine-view ol ul[data-indent-new=\"3\"],.am-engine ul ul[data-indent-new=\"3\"],.am-engine-view ul ul[data-indent-new=\"3\"],.am-engine ol ul[data-indent-new=\"6\"],.am-engine-view ol ul[data-indent-new=\"6\"],.am-engine ul ul[data-indent-new=\"6\"],.am-engine-view ul ul[data-indent-new=\"6\"] {\r\n\tlist-style-type: disc;\r\n}\r\n\r\n.am-engine ol ul[data-indent-new=\"1\"],.am-engine-view ol ul[data-indent-new=\"1\"],.am-engine ul ul[data-indent-new=\"1\"],.am-engine-view ul ul[data-indent-new=\"1\"],.am-engine ol ul[data-indent-new=\"4\"],.am-engine-view ol ul[data-indent-new=\"4\"],.am-engine ul ul[data-indent-new=\"4\"],.am-engine-view ul ul[data-indent-new=\"4\"],.am-engine ol ul[data-indent-new=\"7\"],.am-engine-view ol ul[data-indent-new=\"7\"],.am-engine ul ul[data-indent-new=\"7\"],.am-engine-view ul ul[data-indent-new=\"7\"] {\r\n\tlist-style-type: circle;\r\n}\r\n\r\n.am-engine ol ul[data-indent-new=\"2\"],.am-engine-view ol ul[data-indent-new=\"2\"],.am-engine ul ul[data-indent-new=\"2\"],.am-engine-view ul ul[data-indent-new=\"2\"],.am-engine ol ul[data-indent-new=\"5\"],.am-engine-view ol ul[data-indent-new=\"5\"],.am-engine ul ul[data-indent-new=\"5\"],.am-engine-view ul ul[data-indent-new=\"5\"],.am-engine ol ul[data-indent-new=\"8\"],.am-engine-view ol ul[data-indent-new=\"8\"],.am-engine ul ul[data-indent-new=\"8\"],.am-engine-view ul ul[data-indent-new=\"8\"] {\r\n\tlist-style-type: square;\r\n}\r\n\r\n.am-engine li,.am-engine-view li {\r\n\tmargin-left: 23px;\r\n}\r\n\r\n.am-engine ol,.am-engine-view ol,.am-engine ol[data-indent=\"3\"],.am-engine-view ol[data-indent=\"3\"],.am-engine ol[data-indent=\"6\"],.am-engine-view ol[data-indent=\"6\"] {\r\n\tlist-style-type: decimal;\r\n}\r\n\r\n.am-engine ol[data-indent=\"1\"],.am-engine-view ol[data-indent=\"1\"],.am-engine ol[data-indent=\"4\"],.am-engine-view ol[data-indent=\"4\"],.am-engine ol[data-indent=\"7\"],.am-engine-view ol[data-indent=\"7\"] {\r\n\tlist-style-type: lower-alpha;\r\n}\r\n\r\n.am-engine ol[data-indent=\"2\"],.am-engine-view ol[data-indent=\"2\"],.am-engine ol[data-indent=\"5\"],.am-engine-view ol[data-indent=\"5\"],.am-engine ol[data-indent=\"8\"],.am-engine-view ol[data-indent=\"8\"] {\r\n\tlist-style-type: lower-roman;\r\n}\r\n\r\n.am-engine ul,.am-engine-view ul,.am-engine ul[data-indent=\"3\"],.am-engine-view ul[data-indent=\"3\"],.am-engine ul[data-indent=\"6\"],.am-engine-view ul[data-indent=\"6\"] {\r\n\tlist-style-type: disc;\r\n}\r\n\r\n.am-engine ul[data-indent=\"1\"],.am-engine-view ul[data-indent=\"1\"],.am-engine ul[data-indent=\"4\"],.am-engine-view ul[data-indent=\"4\"],.am-engine ul[data-indent=\"7\"],.am-engine-view ul[data-indent=\"7\"] {\r\n\tlist-style-type: circle;\r\n}\r\n\r\n.am-engine ul[data-indent=\"2\"],.am-engine-view ul[data-indent=\"2\"],.am-engine ul[data-indent=\"5\"],.am-engine-view ul[data-indent=\"5\"],.am-engine ul[data-indent=\"8\"],.am-engine-view ul[data-indent=\"8\"] {\r\n\tlist-style-type: square;\r\n}\r\n\r\n.am-engine ol[data-indent=\"1\"],.am-engine-view ol[data-indent=\"1\"],.am-engine ul[data-indent=\"1\"],.am-engine-view ul[data-indent=\"1\"] {\r\n\tpadding-left: 2em;\r\n}\r\n\r\n.am-engine ol[data-indent=\"2\"],.am-engine-view ol[data-indent=\"2\"],.am-engine ul[data-indent=\"2\"],.am-engine-view ul[data-indent=\"2\"] {\r\n\tpadding-left: 4em;\r\n}\r\n\r\n.am-engine ol[data-indent=\"3\"],.am-engine-view ol[data-indent=\"3\"],.am-engine ul[data-indent=\"3\"],.am-engine-view ul[data-indent=\"3\"] {\r\n\tpadding-left: 6em;\r\n}\r\n\r\n.am-engine ol[data-indent=\"4\"],.am-engine-view ol[data-indent=\"4\"],.am-engine ul[data-indent=\"4\"],.am-engine-view ul[data-indent=\"4\"] {\r\n\tpadding-left: 8em;\r\n}\r\n\r\n.am-engine ol[data-indent=\"5\"],.am-engine-view ol[data-indent=\"5\"],.am-engine ul[data-indent=\"5\"],.am-engine-view ul[data-indent=\"5\"] {\r\n\tpadding-left: 10em;\r\n}\r\n\r\n.am-engine ol[data-indent=\"6\"],.am-engine-view ol[data-indent=\"6\"],.am-engine ul[data-indent=\"6\"],.am-engine-view ul[data-indent=\"6\"] {\r\n\tpadding-left: 12em;\r\n}\r\n\r\n.am-engine ol[data-indent=\"7\"],.am-engine-view ol[data-indent=\"7\"],.am-engine ul[data-indent=\"7\"],.am-engine-view ul[data-indent=\"7\"] {\r\n\tpadding-left: 14em;\r\n}\r\n\r\n.am-engine ol[data-indent=\"8\"],.am-engine-view ol[data-indent=\"8\"],.am-engine ul[data-indent=\"8\"],.am-engine-view ul[data-indent=\"8\"] {\r\n\tpadding-left: 16em;\r\n}\r\n\r\n.am-engine .data-list,.am-engine-view .data-list {\r\n\tcolor: #262626;\r\n\ttext-indent: 0;\r\n}\r\n\r\n.am-engine .data-list-item,.am-engine-view .data-list-item {\r\n    line-height: inherit;\r\n    position: relative;\r\n    list-style: none;\r\n    text-indent: 0;\r\n}";
styleInject(css_248z$4);

var ListEntry = /*#__PURE__*/function (_BlockEntry) {
  _inherits(ListEntry, _BlockEntry);

  var _super = _createSuper(ListEntry);

  function ListEntry() {
    var _this;

    _classCallCheck(this, ListEntry);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));
    _this.cardName = void 0;
    _this.isPasteList = false;
    return _this;
  }

  _createClass(ListEntry, [{
    key: "init",
    value: function init() {
      var _this2 = this;

      _get(_getPrototypeOf(ListEntry.prototype), "init", this).call(this);

      var editor = this.editor;

      if (isEngine(editor)) {
        editor.on('paste:before', function (fragment) {
          return _this2.pasteBefore(fragment);
        });
        editor.on('paste:insert', function () {
          return _this2.pasteInsert();
        });
        editor.on('paste:after', function () {
          return _this2.pasteAfter();
        });
      }
    }
  }, {
    key: "queryState",
    value: function queryState() {
      var editor = this.editor;
      if (!isEngine(editor)) return false;
      return editor.list.getPluginNameByNodes(editor.change.blocks) === this.constructor.pluginName;
    }
  }, {
    key: "pasteBefore",
    value: function pasteBefore(documentFragment) {
      if (!this.cardName || !this.editor) return;
      var list = this.editor.list;
      var node = $(documentFragment);
      var children = node.allChildren();
      children.forEach(function (child) {
        var domChild = $(child);

        if (domChild.name === 'li' && domChild.hasClass(list.CUSTOMZIE_LI_CLASS)) {
          var _domChild$first;

          //自定义列表，没有卡片节点，就作为普通列表
          if (!((_domChild$first = domChild.first()) === null || _domChild$first === void 0 ? void 0 : _domChild$first.isCard())) {
            domChild.removeClass(list.CUSTOMZIE_LI_CLASS);
            domChild.closest('ul').removeClass(list.CUSTOMZIE_UL_CLASS);
            return;
          } else {
            domChild.closest('ul').addClass(list.CUSTOMZIE_UL_CLASS);
          }
        }
      });
      this.isPasteList = children.some(function (child) {
        return child.nodeName.toLowerCase() === 'li';
      });
    }
  }, {
    key: "pasteInsert",
    value: function pasteInsert() {
      var _this3 = this;

      if (!this.cardName || !isEngine(this.editor)) return;
      var _this$editor = this.editor,
          change = _this$editor.change,
          list = _this$editor.list;
      var range = change.getRange();
      var rootBlock = range.getRootBlock();
      var nextBlock = rootBlock === null || rootBlock === void 0 ? void 0 : rootBlock.next();
      var customizeItems = nextBlock === null || nextBlock === void 0 ? void 0 : nextBlock.find("li.".concat(list.CUSTOMZIE_LI_CLASS));

      if (customizeItems && customizeItems.length > 0) {
        customizeItems.each(function (node) {
          var domNode = $(node);
          if (0 === domNode.find("[".concat(CARD_KEY, "=").concat(_this3.cardName, "],[").concat(READY_CARD_KEY, "=").concat(_this3.cardName, "]")).length) list.addReadyCardToCustomize(domNode, _this3.cardName);
        });
      }
    }
  }, {
    key: "pasteAfter",
    value: function pasteAfter() {
      if (this.isPasteList) {
        var _this$editor2;

        (_this$editor2 = this.editor) === null || _this$editor2 === void 0 ? void 0 : _this$editor2.list.merge();
      }
    }
  }]);

  return ListEntry;
}(BlockEntry);

var MarkEntry = /*#__PURE__*/function (_ElementPluginEntry) {
  _inherits(MarkEntry, _ElementPluginEntry);

  var _super = _createSuper(MarkEntry);

  function MarkEntry() {
    var _this;

    _classCallCheck(this, MarkEntry);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));
    _this.kind = 'mark';
    _this.markdown = void 0;
    _this.copyOnEnter = void 0;
    _this.followStyle = true;
    _this.combineValueByWrap = false;
    return _this;
  }

  _createClass(MarkEntry, [{
    key: "init",
    value: function init() {
      var _this2 = this;

      _get(_getPrototypeOf(MarkEntry.prototype), "init", this).call(this);

      var editor = this.editor;

      if (isEngine(editor) && this.markdown) {
        editor.on('paste:markdown', function (node) {
          return _this2.pasteMarkdown(node);
        });
      }
    }
  }, {
    key: "execute",
    value: function execute() {
      var editor = this.editor;
      if (!isEngine(editor)) return;
      var change = editor.change,
          mark = editor.mark;
      var markNode = $("<".concat(this.tagName, " />"));

      for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        args[_key2] = arguments[_key2];
      }

      this.setStyle.apply(this, [markNode].concat(args));
      this.setAttributes.apply(this, [markNode].concat(args));
      var trigger = this.isTrigger ? this.isTrigger.apply(this, args) : !this.queryState();

      if (trigger) {
        if (!this.followStyle && change.getRange().collapsed) {
          return;
        }

        mark.wrap(markNode);
      } else {
        mark.unwrap(markNode);
      }
    }
  }, {
    key: "queryState",
    value: function queryState() {
      var _this3 = this;

      if (!isEngine(this.editor)) return;
      var change = this.editor.change; //如果没有属性和样式限制，直接查询是否包含当前标签名称

      if (!this.style && !this.attributes) return change.marks.some(function (node) {
        return node.name === _this3.tagName;
      }); //获取属性和样式限制内的值集合

      var values = [];
      change.marks.forEach(function (node) {
        values.push.apply(values, _toConsumableArray(Object.values(_this3.getStyle(node))));
        values.push.apply(values, _toConsumableArray(Object.values(_this3.getAttributes(node))));
      });
      return values.length === 0 ? undefined : values;
    }
  }, {
    key: "schema",
    value: function schema() {
      var schema = _get(_getPrototypeOf(MarkEntry.prototype), "schema", this).call(this);

      if (Array.isArray(schema)) {
        return schema.map(function (schema) {
          return _objectSpread({}, schema);
        });
      }

      return _objectSpread({}, schema);
    }
    /**
     * 解析markdown
     * @param event 事件
     * @param text markdown文本
     * @param node 触发节点
     */

  }, {
    key: "triggerMarkdown",
    value: function triggerMarkdown(event, text, node) {
      var editor = this.editor;
      if (!isEngine(editor) || !this.markdown) return;
      var block = editor.block,
          change = editor.change,
          command = editor.command;
      var key = this.markdown.replace(/(\*|\^|\$)/g, '\\$1');
      var match = new RegExp("^(.*)".concat(key, "(.+?)").concat(key, "$")).exec(text);

      if (match) {
        //限制block下某些禁用的mark插件
        var blockPlugin = block.findPlugin(node);
        var pluginName = this.constructor.pluginName;
        if (blockPlugin && blockPlugin.disableMark && blockPlugin.disableMark.indexOf(pluginName) > -1) return;
        var range = change.getRange();
        var visibleChar = match[1] && /\S$/.test(match[1]);
        var codeChar = match[2];
        event.preventDefault();
        var leftText = text.substr(0, text.length - codeChar.length - 2 * this.markdown.length);
        node.get().splitText((leftText + codeChar).length + 2 * this.markdown.length);

        if (visibleChar) {
          leftText += ' ';
        }

        node[0].nodeValue = leftText + codeChar;
        range.setStart(node[0], leftText.length);
        range.setEnd(node[0], (leftText + codeChar).length);
        change.select(range);
        command.execute(this.constructor.pluginName);
        range = change.getRange();
        range.collapse(false);
        change.select(range);
        editor.node.insertText('\xa0');
        return false;
      }

      return;
    }
  }, {
    key: "pasteMarkdown",
    value: function pasteMarkdown(node) {
      if (!isEngine(this.editor) || !this.markdown) return;
      if (!node.isText()) return;
      var text = node.text();
      if (!text) return;
      var key = this.markdown.replace(/(\*|\^|\$)/g, '\\$1');
      var reg = key === '_' ? new RegExp("\\s+(".concat(key, "([^").concat(key, "\\r\\n]+)").concat(key, ")\\s+")) : new RegExp("(".concat(key, "([^").concat(key, "\\r\\n]+)").concat(key, ")"));
      var match = reg.exec(text);
      if (!match) return;
      var newText = '';
      var textNode = node.clone(true).get();

      while (textNode.textContent && (match = reg.exec(textNode.textContent))) {
        var _markNode$get;

        //从匹配到的位置切断
        var regNode = textNode.splitText(match.index);
        newText += textNode.textContent; //从匹配结束位置分割

        textNode = regNode.splitText(match[0].length); //获取中间字符

        var markNode = $("<".concat(this.tagName, ">").concat(match[2], "</").concat(this.tagName, ">"));
        this.setStyle(markNode);
        this.setAttributes(markNode);
        newText += (_markNode$get = markNode.get()) === null || _markNode$get === void 0 ? void 0 : _markNode$get.outerHTML;
      }

      newText += textNode.textContent;
      node.text(newText);
    }
  }]);

  return MarkEntry;
}(ElementPluginEntry);

var PluginModel = /*#__PURE__*/function () {
  function PluginModel(editor) {
    _classCallCheck(this, PluginModel);

    this.data = {};
    this.components = {};
    this.editor = void 0;
    this.editor = editor;
  }

  _createClass(PluginModel, [{
    key: "init",
    value: function init(plugins, config) {
      var _this = this;

      plugins.forEach(function (pluginClazz) {
        _this.data[pluginClazz.pluginName] = pluginClazz;
        var plugin = new pluginClazz(_this.editor, config[pluginClazz.pluginName]);
        _this.components[pluginClazz.pluginName] = plugin;
        if (plugin.init) plugin.init();
      });
    }
  }, {
    key: "add",
    value: function add(clazz, options) {
      this.data[clazz.pluginName] = clazz;
      options = _objectSpread(_objectSpread({}, options), {}, {
        editor: this.editor
      });

      if (isEngine(this.editor)) {
        var plugin = new clazz(this.editor, options);
        if (plugin.init) plugin.init();
        this.components[clazz.pluginName] = plugin;
      }
    }
  }, {
    key: "each",
    value: function each(callback) {
      var _this2 = this;

      Object.keys(this.data).forEach(function (name, index) {
        if (callback && callback(name, _this2.data[name], index) === false) return;
      });
    }
  }]);

  return PluginModel;
}();

var Enter = /*#__PURE__*/function () {
  function Enter(engine) {
    _classCallCheck(this, Enter);

    this.engine = void 0;
    this.engine = engine;
  }
  /**
   * 在卡片处插入空段落
   * @param range 光标
   * @param card 卡片
   * @param isStart 是否聚焦到开始位置
   */


  _createClass(Enter, [{
    key: "insertNewline",
    value: function insertNewline(range, card, isStart) {
      var _this$engine = this.engine,
          change = _this$engine.change,
          block = _this$engine.block;
      range.select(card.root);
      range.collapse(isStart);
      change.select(range);
      var emptyBlock = $('<p><br /></p>');
      block.insert(emptyBlock);

      if (isStart) {
        card.focus(range, true);
      } else {
        range.select(emptyBlock, true);
        range.collapse(false);
      }

      change.select(range);
    }
    /**
     * 在卡片节点处按下enter键
     */

  }, {
    key: "trigger",
    value: function trigger(event) {
      var change = this.engine.change;
      var range = change.getRange(); // 查找当前光标所在卡片

      var card = this.engine.card.find(range.startNode);
      if (!card) return true;

      if (card.constructor.cardType === CardType.INLINE) {
        // 左侧光标
        var cardLeft = range.startNode.closest(CARD_LEFT_SELECTOR);

        if (cardLeft.length > 0) {
          range.select(card.root);
          range.collapse(true);
          change.select(range);
        } // 右侧光标


        var cardRight = range.startNode.closest(CARD_RIGHT_SELECTOR);

        if (cardRight.length > 0) {
          range.select(card.root);
          range.collapse(false);
          change.select(range);
        }
      } else {
        // 左侧光标
        var _cardLeft = range.startNode.closest(CARD_LEFT_SELECTOR);

        if (_cardLeft.length > 0) {
          event.preventDefault();
          var prev = card.root.prev();

          if (!prev || prev.isCard()) {
            card.focusPrevBlock(range, true);
            change.select(range);
          } else {
            this.insertNewline(range, card, true);
          }

          return false;
        } // 右侧光标


        var _cardRight = range.startNode.closest(CARD_RIGHT_SELECTOR);

        if (_cardRight.length > 0) {
          event.preventDefault();
          var next = card.root.next();

          if (!next || next.isCard()) {
            card.focusNextBlock(range, true);
            change.select(range);
          } else {
            this.insertNewline(range, card, false);
          }

          return false;
        }
      }

      return true;
    }
  }]);

  return Enter;
}();

var Backspace = /*#__PURE__*/function () {
  function Backspace(engine) {
    _classCallCheck(this, Backspace);

    this.engine = void 0;
    this.engine = engine;
  }
  /**
   * 在卡片节点处按下backspace键
   */


  _createClass(Backspace, [{
    key: "trigger",
    value: function trigger(event) {
      var change = this.engine.change;
      var range = change.getRange();
      if (!range.collapsed) return; // 查找当前光标所在卡片

      var card = this.engine.card.find(range.startNode);

      if (!card) {
        // 光标前面有Card，并且不是自定义列表，移除卡片
        var prevNode = range.getPrevNode();

        if (!event['isDelete'] && prevNode && prevNode.isCard() && !this.engine.node.isCustomize(prevNode)) {
          event.preventDefault();
          this.engine.card.remove(prevNode);
          return false;
        }

        return true;
      } // inline 卡片


      if (card.constructor.cardType === CardType.INLINE) {
        // 左侧光标
        var cardLeft = range.startNode.closest(CARD_LEFT_SELECTOR);

        if (cardLeft.length > 0) {
          range.select(card.root).collapse(true);
          change.select(range);
        } // 右侧光标


        var cardRight = range.startNode.closest(CARD_RIGHT_SELECTOR);

        if (cardRight.length > 0) {
          event.preventDefault();
          this.engine.card.remove(card.id);
          range.addOrRemoveBr();
          return false;
        }
      } else {
        // 左侧光标
        var _cardLeft = range.startNode.closest(CARD_LEFT_SELECTOR);

        if (_cardLeft.length > 0) {
          var _card$root$parent;

          event.preventDefault();

          if ((_card$root$parent = card.root.parent()) === null || _card$root$parent === void 0 ? void 0 : _card$root$parent.inEditor()) {
            change.unwrapNode(card.root.parent());
          } else {
            change.focusPrevBlock(card.root, true);
          }

          return false;
        } // 右侧光标


        var _cardRight = range.startNode.closest(CARD_RIGHT_SELECTOR);

        if (_cardRight.length > 0) {
          event.preventDefault();
          change.focusPrevBlock(card.root);
          this.engine.card.remove(card.id);
          return false;
        }
      } //改变了光标选区，再次触发事件


      if (this.engine.trigger('keydown:backspace') === false) return false;
      return true;
    }
  }]);

  return Backspace;
}();

var Left = /*#__PURE__*/function () {
  function Left(engine) {
    _classCallCheck(this, Left);

    this.engine = void 0;
    this.engine = engine;
  }

  _createClass(Left, [{
    key: "inline",
    value: function inline(card, event) {
      var change = this.engine.change;
      var range = change.getRange();
      var singleSelectable = card.constructor.singleSelectable; // 左侧光标

      var cardLeft = range.commonAncestorNode.closest(CARD_LEFT_SELECTOR);

      if (cardLeft.length > 0) {
        range.setStartBefore(card.root[0]);
        range.collapse(true);
        change.select(range);
        return true;
      } // 右侧光标


      var cardRight = range.commonAncestorNode.closest(CARD_RIGHT_SELECTOR);

      if (cardRight.length > 0) {
        event.preventDefault();

        if (singleSelectable !== false) {
          this.engine.card.select(card);
        } else {
          card.focus(range, true);
          change.select(range);
        }

        return false;
      }

      return true;
    }
  }, {
    key: "block",
    value: function block(card, event) {
      var change = this.engine.change;
      var range = change.getRange(); // 左侧光标

      var cardLeft = range.commonAncestorNode.closest(CARD_LEFT_SELECTOR);

      if (cardLeft.length > 0) {
        event.preventDefault();
        card.focusPrevBlock(range, false);
        change.select(range);
        return false;
      } // 右侧光标


      var cardRight = range.commonAncestorNode.closest(CARD_RIGHT_SELECTOR);

      if (cardRight.length > 0) {
        event.preventDefault();
        this.engine.card.select(card);
        return false;
      }

      if (this.engine.card.getSingleSelectedCard(range)) {
        event.preventDefault();
        card.focus(range, true);
        change.select(range);
        return false;
      }

      return true;
    }
  }, {
    key: "trigger",
    value: function trigger(event) {
      var change = this.engine.change;
      var range = change.getRange();
      var card = this.engine.card.getSingleCard(range);
      if (!card) return true;
      var cardEntry = card.constructor;
      return cardEntry.cardType === CardType.INLINE ? this.inline(card, event) : this.block(card, event);
    }
  }]);

  return Left;
}();

var Right = /*#__PURE__*/function () {
  function Right(engine) {
    _classCallCheck(this, Right);

    this.engine = void 0;
    this.engine = engine;
  }

  _createClass(Right, [{
    key: "inline",
    value: function inline(card, event) {
      var change = this.engine.change;
      var range = change.getRange();
      var singleSelectable = card.constructor.singleSelectable; // 左侧光标

      var cardLeft = range.commonAncestorNode.closest(CARD_LEFT_SELECTOR);

      if (cardLeft.length > 0) {
        event.preventDefault();

        if (singleSelectable !== false) {
          this.engine.card.select(card);
        } else {
          card.focus(range, false);
          change.select(range);
        }

        return false;
      } // 右侧光标


      var cardRight = range.commonAncestorNode.closest(CARD_RIGHT_SELECTOR);

      if (cardRight.length > 0) {
        var cardNext = card.root.next();

        if (cardNext && cardNext.isCard()) {
          event.preventDefault();
          this.engine.card.select(this.engine.card.find(cardNext));
          return false;
        } else {
          range.setEndAfter(card.root[0]);
          range.collapse(false);
          change.select(range);
        }
      }

      return true;
    }
  }, {
    key: "block",
    value: function block(card, event) {
      var change = this.engine.change;
      var range = change.getRange(); // 左侧光标

      var cardLeft = range.commonAncestorNode.closest(CARD_LEFT_SELECTOR);

      if (cardLeft.length > 0) {
        event.preventDefault();
        this.engine.card.select(card);
        return false;
      } // 右侧光标


      var cardRight = range.commonAncestorNode.closest(CARD_RIGHT_SELECTOR);

      if (cardRight.length > 0) {
        event.preventDefault();
        card.focusNextBlock(range, false);
        change.select(range);
        return false;
      }

      if (this.engine.card.getSingleSelectedCard(range)) {
        event.preventDefault();
        card.focus(range, false);
        change.select(range);
        return false;
      }

      return true;
    }
  }, {
    key: "trigger",
    value: function trigger(event) {
      var change = this.engine.change;
      var range = change.getRange();
      var card = this.engine.card.getSingleCard(range);
      if (!card) return true;
      var cardEntry = card.constructor;
      return cardEntry.cardType === CardType.INLINE ? this.inline(card, event) : this.block(card, event);
    }
  }]);

  return Right;
}();

var Up = /*#__PURE__*/function () {
  function Up(engine) {
    _classCallCheck(this, Up);

    this.engine = void 0;
    this.engine = engine;
  }

  _createClass(Up, [{
    key: "inline",
    value: function inline(card, event) {
      var change = this.engine.change;
      var range = change.getRange();
      event.preventDefault();
      card.focusPrevBlock(range, false);
      change.select(range);
      return false;
    }
  }, {
    key: "block",
    value: function block(card, event) {
      var change = this.engine.change;
      var range = change.getRange();
      event.preventDefault();
      card.focusPrevBlock(range, false);
      change.select(range);
      return false;
    }
  }, {
    key: "trigger",
    value: function trigger(event) {
      var change = this.engine.change;
      var range = change.getRange();
      var card = this.engine.card.getSingleCard(range);
      if (!card) return true;
      var cardEntry = card.constructor;
      return cardEntry.cardType === CardType.INLINE ? this.inline(card, event) : this.block(card, event);
    }
  }]);

  return Up;
}();

var Down = /*#__PURE__*/function () {
  function Down(engine) {
    _classCallCheck(this, Down);

    this.engine = void 0;
    this.engine = engine;
  }

  _createClass(Down, [{
    key: "inline",
    value: function inline(card, event) {
      var change = this.engine.change;
      var range = change.getRange();
      event.preventDefault();
      card.focusNextBlock(range, false);
      change.select(range);
      return false;
    }
  }, {
    key: "block",
    value: function block(card, event) {
      var change = this.engine.change;
      var range = change.getRange();
      event.preventDefault();
      card.focusNextBlock(range, false);
      change.select(range);
      return false;
    }
  }, {
    key: "trigger",
    value: function trigger(event) {
      var change = this.engine.change;
      var range = change.getRange();
      var card = this.engine.card.getSingleCard(range);
      if (!card) return true;
      var cardEntry = card.constructor;
      return cardEntry.cardType === CardType.INLINE ? this.inline(card, event) : this.block(card, event);
    }
  }]);

  return Down;
}();

var Default = /*#__PURE__*/function () {
  function Default(engine) {
    _classCallCheck(this, Default);

    this.engine = void 0;
    this.engine = engine;
  }

  _createClass(Default, [{
    key: "block",
    value: function block(card, event) {
      var change = this.engine.change;
      var range = change.getRange(); // 左侧光标

      var cardLeft = range.commonAncestorNode.closest(CARD_LEFT_SELECTOR);

      if (cardLeft.length > 0) {
        // 其它情况
        if (!event.metaKey && !event.ctrlKey) {
          card.focusPrevBlock(range, true);
          change.select(range);
        }

        return true;
      } // 右侧光标


      var cardRight = range.commonAncestorNode.closest(CARD_RIGHT_SELECTOR);

      if (cardRight.length > 0) {
        // 其它情况
        if (!event.metaKey && !event.ctrlKey) {
          card.focusNextBlock(range, true);
          change.select(range);
        }
      }

      return true;
    }
  }, {
    key: "trigger",
    value: function trigger(event) {
      var change = this.engine.change;
      var range = change.getRange();
      var card = this.engine.card.getSingleCard(range);
      if (!card) return true;
      var cardEntry = card.constructor;
      if (cardEntry.cardType === CardType.BLOCK) return this.block(card, event);
      return true;
    }
  }]);

  return Default;
}();

var css_248z$5 = ".am-engine .card-selected [data-card-element=\"center\"].data-card-background-selected {\r\n    background: rgba(27, 162, 227, 0.2);\r\n}\r\n\r\n.am-engine .card-selected [data-card-element=\"center\"].data-card-border-selected {\r\n    outline: 2px solid #1890FF;\r\n}";
styleInject(css_248z$5);

var CardModel = /*#__PURE__*/function () {
  function CardModel(editor) {
    _classCallCheck(this, CardModel);

    this.classes = void 0;
    this.components = void 0;
    this.editor = void 0;
    this.classes = {};
    this.components = [];
    this.editor = editor;
  }

  _createClass(CardModel, [{
    key: "active",
    get: function get() {
      return this.components.find(function (component) {
        return component.activated;
      });
    }
  }, {
    key: "length",
    get: function get() {
      return this.components.length;
    }
  }, {
    key: "init",
    value: function init(cards) {
      var _this = this;

      var editor = this.editor;

      if (isEngine(editor)) {
        var _typing$getHandleList, _typing$getHandleList2, _typing$getHandleList3, _typing$getHandleList4, _typing$getHandleList5, _typing$getHandleList6, _typing$getHandleList7;

        var typing = editor.typing; //绑定回车事件

        var enter = new Enter(editor);
        (_typing$getHandleList = typing.getHandleListener('enter', 'keydown')) === null || _typing$getHandleList === void 0 ? void 0 : _typing$getHandleList.on(function (event) {
          return enter.trigger(event);
        }); //删除事件

        var backspace = new Backspace(editor);
        (_typing$getHandleList2 = typing.getHandleListener('backspace', 'keydown')) === null || _typing$getHandleList2 === void 0 ? void 0 : _typing$getHandleList2.on(function (event) {
          return backspace.trigger(event);
        }); //方向键事件

        var left = new Left(editor);
        (_typing$getHandleList3 = typing.getHandleListener('left', 'keydown')) === null || _typing$getHandleList3 === void 0 ? void 0 : _typing$getHandleList3.on(function (event) {
          return left.trigger(event);
        });
        var right = new Right(editor);
        (_typing$getHandleList4 = typing.getHandleListener('right', 'keydown')) === null || _typing$getHandleList4 === void 0 ? void 0 : _typing$getHandleList4.on(function (event) {
          return right.trigger(event);
        });
        var up = new Up(editor);
        (_typing$getHandleList5 = typing.getHandleListener('up', 'keydown')) === null || _typing$getHandleList5 === void 0 ? void 0 : _typing$getHandleList5.on(function (event) {
          return up.trigger(event);
        });
        var down = new Down(editor);
        (_typing$getHandleList6 = typing.getHandleListener('down', 'keydown')) === null || _typing$getHandleList6 === void 0 ? void 0 : _typing$getHandleList6.on(function (event) {
          return down.trigger(event);
        });

        var _default = new Default(editor);

        (_typing$getHandleList7 = typing.getHandleListener('default', 'keydown')) === null || _typing$getHandleList7 === void 0 ? void 0 : _typing$getHandleList7.on(function (event) {
          return _default.trigger(event);
        });
      }

      cards.forEach(function (card) {
        _this.classes[card.cardName] = card;
      });
    }
  }, {
    key: "add",
    value: function add(clazz) {
      this.classes[clazz.cardName] = clazz;
    }
  }, {
    key: "each",
    value: function each(callback) {
      this.components.every(function (card, index) {
        if (callback && callback(card, index) === false) return false;
        return true;
      });
    }
  }, {
    key: "closest",
    value: function closest(selector, ignoreEditable) {
      if (isNode(selector)) selector = $(selector);

      if (isNodeEntry(selector) && !selector.isCard()) {
        var card = selector.closest(CARD_SELECTOR, function (node) {
          if (node && ignoreEditable ? $(node).isRoot() : $(node).isEditable()) {
            return;
          }

          return node.parentNode || undefined;
        });
        if (!card || card.length === 0) return;
        selector = card;
      }

      return selector;
    }
  }, {
    key: "find",
    value: function find(selector, ignoreEditable) {
      if (typeof selector !== 'string') {
        var cardNode = this.closest(selector, ignoreEditable);
        if (!cardNode) return;
        selector = cardNode;
      }

      var getValue = function getValue(node) {
        if (isNode(node)) node = $(node);
        var value = node.attributes(CARD_VALUE_KEY);
        return value ? decodeCardValue(value) : {};
      };

      var cards = this.components.filter(function (item) {
        if (typeof selector === 'string') return item.id === selector;
        return item.root.equal(selector) || item.id === getValue(selector).id;
      });
      if (cards.length === 0) return;
      return cards[0];
    }
  }, {
    key: "findBlock",
    value: function findBlock(selector) {
      if (isNode(selector)) selector = $(selector);
      if (!selector.get()) return;
      var parent = selector.parent();
      if (!parent) return;
      var card = this.find(parent);
      if (!card) return;
      if (card.constructor.cardType === CardType.BLOCK) return card;
      return this.findBlock(card.root);
    }
  }, {
    key: "getSingleCard",
    value: function getSingleCard(range) {
      var card = this.find(range.commonAncestorNode);
      if (!card) card = this.getSingleSelectedCard(range);
      return card;
    }
  }, {
    key: "getSingleSelectedCard",
    value: function getSingleSelectedCard(range) {
      var elements = range.findElementsInSimpleRange();
      var node = elements[0];

      if (elements.length === 1 && node) {
        var domNode = $(node);

        if (domNode.isCard()) {
          return this.find(domNode);
        }
      }

      return;
    } // 插入Card

  }, {
    key: "insertNode",
    value: function insertNode(range, card) {
      var isInline = card.constructor.cardType === 'inline';
      var editor = this.editor; // 范围为折叠状态时先删除内容

      if (!range.collapsed && isEngine(editor)) {
        editor.change.deleteContent(range);
      }

      this.gc();
      var inline = editor.inline,
          block = editor.block,
          node = editor.node; // 插入新 Card

      if (isInline) {
        inline.insert(card.root, range);
      } else {
        block.insert(card.root, range, function (container) {
          //获取最外层的block嵌套节点
          var blockParent = container.parent();

          while (blockParent && !blockParent.isEditable()) {
            container = blockParent;
            var parent = blockParent.parent();

            if (parent && node.isBlock(parent)) {
              blockParent = parent;
            } else break;
          }

          return container;
        });
      }

      this.components.push(card);
      card.focus(range); // 矫正错误 HTML 结构

      var rootParent = card.root.parent();

      if (!isInline && rootParent && rootParent.inEditor() && node.isBlock(rootParent)) {
        block.unwrap(rootParent, range);
      }

      var result = card.render();
      var center = card.getCenter();

      if (result !== undefined) {
        card.getCenter().append(typeof result === 'string' ? $(result) : result);
      }

      if (card.contenteditable.length > 0) {
        center.find(card.contenteditable.join(',')).each(function (node) {
          var child = $(node);
          child.attributes('contenteditable', 'true');
          child.attributes(DATA_ELEMENT, EDITABLE);
        });
      } //创建工具栏


      card.didRender();

      if (card.didInsert) {
        card.didInsert();
      }

      return card;
    } // 移除Card

  }, {
    key: "removeNode",
    value: function removeNode(card) {
      if (card.destroy) card.destroy();

      if (isEngine(this.editor) && card.constructor.cardType === CardType.BLOCK) {
        this.editor.readonly = false;
      }

      this.removeComponent(card);
      card.root.remove();
    } // 更新Card

  }, {
    key: "updateNode",
    value: function updateNode(card, value) {
      if (card.destroy) card.destroy();
      var container = card.findByKey('center');
      container.empty();
      card.setValue(value);
      var result = card.render();

      if (result !== undefined) {
        card.getCenter().append(typeof result === 'string' ? $(result) : result);
      }

      if (card.didUpdate) {
        card.didUpdate();
      }
    } // 将指定节点替换成等待创建的Card DOM 节点

  }, {
    key: "replaceNode",
    value: function replaceNode(node, name, value) {
      var clazz = this.classes[name];
      if (!clazz) throw ''.concat(name, ': This card does not exist');
      value = encodeCardValue(value);
      var cardNode = transformCustomTags("<card type=\"".concat(clazz.cardType, "\" name=\"").concat(name, "\" value=\"").concat(value, "\"></card>"));
      var readyCard = $(cardNode);
      node.before(readyCard);
      readyCard.append(node);
      return readyCard;
    }
  }, {
    key: "activate",
    value: function activate(node) {
      var trigger = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : ActiveTrigger.MANUAL;
      var event = arguments.length > 2 ? arguments[2] : undefined;
      var editor = this.editor;
      if (!isEngine(editor)) return; //获取当前卡片所在编辑器的根节点

      var container = node.getRoot(); //如果当前编辑器根节点和引擎的根节点不匹配就不执行，主要是子父编辑器的情况

      if (!container.get() || editor.container.equal(container)) {
        var card = this.find(node);
        var editableElement = node.closest(EDITABLE_SELECTOR);

        if (!card && editableElement.length > 0) {
          var editableParent = editableElement.parent();
          card = editableParent ? this.find(editableParent) : undefined;
        }

        var blockCard = card ? this.findBlock(card.root) : undefined;

        if (blockCard) {
          card = blockCard;
        }

        if (card && card.isCursor(node)) card = undefined;
        var isCurrentActiveCard = card && this.active && this.active.root.equal(card.root);

        if (trigger === ActiveTrigger.UPDATE_CARD) {
          isCurrentActiveCard = false;
        }

        if (this.active && !isCurrentActiveCard) {
          var _this$active$toolbarM;

          (_this$active$toolbarM = this.active.toolbarModel) === null || _this$active$toolbarM === void 0 ? void 0 : _this$active$toolbarM.hide();
          var type = this.active.constructor.cardType;
          this.active.select(false);
          this.active.activate(false);

          if (type === CardType.BLOCK) {
            editor.readonly = false;
          }
        }

        if (card) {
          if (card.activatedByOther) return;

          if (!isCurrentActiveCard) {
            var _card$toolbarModel;

            (_card$toolbarModel = card.toolbarModel) === null || _card$toolbarModel === void 0 ? void 0 : _card$toolbarModel.show(event);

            if (card.constructor.cardType === CardType.INLINE && card.constructor.autoSelected !== false && (trigger !== ActiveTrigger.CLICK || !card.readonly)) {
              this.select(card);
            }

            card.activate(true);
          }

          if (card.constructor.cardType === CardType.BLOCK) {
            card.select(false); //不在可编辑器区域内，设置为只读

            if (editableElement.length === 0) {
              editor.readonly = true;
            } else editor.readonly = false;
          }

          if (!isCurrentActiveCard && trigger === ActiveTrigger.MOUSE_DOWN) {
            editor.trigger('focus');
          }

          editor.change.onSelect();
        }
      }
    }
  }, {
    key: "select",
    value: function select(card) {
      var editor = this.editor;
      if (!isEngine(editor)) return;

      if (card.constructor.singleSelectable !== false && (card.constructor.cardType !== CardType.BLOCK || !card.activated)) {
        var range = editor.change.getRange();
        var root = card.root;
        var parentNode = root.parent();
        var index = parentNode.children().toArray().findIndex(function (child) {
          return child.equal(root);
        });
        range.setStart(parentNode, index);
        range.setEnd(parentNode, index + 1);
        editor.change.select(range);
      }
    }
  }, {
    key: "focus",
    value: function focus(card) {
      var toStart = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      var editor = this.editor;
      if (!isEngine(editor)) return;
      var change = editor.change,
          container = editor.container,
          scrollNode = editor.scrollNode;
      var range = change.getRange();
      card.focus(range, toStart);
      change.select(range);
      editor.readonly = false;
      this.activate(range.startNode, ActiveTrigger.MOUSE_DOWN);
      change.onSelect();
      if (scrollNode) range.scrollIntoViewIfNeeded(container, scrollNode);
    }
  }, {
    key: "insert",
    value: function insert(name, value) {
      if (!isEngine(this.editor)) throw 'Engine not found';
      var component = this.create(name, {
        value: value
      });
      var change = this.editor.change;
      var range = change.getSafeRange();
      var card = this.insertNode(range, component);
      var type = component.constructor.cardType;

      if (type === 'inline') {
        card.focus(range, false);
      }

      change.select(range);

      if (type === 'block' && component.constructor.autoActivate !== false) {
        this.activate(card.root, ActiveTrigger.INSERT_CARD);
      }

      change.change();
      return card;
    }
  }, {
    key: "update",
    value: function update(selector, value) {
      if (!isEngine(this.editor)) return;
      var change = this.editor.change;
      var card = this.find(selector);

      if (card) {
        this.updateNode(card, value);
        var range = change.getRange();
        card.focus(range, false);
        change.change();
      }
    }
  }, {
    key: "remove",
    value: function remove(selector) {
      if (!isEngine(this.editor)) return;
      var _this$editor = this.editor,
          change = _this$editor.change,
          list = _this$editor.list,
          node = _this$editor.node;
      var range = change.getRange();
      var card = this.find(selector);
      if (!card) return;

      if (card.constructor.cardType === CardType.INLINE) {
        range.setEndAfter(card.root[0]);
        range.collapse(false);
      } else {
        card.focusPrevBlock(range, true);
      }

      var parent = card.root.parent();
      this.removeNode(card);
      list.addBr(range.startNode);

      if (parent && node.isEmpty(parent)) {
        if (parent.isEditable()) {
          node.html(parent, '<p><br /></p>');
          range.select(parent, true);
          range.shrinkToElementNode();
          range.collapse(false);
        } else {
          node.html(parent, '<br />');
          range.select(parent, true);
          range.collapse(false);
        }
      }

      change.apply(range);
    }
  }, {
    key: "removeRemote",
    value: function removeRemote(selector) {
      if (!isEngine(this.editor)) return;
      var node = this.editor.node;
      var card = this.find(selector);
      if (!card) return;
      var parent = card.root.parent();
      this.removeNode(card);

      if (parent && node.isEmpty(parent)) {
        if (parent.isEditable()) {
          node.html(parent, '<p><br /></p>');
        } else {
          node.html(parent, '<br />');
        }
      }
    } // 创建Card DOM 节点

  }, {
    key: "create",
    value: function create(name, options) {
      var clazz = this.classes[name];
      if (!clazz) throw ''.concat(name, ': This card does not exist');

      if (['inline', 'block'].indexOf(clazz.cardType) < 0) {
        throw ''.concat(name, ': the type of card must be "inline", "block"');
      }

      if (options === null || options === void 0 ? void 0 : options.root) options.root.empty();
      var component = new clazz({
        editor: this.editor,
        value: options === null || options === void 0 ? void 0 : options.value,
        root: options === null || options === void 0 ? void 0 : options.root
      });
      component.root.attributes(CARD_TYPE_KEY, clazz.cardType);
      component.root.attributes(CARD_KEY, name); //如果没有指定是否能聚集，那么当card不是只读的时候就可以聚焦

      var hasFocus = clazz.focus !== undefined ? clazz.focus : !component.readonly;
      var tagName = clazz.cardType === CardType.INLINE ? 'span' : 'div'; //center

      var center = $("<".concat(tagName, " ").concat(component.isEditable ? DATA_TRANSIENT_ATTRIBUTES + "='*'" : '', "/>"));
      center.attributes(CARD_ELEMENT_KEY, 'center');

      if (hasFocus) {
        center.attributes('contenteditable', 'false');
      } else {
        component.root.attributes('contenteditable', 'false');
      } //body


      var body = $('<'.concat(tagName, ' ').concat(CARD_ELEMENT_KEY, '="body" />')); //可以聚焦的情况下，card左右两边添加光标位置

      if (hasFocus) {
        //left
        var left = $("<span ".concat(CARD_ELEMENT_KEY, "=\"left\" ").concat(DATA_TRANSIENT_ELEMENT, "=\"true\">&#8203;</span>")); //right

        var right = $("<span ".concat(CARD_ELEMENT_KEY, "=\"right\" ").concat(DATA_TRANSIENT_ELEMENT, "=\"true\">&#8203;</span>"));
        body.append(left);
        body.append(center);
        body.append(right);
      } else {
        body.append(center);
      }

      component.root.append(body);
      if (component.init) component.init();
      return component;
    }
    /**
     * 渲染
     * @param container 需要重新渲染包含卡片的节点，如果不传，则渲染全部待创建的卡片节点
     * @param asyncRender 是否异步渲染， 全部异步渲染完成后触发
     */

  }, {
    key: "render",
    value: function render(container, asyncRender) {
      var _this2 = this;

      var cards = container ? container.isCard() ? container : container.find("".concat(READY_CARD_SELECTOR)) : this.editor.container.find(READY_CARD_SELECTOR);
      this.gc();
      var setp = 0;

      var render = function render(card) {
        var result = card.render();
        var center = card.getCenter();

        if (result !== undefined) {
          center.append(typeof result === 'string' ? $(result) : result);
        }

        if (card.contenteditable.length > 0) {
          center.find(card.contenteditable.join(',')).each(function (node) {
            var child = $(node);
            if (!child.attributes('contenteditable')) child.attributes('contenteditable', 'true');
            child.attributes(DATA_ELEMENT, EDITABLE);
          });
        }

        card.didRender();
      };

      var asyncRenderCards = [];
      cards.each(function (node) {
        var cardNode = $(node);
        var readyKey = cardNode.attributes(READY_CARD_KEY);
        var key = cardNode.attributes(CARD_KEY);
        var name = readyKey || key;

        if (_this2.classes[name]) {
          var value = cardNode.attributes(CARD_VALUE_KEY);
          var attributes = cardNode.attributes();
          var card;

          if (key) {
            card = _this2.find(cardNode);

            if (card) {
              if (card.destroy) card.destroy();

              _this2.removeComponent(card);
            }
          } //ready_card_key 待创建的需要重新生成节点，并替换当前待创建节点


          card = _this2.create(name, {
            value: decodeCardValue(value),
            root: key ? cardNode : undefined
          });

          if (asyncRender) {
            card.root.attributes(CARD_ASYNC_RENDER, 'true');
          }

          Object.keys(attributes).forEach(function (attributesName) {
            if (attributesName.indexOf('data-') === 0 && attributesName.indexOf('data-card') !== 0) {
              card.root.attributes(attributesName, attributes[attributesName]);
            }
          });
          if (readyKey) cardNode.replaceWith(card.root);

          _this2.components.push(card); // 重新渲染


          if (!asyncRender) {
            render(card);
          } else {
            asyncRenderCards.push(card);
          }

          if (readyKey) {
            card.root.removeAttributes(READY_CARD_KEY);
          }
        }
      });
      if (!asyncRender) return;
      asyncRenderCards.forEach(function (card) {
        setTimeout(function () {
          render(card); //协同记录后移除标记属性

          setTimeout(function () {
            card.root.removeAttributes(CARD_ASYNC_RENDER);
          }, 50);
          setp++;

          if (setp === asyncRenderCards.length && typeof asyncRender === 'function') {
            asyncRender(asyncRenderCards.length);
          }
        }, 20);
      });

      if (asyncRenderCards.length === 0 && typeof asyncRender === 'function') {
        asyncRender(0);
      }
    }
  }, {
    key: "removeComponent",
    value: function removeComponent(card) {
      var _this3 = this;

      this.each(function (c, index) {
        if (c.root.equal(card.root)) {
          _this3.components.splice(index, 1);

          return false;
        }

        return;
      });
    }
  }, {
    key: "gc",
    value: function gc() {
      for (var i = 0; i < this.components.length; i++) {
        var component = this.components[i];

        if (!component.root[0] || component.root.closest('body').length === 0) {
          if (component.destroy) component.destroy();
          this.components.splice(i, 1);
          i--;
        }
      }
    }
  }]);

  return CardModel;
}();

var isDragEvent = function isDragEvent(event) {
  return !!event.dataTransfer;
};

var Clipboard = /*#__PURE__*/function () {
  function Clipboard(editor) {
    _classCallCheck(this, Clipboard);

    this.editor = void 0;
    this.editor = editor;
  }

  _createClass(Clipboard, [{
    key: "getData",
    value: function getData(event) {
      var _html;

      var transfer = isDragEvent(event) ? event.dataTransfer : event.clipboardData;
      var html = transfer === null || transfer === void 0 ? void 0 : transfer.getData('text/html');
      var text = transfer === null || transfer === void 0 ? void 0 : transfer.getData('text');
      var files = []; // Edge 兼容性处理

      try {
        if ((transfer === null || transfer === void 0 ? void 0 : transfer.items) && transfer.items.length > 0) {
          Array.from(transfer.items).forEach(function (item) {
            var file = item.kind === 'file' ? item.getAsFile() : null;

            if (file !== null) {
              var _text;

              if (file.type && file.type.indexOf('image/png') > -1 && !file.lastModified) {
                file = new File([file], 'image.png', {
                  type: file.type
                });
              }

              file['ext'] = (_text = text) === null || _text === void 0 ? void 0 : _text.split('.').pop();
            }

            if (file) files.push(file);
          });
        } else if ((transfer === null || transfer === void 0 ? void 0 : transfer.files) && transfer.files.length > 0) {
          files = Array.from(transfer.files);
        }
      } catch (err) {
        if ((transfer === null || transfer === void 0 ? void 0 : transfer.files) && transfer.files.length > 0) {
          files = Array.from(transfer.files);
        }
      } // 从 Mac OS Finder 复制文件


      if (html === '' && text && /^.+\.\w+$/.test(text) && files.length > 0) {
        text = ''; // 在图片上，点击右键复制
      } else if (text === '' && html && /^(<meta.+?>)?<img.+?>$/.test(html) && files.length > 0) {
        html = ''; // 从 Excel、Numbers 复制
      } else if ((html || text) && files.length > 0 && !((_html = html) === null || _html === void 0 ? void 0 : _html.startsWith('<img')) && (html || !/^https(s)?:/.test(text || ''))) {
        files = [];
      }

      return {
        html: html,
        text: text,
        files: files
      };
    }
  }, {
    key: "write",
    value: function write(event) {
      var range = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
      var callback = arguments.length > 2 ? arguments[2] : undefined;
      if (!range) range = Range.from(this.editor);
      if (!range) throw 'Range is null';
      range = range.cloneRange();
      var card = range.startNode.closest('[data-card-key]', function (node) {
        return $(node).isEditable() ? undefined : node.parentNode || undefined;
      });

      if (card.length > 0 && !range.collapsed && range.endOffset === 0) {
        if (range.endContainer.previousSibling) {
          range.setEndAfter(range.endContainer.previousSibling);
        }

        if (!range.collapsed && range.endOffset > 0 && range.endContainer.childNodes[range.endOffset - 1] === card[0]) {
          var cardCenter = range.startNode.closest('[data-card-element="center"]', function (node) {
            return $(node).isEditable() ? undefined : node.parentNode || undefined;
          });

          if (cardCenter.length > 0) {
            range.setEnd(cardCenter[0], cardCenter[0].childNodes.length);
          } else {
            range.setEnd(card[0], card[0].childNodes.length);
          }
        }
      }

      var root = range.commonAncestorNode;
      card = root.closest('[data-card-key]', function (node) {
        return $(node).isEditable() ? undefined : node.parentNode || undefined;
      });

      if (card.length > 0) {
        var _cardCenter = root.closest('[data-card-element="center"]', function (node) {
          return $(node).isEditable() ? undefined : node.parentNode || undefined;
        });

        if (_cardCenter.length === 0) {
          range.select(card);
          root = range.commonAncestorNode;
        }
      }

      var nodes = root.name === '#text' ? [document.createElement('span')] : [];
      card = root.closest('[data-card-key]', function (node) {
        if ($(node).isEditable()) return;

        if (node.nodeType === getWindow().Node.ELEMENT_NODE) {
          var display = window.getComputedStyle(node).getPropertyValue('display');

          if (display === 'inline') {
            nodes.push(node.cloneNode());
          }
        }

        return node.parentNode || undefined;
      });
      var hasChildEngine = root.find('.am-engine-view').length > 0 || root.find('.am-engine').length > 0;
      var hasParentEngine = root.closest('.am-engine-view').length > 0 || root.closest('.am-engine').length > 0;

      if (card.length <= 0 && (hasChildEngine || hasParentEngine)) {
        if (hasParentEngine && !root.isEditable()) {
          var block = this.editor.block.closest(root);

          if (['h1', 'h2', 'h3', 'h4', 'h5', 'h6'].indexOf(block.name) > -1) {
            nodes.push(block[0].cloneNode());
          }
        }

        event.preventDefault();

        if (range.collapsed) {
          var _event$clipboardData, _event$clipboardData2;

          (_event$clipboardData = event.clipboardData) === null || _event$clipboardData === void 0 ? void 0 : _event$clipboardData.setData('text/html', '');
          (_event$clipboardData2 = event.clipboardData) === null || _event$clipboardData2 === void 0 ? void 0 : _event$clipboardData2.setData('text', '');
        } else {
          var _event$clipboardData3, _event$clipboardData4;

          var contents = range.cloneContents();

          var _this$setNodes = this.setNodes(nodes),
              inner = _this$setNodes.inner,
              outter = _this$setNodes.outter;

          var parser = new Parser(contents, this.editor);

          var _parser$toHTML = parser.toHTML(inner, outter),
              html = _parser$toHTML.html,
              text = _parser$toHTML.text;

          if (callback) {
            callback({
              html: html,
              text: text
            });
          }

          if (html) html = '<meta name="source" content="aomao" />' + html;
          (_event$clipboardData3 = event.clipboardData) === null || _event$clipboardData3 === void 0 ? void 0 : _event$clipboardData3.setData('text/html', html);
          (_event$clipboardData4 = event.clipboardData) === null || _event$clipboardData4 === void 0 ? void 0 : _event$clipboardData4.setData('text', text);
        }
      }
    }
  }, {
    key: "copy",
    value: function copy(data) {
      var _this = this;

      var trigger = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

      if (typeof data === 'string') {
        return copyTo(data);
      }

      var editor = this.editor;
      var selection = window.getSelection();
      var range = selection ? Range.from(editor, selection) || Range.create(editor) : Range.create(editor);
      var cloneRange = range.cloneRange();
      var block = $('<div class="am-engine-view">&#8203;</div>');
      block.css({
        position: 'fixed',
        top: 0,
        clip: 'rect(0, 0, 0, 0)'
      });
      block.on('copy', function (e) {
        _this.write(e);
      });
      $(document.body).append(block);
      block.append(editor.node.clone($(data), true));

      if (trigger) {
        block.allChildren().forEach(function (child) {
          editor.trigger('copy', $(child));
        });
      }

      block.append($('&#8203;', null));
      range.select(block, true);
      selection === null || selection === void 0 ? void 0 : selection.removeAllRanges();
      selection === null || selection === void 0 ? void 0 : selection.addRange(range.toRange());
      var success = false;

      try {
        success = document.execCommand('copy');

        if (!success) {
          throw 'copy command was unsuccessful';
        }
      } catch (err) {
        console.log('unable to copy using execCommand: ', err);
      } finally {
        block.remove();
        selection === null || selection === void 0 ? void 0 : selection.removeAllRanges();
        selection === null || selection === void 0 ? void 0 : selection.addRange(cloneRange.toRange());
      }

      return success;
    }
  }, {
    key: "cut",
    value: function cut() {
      var range = Range.from(this.editor);
      if (!range) return;
      var root = range.commonAncestorNode;
      this.editor.change.deleteContent(range);
      var listElements = -1 !== ['ul', 'ol'].indexOf(root.name) ? root : root.find('ul,ol');

      for (var i = 0; i < listElements.length; i++) {
        var list = $(listElements[i]);
        var childs = list.find('li');
        childs.each(function (child) {
          if ('' === child.innerText || isSafari && '\n' === child.innerText) {
            var _child$parentNode;

            (_child$parentNode = child.parentNode) === null || _child$parentNode === void 0 ? void 0 : _child$parentNode.removeChild(child);
          }
        });

        if (list.children().length === 0) {
          list.remove();
        }
      }
    }
  }, {
    key: "setNodes",
    value: function setNodes(nodes) {
      if (0 === nodes.length) return {};

      for (var i = nodes.length - 1; i > 0; i--) {
        var node = nodes[i];
        node.appendChild(nodes[i - 1]);
      }

      return {
        inner: nodes[0],
        outter: nodes[nodes.length - 1]
      };
    }
  }]);

  return Clipboard;
}();

var Language = /*#__PURE__*/function () {
  function Language(lange) {
    var data = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    _classCallCheck(this, Language);

    this.data = {};
    this.lange = 'zh-cn';
    this.lange = lange;
    this.data = data;
  }

  _createClass(Language, [{
    key: "add",
    value: function add(data) {
      this.data = merge(this.data, data);
    }
  }, {
    key: "get",
    value: function get() {
      for (var _len = arguments.length, keys = new Array(_len), _key = 0; _key < _len; _key++) {
        keys[_key] = arguments[_key];
      }

      var get = function get() {
        var start = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
        var language = arguments.length > 1 ? arguments[1] : undefined;

        for (var i = start; i < keys.length; i++) {
          var value = language[keys[i]];

          if (_typeof(value) === 'object') {
            return get(i + 1, value);
          }

          return value || '';
        }

        return language;
      };

      return get(0, this.data[this.lange]);
    }
  }]);

  return Language;
}();

var Backspace$1 = /*#__PURE__*/function () {
  function Backspace(engine) {
    _classCallCheck(this, Backspace);

    this.engine = void 0;
    this.engine = engine;
  }
  /**
   * 在inline节点处按下backspace键
   */


  _createClass(Backspace, [{
    key: "trigger",
    value: function trigger(event) {
      var _this$engine = this.engine,
          change = _this$engine.change,
          node = _this$engine.node;
      var range = change.getRange();

      var _range$cloneRange$shr = range.cloneRange().shrinkToTextNode(),
          collapsed = _range$cloneRange$shr.collapsed,
          endNode = _range$cloneRange$shr.endNode,
          startNode = _range$cloneRange$shr.startNode,
          startOffset = _range$cloneRange$shr.startOffset,
          endOffset = _range$cloneRange$shr.endOffset;

      if (endNode.type === Node.TEXT_NODE || startNode.type === Node.TEXT_NODE) {
        //光标展开的情况下，判断光标结束位置是否在mark节点内侧零宽字符后面，不在后面让光标选在后面
        if (!collapsed) {
          var _markNode = endNode.parent();

          if (_markNode && endNode.type === Node.TEXT_NODE && node.isMark(_markNode)) {
            var text = endNode.text();
            var rightText = text.substr(endOffset); //不位于零宽字符前，不处理

            if (!/^\u200b$/.test(rightText)) return true;
            range.setEnd(endNode, endOffset + 1);
          }

          return true;
        }

        var markNode = startNode.parent(); //开始节点在mark标签内

        if (markNode && startNode.type === Node.TEXT_NODE && node.isMark(markNode)) {
          if (startOffset < 1) return true;

          var _text = startNode.text();

          var leftText = _text.substr(startOffset - 1, 1); //不位于零宽字符后，不处理


          if (!/^\u200b$/.test(leftText)) return true;

          if (startOffset === 1) {
            var prev = markNode.prev();

            if (prev && !node.isEmpty(prev)) {
              var _range$cloneRange$sel = range.cloneRange().select(prev, true).shrinkToTextNode().collapse(false),
                  _startNode = _range$cloneRange$sel.startNode,
                  _startOffset = _range$cloneRange$sel.startOffset;

              range.setStart(_startNode, _startOffset - 1);
            } //在段落的开始位置
            else if (!prev && node.isEmpty(markNode)) {
                var parent = markNode.parent();
                var offset = markNode.offset();
                markNode.remove();
                if (parent) range.setStart(parent, offset === 0 ? 0 : offset - 1);
              }
          } else {
            range.setStart(startNode, startOffset - 1);
          }

          return true;
        } //mark 标签后面的零宽字符后删除零宽字符前面的字符


        markNode = startNode.prev() || undefined;

        if (markNode && startNode.type === Node.TEXT_NODE && node.isMark(markNode)) {
          var _text2 = startNode.text();

          var _leftText = _text2.substr(startOffset - 1, 1); //不位于零宽字符后，不处理


          if (!/^\u200b$/.test(_leftText)) return true;

          if (startOffset === 1) {
            var _range$cloneRange$sel2 = range.cloneRange().select(markNode, true).shrinkToTextNode().collapse(false),
                _startNode2 = _range$cloneRange$sel2.startNode,
                _startOffset2 = _range$cloneRange$sel2.startOffset;

            range.setStart(_startNode2, _startOffset2 - 1);
          } else {
            range.setStart(startNode, startOffset - 1);
          }
        }
      }

      return true;
    }
  }]);

  return Backspace;
}();

var Mark = /*#__PURE__*/function () {
  function Mark(editor) {
    _classCallCheck(this, Mark);

    this.editor = void 0;
    this.editor = editor;
  }

  _createClass(Mark, [{
    key: "init",
    value: function init() {
      var _this = this;

      var editor = this.editor;

      if (isEngine(editor)) {
        var _editor$typing$getHan;

        //删除事件
        var backspace = new Backspace$1(editor);
        (_editor$typing$getHan = editor.typing.getHandleListener('backspace', 'keydown')) === null || _editor$typing$getHan === void 0 ? void 0 : _editor$typing$getHan.on(function (event) {
          return backspace.trigger(event);
        });
        editor.on('keydown:space', function (event) {
          return _this.triggerMarkdown(event);
        });
      }
    }
    /**
     * 解析markdown
     * @param event 事件
     */

  }, {
    key: "triggerMarkdown",
    value: function triggerMarkdown(event) {
      var editor = this.editor;
      if (!isEngine(editor)) return;
      var change = editor.change;
      var range = change.getRange();
      if (!range.collapsed || change.isComposing()) return;
      var startNode = range.startNode,
          startOffset = range.startOffset;
      var node = startNode.type === Node.TEXT_NODE ? startNode : startNode.children().eq(startOffset - 1);
      if (!node) return;
      var text = node.type === Node.TEXT_NODE ? node.text().substr(0, startOffset) : node.text();
      return !Object.keys(editor.plugin.components).some(function (pluginName) {
        var plugin = editor.plugin.components[pluginName];

        if (isMarkPlugin(plugin) && !!plugin.markdown) {
          var reuslt = plugin.triggerMarkdown(event, text, node);
          if (reuslt === false) return true;
        }

        return;
      });
    }
    /**
     * 根据节点查找mark插件实例
     * @param node 节点
     */

  }, {
    key: "findPlugin",
    value: function findPlugin(mark) {
      var _this$editor = this.editor,
          node = _this$editor.node,
          plugin = _this$editor.plugin,
          schema = _this$editor.schema;
      if (!node.isMark(mark)) return;
      var result = undefined;
      Object.keys(plugin.components).some(function (pluginName) {
        var markPlugin = plugin.components[pluginName];

        if (isMarkPlugin(markPlugin) && mark.name === markPlugin.tagName) {
          var schemaRule = markPlugin.schema();
          if ((markPlugin.attributes || markPlugin.style) && !(Array.isArray(schemaRule) ? schemaRule.find(function (rule) {
            return schema.checkNode(mark, rule.attributes);
          }) : schema.checkNode(mark, schemaRule.attributes))) return;
          result = markPlugin;
          return true;
        }

        return;
      });
      return result;
    }
    /**
     * 获取最近的 Mark 节点，找不到返回 node
     */

  }, {
    key: "closest",
    value: function closest(node) {
      var nodeApi = this.editor.node;

      while (node && node.parent() && !nodeApi.isBlock(node)) {
        if (node.isEditable()) break;
        if (nodeApi.isMark(node)) return node;
        var parentNode = node.parent();
        if (!parentNode) break;
        node = parentNode;
      }

      return node;
    }
    /**
     * 获取向上第一个非 Mark 节点
     */

  }, {
    key: "closestNotMark",
    value: function closestNotMark(node) {
      while (this.editor.node.isMark(node) || node.isText()) {
        if (node.isEditable()) break;
        var parent = node.parent();
        if (!parent) break;
        node = parent;
      }

      return node;
    }
    /**
     * 比较两个节点是否相同，包括attributes、style、class
     * @param source 源节点
     * @param target 目标节点
     * @param isCompareValue 是否比较每项属性的值
     */

  }, {
    key: "compare",
    value: function compare(source, target, isCompareValue) {
      //节点名称不一致
      if (source.name !== target.name) return false; //获取节点属性

      var sourceAttributes = source.attributes();
      delete sourceAttributes['style'];
      var targetAttributes = target.attributes();
      delete targetAttributes['style']; //获取节点样式属性

      var sourceStyles = source.css();
      var targetStyles = target.css();
      delete sourceAttributes['class'];
      delete targetAttributes['class']; //获取节点样式名称

      var sourceClassName = source.get().className.trim();
      var targetClassName = target.get().className.trim();
      var sourceClasses = sourceClassName !== '' ? sourceClassName.split(/\s+/) : [];
      var targetClasses = targetClassName !== '' ? targetClassName.split(/\s+/) : []; //样式名称可能是可变的，如data-fontsize-12和data-fontsize-14代表的是不同的值，如果不需要比较值，直接获取标签样式规则比较

      var schema = this.editor.schema;
      var schemas = schema.find(function (rule) {
        return rule.name === source.name;
      });

      var compareClass = function compareClass(classNames) {
        for (var i = 0; i < schemas.length; i++) {
          var schemaRule = schemas[i];

          if (schemaRule.attributes && schema.checkValue(schemaRule.attributes, 'class', classNames.join(' ').trim())) {
            return schemaRule.attributes['class'].toString();
          }
        }

        return classNames.join(' ').trim();
      };

      if (!isCompareValue) {
        sourceClasses = sourceClasses.length > 0 ? [compareClass(sourceClasses)] : [];
        targetClasses = targetClasses.length > 0 ? [compareClass(targetClasses)] : [];
      } //属性长度不一致


      if (Object.keys(sourceAttributes).length !== Object.keys(targetAttributes).length) return false; //属性名称或值不一致

      if (!Object.keys(sourceAttributes).every(function (attributesName) {
        return isCompareValue ? sourceAttributes[attributesName] === targetAttributes[attributesName] : !!targetAttributes[attributesName];
      })) return false; //样式属性长度不一致

      if (Object.keys(sourceStyles).length !== Object.keys(targetStyles).length) return false; //样式属性名称或值不一致

      if (!Object.keys(sourceStyles).every(function (styleName) {
        return isCompareValue ? sourceStyles[styleName] === targetStyles[styleName] : !!targetStyles[styleName];
      })) return false; //样式名称长度不一致

      if (sourceClasses.length !== targetClasses.length) return false; //样式名称不一致

      if (!sourceClasses.every(function (sourceClass) {
        return targetClasses.indexOf(sourceClass) !== -1;
      })) return false;
      return true;
    }
    /**
     * 判断源节点是否包含目标节点的所有属性名称和样式名称
     * @param source 源节点
     * @param target 目标节点
     */

  }, {
    key: "contain",
    value: function contain(source, target) {
      var attributes = target.attributes();
      var styles = attributes['style'] || {};
      delete attributes['style'];
      var sourceAttributes = source.attributes();
      var sourceStyles = sourceAttributes['style'] || {};
      delete sourceAttributes['style'];
      return Object.keys(attributes).every(function (key) {
        return !!sourceAttributes[key];
      }) && Object.keys(styles).every(function (key) {
        return !sourceStyles[key];
      });
    }
    /**
     * 去除一个节点下的所有空 Mark，通过 callback 可以设置其它条件
     * @param root 节点
     * @param callback 回调
     */

  }, {
    key: "unwrapEmptyMarks",
    value: function unwrapEmptyMarks(root, callback) {
      var node = this.editor.node;
      var children = root.allChildren();
      children.forEach(function (childNode) {
        var child = $(childNode);

        if (node.isEmpty(child) && node.isMark(child) && (!callback || callback(child))) {
          node.unwrap(child);
        }
      });
    }
    /**
     * 在光标重叠位置时分割，在分割时会清空父节点内容再重组，如果需要保持光标右边某节点的追踪，请传入该节点
     * @param range 光标
     * @param removeMark 要移除的mark空节点
     * @param keelpNode 分割光标右侧需要保持追踪的节点
     */

  }, {
    key: "splitOnCollapsed",
    value: function splitOnCollapsed(range, removeMark, keelpNode) {
      var _this2 = this;

      if (!range.collapsed) return; //扩大光标选区

      range.enlargeFromTextNode();
      range.shrinkToElementNode();
      var startNode = range.startNode;
      var startParent = startNode.parent(); //获取卡片

      var card = startNode.isCard() ? startNode : startNode.closest(CARD_SELECTOR);
      var node = this.editor.node;

      if ((card.length === 0 || card.attributes(CARD_TYPE_KEY) !== 'inline') && (node.isMark(startNode) || startParent && node.isMark(startParent))) {
        var _rightNodes$find;

        // 获取上面第一个非mark标签
        var parent = this.closestNotMark(startNode); // 插入范围的开始和结束标记

        var selection = range.createSelection(); // 获取标记左侧节点

        var left = selection.getNode(parent, 'left'); // 获取标记右侧节点

        var right = undefined;
        var keelpRoot = undefined;
        var keelpPath = [];

        if (keelpNode) {
          if (isNode(keelpNode)) keelpNode = $(keelpNode); // 获取需要跟踪节点的路径

          var path = keelpNode.getPath(parent.get());
          var cloneParent = parent.clone(true);
          keelpPath = path.slice(1); // 获取需要跟踪节点的root节点

          keelpRoot = $(cloneParent.getChildByPath(path.slice(0, 1)));
          right = selection.getNode(cloneParent, 'right', false);
        } else right = selection.getNode(parent, 'right'); // 删除空标签


        this.unwrapEmptyMarks(left);
        this.unwrapEmptyMarks(right, function (node) {
          if (removeMark && !Array.isArray(removeMark)) removeMark = [removeMark]; //没有传指定的mark，那就都移除。否则比较后一致就移除

          var isUnwrap = !removeMark || removeMark.length === 0 || !node.isCard() && removeMark.some(function (mark) {
            return _this2.compare(node, mark);
          });
          return isUnwrap;
        }); // 清空原父容器，用新的内容代替

        parent.empty();
        var leftChildren = left.children();
        var leftNodes = leftChildren.toArray();
        parent.append(leftChildren);
        var rightChildren = right.children();
        var rightNodes = rightChildren.toArray(); // 根据跟踪节点的root节点和path获取其在rightNodes中的新节点

        if (keelpRoot) keelpNode = (_rightNodes$find = rightNodes.find(function (node) {
          return node.equal(keelpRoot);
        })) === null || _rightNodes$find === void 0 ? void 0 : _rightNodes$find.getChildByPath(keelpPath);
        parent.append(rightChildren); // 找到卡片，重新设置卡片根节点的引用

        parent.find(CARD_SELECTOR).each(function (cardNode) {
          var cardComponent = _this2.editor.card.find(cardNode);

          if (cardComponent && !cardComponent.root.equal(cardNode)) {
            cardComponent.root[0] = cardNode;
          }
        });
        var zeroWidthNode = $("\u200B", null); // 重新设置范围

        if (leftNodes.length === 1 && leftNodes[0].name === 'br') {
          leftNodes[0].remove();
          leftNodes.splice(0, 1);
        }

        if (rightNodes.length === 1 && rightNodes[0].name === 'br') {
          rightNodes[0].remove();
          rightNodes.splice(0, 1);
        }

        if (rightNodes.length > 0) {
          var rightContainer = rightNodes[0]; // 右侧没文本

          if (node.isEmpty(rightContainer)) {
            var firstChild = rightContainer.first();

            while (firstChild && !firstChild.isText()) {
              rightContainer = firstChild;
              firstChild = firstChild.first();
            }

            if (rightContainer.isText()) {
              rightContainer.before(zeroWidthNode);
            } else {
              rightContainer.prepend(zeroWidthNode);
            }
          } // 右侧有文本
          else {
              //在多层mark嵌套的情况下，要移除的mark里面还有其它mark节点，还有要移除的mark外面还有其它mark，需要将其组合起来
              var markParent = node.isMark(startNode) ? startNode : startNode.parent();
              var wrapZeroNode = zeroWidthNode;
              if (removeMark && !Array.isArray(removeMark)) removeMark = [removeMark];

              var _loop = function _loop() {
                var markClone = markParent.clone(); //不是要移除的mark

                if (!removeMark.some(function (mark) {
                  return _this2.compare(markClone, mark);
                })) {
                  var isZero = zeroWidthNode.equal(wrapZeroNode);
                  wrapZeroNode = node.wrap(wrapZeroNode, markClone);
                  if (isZero) zeroWidthNode = wrapZeroNode.first();
                }

                markParent = markParent.parent();
              };

              while (removeMark && removeMark.length > 0 && markParent && node.isMark(markParent)) {
                _loop();
              }

              rightContainer.before(wrapZeroNode);
            }

          range.select(zeroWidthNode).collapse(false);
        } else if (leftNodes.length > 0) {
          var leftContainer = leftNodes[leftNodes.length - 1];
          leftContainer.after(zeroWidthNode);
          range.select(zeroWidthNode).collapse(false);
        } else {
          range.select(parent, true).collapse(true);
        } //替换多个零宽字符为一个零宽字符


        var textWithEmpty = false;
        parent.children().each(function (child) {
          var childNode = $(child);

          if (childNode.isText()) {
            var textContent = child.textContent;
            var text = (textContent === null || textContent === void 0 ? void 0 : textContent.replace(/\u200b+/g, "\u200B")) || '';

            if (textContent !== text) {
              child.textContent = text;
            }

            if (textWithEmpty) {
              //当前第二个连续零宽字符的下一个节点不能是inline节点
              var next = childNode.next(); //当前第二个连续零宽字符没有下一个节点，并且父级节点不能为inline节点

              var _parent = childNode.parent();

              if ((!next && _parent && !node.isInline(_parent) || next && !node.isInline(next)) && text.startsWith("\u200B")) {
                text = text.substring(1);
                if (text) child.textContent = text;else childNode.remove();
              } else textWithEmpty = false;
            }

            if (text.endsWith("\u200B")) textWithEmpty = true;
          } else textWithEmpty = false;
        });
        var nodeApi = node; //移除多余的零宽字符

        if (zeroWidthNode[0].parentNode) {
          var at = zeroWidthNode[0];
          var atText = null;
          var atTextLen = 0;

          var handleAt = function handleAt(node, findPrev) {
            var getAlignNode = function getAlignNode(node) {
              return findPrev ? node.previousSibling : node.nextSibling;
            };

            while (node) {
              if (node.nodeType !== at.nodeType) return;
              var alignNode = getAlignNode(node);

              if (node.textContent === atText) {
                var _node$parentNode;

                //inline 节点位置的零宽字符跳过
                if (alignNode && nodeApi.isInline(alignNode) || !alignNode && node.parentNode && nodeApi.isInline(node.parentNode)) break;
                (_node$parentNode = node.parentNode) === null || _node$parentNode === void 0 ? void 0 : _node$parentNode.removeChild(node);
                node = alignNode;
              } else {
                var _node$textContent3, _node$parentNode2;

                if (findPrev) {
                  while (atText && ((_node$textContent = node.textContent) === null || _node$textContent === void 0 ? void 0 : _node$textContent.endsWith(atText))) {
                    var _node$textContent;

                    //inline 节点位置的零宽字符跳过
                    if (alignNode && nodeApi.isInline(alignNode) || !alignNode && node.parentNode && nodeApi.isInline(node.parentNode)) break;
                    node.textContent = node.textContent.substring(0, node.textContent.length - atTextLen);
                  }
                } else {
                  while (atText && ((_node$textContent2 = node.textContent) === null || _node$textContent2 === void 0 ? void 0 : _node$textContent2.startsWith(atText))) {
                    var _node$textContent2;

                    //inline 节点位置的零宽字符跳过
                    if (alignNode && nodeApi.isInline(alignNode) || !alignNode && node.parentNode && nodeApi.isInline(node.parentNode)) break;
                    node.textContent = node.textContent.substring(atText.length);
                  }
                }

                if (((_node$textContent3 = node.textContent) === null || _node$textContent3 === void 0 ? void 0 : _node$textContent3.length) !== 0) return;
                (_node$parentNode2 = node.parentNode) === null || _node$parentNode2 === void 0 ? void 0 : _node$parentNode2.removeChild(node);
                node = alignNode;
              }
            }
          };

          if (at.nodeType === getWindow().Node.TEXT_NODE) {
            var textContent = at.textContent;
            atText = textContent;
            atTextLen = atText.length;
            handleAt(at.previousSibling, true);
            handleAt(at.nextSibling, false);
          }
        }
      }

      return keelpNode;
    }
    /**
     * 在光标位置不重合时分割
     * @param range 光标
     * @param removeMark 要移除的空mark节点
     */

  }, {
    key: "splitOnExpanded",
    value: function splitOnExpanded(range, removeMark) {
      var _this3 = this;

      if (range.collapsed) return;
      range.enlargeToElementNode();
      range.shrinkToElementNode();
      var startNode = range.startNode,
          endNode = range.endNode;
      var cardStart = startNode.isCard() ? startNode : startNode.closest(CARD_SELECTOR);
      var cardEnd = endNode.isCard() ? endNode : endNode.closest(CARD_SELECTOR);

      if (!(cardStart.length > 0 && 'inline' === cardStart.attributes(CARD_TYPE_KEY) || cardEnd.length > 0 && 'inline' === cardEnd.attributes(CARD_TYPE_KEY))) {
        //开始非mark标签父节点
        var startNotMarkParent = this.closestNotMark(startNode); //结束非mark标签父节点

        var endNotMarkParent = this.closestNotMark(endNode);

        if (!startNotMarkParent.equal(endNotMarkParent)) {
          var startRange = range.cloneRange();
          startRange.collapse(true);
          var endRange = range.cloneRange();
          endRange.collapse(false); //如果开始非mark标签父节点包含结束非mark标签父节点，那么分割的时候会清空 结束非mark标签父节点的内容进行重组。结束非mark标签父节点 将无非找到
          //所以需要从被包含的节点开始分割

          var keelpNode = undefined;
          var startOffset = startRange.startOffset;
          var endOffset = endRange.endOffset; //如果开始节点的父节点包含结尾父节点，会将结尾父节点删除重组，导致光标失效，需要先执行开始节点分割，并跟踪结尾节点

          if (startNotMarkParent.contains(endNotMarkParent)) {
            //先分割开始节点，并跟踪结尾节点
            keelpNode = this.splitOnCollapsed(startRange, removeMark, endRange.endNode);
            range.setStart(startRange.startContainer, startRange.startOffset); //如果有跟踪到，重新设置结尾节点

            if (keelpNode) {
              endRange.setOffset(keelpNode, endOffset, endOffset);
            } //分割结尾节点


            this.splitOnCollapsed(endRange, removeMark);
            range.setEnd(endRange.startContainer, endRange.startOffset);
          } else {
            //结尾父节点包含开始节点父节点
            //先分割结尾节点，并跟踪开始节点
            keelpNode = this.splitOnCollapsed(endRange, removeMark, startRange.startNode);
            range.setEnd(endRange.startContainer, endRange.startOffset); //如果有跟踪到，重新设置开始节点

            if (keelpNode) {
              startRange.setOffset(keelpNode, startOffset, startOffset);
            } //分割开始节点


            this.splitOnCollapsed(startRange, removeMark);
            range.setStart(startRange.startContainer, startRange.startOffset);
          }

          return;
        }

        var node = this.editor.node; // 节点不是样式标签，文本节点时判断父节点

        var startParent = startNode.parent();
        var startIsMark = node.isMark(startNode) || startParent && node.isMark(startParent);
        var endParent = endNode.parent();
        var endIsMark = node.isMark(endNode) || endParent && node.isMark(endParent); // 不是样式标签，无需分割

        if (!startIsMark && !endIsMark) {
          return;
        } // 获取上面第一个非样式标签


        var commonAncestorNode = range.commonAncestorNode;

        if (commonAncestorNode.isText()) {
          commonAncestorNode = commonAncestorNode.parent();
        }

        var parent = this.closestNotMark(commonAncestorNode); // 插入范围的开始和结束标记

        var selection = range.createSelection(); // 子节点分别保存在两个变量

        var left = selection.getNode(parent, 'left');
        var center = selection.getNode(parent);
        var right = selection.getNode(parent, 'right'); // 删除空标签

        this.unwrapEmptyMarks(left);
        this.unwrapEmptyMarks(right); // 清空原父容器，用新的内容代替

        parent.empty();
        parent.append(left.children());
        var centerChildren = center.children();
        var centerNodes = centerChildren.toArray();
        parent.append(centerChildren);
        parent.append(right.children()); // 找到卡片，重新设置卡片根节点的引用

        parent.find(CARD_SELECTOR).each(function (cardNode) {
          var cardComponent = _this3.editor.card.find(cardNode);

          if (cardComponent && !cardComponent.root.equal(cardNode)) {
            cardComponent.root[0] = cardNode;
          }
        }); // 重新设置范围

        range.setStartBefore(centerNodes[0][0]);
        range.setEndAfter(centerNodes[centerNodes.length - 1][0]);
      }
    }
    /**
     * 分割mark标签
     * @param removeMark 需要移除的mark标签
     */

  }, {
    key: "split",
    value: function split(range, removeMark) {
      if (!isEngine(this.editor)) return;
      var change = this.editor.change;
      var safeRange = range || change.getSafeRange();
      var doc = getDocument(safeRange.startContainer);

      if (typeof removeMark === 'string' || !Array.isArray(removeMark) && removeMark && isNode(removeMark)) {
        removeMark = $(removeMark, doc);
      }

      if (safeRange.collapsed) {
        this.splitOnCollapsed(safeRange, removeMark);
      } else {
        this.splitOnExpanded(safeRange, removeMark);
      }

      if (!range) change.apply(safeRange);
    }
    /**
     * 在当前光标选区包裹mark标签
     * @param mark mark标签
     * @param both mark标签两侧节点
     */

  }, {
    key: "wrap",
    value: function wrap(mark, range) {
      var _this4 = this;

      var change = isEngine(this.editor) ? this.editor.change : undefined;
      if (!range && !change) return;
      var node = this.editor.node;
      var safeRange = range || change.getSafeRange();
      var doc = getDocument(safeRange.startContainer);

      if (typeof mark === 'string' || isNode(mark)) {
        mark = $(mark, doc);
      } else mark = mark;

      if (!node.isMark(mark)) return;

      if (safeRange.collapsed) {
        if (mark.children().length === 0) mark.append(doc.createTextNode("\u200B")); //在相通插件下，值不同，插入到同级，不做嵌套

        var startNode = safeRange.startNode;
        var parent = startNode.parent();

        if (parent && startNode.isText() && node.isMark(parent)) {
          if (this.compare(parent, mark)) {
            this.split(safeRange, parent.clone());
          }
        }

        node.insert(mark, safeRange);
        this.merge(safeRange);
        safeRange.addOrRemoveBr();
        if (!range) change === null || change === void 0 ? void 0 : change.apply(safeRange);
        return;
      }

      this.split(safeRange);
      var commonAncestorNode = safeRange.commonAncestorNode;

      if (commonAncestorNode.type === getWindow().Node.TEXT_NODE) {
        commonAncestorNode = commonAncestorNode.parent();
      } // 插入范围的开始和结束标记


      var selection = safeRange.createSelection();

      if (!selection.anchor && !selection.focus) {
        if (!range) change === null || change === void 0 ? void 0 : change.apply(safeRange);
        return;
      } // 遍历范围内的节点，添加 Mark


      var started = false;
      var nodeApi = node;
      commonAncestorNode.traverse(function (node) {
        var child = $(node);
        mark = mark;

        if (!child.equal(selection.anchor)) {
          if (started) {
            if (child.equal(selection.focus)) {
              started = false;
              return false;
            }

            if (nodeApi.isMark(child) && !child.isCard()) {
              if (!nodeApi.isEmpty(child)) {
                //找到最底层mark标签添加包裹，<strong><span style="font-size:16px">abc</span></strong> ，在 span 节点中的text再添加包裹，不在strong外添加包裹
                var targetNode = child;
                var targetChildrens = targetNode.children();

                while (nodeApi.isMark(targetNode) && targetChildrens.length === 1) {
                  var targetChild = targetChildrens.eq(0);

                  if (nodeApi.isMark(targetChild)) {
                    targetNode = targetChild;
                    targetChildrens = targetNode.children();
                  } else if (targetChild.isText()) {
                    targetNode = targetChild;
                  } else break;
                }

                nodeApi.removeZeroWidthSpace(targetNode);

                var _parent2 = targetNode.parent(); //父级和当前要包裹的节点，属性和值都相同，那就不包裹。只有属性一样，并且父节点只有一个节点那就移除父节点包裹,然后按插件情况合并值


                if (targetNode.isText() && _parent2 && nodeApi.isMark(_parent2)) {
                  if (_this4.compare(_parent2.clone(), mark, true)) return true;

                  if (_parent2.children().length === 1) {
                    var plugin = _this4.findPlugin(mark);

                    var curPlugin = _this4.findPlugin(_parent2); //插件一样，并且并表明要合并值


                    if (plugin && plugin === curPlugin && plugin.combineValueByWrap === true) {
                      nodeApi.wrap(_parent2, mark, true);
                      return true;
                    } //插件一样，不合并，直接移除
                    else if (plugin && plugin === curPlugin) nodeApi.unwrap(_parent2);
                  }
                }

                nodeApi.wrap(targetNode, mark);
                return true;
              } else if (child.name !== mark.name) {
                child.remove();
              }
            }

            if (child.isText() && !nodeApi.isEmpty(child)) {
              nodeApi.removeZeroWidthSpace(child);

              var _parent3 = child.parent(); //父级和当前要包裹的节点，属性和值都相同，那就不包裹。只有属性一样，并且父节点只有一个节点那就移除父节点包裹,然后按插件情况合并值


              if (_parent3 && nodeApi.isMark(_parent3)) {
                if (_this4.compare(_parent3.clone(), mark, true)) return true;

                if (_parent3.children().length === 1) {
                  var _plugin = _this4.findPlugin(mark);

                  var _curPlugin = _this4.findPlugin(_parent3); //插件一样，并且并表明要合并值


                  if (_plugin && _plugin === _curPlugin && _plugin.combineValueByWrap === true) {
                    nodeApi.wrap(_parent3, mark, true);
                    return true;
                  } //插件一样，不合并，直接移除
                  else if (_plugin && _plugin === _curPlugin) nodeApi.unwrap(_parent3);
                }
              }

              nodeApi.wrap(child, mark);
            }
          }
        } else {
          started = true;
        }

        return;
      });
      selection.move();
      this.merge(safeRange);
      if (!range) change === null || change === void 0 ? void 0 : change.apply(safeRange);
    }
    /**
     * 合并当前选区的mark节点
     * @param range 光标，默认当前选区光标
     */

  }, {
    key: "merge",
    value: function merge(range) {
      var _this5 = this;

      if (!isEngine(this.editor)) return;
      var _this$editor2 = this.editor,
          change = _this$editor2.change,
          node = _this$editor2.node;
      var safeRange = range || change.getSafeRange();
      var marks = this.findMarks(safeRange);

      if (marks.length === 0) {
        return;
      }

      var mergeMarks = function mergeMarks(marks) {
        marks.forEach(function (mark) {
          var prevMark = mark.prev();
          var nextMark = mark.next();

          if (prevMark && _this5.compare(prevMark, mark, true)) {
            var selection = safeRange.shrinkToElementNode().createSelection();
            node.merge(prevMark, mark);
            selection.move(); // 原来 mark 已经被移除，重新指向

            mark = prevMark;
          }

          if (nextMark && _this5.compare(nextMark, mark, true)) {
            var _selection = safeRange.shrinkToElementNode().createSelection();

            node.merge(mark, nextMark);

            _selection.move();
          } //合并子级mark


          var childMarks = [];
          mark.children().each(function (childNode) {
            var child = $(childNode);

            if (node.isMark(child)) {
              childMarks.push(child);
            }
          });

          if (childMarks.length > 0) {
            mergeMarks(childMarks);
          }
        });
      };

      mergeMarks(marks);
      safeRange.addOrRemoveBr();
      if (!range) change.apply(safeRange);
    }
    /**
     * 去掉mark包裹
     * @param range 光标
     * @param removeMark 要移除的mark标签
     */

  }, {
    key: "unwrap",
    value: function unwrap(removeMark, range) {
      var _this6 = this;

      if (!isEngine(this.editor)) return;
      var _this$editor3 = this.editor,
          change = _this$editor3.change,
          node = _this$editor3.node;
      var safeRange = range || change.getSafeRange();
      var doc = getDocument(safeRange.startContainer) || document;

      if (removeMark !== undefined && !Array.isArray(removeMark) && (typeof removeMark === 'string' || isNode(removeMark))) {
        removeMark = $(removeMark, doc);
      }

      this.split(safeRange, removeMark);

      if (safeRange.collapsed) {
        if (!range) change.apply(safeRange);
        return;
      }

      var ancestor = safeRange.commonAncestorNode;

      if (ancestor.type === getWindow().Node.TEXT_NODE) {
        ancestor = ancestor.parent();
      } // 插入范围的开始和结束标记


      var selection = safeRange.createSelection();

      if (!selection.has()) {
        if (!range) change.apply(safeRange);
        return;
      } // 遍历范围内的节点，获取目标 Mark


      var markNodes = [];
      var started = false;
      ancestor.traverse(function (childNode) {
        var child = $(childNode);

        if (!child.equal(selection.anchor)) {
          if (started) {
            if (!child.equal(selection.focus)) {
              if (node.isMark(child) && !child.isCard() && safeRange.isPointInRange(child, 0)) {
                markNodes.push(child);
              }
            }
          }
        } else {
          started = true;
        }
      }); // 清除 Mark

      var nodeApi = node;
      markNodes.forEach(function (node) {
        removeMark = removeMark;

        if (!removeMark || !node.isCard() && _this6.compare(node, removeMark)) {
          nodeApi.unwrap(node);
        } else if (removeMark) {
          var _removeMark$get;

          var styleMap = removeMark.css();
          Object.keys(styleMap).forEach(function (key) {
            node.css(key, '');
          }); //移除符合规则的class

          var removeClass = (_removeMark$get = removeMark.get()) === null || _removeMark$get === void 0 ? void 0 : _removeMark$get.className.split(/\s+/);

          if (removeClass) {
            (function () {
              var schema = _this6.editor.schema;
              var schemas = schema.find(function (rule) {
                return rule.name === node.name;
              });

              var _loop2 = function _loop2(i) {
                var schemaRule = schemas[i];
                removeClass.forEach(function (className) {
                  className = className.trim();
                  if (className === '') return;

                  if (schemaRule.attributes && schema.checkValue(schemaRule.attributes, 'class', className)) {
                    node.removeClass(className);
                  }
                });
              };

              for (var i = 0; i < schemas.length; i++) {
                _loop2(i);
              }
            })();
          }
        } else {
          node.removeAttributes('class');
          node.removeAttributes('style');
        }
      });
      selection.move();
      this.merge(safeRange);
      if (!range) change.apply(safeRange);
    }
    /**
     * 光标处插入mark标签
     * @param mark mark标签
     * @param range 指定光标，默认为编辑器选中的光标
     */

  }, {
    key: "insert",
    value: function insert(mark, range) {
      var _node$insert;

      if (!isEngine(this.editor)) return;
      var _this$editor4 = this.editor,
          change = _this$editor4.change,
          node = _this$editor4.node;
      var safeRange = range || change.getSafeRange();

      if (typeof mark === 'string' || isNode(mark)) {
        var doc = getDocument(safeRange.startContainer);
        mark = $(mark, doc);
      } // 范围为折叠状态时先删除内容


      if (!safeRange.collapsed) {
        change.deleteContent(safeRange);
      } // 插入新 Mark


      (_node$insert = node.insert(mark, safeRange)) === null || _node$insert === void 0 ? void 0 : _node$insert.addOrRemoveBr().select(mark).collapse(false);
      if (!range) change.apply(safeRange);
    }
    /**
     * 查找对范围有效果的所有 Mark
     * @param range 范围
     */

  }, {
    key: "findMarks",
    value: function findMarks(range) {
      var cloneRange = range.cloneRange();
      var node = this.editor.node;

      var handleRange = function handleRange(allowBlock, range) {
        var toStart = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
        if (!range.collapsed) return;
        var startNode = range.startNode,
            startOffset = range.startOffset; //没有父节点

        var startParent = startNode.findParent();
        if (!startParent) return; //选择父节点内容

        var cloneRange = range.cloneRange();
        cloneRange.select(startParent, true); //开始位置

        if (toStart) {
          cloneRange.setEnd(startNode, startOffset);
          cloneRange.enlargeFromTextNode();
          cloneRange.enlargeToElementNode(true);
          var startChildren = startNode.children();
          var _endNode = cloneRange.endNode,
              endOffset = cloneRange.endOffset;

          var endChildren = _endNode.children();

          var endOffsetNode = endChildren.eq(endOffset);
          var startOffsetNode = startChildren.eq(startOffset) || startChildren.eq(startOffset - 1);
          if (!allowBlock && _endNode.type === Node.ELEMENT_NODE && endOffsetNode && node.isBlock(endOffsetNode) && (startNode.type !== Node.ELEMENT_NODE || !!startOffsetNode && !node.isBlock(startOffsetNode))) return;
          cloneRange.select(startParent, true);
          cloneRange.setStart(_endNode, endOffset);
          cloneRange.shrinkToElementNode();
          cloneRange.shrinkToTextNode();
          range.setStart(cloneRange.startContainer, cloneRange.startOffset);
          range.collapse(true);
        } else {
          cloneRange.setStart(startNode, startOffset);
          cloneRange.enlargeFromTextNode();
          cloneRange.enlargeToElementNode(true);

          var _startChildren = startNode.children();

          var startNodeClone = cloneRange.startNode;
          var startOffsetClone = cloneRange.startOffset;
          var startNodeCloneChildren = startNodeClone.children();

          var _startOffsetNode = startNodeCloneChildren.eq(startOffsetClone);

          var startChildrenOffsetNode = _startChildren.eq(startOffset) || _startChildren.eq(startOffset - 1);

          if (!allowBlock && startNodeClone.type === Node.ELEMENT_NODE && _startOffsetNode && node.isBlock(_startOffsetNode) && (startNode.type !== Node.ELEMENT_NODE || startChildrenOffsetNode && !node.isBlock(startChildrenOffsetNode))) return;
          cloneRange.select(startParent, true);
          cloneRange.setEnd(startNodeClone, startOffsetClone);
          cloneRange.shrinkToElementNode();
          cloneRange.shrinkToTextNode();
          range.setEnd(cloneRange.endContainer, cloneRange.endOffset);
          range.collapse(false);
        }
      }; // 左侧不动，只缩小右侧边界
      // <anchor /><strong>foo</strong><focus />bar
      // 改成
      // <anchor /><strong>foo<focus /></strong>bar


      if (!range.collapsed) {
        var leftRange = range.cloneRange();
        var rightRange = range.cloneRange();
        leftRange.collapse(true);
        rightRange.collapse(false);
        handleRange(true, leftRange, true);
        handleRange(true, rightRange);
        cloneRange.setStart(leftRange.startContainer, leftRange.startOffset), cloneRange.setEnd(rightRange.startContainer, rightRange.startOffset);
      }

      handleRange(false, cloneRange);
      var sc = cloneRange.startContainer;
      var so = cloneRange.startOffset;
      var ec = cloneRange.endContainer;
      var eo = cloneRange.endOffset;
      var startNode = sc;
      var endNode = ec;

      if (sc.nodeType === getWindow().Node.ELEMENT_NODE && sc.childNodes[so]) {
        startNode = sc.childNodes[so] || sc;
      }

      if (ec.nodeType === getWindow().Node.ELEMENT_NODE && eo > 0 && ec.childNodes[eo - 1]) {
        endNode = ec.childNodes[eo - 1] || sc;
      } // 折叠状态时，按右侧位置的方式处理


      if (range.collapsed) {
        startNode = endNode;
      } // 不存在时添加


      var addNode = function addNode(nodes, nodeB) {
        if (!nodes.some(function (nodeA) {
          return nodeA.equal(nodeB);
        })) {
          nodes.push(nodeB);
        }
      };

      var nodeApi = node; // 向上寻找

      var findNodes = function findNodes(node) {
        var nodes = [];

        while (node) {
          if (node.type === getWindow().Node.ELEMENT_NODE && node.isEditable()) {
            break;
          }

          if (nodeApi.isMark(node) && !node.attributes(CARD_KEY) && !node.attributes(CARD_ELEMENT_KEY)) {
            nodes.push(node);
          }

          var parent = node.parent();
          if (!parent) break;
          node = parent;
        }

        return nodes;
      };

      var nodes = findNodes($(startNode));

      if (!range.collapsed) {
        findNodes($(endNode)).forEach(function (nodeB) {
          return addNode(nodes, nodeB);
        });

        if (sc !== ec) {
          var isBegin = false;
          var isEnd = false;
          range.commonAncestorNode.traverse(function (child) {
            if (isEnd) return false; //节点不是开始节点

            if (!child.equal(sc)) {
              if (isBegin) {
                //节点是结束节点，标记为结束
                if (child.equal(ec)) {
                  isEnd = true;
                  return false;
                }

                if (nodeApi.isMark(child) && !child.attributes(CARD_KEY) && !child.attributes(CARD_ELEMENT_KEY)) {
                  addNode(nodes, child);
                }
              }
            } else {
              //如果是开始节点，标记为开始
              isBegin = true;
            }

            return;
          });
        }
      }

      return nodes;
    }
    /**
     * 从下开始往上遍历删除空 Mark，当遇到空 Block，添加 BR 标签
     * @param node 节点
     * @param addBr 是否添加br
     */

  }, {
    key: "removeEmptyMarks",
    value: function removeEmptyMarks(node, addBr) {
      if (node.length === 0 || node.isEditable() || node.isCard() || node.attributes(DATA_ELEMENT)) {
        return;
      }

      var nodeApi = this.editor.node;

      if (!node.attributes(DATA_ELEMENT)) {
        var _node$first;

        var parent = node.parent(); // 包含光标标签
        // <p><strong><cursor /></strong></p>

        if (node.children().length === 1 && ((_node$first = node.first()) === null || _node$first === void 0 ? void 0 : _node$first.attributes(DATA_ELEMENT))) {
          if (nodeApi.isMark(node)) {
            node.before(node.first());
            node.remove();
            if (parent) this.removeEmptyMarks(parent, true);
          } else if (addBr && nodeApi.isBlock(node)) {
            node.prepend('<br />');
          }

          return;
        }

        var html = nodeApi.html(node);

        if (html === '' || html === "\u200B") {
          if (nodeApi.isMark(node)) {
            node.remove();
            if (parent) this.removeEmptyMarks(parent, true);
          } else if (addBr && nodeApi.isBlock(node)) {
            nodeApi.html(node, '<br />');
          }
        }
      }
    }
  }]);

  return Mark;
}();

var Backspace$2 = /*#__PURE__*/function () {
  function Backspace(engine) {
    _classCallCheck(this, Backspace);

    this.engine = void 0;
    this.engine = engine;
  }
  /**
   * 列表删除事件
   * @param e 事件
   * @param isDeepMerge 是否深度合并
   */


  _createClass(Backspace, [{
    key: "trigger",
    value: function trigger(event, isDeepMerge) {
      var _this$engine = this.engine,
          change = _this$engine.change,
          command = _this$engine.command,
          list = _this$engine.list,
          node = _this$engine.node;
      var range = change.getRange();

      if (range.collapsed) {
        var _block = this.engine.block.closest(range.startNode);

        if ('li' === _block.name && list.isFirst(range)) {
          event.preventDefault();
          command.execute(list.getPluginNameByNode(_block));
          return false;
        }
      } else {
        var startBlock = this.engine.block.closest(range.startNode);
        var endBlock = this.engine.block.closest(range.endNode);

        if ('li' === startBlock.name || 'li' === endBlock.name) {
          event.preventDefault();
          var cloneRange = range.cloneRange();
          change.deleteContent(undefined, isDeepMerge);
          list.addBr(startBlock);
          list.addBr(endBlock);
          range.setStart(cloneRange.startContainer, cloneRange.startOffset);
          range.collapse(true);
          list.merge();
          change.select(range);
          return false;
        }
      }

      if (!this.engine.block.isFirstOffset(range, 'start')) return;
      var block = this.engine.block.closest(range.startNode); // 在列表里

      if (['ul', 'ol'].indexOf(block.name) >= 0) {
        // 矫正这种情况，<ul><cursor /><li>foo</li></ul>
        var li = block.first();

        if (!li || li.isText()) {
          // <ul><cursor />foo</ul>
          event.preventDefault();
          change.mergeAfterDeletePrevNode(block);
          return false;
        } else {
          block = li;
          range.setStart(block[0], 0);
          range.collapse(true);
          change.select(range);
        }
      }

      if (block.name === 'li') {
        var _block$parent;

        if (node.isCustomize(block)) {
          return;
        }

        event.preventDefault();
        var listRoot = block.closest('ul');

        if ((_block$parent = block.parent()) === null || _block$parent === void 0 ? void 0 : _block$parent.isEditable()) {
          // <p>foo</p><li><cursor />bar</li>
          change.mergeAfterDeletePrevNode(block);
          return false;
        }

        if (listRoot.length > 0) {
          command.execute(list.getPluginNameByNode(listRoot));
        } else {
          // <p><li><cursor />foo</li></p>
          change.unwrapNode(block);
        }

        return false;
      }

      return true;
    }
  }]);

  return Backspace;
}();

var Enter$1 = /*#__PURE__*/function () {
  function Enter(engine) {
    _classCallCheck(this, Enter);

    this.engine = void 0;
    this.backspace = void 0;
    this.engine = engine;
    this.backspace = new Backspace$2(engine);
  }

  _createClass(Enter, [{
    key: "trigger",
    value: function trigger(event) {
      var _this$engine = this.engine,
          change = _this$engine.change,
          command = _this$engine.command,
          list = _this$engine.list;
      var range = change.getRange();
      range.shrinkToElementNode();
      var startBlock = this.engine.block.closest(range.startNode);
      var endBlock = this.engine.block.closest(range.endNode); //选区开始或结束位置为li

      if ('li' === startBlock.name || 'li' === endBlock.name) {
        //选区为展开状态，先删除
        if (!range.collapsed) {
          this.backspace.trigger(event, startBlock.name !== endBlock.name);
          range = change.getRange();
        }

        event.preventDefault(); //如果光标在列表结尾或者开始位置

        var pluginName = list.getPluginNameByNode(startBlock);

        if (list.isLast(range) && list.isFirst(range)) {
          command.execute(pluginName);
        } else {
          this.engine.block.split();
          range = change.getRange();
          var selection = range.createSelection();
          var block = this.engine.block.closest(range.endNode);
          var plugin = list.getPlugins().find(function (plugin) {
            return pluginName === plugin.constructor.pluginName;
          });
          if (!plugin) return;

          if (plugin.cardName) {
            var prev = block.prev();

            if (prev) {
              list.addCardToCustomize(prev, plugin.cardName);
              list.addBr(prev);
            }

            list.addCardToCustomize(block, plugin.cardName);
            list.addBr(block);
            var next = block.next();

            if (next) {
              list.addCardToCustomize(next, plugin.cardName);
              list.addBr(next);
            }
          }

          selection.move();
          range.collapse(false);
          change.select(range);
          list.merge();
          list.addBr(range.startNode.closest('ul'));
        }

        range.scrollIntoView();
        return false;
      }

      return true;
    }
  }]);

  return Enter;
}();

var List = /*#__PURE__*/function () {
  /**
   * 自定义列表样式
   */

  /**
   * 自定义列表样式
   */

  /**
   * 列表缩进key
   */
  function List(editor) {
    _classCallCheck(this, List);

    this.editor = void 0;
    this.CUSTOMZIE_UL_CLASS = 'data-list';
    this.CUSTOMZIE_LI_CLASS = 'data-list-item';
    this.INDENT_KEY = 'data-indent';
    this.editor = editor;
  }

  _createClass(List, [{
    key: "init",
    value: function init() {
      if (isEngine(this.editor)) {
        var _this$editor$typing$g, _this$editor$typing$g2;

        //绑定回车事件
        var enter = new Enter$1(this.editor);
        (_this$editor$typing$g = this.editor.typing.getHandleListener('enter', 'keydown')) === null || _this$editor$typing$g === void 0 ? void 0 : _this$editor$typing$g.on(function (event) {
          return enter.trigger(event);
        }); //删除事件

        var backspace = new Backspace$2(this.editor);
        (_this$editor$typing$g2 = this.editor.typing.getHandleListener('backspace', 'keydown')) === null || _this$editor$typing$g2 === void 0 ? void 0 : _this$editor$typing$g2.on(function (event) {
          return backspace.trigger(event);
        });
      }
    }
    /**
     * 判断列表项节点是否为空
     * @param node 节点
     */

  }, {
    key: "isEmptyItem",
    value: function isEmptyItem(node) {
      var _node$first, _node$first2, _node$first3, _node$last;

      return (//节点名称必须为li
        'li' === node.name && ( //空节点
        this.editor.node.isEmpty(node) || ( //子节点只有一个，如果是自定义列表并且第一个是卡片 或者第一个节点是 br标签，就是空节点
        1 === node.children().length ? node.hasClass(this.CUSTOMZIE_LI_CLASS) && ((_node$first = node.first()) === null || _node$first === void 0 ? void 0 : _node$first.isCard()) || 'br' === ((_node$first2 = node.first()) === null || _node$first2 === void 0 ? void 0 : _node$first2.name) : //子节点有两个，并且是自定义列表而且第一个是卡片，并且第二个节点是br标签
        2 === node.children().length && node.hasClass(this.CUSTOMZIE_LI_CLASS) && !!((_node$first3 = node.first()) === null || _node$first3 === void 0 ? void 0 : _node$first3.isCard()) && 'br' === ((_node$last = node.last()) === null || _node$last === void 0 ? void 0 : _node$last.name)))
      );
    }
    /**
     * 判断两个节点是否是一样的List节点
     * @param sourceNode 源节点
     * @param targetNode 目标节点
     */

  }, {
    key: "isSame",
    value: function isSame(sourceNode, targetNode) {
      //节点名称不一样
      if (sourceNode.name !== targetNode.name) return false;
      var node = this.editor.node; //自定义列表类型不一致，要么都是，要么都不是

      if (node.isCustomize(sourceNode) !== node.isCustomize(targetNode)) return false; //判断缩进是否一致

      var sourceIndent = parseInt(sourceNode.attributes(this.INDENT_KEY), 10) || 0;
      var targetIndent = parseInt(targetNode.attributes(this.INDENT_KEY), 10) || 0;
      return sourceIndent === targetIndent;
    }
    /**
     * 判断节点集合是否是指定类型的List列表
     * @param blocks 节点集合
     * @param name 节点标签类型
     * @param card 是否是指定的自定义列表项的卡片名称
     */

  }, {
    key: "isSpecifiedType",
    value: function isSpecifiedType(blocks) {
      var name = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'ul';
      var card = arguments.length > 2 ? arguments[2] : undefined;
      var node = this.editor.node;
      var isSame = true;
      blocks.forEach(function (block) {
        var _block$parent;

        //如果节点内包含了列表节点，则跳过此节点
        if (['li', 'p'].indexOf(block.name) === -1 && (block.name === name || block.find(name).length > 0)) return;

        switch (block.name) {
          case 'li':
            //有指定卡片，判断是否是自定义列表项的卡片相同
            if (card) {
              var _block$first;

              isSame = isSame && node.isCustomize(block) && (((_block$first = block.first()) === null || _block$first === void 0 ? void 0 : _block$first.attributes(CARD_KEY)) || '') === card;
            } else {
              isSame = isSame && !node.isCustomize(block);
            }

            break;

          case 'p':
            if (block.parent() && ((_block$parent = block.parent()) === null || _block$parent === void 0 ? void 0 : _block$parent.name) !== 'li') {
              isSame = false;
            }

            break;

          default:
            isSame = false;
            break;
        }
      });
      return isSame;
    }
  }, {
    key: "getPlugins",
    value: function getPlugins() {
      var _this = this;

      var plugins = [];
      Object.keys(this.editor.plugin.components).forEach(function (name) {
        var plugin = _this.editor.plugin.components[name];

        if (!!plugin.isCurrent) {
          plugins.push(plugin);
        }
      });
      return plugins;
    }
    /**
     * 根据节点获取列表插件名称
     * @param block 节点
     */

  }, {
    key: "getPluginNameByNode",
    value: function getPluginNameByNode(block) {
      var _this2 = this;

      var name = block.name;

      var getName = function getName(node) {
        var name = '';

        _this2.getPlugins().some(function (plugin) {
          if (plugin.isCurrent(node)) {
            name = plugin.constructor.pluginName;
            return true;
          }

          return;
        });

        return name;
      }; //如果是自定义列表


      if (this.editor.node.isCustomize(block)) return getName(block); //如果是li标签

      if ('li' === name && block.parent()) return getName(block.parent());
      return '';
    }
    /**
     * 获取列表插件名称
     * @param blocks 节点集合
     */

  }, {
    key: "getPluginNameByNodes",
    value: function getPluginNameByNodes(blocks) {
      var listType = '';

      for (var i = 0; i < blocks.length; i++) {
        var block = blocks[i]; //节点父级

        var parent = block.parent();
        var type = '';

        switch (block.name) {
          case 'li':
          case 'ul':
          case 'ol':
            type = this.getPluginNameByNode(blocks[i]);
            break;

          case 'p':
            if (parent && parent.name === 'li') {
              type = this.getPluginNameByNode(parent);
            } else {
              return '';
            }

            break;

          default:
            //如果节点内包含了列表节点，则跳过此节点
            if (this.editor.node.isBlock(block) && block.find('ul,ol').length > 0) break;else return '';
        }

        if (listType && type && listType !== type) {
          return '';
        }

        listType = type;
      }

      return listType;
    }
    /**
     * 清除自定义列表节点相关属性
     * @param node 节点
     */

  }, {
    key: "unwrapCustomize",
    value: function unwrapCustomize(node) {
      if (this.editor.node.isCustomize(node)) {
        switch (node.name) {
          case 'li':
            node.removeAttributes('class');
            node.find(CARD_SELECTOR).remove();
            return node;

          case 'ul':
            node.removeAttributes('class');
            return node;

          default:
            return node;
        }
      }

      return node;
    }
    /**
     * 取消节点的列表
     * @param blocks 节点集合
     */

  }, {
    key: "unwrap",
    value: function unwrap(blocks) {
      var _this3 = this;

      var indent = 0;
      var node = this.editor.node;
      var normalBlock = $('<p />');
      blocks.forEach(function (block) {
        _this3.unwrapCustomize(block);

        if (node.isList(block)) {
          indent = parseInt(block.attributes(_this3.INDENT_KEY), 10) || 0;
          node.unwrap(block);
        }

        if (block.name === 'li') {
          var toBlock = node.clone(normalBlock, false);

          if (indent !== 0) {
            toBlock.css('padding-left', indent * 2 + 'em');
          }

          node.replace(block, toBlock);
        }
      });
    }
    /**
     * 获取当前选区的修复列表后的节点集合
     */

  }, {
    key: "normalize",
    value: function normalize() {
      if (!isEngine(this.editor)) return [];
      var _this$editor = this.editor,
          change = _this$editor.change,
          block = _this$editor.block,
          node = _this$editor.node;
      var range = change.getRange();
      var blocks = block.getBlocks(range);
      var listNodes = [];
      blocks.forEach(function (block, i) {
        var parent = block.parent(); //节点是p标签

        if (block.name === 'p') {
          //p标签被li节点包裹时，去除p标签，并保留子节点和内容
          if ((parent === null || parent === void 0 ? void 0 : parent.name) === 'li') {
            if (i === 0) {
              listNodes.push(parent);
            } //去除包裹


            node.unwrap(block);
            return;
          } // <ul><p>a</p></ul> => <ul><li>a</li></ul>


          if (parent && ['ul', 'ol'].indexOf(parent.name) > -1) {
            block = node.replace(block, $('<li />'));
            listNodes.push(block);
            return;
          }
        }

        if (block.name === 'li' && (parent === null || parent === void 0 ? void 0 : parent.name) === 'li') {
          // <li><li>a</li></li> => <li>a</li>
          if (i === 0) {
            listNodes.push(parent);
          }

          node.unwrap(block);
          return;
        }

        if (['ul', 'ol'].indexOf(block.name) > -1) {
          // <li><ul>...</ul></li> => <li>...</li>
          if ((parent === null || parent === void 0 ? void 0 : parent.name) === 'li') {
            node.unwrap(block);
            return;
          }
        }

        listNodes.push(block);
      }); // 最后一个 li 如果没选中内容，会在 getBlocks 时抛弃掉，这里需要补回来

      var lastBlock = range.endNode.closest('li');

      if (!listNodes.some(function (block) {
        return block[0] === lastBlock[0];
      })) {
        listNodes.push(lastBlock);
      }

      return listNodes;
    }
    /**
     * 将选中列表项列表分割出来单独作为一个列表
     */

  }, {
    key: "split",
    value: function split() {
      if (!isEngine(this.editor)) return;
      var _this$editor2 = this.editor,
          change = _this$editor2.change,
          node = _this$editor2.node;
      var range = change.getSafeRange();
      var blocks = this.normalize();

      if (blocks.length > 0 && (blocks[0].name === 'li' || blocks[blocks.length - 1].name === 'li')) {
        var selection = range.createSelection();
        var firstBlock = blocks[0];
        var lastBlock = blocks[blocks.length - 1];
        var middleList = [];
        var rightList = [];
        var beforeListElement; //选区中最后的li节点的列表节点

        var afterListElement; // 当前选中的第一个节点是li，并且这个节点前面还有节点，那就获取选中的节点所在列表的后续节点然后放入middleList

        if (firstBlock.prev()) {
          beforeListElement = firstBlock.parent();
          var indexInRange = 0;

          while (blocks[indexInRange] && blocks[indexInRange].name === 'li') {
            middleList.push(blocks[indexInRange]);
            indexInRange += 1;
          }
        } // 当前选中的最后一个节点是li，那么获取到这个列表的最后所有的li节点放入rightList


        if (lastBlock.next()) {
          afterListElement = lastBlock.parent();
          var nextBlock = lastBlock.next();

          while (nextBlock && nextBlock.name === 'li') {
            rightList.push(nextBlock);
            nextBlock = nextBlock.next();
          }
        } //将 rightList 集合添加到最后的列表节点内


        var afterListElementClone;

        if (rightList.length > 0 && afterListElement) {
          afterListElementClone = node.clone(afterListElement, false);
          rightList.forEach(function (li) {
            var _afterListElementClon;

            (_afterListElementClon = afterListElementClone) === null || _afterListElementClon === void 0 ? void 0 : _afterListElementClon.append(li[0]);
          });
          afterListElement.after(afterListElementClone);
        }

        var beforeListElementClone; //将 middleList 集合添加到前方列表节点内

        if (middleList.length > 0 && beforeListElement) {
          beforeListElementClone = node.clone(beforeListElement, false);
          middleList.forEach(function (li) {
            var _beforeListElementClo;

            (_beforeListElementClo = beforeListElementClone) === null || _beforeListElementClo === void 0 ? void 0 : _beforeListElementClo.append(li[0]);
          });
          beforeListElement.after(beforeListElementClone);
        } //有序列表设置start属性


        if (beforeListElement && afterListElement && afterListElement.equal(beforeListElement) && beforeListElement.name === 'ol') {
          var newStart = (parseInt(beforeListElement.attributes('start'), 10) || 1) + beforeListElement.find('li').length;
          afterListElementClone.attributes('start', newStart);
        }

        selection.move();
      }

      change.apply(range);
    }
  }, {
    key: "merge",
    value: function merge(blocks, range) {
      var _this4 = this;

      if (!isEngine(this.editor)) return;
      var _this$editor3 = this.editor,
          change = _this$editor3.change,
          block = _this$editor3.block,
          node = _this$editor3.node;
      var safeRange = range || change.getSafeRange();
      blocks = blocks || block.getBlocks(safeRange);
      blocks.forEach(function (block) {
        block = block.closest('ul,ol');

        if (!node.isList(block)) {
          return;
        }

        var prevBlock = block.prev();
        var nextBlock = block.next();

        if (prevBlock && _this4.isSame(prevBlock, block)) {
          var selection = safeRange.createSelection();
          node.merge(prevBlock, block);
          selection.move(); // 原来 block 已经被移除，重新指向

          block = prevBlock;
        }

        if (nextBlock && _this4.isSame(nextBlock, block)) {
          var _selection = safeRange.createSelection();

          node.merge(block, nextBlock);

          _selection.move();
        }
      });
      blocks = block.getBlocks(safeRange);

      if (blocks.length > 0) {
        var _block = blocks[0].closest('ul,ol');

        this.addStart(_block);
      }

      if (!range) change.apply(safeRange);
    }
    /**
     * 给列表添加start序号
     * @param block 列表节点
     */

  }, {
    key: "addStart",
    value: function addStart(block) {
      if (!isEngine(this.editor)) return;
      var _this$editor4 = this.editor,
          change = _this$editor4.change,
          node = _this$editor4.node;

      if (!block) {
        var blocks = this.editor.block.getBlocks(change.getRange());
        if (blocks.length === 0) return;
        block = blocks[0].closest('ul,ol');
      }

      if (!block || !node.isList(block)) return;
      var startIndent = parseInt(block.attributes(this.INDENT_KEY), 10) || 0; // 当前选区起始位置如果不是第一层级，需要向前遍历，找到各层级的前序序号
      // 直到遇到一个非列表截止，比如 p

      var startCache = [];
      var cacheIndent = startIndent;
      var prevNode = block.prev();

      while (prevNode && node.isList(prevNode)) {
        if (prevNode.name === 'ol') {
          var prevIndent = parseInt(prevNode.attributes(this.INDENT_KEY), 10) || 0;
          var prevStart = parseInt(prevNode.attributes('start'), 10) || 1;
          var len = prevNode.find('li').length;

          if (prevIndent === 0) {
            startCache[prevIndent] = prevStart + len;
            break;
          }

          if (prevIndent <= cacheIndent) {
            cacheIndent = prevIndent;
            startCache[prevIndent] = startCache[prevIndent] || prevStart + len;
          }
        } else cacheIndent = parseInt(prevNode.attributes(this.INDENT_KEY), 10) || 0;

        prevNode = prevNode.prev();
      }

      var nextNode = block;

      while (nextNode) {
        if (node.isList(nextNode)) {
          var nextIndent = parseInt(nextNode.attributes(this.INDENT_KEY), 10) || 0;
          var nextStart = parseInt(nextNode.attributes('start'), 10);
          var _len = nextNode.find('li').length;

          if (nextNode.name === 'ol') {
            var currentStart = startCache[nextIndent];

            if (nextIndent > 0) {
              currentStart = currentStart || 1;
              if (currentStart > 1) nextNode.attributes('start', currentStart);else nextNode.removeAttributes('start');
              startCache[nextIndent] = currentStart + _len;
            } else {
              if (currentStart && currentStart !== nextStart) {
                if (currentStart > 1) nextNode.attributes('start', currentStart);else nextNode.removeAttributes('start');
                startCache[nextIndent] = currentStart + _len;
              } else {
                startCache[nextIndent] = (nextStart || 1) + _len;
                startCache = startCache.slice(0, nextIndent + 1);
              }
            }
          }
        } else startCache = [];

        var next = nextNode.next();
        if (!next) break;
        nextNode = next;
      }
    }
    /**
     * 给列表节点增加缩进
     * @param block 列表节点
     * @param value 缩进值
     */

  }, {
    key: "addIndent",
    value: function addIndent(block, value, maxValue) {
      if (this.editor.node.isList(block)) {
        var indentValue = this.getIndent(block);
        value = indentValue + (value < 0 ? -1 : 1);
        if (maxValue && value > maxValue) value = maxValue;

        if (value < 1) {
          block.removeAttributes(this.INDENT_KEY);
        } else {
          block.attributes(this.INDENT_KEY, value);
        }
      }
    }
    /**
     * 获取列表节点 indent 值
     * @param block 列表节点
     * @returns
     */

  }, {
    key: "getIndent",
    value: function getIndent(block) {
      if (this.editor.node.isList(block)) {
        return parseInt(block.attributes(this.INDENT_KEY), 10) || 0;
      }

      return 0;
    }
    /**
     * 为自定义列表项添加卡片节点
     * @param node 列表节点项
     * @param cardName 卡片名称，必须是支持inline卡片类型
     * @param value 卡片值
     */

  }, {
    key: "addCardToCustomize",
    value: function addCardToCustomize(node, cardName, value) {
      var _node$first4;

      if (isNode(node)) node = $(node); //必须是li标签

      if (node.name !== 'li') return; //第一个子节点必须不是卡片

      if ((_node$first4 = node.first()) === null || _node$first4 === void 0 ? void 0 : _node$first4.isCard()) return; //创建卡片

      var card = this.editor.card;
      var component = card.create(cardName, {
        value: value
      }); //设置光标选中

      var range = Range.create(this.editor);
      range.select(node, true);
      range.collapse(true); //插入卡片

      card.insertNode(range, component);
      return component;
    }
    /**
     * 为自定义列表项添加待渲染卡片节点
     * @param node 列表节点项
     * @param cardName 卡片名称，必须是支持inline卡片类型
     * @param value 卡片值
     */

  }, {
    key: "addReadyCardToCustomize",
    value: function addReadyCardToCustomize(node, cardName, value) {
      var _node$first5;

      if (isNode(node)) node = $(node); //必须是li标签

      if (node.name !== 'li') return; //第一个子节点必须不是卡片

      if ((_node$first5 = node.first()) === null || _node$first5 === void 0 ? void 0 : _node$first5.isCard()) return;
      var cardRoot = $('<span />');
      node.prepend(cardRoot);
      this.editor.card.replaceNode(cardRoot, cardName, value);
      return cardRoot;
    }
    /**
     * 给列表添加BR标签
     * @param node 列表节点项
     */

  }, {
    key: "addBr",
    value: function addBr(node) {
      var _this5 = this;

      var nodeApi = this.editor.node;

      if (nodeApi.isList(node)) {
        node.find('li').each(function (node) {
          _this5.addBr($(node));
        });
      } else if (node.name === 'li') {
        var _child;

        var child = node.last();
        if ((_child = child) === null || _child === void 0 ? void 0 : _child.isCursor()) child = child.prev();

        if (child) {
          //最后一个节点是br节点
          if (child.name === 'br') return; //自定义节点，并且最后一个是卡片

          if (nodeApi.isCustomize(node) && child.isCard()) {
            node.append($('<br />'));
            return;
          }

          while (child) {
            if (child.equal(node.first()) && (child.isCard() || child.text() === '')) {
              node.append($('<br />'));
              return;
            } //文本


            if (child.type === getWindow().Node.TEXT_NODE) {
              if (child.text() !== '') return;
              child = child.prev();
            } //节点
            else if (child.type === getWindow().Node.ELEMENT_NODE) {
                if (!nodeApi.isMark(child) || child.text() !== '') return;
                child = node.prev();
              } //其它
              else child = child.prev();
          }
        } else node.append($('<br />'));
      }
    }
    /**
     * 将节点转换为自定义节点
     * @param blocks 节点
     * @param cardName 卡片名称
     * @param value 卡片值
     */

  }, {
    key: "toCustomize",
    value: function toCustomize(blocks, cardName, value) {
      var _this6 = this;

      var node = this.editor.node;

      if (Array.isArray(blocks)) {
        var nodes = [];
        blocks.forEach(function (block) {
          if (!node.isCustomize(block)) {
            var _node = _this6.toCustomize(block, cardName, value);

            nodes = nodes.concat(_node);
          }
        });
        return nodes;
      } else {
        var indent;
        var customizeRoot = $("<ul class=\"".concat(this.CUSTOMZIE_UL_CLASS, "\"/>"));
        var customizeItem = $("<li class=\"".concat(this.CUSTOMZIE_LI_CLASS, "\"/>"));

        switch (blocks.name) {
          case 'li':
            blocks.addClass(this.CUSTOMZIE_LI_CLASS);
            this.addCardToCustomize(blocks, cardName, value);
            return blocks;

          case 'ul':
            blocks.addClass(this.CUSTOMZIE_UL_CLASS);
            return blocks;

          case 'ol':
            customizeRoot.attributes(blocks.attributes());
            blocks = node.replace(blocks, customizeRoot);
            return blocks;

          case 'p':
            indent = removeUnit(blocks.css('padding-left')) / 2;
            blocks = node.replace(blocks, customizeItem);
            this.addCardToCustomize(blocks, cardName, value);

            if (indent) {
              customizeRoot.attributes(this.INDENT_KEY, indent);
            }

            blocks = node.wrap(blocks, customizeRoot);
            return blocks;

          default:
            if (node.isSimpleBlock(blocks) && !blocks.isBlockCard()) {
              blocks = node.replace(blocks, customizeItem);
              this.addCardToCustomize(blocks, cardName, value);
              blocks = node.wrap(blocks, customizeRoot);
            }

            return blocks;
        }
      }
    }
    /**
     * 将节点转换为列表节点
     * @param blocks 节点
     * @param tagName 列表节点名称，ul 或者 ol
     * @param start 有序列表开始序号
     */

  }, {
    key: "toNormal",
    value: function toNormal(blocks) {
      var _this7 = this,
          _blocks$parent;

      var tagName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'ul';
      var start = arguments.length > 2 ? arguments[2] : undefined;
      var node = this.editor.node;

      if (Array.isArray(blocks)) {
        var nodes = [];
        blocks.forEach(function (block) {
          var node = _this7.toNormal(block, tagName, start);

          nodes = nodes.concat(node);
        });
        return nodes;
      } else {
        this.unwrapCustomize(blocks);
        var indent;
        var targetNode = $('<'.concat(tagName, ' />'));
        var itemNode = $('<li />');

        switch (blocks.name) {
          case 'li':
          case tagName:
            return blocks;

          case 'ol':
          case 'ul':
            targetNode.attributes(blocks.attributes());
            if (targetNode.name === 'ul') targetNode.removeAttributes('start');
            blocks = node.replace(blocks, targetNode);
            return blocks;

          case 'p':
            if (((_blocks$parent = blocks.parent()) === null || _blocks$parent === void 0 ? void 0 : _blocks$parent.name) === 'li') {
              node.unwrap(blocks);
              return blocks;
            }

            indent = removeUnit(blocks.css('padding-left')) / 2;
            blocks = node.replace(blocks, itemNode);

            if (indent) {
              targetNode.attributes(this.INDENT_KEY, indent);
            }

            if (start) {
              targetNode.attributes('start', start);
            }

            blocks = node.wrap(blocks, targetNode);
            return blocks;

          default:
            if (node.isSimpleBlock(blocks) && !blocks.isBlockCard()) {
              blocks = node.replace(blocks, itemNode);
              blocks = node.wrap(blocks, targetNode);
            }

            return blocks;
        }
      }
    }
    /**
     * 判断选中的区域是否在List列表的开始
     */

  }, {
    key: "isFirst",
    value: function isFirst(range) {
      //获取选区开始节点和位置偏移值
      var startNode = range.startNode,
          startOffset = range.startOffset; //复制选区

      var cloneRange = range.cloneRange(); //找到li节点

      var node = 'li' === startNode.name ? startNode : startNode.closest('li'); //如果没有li节点

      if (!node[0]) return false; //让选区选择li节点

      cloneRange.select(node, true); //设置选区结束位置偏移值

      cloneRange.setEnd(startNode[0], startOffset); //复制选区内容

      var contents = cloneRange.cloneContents(); //如果选区中没有节点

      if (!contents.firstChild) return true; //如果选区中只有一个节点，并且是br标签

      if (1 === contents.childNodes.length && 'br' === $(contents.firstChild).name) return true; //如果选区中只有一个节点，并且是自定义列表并且第一个是Card

      if (1 === contents.childNodes.length && node.hasClass('data-list-item') && $(contents.firstChild).isCard()) return true;
      var nodeApi = this.editor.node; //如果选区中只有两个节点，并且是自定义列表并且第一个是Card，最后一个为空节点

      if (2 === contents.childNodes.length && node.hasClass('data-list-item') && $(contents.firstChild).isCard() && nodeApi.isEmpty($(contents.lastChild || []))) return true; //判断选区内容是否是空节点

      var block = $('<div />');
      block.append(contents);
      return nodeApi.isEmpty(block);
    }
    /**
     * 判断选中的区域是否在List列表的末尾
     */

  }, {
    key: "isLast",
    value: function isLast(range) {
      //获取选区范围结束节点和结束位置偏移值
      var endNode = range.endNode,
          endOffset = range.endOffset; //复制选区

      var cloneRange = range.cloneRange(); //找到li节点

      var node = 'li' === endNode.name ? endNode : endNode.closest('li'); //如果没有li节点

      if (!node[0]) return false; //让选区选择li节点

      cloneRange.select(node, true); //设置选区开始位置偏移值

      cloneRange.setStart(endNode, endOffset); //复制选区内容

      var contents = cloneRange.cloneContents(); //如果选区中没有节点

      if (!contents.firstChild) return true; //如果选区中只有一个节点，并且是br标签

      if (1 === contents.childNodes.length && 'br' === $(contents.firstChild).name) return true; //判断选区内容是否是空节点

      var block = $('<div />');
      block.append(contents);
      return this.editor.node.isEmpty(block);
    }
  }]);

  return List;
}();

var Backspace$3 = /*#__PURE__*/function () {
  function Backspace(engine) {
    _classCallCheck(this, Backspace);

    this.type = 'keydown';
    this.hotkey = 'backspace';
    this.engine = void 0;
    this.listeners = [];
    this.engine = engine;
  }

  _createClass(Backspace, [{
    key: "on",
    value: function on(listener) {
      this.listeners.push(listener);
    }
  }, {
    key: "off",
    value: function off(listener) {
      for (var i = 0; i < this.listeners.length; i++) {
        if (this.listeners[i] === listener) {
          this.listeners.splice(i, 1);
          break;
        }
      }
    }
  }, {
    key: "trigger",
    value: function trigger(event) {
      var change = this.engine.change;
      var range = change.getRange(); // 编辑器没有内容

      if (change.isEmpty()) {
        event.preventDefault();
        change.setValue('<p><br /><cursor /></p>');
        return;
      } // 处理 BR


      var startNode = range.startNode,
          startOffset = range.startOffset;

      if (startNode.isEditable()) {
        var child = startNode[0].childNodes[startOffset - 1];
        var lastNode = $(child);

        if (lastNode.name === 'br') {
          event.preventDefault();
          lastNode.remove();
          return;
        }
      }

      var result = true;

      for (var i = 0; i < this.listeners.length; i++) {
        var listener = this.listeners[i];
        result = listener(event);
        if (result === false) break;
      }

      if (result === false) return; // 范围为展开状态

      if (!range.collapsed) {
        event.preventDefault();
        change.deleteContent();
        return;
      }
    }
  }, {
    key: "destroy",
    value: function destroy() {
      this.listeners = [];
    }
  }]);

  return Backspace;
}();

var DefaultKeydown = /*#__PURE__*/function () {
  function DefaultKeydown(engine) {
    _classCallCheck(this, DefaultKeydown);

    this.type = 'keydown';
    this.hotkey = '';
    this.engine = void 0;
    this.listeners = [];
    this.engine = engine;
  }

  _createClass(DefaultKeydown, [{
    key: "on",
    value: function on(listener) {
      this.listeners.push(listener);
    }
  }, {
    key: "off",
    value: function off(listener) {
      for (var i = 0; i < this.listeners.length; i++) {
        if (this.listeners[i] === listener) {
          this.listeners.splice(i, 1);
          break;
        }
      }
    }
  }, {
    key: "trigger",
    value: function trigger(event) {
      for (var i = 0; i < this.listeners.length; i++) {
        var listener = this.listeners[i];
        var result = listener(event);
        if (result === false) break;
      }
    }
  }, {
    key: "destroy",
    value: function destroy() {
      this.listeners = [];
    }
  }]);

  return DefaultKeydown;
}();

var Delete = /*#__PURE__*/function () {
  function Delete(engine) {
    _classCallCheck(this, Delete);

    this.engine = void 0;
    this.type = 'keydown';
    this.hotkey = 'delete';
    this.listeners = [];
    this.engine = engine;
  }

  _createClass(Delete, [{
    key: "on",
    value: function on(listener) {
      this.listeners.push(listener);
    }
  }, {
    key: "off",
    value: function off(listener) {
      for (var i = 0; i < this.listeners.length; i++) {
        if (this.listeners[i] === listener) {
          this.listeners.splice(i, 1);
          break;
        }
      }
    }
  }, {
    key: "getNext",
    value: function getNext(node) {
      return $(node).isEditable() ? null : node.nextSibling ? node.nextSibling : node.parentNode === null ? null : this.getNext(node.parentNode);
    }
  }, {
    key: "getRange",
    value: function getRange(node) {
      var hasNext = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      if ($(node).isEditable()) return null;

      if (!hasNext) {
        var next = this.getNext(node);
        if (!next) return null;
        node = next;
      }

      while (node) {
        var nodeDom = $(node);

        if (nodeDom.attributes(CARD_KEY)) {
          if (!node.ownerDocument) return null;
          var range = Range.create(this.engine, node.ownerDocument);
          range.setStartAfter(node);
          range.collapse(true);
          return range;
        }

        if (this.engine.node.isBlock(nodeDom)) {
          if (!node.ownerDocument) return null;

          var _range = Range.create(this.engine, node.ownerDocument);

          _range.select(nodeDom, true).collapse(true);

          return _range;
        }

        if (nodeDom.name === 'br') {
          var _node$parentNode;

          if (((_node$parentNode = node.parentNode) === null || _node$parentNode === void 0 ? void 0 : _node$parentNode.childNodes.length) === 1) return null;
          if (!node.ownerDocument) return null;

          var _range2 = Range.create(this.engine, node.ownerDocument);

          _range2.setStartAfter(node);

          _range2.collapse(true);

          return _range2;
        }

        if (node.nodeType === getWindow().Node.TEXT_NODE) {
          if (node['data'].length === 0) return this.getRange(node);
          if (!node.ownerDocument) return null;

          var _range3 = Range.create(this.engine, node.ownerDocument);

          _range3.setStart(node, 1);

          _range3.collapse(true);

          return _range3;
        }

        if (node.childNodes.length === 0) return this.getRange(node);
        node = node.childNodes[0];
      }

      return null;
    }
  }, {
    key: "trigger",
    value: function trigger(event) {
      var change = this.engine.change;
      var range = change.getRange();

      if (!range.collapsed) {
        event.preventDefault();
        change.deleteContent();
        return;
      }

      var card = this.engine.card.find(range.startNode);
      var hasNext = false;
      var nextNode;

      if (card) {
        if (card.isLeftCursor(range.startNode)) {
          event.preventDefault();
          this.engine.card.select(card);
          change.deleteContent();
          return;
        }

        if (!card.isRightCursor(range.startNode)) return;
        nextNode = card.root[0];
      } else if (range.endContainer.nodeType === getWindow().Node.TEXT_NODE) {
        if (range.endContainer['data'].length > range.endOffset) {
          event.preventDefault();
          var cloneRange = range.cloneRange();
          cloneRange.setEnd(range.endContainer, range.endOffset + 1);
          change.select(cloneRange);
          change.deleteContent();
          return;
        }

        nextNode = range.endContainer;
      } else {
        if (range.endContainer.nodeType !== getWindow().Node.ELEMENT_NODE) return;

        if (range.endContainer.childNodes.length === 0) {
          nextNode = range.endContainer;
        } else if (range.endOffset === 0) {
          var _range$endContainer$f;

          if (range.endContainer.childNodes.length !== 1 || ((_range$endContainer$f = range.endContainer.firstChild) === null || _range$endContainer$f === void 0 ? void 0 : _range$endContainer$f.nodeName) !== 'BR') {
            hasNext = true;
          }

          nextNode = range.endContainer.childNodes[range.endOffset];
        } else {
          nextNode = range.endContainer.childNodes[range.endOffset - 1];
        }
      }

      var nodeRange = this.getRange(nextNode, hasNext);

      if (nodeRange) {
        event.preventDefault();
        var startOffset = range.startOffset;
        if (startOffset === 1 && range.startContainer.childNodes.length === 1 && range.startContainer.childNodes[0].nodeName === 'BR') startOffset = 0;
        nodeRange.setStart(range.startContainer, startOffset);
        change.select(nodeRange);
        change.deleteContent();
      }

      for (var i = 0; i < this.listeners.length; i++) {
        var listener = this.listeners[i];
        var result = listener(event);
        if (result === false) break;
      }
    }
  }, {
    key: "destroy",
    value: function destroy() {
      this.listeners = [];
    }
  }]);

  return Delete;
}();

var Enter$2 = /*#__PURE__*/function () {
  function Enter(engine) {
    _classCallCheck(this, Enter);

    this.type = 'keydown';
    this.hotkey = 'enter';
    this.engine = void 0;
    this.listeners = [];
    this.engine = engine;
  }

  _createClass(Enter, [{
    key: "on",
    value: function on(listener) {
      this.listeners.push(listener);
    }
  }, {
    key: "off",
    value: function off(listener) {
      for (var i = 0; i < this.listeners.length; i++) {
        if (this.listeners[i] === listener) {
          this.listeners.splice(i, 1);
          break;
        }
      }
    }
  }, {
    key: "trigger",
    value: function trigger(event) {
      var change = this.engine.change;
      var range = change.getRange(); // 选区选中最后的节点

      var block = this.engine.block.closest(range.endNode); // 无段落

      if (block.isEditable()) {
        this.engine.block.wrap('<p />');
      }

      for (var i = 0; i < this.listeners.length; i++) {
        var listener = this.listeners[i];
        var result = listener(event);
        if (result === false) break;
      }

      if (this.engine.scrollNode) this.engine.change.getRange().scrollIntoViewIfNeeded(this.engine.container, this.engine.scrollNode);
    }
  }, {
    key: "destroy",
    value: function destroy() {
      this.listeners = [];
    }
  }]);

  return Enter;
}();

var Tab = /*#__PURE__*/function () {
  function Tab(engine) {
    _classCallCheck(this, Tab);

    this.engine = void 0;
    this.type = 'keydown';
    this.hotkey = 'tab';
    this.listeners = [];
    this.engine = engine;
  }

  _createClass(Tab, [{
    key: "on",
    value: function on(listener) {
      this.listeners.push(listener);
    }
  }, {
    key: "off",
    value: function off(listener) {
      for (var i = 0; i < this.listeners.length; i++) {
        if (this.listeners[i] === listener) {
          this.listeners.splice(i, 1);
          break;
        }
      }
    }
  }, {
    key: "trigger",
    value: function trigger(event) {
      var node = this.engine.node;
      event.preventDefault();
      node.insertText('    ');
    }
  }, {
    key: "destroy",
    value: function destroy() {
      this.listeners = [];
    }
  }]);

  return Tab;
}();

var ShiftTab = /*#__PURE__*/function (_Default) {
  _inherits(ShiftTab, _Default);

  var _super = _createSuper(ShiftTab);

  function ShiftTab() {
    var _this;

    _classCallCheck(this, ShiftTab);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));
    _this.hotkey = 'shift+tab';
    return _this;
  }

  return ShiftTab;
}(DefaultKeydown);

var At = /*#__PURE__*/function (_Default) {
  _inherits(At, _Default);

  var _super = _createSuper(At);

  function At() {
    var _this;

    _classCallCheck(this, At);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));

    _this.hotkey = function (event) {
      return event.key === '@' || event.shiftKey && event.keyCode === 229 && event.code === 'Digit2';
    };

    return _this;
  }

  return At;
}(DefaultKeydown);

var Space = /*#__PURE__*/function (_Default) {
  _inherits(Space, _Default);

  var _super = _createSuper(Space);

  function Space() {
    var _this;

    _classCallCheck(this, Space);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));

    _this.hotkey = function (event) {
      return event.key === ' ';
    };

    return _this;
  }

  return Space;
}(DefaultKeydown);

var Slash = /*#__PURE__*/function (_Default) {
  _inherits(Slash, _Default);

  var _super = _createSuper(Slash);

  function Slash() {
    var _this;

    _classCallCheck(this, Slash);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));

    _this.hotkey = function (event) {
      return event.key === '/' || isHotkey('/', event) || event.keyCode === 229 && event.code === 'Slash';
    };

    return _this;
  }

  return Slash;
}(DefaultKeydown);

var All = /*#__PURE__*/function (_Default) {
  _inherits(All, _Default);

  var _super = _createSuper(All);

  function All() {
    var _this;

    _classCallCheck(this, All);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));
    _this.hotkey = 'mod+a';
    return _this;
  }

  return All;
}(DefaultKeydown);

var Left$1 = /*#__PURE__*/function () {
  function Left() {
    _classCallCheck(this, Left);

    this.type = 'keydown';

    this.hotkey = function (event) {
      return isHotkey('left', event) || isHotkey('shift+left', event) || isHotkey('ctrl+a', event) || isHotkey('ctrl+b', event);
    };

    this.listeners = [];
  }

  _createClass(Left, [{
    key: "on",
    value: function on(listener) {
      this.listeners.push(listener);
    }
  }, {
    key: "off",
    value: function off(listener) {
      for (var i = 0; i < this.listeners.length; i++) {
        if (this.listeners[i] === listener) {
          this.listeners.splice(i, 1);
          break;
        }
      }
    }
  }, {
    key: "trigger",
    value: function trigger(event) {
      for (var i = 0; i < this.listeners.length; i++) {
        var listener = this.listeners[i];
        var result = listener(event);
        if (result === false) break;
      }
    }
  }, {
    key: "destroy",
    value: function destroy() {
      this.listeners = [];
    }
  }]);

  return Left;
}();

var Right$1 = /*#__PURE__*/function () {
  function Right(engine) {
    _classCallCheck(this, Right);

    this.type = 'keydown';

    this.hotkey = function (event) {
      return isHotkey('right', event) || isHotkey('shift+right', event) || isHotkey('ctrl+e', event) || isHotkey('ctrl+f', event);
    };

    this.engine = void 0;
    this.listeners = [];
    this.engine = engine;
  }

  _createClass(Right, [{
    key: "on",
    value: function on(listener) {
      this.listeners.push(listener);
    }
  }, {
    key: "off",
    value: function off(listener) {
      for (var i = 0; i < this.listeners.length; i++) {
        if (this.listeners[i] === listener) {
          this.listeners.splice(i, 1);
          break;
        }
      }
    }
  }, {
    key: "trigger",
    value: function trigger(event) {
      for (var i = 0; i < this.listeners.length; i++) {
        var listener = this.listeners[i];
        var result = listener(event);
        if (result === false) break;
      }
    }
  }, {
    key: "destroy",
    value: function destroy() {
      this.listeners = [];
    }
  }]);

  return Right;
}();

var Up$1 = /*#__PURE__*/function (_Default) {
  _inherits(Up, _Default);

  var _super = _createSuper(Up);

  function Up() {
    var _this;

    _classCallCheck(this, Up);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));

    _this.hotkey = function (event) {
      return isHotkey('up', event) || isHotkey('ctrl+p', event);
    };

    return _this;
  }

  return Up;
}(DefaultKeydown);

var Down$1 = /*#__PURE__*/function (_Default) {
  _inherits(Down, _Default);

  var _super = _createSuper(Down);

  function Down() {
    var _this;

    _classCallCheck(this, Down);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));

    _this.hotkey = function (event) {
      return isHotkey('down', event) || isHotkey('ctrl+n', event);
    };

    return _this;
  }

  return Down;
}(DefaultKeydown);

var defaultHandles = [{
  name: 'default',
  handle: DefaultKeydown
}, {
  name: 'enter',
  handle: Enter$2,
  triggerName: 'keydown:enter'
}, {
  name: 'backspace',
  handle: Backspace$3,
  triggerName: 'keydown:backspace'
}, {
  name: 'delete',
  handle: Delete,
  triggerName: 'keydown:backspace'
}, {
  name: 'tab',
  handle: Tab,
  triggerName: 'keydown:tab'
}, {
  name: 'shift-tab',
  handle: ShiftTab,
  triggerName: 'keydown:shift-tab'
}, {
  name: 'at',
  handle: At,
  triggerName: 'keydown:at'
}, {
  name: 'space',
  handle: Space,
  triggerName: 'keydown:space'
}, {
  name: 'slash',
  handle: Slash,
  triggerName: 'keydown:slash'
}, {
  name: 'all',
  handle: All,
  triggerName: 'keydown:all'
}, {
  name: 'left',
  handle: Left$1,
  triggerName: 'keydown:left'
}, {
  name: 'right',
  handle: Right$1,
  triggerName: 'keydown:right'
}, {
  name: 'up',
  handle: Up$1,
  triggerName: 'keydown:up'
}, {
  name: 'down',
  handle: Down$1,
  triggerName: 'keydown:down'
}];

var DefaultKeyup = /*#__PURE__*/function (_DefaultKeydown) {
  _inherits(DefaultKeyup, _DefaultKeydown);

  var _super = _createSuper(DefaultKeyup);

  function DefaultKeyup() {
    var _this;

    _classCallCheck(this, DefaultKeyup);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));
    _this.type = 'keyup';
    return _this;
  }

  return DefaultKeyup;
}(DefaultKeydown);

var Enter$3 = /*#__PURE__*/function (_Default) {
  _inherits(Enter, _Default);

  var _super = _createSuper(Enter);

  function Enter() {
    var _this;

    _classCallCheck(this, Enter);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));
    _this.hotkey = 'enter';
    return _this;
  }

  return Enter;
}(DefaultKeyup);

var Backspace$4 = /*#__PURE__*/function () {
  function Backspace(engine) {
    _classCallCheck(this, Backspace);

    this.type = 'keyup';
    this.hotkey = 'backspace';
    this.engine = void 0;
    this.listeners = [];
    this.engine = engine;
  }

  _createClass(Backspace, [{
    key: "on",
    value: function on(listener) {
      this.listeners.push(listener);
    }
  }, {
    key: "off",
    value: function off(listener) {
      for (var i = 0; i < this.listeners.length; i++) {
        if (this.listeners[i] === listener) {
          this.listeners.splice(i, 1);
          break;
        }
      }
    }
  }, {
    key: "trigger",
    value: function trigger(event) {
      var change = this.engine.change;
      var range = change.getRange(); // 编辑器没有内容

      if (change.isEmpty()) {
        event.preventDefault();
        change.setValue('<p><br /><cursor /></p>');
        return;
      }

      var result = true;

      for (var i = 0; i < this.listeners.length; i++) {
        var listener = this.listeners[i];
        result = listener(event);
        if (result === false) break;
      }
    }
  }, {
    key: "destroy",
    value: function destroy() {
      this.listeners = [];
    }
  }]);

  return Backspace;
}();

var Tab$1 = /*#__PURE__*/function (_Default) {
  _inherits(Tab, _Default);

  var _super = _createSuper(Tab);

  function Tab() {
    var _this;

    _classCallCheck(this, Tab);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));
    _this.hotkey = 'tab';
    return _this;
  }

  return Tab;
}(DefaultKeyup);

var Space$1 = /*#__PURE__*/function (_Default) {
  _inherits(Space, _Default);

  var _super = _createSuper(Space);

  function Space() {
    var _this;

    _classCallCheck(this, Space);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));

    _this.hotkey = function (event) {
      return event.key === ' ';
    };

    return _this;
  }

  return Space;
}(DefaultKeyup);

var defaultHandles$1 = [{
  name: 'default',
  handle: DefaultKeyup
}, {
  name: 'enter',
  handle: Enter$3,
  triggerName: 'keyup:enter'
}, {
  name: 'backspace',
  handle: Backspace$4,
  triggerName: 'keyup:backspace'
}, {
  name: 'tab',
  handle: Tab$1,
  triggerName: 'keyup:tab'
}, {
  name: 'space',
  handle: Space$1,
  triggerName: 'keyup:space'
}];

var Typing = /*#__PURE__*/function () {
  function Typing(engine) {
    var _this = this;

    _classCallCheck(this, Typing);

    this.engine = void 0;
    this.handleListeners = [];
    this.engine = engine;
    defaultHandles.concat(defaultHandles$1).forEach(function (handle) {
      _this.addHandleListener(handle.name, handle.handle, handle.triggerName);
    });
    var container = engine.container;
    container.on('keydown', function (e) {
      return _this.bindKeydown(e);
    });
    container.on('keyup', function (e) {
      return _this.bindKeyup(e);
    });
  }

  _createClass(Typing, [{
    key: "addHandleListener",
    value: function addHandleListener(name, handle, triggerName, triggerParams) {
      this.handleListeners.push({
        name: name,
        handle: new handle(this.engine),
        triggerName: triggerName,
        triggerParams: triggerParams
      });
    }
  }, {
    key: "getHandleListener",
    value: function getHandleListener(name, type) {
      var _this$handleListeners;

      return (_this$handleListeners = this.handleListeners.find(function (listener) {
        return listener.name === name && listener.handle.type === type;
      })) === null || _this$handleListeners === void 0 ? void 0 : _this$handleListeners.handle;
    }
  }, {
    key: "removeHandleListener",
    value: function removeHandleListener(name, type) {
      for (var i = 0; i < this.handleListeners.length; i++) {
        if (this.handleListeners[i].name === name && this.handleListeners[i].handle.type === type) {
          this.handleListeners[i].handle.destroy();
          this.handleListeners.splice(i, 1);
          break;
        }
      }
    }
  }, {
    key: "bindKeydown",
    value: function bindKeydown(event) {
      var _this$engine = this.engine,
          readonly = _this$engine.readonly,
          card = _this$engine.card; //只读状态

      if (readonly) {
        //全选禁止默认事件触发
        if (isHotkey('mod+a', event)) event.preventDefault();
        return;
      } //跳过卡片


      if (event.target && card.find($(event.target))) return;
      this.trigger('keydown', event);
    }
  }, {
    key: "bindKeyup",
    value: function bindKeyup(event) {
      var _this$engine2 = this.engine,
          readonly = _this$engine2.readonly,
          card = _this$engine2.card; //只读状态

      if (readonly) return; //跳过卡片

      if (event.target && card.find($(event.target))) return;
      this.trigger('keyup', event);
    }
  }, {
    key: "trigger",
    value: function trigger(type, event) {
      var _this2 = this;

      //循环事件
      var result = this.handleListeners.filter(function (_ref) {
        var handle = _ref.handle;
        return handle.type === type;
      }).some(function (listener) {
        var name = listener.name,
            handle = listener.handle,
            triggerName = listener.triggerName,
            triggerParams = listener.triggerParams;
        if (name === 'default' || !!!handle.hotkey) return false;

        if (typeof handle.hotkey === 'function' ? handle.hotkey(event) : isHotkey(handle.hotkey, event)) {
          var _this2$engine;

          var params = [event];
          if (typeof triggerParams === 'function') params = triggerParams(_this2.engine, event);

          if (!triggerName || (_this2$engine = _this2.engine).trigger.apply(_this2$engine, [triggerName].concat(_toConsumableArray(params))) !== false) {
            handle.trigger(event);
          }

          return true;
        }

        return false;
      }); //触发默认事件

      if (result === false) {
        var _this$getHandleListen;

        (_this$getHandleListen = this.getHandleListener('default', type)) === null || _this$getHandleListen === void 0 ? void 0 : _this$getHandleListen.trigger(event);
      }
    }
  }]);

  return Typing;
}();

var Container = /*#__PURE__*/function () {
  function Container(selector, options) {
    _classCallCheck(this, Container);

    this.options = void 0;
    this.node = void 0;
    this._focused = false;
    this.node = $(selector);
    this.options = options;

    this._init();

    this._focused = document.activeElement !== null && this.node.equal(document.activeElement);
  }

  _createClass(Container, [{
    key: "_init",
    value: function _init() {
      var _this = this;

      var _this$options = this.options,
          lang = _this$options.lang,
          tabIndex = _this$options.tabIndex,
          className = _this$options.className;
      this.node.attributes(DATA_ELEMENT, ROOT);
      this.node.attributes({
        contenteditable: 'true',
        role: 'textbox',
        autocorrect: lang === 'en' ? 'on' : 'off',
        autocomplete: 'off',
        spellcheck: lang === 'en' ? 'true' : 'false',
        'data-gramm': 'false'
      });

      if (tabIndex !== undefined) {
        this.node.attributes('tabindex', tabIndex);
      }

      if (!this.node.hasClass('am-engine')) {
        this.node.addClass('am-engine');
      }

      if (isMobile) this.node.addClass('am-engine-mobile');

      if (className !== undefined) {
        (Array.isArray(className) ? className : className.split(/\s+/)).forEach(function (name) {
          if (name.trim() !== '') _this.node.addClass(name);
        });
      }
    }
  }, {
    key: "init",
    value: function init() {
      var _this2 = this;

      var engine = this.options.engine;
      this.node.on('input', function (e) {
        if (engine.readonly) {
          return;
        }

        if (engine.card.find(e.target)) {
          return;
        }

        var range = engine.change.getRange();
        range.addOrRemoveBr(true);
      }); // 编辑器文档尾部始终保持一行

      this.node.on('click', function (event) {
        if (event.target && $(event.target).isEditable()) {
          //获取到编辑器内最后一个子节点
          var block = _this2.node.last();

          if (block) {
            var _block$get, _block$get2;

            //不是卡片不处理
            if (!block.isCard()) return; //节点不可见不处理

            if ((((_block$get = block.get()) === null || _block$get === void 0 ? void 0 : _block$get.offsetTop) || 0) + (((_block$get2 = block.get()) === null || _block$get2 === void 0 ? void 0 : _block$get2.clientHeight) || 0) > event.offsetY) return;
          }

          var node = $('<p><br /></p>');

          _this2.node.append(node);

          var range = engine.change.getRange();
          range.select(node, true).collapse(false);
          engine.change.select(range);
        }
      });
      this.node.on('focus', function () {
        _this2._focused = true;
        return engine.trigger('focus');
      });
      this.node.on('blur', function () {
        _this2._focused = false;
        return engine.trigger('blur');
      });
    }
  }, {
    key: "isFocus",
    value: function isFocus() {
      return this._focused;
    }
  }, {
    key: "getNode",
    value: function getNode() {
      return this.node;
    }
  }, {
    key: "setReadonly",
    value: function setReadonly(readonly) {
      this.node.attributes('contenteditable', readonly ? 'false' : 'true');
    }
  }, {
    key: "destroy",
    value: function destroy() {
      var _this3 = this;

      var _this$options2 = this.options,
          className = _this$options2.className,
          engine = _this$options2.engine;
      this.node.removeAttributes(DATA_ELEMENT);
      this.node.removeAttributes('contenteditable');
      this.node.removeAttributes('role');
      this.node.removeAttributes('autocorrect');
      this.node.removeAttributes('autocomplete');
      this.node.removeAttributes('spellcheck');
      this.node.removeAttributes('data-gramm');
      this.node.removeAttributes('tabindex');

      if (this.options.className) {
        (Array.isArray(className) ? className : (className || '').split(/\s+/)).forEach(function (name) {
          if (name.trim() !== '') _this3.node.removeClass(name);
        });
      }

      if (engine.card.closest(this.node)) this.node.removeClass('am-engine');
      this.node.removeAllEvents();
    }
  }]);

  return Container;
}();

var Backspace$5 = /*#__PURE__*/function () {
  function Backspace(engine) {
    _classCallCheck(this, Backspace);

    this.engine = void 0;
    this.engine = engine;
  }
  /**
   * 在inline节点处按下backspace键
   */


  _createClass(Backspace, [{
    key: "trigger",
    value: function trigger(event) {
      var _this$engine = this.engine,
          change = _this$engine.change,
          mark = _this$engine.mark,
          inline = _this$engine.inline,
          node = _this$engine.node;
      var range = change.getRange();

      var _range$cloneRange$shr = range.cloneRange().shrinkToTextNode(),
          collapsed = _range$cloneRange$shr.collapsed,
          endNode = _range$cloneRange$shr.endNode,
          startNode = _range$cloneRange$shr.startNode,
          startOffset = _range$cloneRange$shr.startOffset;

      if (endNode.type === Node.TEXT_NODE || startNode.type === Node.TEXT_NODE) {
        //光标展开的情况下，判断光标结束位置是否在inline节点内左侧零宽字符后面，并且inline节点为空
        if (!collapsed) {
          var _inlineNode = inline.closest(endNode);

          if (node.isInline(_inlineNode) && !_inlineNode.isCard() && node.isEmpty(_inlineNode)) {
            //offset 大于 1 说明至少不在左侧，不处理
            if (startOffset > 1) return true;
            var prev = endNode.prev();
            var parent = endNode.parent(); //位于inline 1级子节点下

            if (parent && node.isInline(parent)) {
              //前面还有节点，不处理
              if (prev) return true;
              var text = endNode.text();
              var leftText = text.substr(0, startOffset); //不位于零宽字符后，不处理

              if (!/^\u200b$/.test(leftText)) return true; //光标结束位置选中inline节点零宽字符后

              var inlineNext = _inlineNode.next();

              var nextText = inlineNext === null || inlineNext === void 0 ? void 0 : inlineNext.text();

              if (inlineNext && inlineNext.isText() && nextText && /^\u200b/.test(nextText)) {
                range.setEnd(inlineNext, 1);
                return false;
              }
            }
          }

          if (node.isInline(_inlineNode) && !_inlineNode.isCard()) {
            setTimeout(function () {
              inline.repairCursor(_inlineNode);
            }, 100);
          }

          return true;
        }

        var inlineNode = inline.closest(startNode); //开始节点在inline标签内

        if (node.isInline(inlineNode)) {
          if (inlineNode.isCard()) return true; //offset 大于 1 说明至少不在左侧，不处理

          if (startOffset > 1) return true;

          var _prev = startNode.prev();

          var _parent = startNode.parent(); //位于inline 1级子节点下


          if (_parent && node.isInline(_parent)) {
            //前面还有节点，不处理
            if (_prev) return true;

            var _text = startNode.text();

            var _leftText = _text.substr(0, startOffset); //不位于零宽字符后，不处理


            if (!/\u200b$/.test(_leftText)) return true;
          } // 其它内嵌节点内
          else if (startOffset === 0) {
              //循环判断是否处于inline节点内的第一个零宽字符后面，可能inline节点内包含多个mark或其它标签
              while (!_prev && _parent && !node.isInline(_parent)) {
                var _parent2;

                _prev = _parent.prev();
                _parent = (_parent2 = _parent) === null || _parent2 === void 0 ? void 0 : _parent2.parent();
              } //前面有节点，并且不是 text 节点，不处理


              if (_prev && !_prev.isText()) return true; //前面有text节点

              if (_prev) {
                //不位于零宽字符后，不处理
                if (!/\u200b$/.test(_prev.text())) return true;
              }
            } else return true; //让光标选择在inline节点前面零宽字符节点前面


          var inlinePrev = inlineNode.prev();
          var prevText = inlinePrev === null || inlinePrev === void 0 ? void 0 : inlinePrev.text();

          if (inlinePrev && inlinePrev.isText() && prevText && /\u200b$/.test(prevText)) {
            range.setStart(inlinePrev, prevText.length - 1); //如果inlne节点中没有内容了，选择在inline标签前后零宽字符两侧

            if (node.isEmpty(inlineNode)) {
              var _inlineNext = inlineNode.next();

              var _nextText = _inlineNext === null || _inlineNext === void 0 ? void 0 : _inlineNext.text();

              if (_inlineNext && _inlineNext.isText() && _nextText && /^\u200b/.test(_nextText)) {
                range.setEnd(_inlineNext, 1);
              }
            } else {
              range.collapse(true);
            }

            change.select(range);
            return false;
          }

          return true;
        } // 在inline节点外
        else {
            var _prev2 = startNode.prev();

            var _parent3 = startNode.parent();

            var _inlineNode2 = undefined; //在零宽字符后面，并且前面有inline节点

            if (_prev2) {
              if (node.isInline(_prev2) && !_prev2.isCard()) {
                var _text2 = startNode.text();

                var _leftText2 = _text2.substr(0, startOffset); //不位于零宽字符后，不处理


                if (!/^\u200b$/.test(_leftText2)) return true;
                _inlineNode2 = _prev2;
              }
            } //前方没有节点，考虑是否在mark节点或其它节点内
            else if (startOffset === 0) {
                //循环判断是否处于inline节点内的第一个零宽字符后面，可能inline节点内包含多个mark或其它标签
                while (!_prev2 && _parent3 && !node.isBlock(_parent3)) {
                  var _parent4;

                  _prev2 = _parent3.prev();
                  _parent3 = (_parent4 = _parent3) === null || _parent4 === void 0 ? void 0 : _parent4.parent();
                } //前面有节点，并且不是 text 节点，不处理


                if (_prev2 && !_prev2.isText()) return true; //前面有text节点

                if (_prev2) {
                  //不位于零宽字符后，不处理
                  if (!/^\u200b$/.test(_prev2.text())) return true; //位于零宽字符后，并且零宽字符前面是inline节点

                  _prev2 = _prev2.prev();

                  if (_prev2 && node.isInline(_prev2)) {
                    _inlineNode2 = _prev2;
                  } else return true;
                }
              } else return true; //让光标选择inline内部最后一个零宽字符前


            if (_inlineNode2) {
              var last = _inlineNode2.last();

              var _text3 = last === null || last === void 0 ? void 0 : last.text();

              if (last && last.isText() && _text3 && /\u200b$/.test(_text3)) {
                range.setStart(last, _text3.length - 1);
                range.collapse(true);
                change.select(range);
                return false;
              }
            }
          }
      }

      return true;
    }
  }]);

  return Backspace;
}();

var Left$2 = /*#__PURE__*/function () {
  function Left(engine) {
    _classCallCheck(this, Left);

    this.engine = void 0;
    this.engine = engine;
  }

  _createClass(Left, [{
    key: "trigger",
    value: function trigger(event) {
      var _this$engine = this.engine,
          change = _this$engine.change,
          inline = _this$engine.inline,
          node = _this$engine.node;
      var range = change.getRange().cloneRange().shrinkToTextNode();
      var startNode = range.startNode,
          startOffset = range.startOffset;
      var card = this.engine.card.getSingleCard(range);

      if (!card && startNode.type === Node.TEXT_NODE) {
        //&#8203<inline>&#8203<cursor />&#8203</inline>&#8203 -> <cursor />&#8203<inline>&#8203&#8203</inline>&#8203
        var inlineNode = inline.closest(startNode); //在inline节点内，靠近左侧位置

        if (node.isInline(inlineNode)) {
          if (inlineNode.isCard()) return; //offset 大于 1 说明至少不在左侧，不处理

          if (startOffset > 1) return true;
          var prev = startNode.prev();
          var parent = startNode.parent(); //位于inline 1级子节点下

          if (parent && node.isInline(parent)) {
            //前面还有节点，不处理
            if (prev) return true;
            var text = startNode.text();
            var leftText = text.substr(0, startOffset); //不位于零宽字符后，不处理

            if (!/^\u200b$/.test(leftText)) return true;
          } // 其它内嵌节点内
          else if (startOffset === 0) {
              //循环判断是否处于inline节点内的第一个零宽字符后面，可能inline节点内包含多个mark或其它标签
              while (!prev && parent && !node.isInline(parent)) {
                var _parent;

                prev = parent.prev();
                parent = (_parent = parent) === null || _parent === void 0 ? void 0 : _parent.parent();
              } //前面有节点，并且不是 text 节点，不处理


              if (prev && !prev.isText()) return true; //前面有text节点

              if (prev) {
                //不位于零宽字符后，不处理
                if (!/^\u200b$/.test(prev.text())) return true;
              }
            } else return true; //让光标选择在inline节点前面节点


          var inlinePrev = inlineNode.prev();
          var prevText = inlinePrev === null || inlinePrev === void 0 ? void 0 : inlinePrev.text();

          if (inlinePrev && inlinePrev.isText() && prevText && /\u200b$/.test(prevText)) {
            event.preventDefault();

            var _range$cloneRange = range.cloneRange(),
                collapsed = _range$cloneRange.collapsed;

            range.setStart(inlinePrev, prevText.length - 1);
            if (collapsed) range.collapse(true);
            change.select(range);
            return false;
          }

          return true;
        } // 在inline节点外
        else {
            var _prev = startNode.prev();

            var _parent2 = startNode.parent();

            var _inlineNode = undefined; //在零宽字符后面，并且前面有inline节点

            if (_prev) {
              if (node.isInline(_prev) && !_prev.isCard()) {
                var _text = startNode.text();

                var _leftText = _text.substr(0, startOffset); //不位于零宽字符后，不处理


                if (!/^\u200b$/.test(_leftText)) return true;
                _inlineNode = _prev;
              }
            } //前方没有节点，考虑是否在mark节点或其它节点内
            else if (startOffset === 0) {
                //循环判断是否处于inline节点内的第一个零宽字符后面，可能inline节点内包含多个mark或其它标签
                while (!_prev && _parent2 && !node.isBlock(_parent2)) {
                  var _parent3;

                  _prev = _parent2.prev();
                  _parent2 = (_parent3 = _parent2) === null || _parent3 === void 0 ? void 0 : _parent3.parent();
                } //前面有节点，并且不是 text 节点，不处理


                if (_prev && !_prev.isText()) return true; //前面有text节点

                if (_prev) {
                  //不位于零宽字符后，不处理
                  if (!/^\u200b$/.test(_prev.text())) return true; //位于零宽字符后，并且零宽字符前面是inline节点

                  _prev = _prev.prev();

                  if (_prev && node.isInline(_prev)) {
                    _inlineNode = _prev;
                  } else return true;
                }
              } else return true; //让光标选择inline内部最后一个零宽字符前


            if (_inlineNode) {
              event.preventDefault();

              var last = _inlineNode.last();

              var _text2 = last === null || last === void 0 ? void 0 : last.text();

              if (last && last.isText() && _text2 && /\u200b$/.test(_text2)) {
                var _range$cloneRange2 = range.cloneRange(),
                    _collapsed = _range$cloneRange2.collapsed;

                range.setStart(last, _text2.length - 1);

                if (_collapsed && !isHotkey('shift+left', event)) {
                  range.collapse(true);
                }

                change.select(range);
                return false;
              }
            }
          }
      }

      return true;
    }
  }]);

  return Left;
}();

var Right$2 = /*#__PURE__*/function () {
  function Right(engine) {
    _classCallCheck(this, Right);

    this.engine = void 0;
    this.engine = engine;
  }

  _createClass(Right, [{
    key: "trigger",
    value: function trigger(event) {
      var _this$engine = this.engine,
          change = _this$engine.change,
          inline = _this$engine.inline,
          node = _this$engine.node;
      var range = change.getRange().cloneRange().shrinkToTextNode();
      var endNode = range.endNode,
          endOffset = range.endOffset;
      var card = this.engine.card.getSingleCard(range);

      if (!card && endNode.type === Node.TEXT_NODE) {
        //&#8203<inline>&#8203<cursor />&#8203</inline>&#8203 -> &#8203<inline>&#8203&#8203</inline>&#8203<cursor />
        var inlineNode = inline.closest(endNode);
        var text = endNode.text(); //在inline节点内，靠近右侧位置

        if (node.isInline(inlineNode)) {
          if (inlineNode.isCard()) return; //offset 小于 文本长度 说明至少不在右侧，不处理

          if (endOffset < text.length - 1) return true;
          var next = endNode.next();
          var parent = endNode.parent(); //位于inline 1级子节点下

          if (parent && node.isInline(parent)) {
            //后面还有节点，不处理
            if (next) return true;
            var rightText = text.substr(endOffset); //不位于零宽字符前，不处理

            if (!/^\u200b$/.test(rightText)) return true;
          } // 其它内嵌节点内，并且开始offset在最后位置，或者在倒数第一前面的位置，浏览器默认会跳出inline节点
          else if (endOffset === text.length || endOffset === text.length - 1) {
              //循环判断是否处于inline节点内的第一个零宽字符前面，可能inline节点内包含多个mark或其它标签
              while (!next && parent && !node.isInline(parent)) {
                var _parent;

                next = parent.next();
                parent = (_parent = parent) === null || _parent === void 0 ? void 0 : _parent.parent();
              } //后面有节点，并且不是 text 节点，不处理


              if (next && !next.isText()) return true; //后面有text节点

              if (next) {
                //不位于零宽字符前，不处理
                if (!/^\u200b$/.test(next.text())) return true; //选中零宽字符前面

                if (endOffset === text.length - 1) {
                  event.preventDefault();

                  var _range$cloneRange = range.cloneRange(),
                      collapsed = _range$cloneRange.collapsed;

                  range.setEnd(next, 0);
                  if (collapsed) range.collapse(false);
                  change.select(range);
                  return false;
                }
              }
            } else return true; //让光标选择在inline节点的下一个零宽字符后面


          var inlineNext = inlineNode.next();
          var nextText = inlineNext === null || inlineNext === void 0 ? void 0 : inlineNext.text();

          if (inlineNext && inlineNext.isText() && nextText && /^\u200b/.test(nextText)) {
            event.preventDefault();

            var _range$cloneRange2 = range.cloneRange(),
                _collapsed = _range$cloneRange2.collapsed;

            range.setEnd(inlineNext, 1);
            if (_collapsed) range.collapse(false);
            change.select(range);
            return false;
          }

          return true;
        } // 在inline节点外
        else {
            var _next = endNode.next();

            var _parent2 = endNode.parent();

            var _inlineNode = undefined; //在零宽字符后面，并且后面有inline节点

            if (_next) {
              if (node.isInline(_next) && !_next.isCard()) {
                var _rightText = text.substr(endOffset); //不位于零宽字符前，不处理


                if (!/^\u200b$/.test(_rightText)) return true;
                _inlineNode = _next;
              }
            } //后方没有节点，考虑是否在mark节点或其它节点内
            else if (endOffset === text.length) {
                //循环判断是否处于inline节点内的最后一个零宽字符前面，可能inline节点内包含多个mark或其它标签
                while (!_next && _parent2 && !node.isBlock(_parent2)) {
                  var _parent3;

                  _next = _parent2.next();
                  _parent2 = (_parent3 = _parent2) === null || _parent3 === void 0 ? void 0 : _parent3.parent();
                } //后面有节点，并且不是 text 节点，不处理


                if (_next && !_next.isText()) return true; //后面有text节点

                if (_next) {
                  //不位于零宽字符前，不处理
                  if (!/^\u200b$/.test(_next.text())) return true; //位于零宽字符前，并且零宽字符后面是inline节点

                  _next = _next.next();

                  if (_next && node.isInline(_next)) {
                    _inlineNode = _next;
                  } else return true;
                }
              } else return true; //让光标选择inline内部最第一个零宽字符后


            if (_inlineNode) {
              event.preventDefault();

              var first = _inlineNode.first();

              var _text = first === null || first === void 0 ? void 0 : first.text();

              if (first && first.isText() && _text && /^\u200b/.test(_text)) {
                var _range$cloneRange3 = range.cloneRange(),
                    _collapsed2 = _range$cloneRange3.collapsed;

                range.setEnd(first, 1);

                if (_collapsed2 && !isHotkey('shift+right', event)) {
                  range.collapse(false);
                }

                change.select(range);
                return false;
              }
            }
          }
      }

      return true;
    }
  }]);

  return Right;
}();

var Inline = /*#__PURE__*/function () {
  function Inline(editor) {
    _classCallCheck(this, Inline);

    this.editor = void 0;
    this.editor = editor;
  }

  _createClass(Inline, [{
    key: "init",
    value: function init() {
      var _this = this;

      var editor = this.editor;

      if (isEngine(editor)) {
        var _typing$getHandleList, _typing$getHandleList2, _typing$getHandleList3;

        var typing = editor.typing,
            event = editor.event; //删除事件

        var backspace = new Backspace$5(editor);
        (_typing$getHandleList = typing.getHandleListener('backspace', 'keydown')) === null || _typing$getHandleList === void 0 ? void 0 : _typing$getHandleList.on(function (event) {
          return backspace.trigger(event);
        }); //左方向键

        var left = new Left$2(editor);
        (_typing$getHandleList2 = typing.getHandleListener('left', 'keydown')) === null || _typing$getHandleList2 === void 0 ? void 0 : _typing$getHandleList2.on(function (event) {
          return left.trigger(event);
        }); //右方向键

        var right = new Right$2(editor);
        (_typing$getHandleList3 = typing.getHandleListener('right', 'keydown')) === null || _typing$getHandleList3 === void 0 ? void 0 : _typing$getHandleList3.on(function (event) {
          return right.trigger(event);
        }); //markdown

        event.on('keydown:space', function (event) {
          return _this.triggerMarkdown(event);
        });
      }
    }
    /**
     * 修复光标选区位置，&#8203;<a>&#8203;<anchor />acde<focus />&#8203;</a>&#8203; -> <anchor />&#8203;<a>&#8203;acde&#8203;</a>&#8203;<focus />
     * 否则在ot中，可能无法正确的应用inline节点两边&#8203;的更改
     */

  }, {
    key: "repairRange",
    value: function repairRange(range) {
      var _this$editor = this.editor,
          change = _this$editor.change,
          node = _this$editor.node;
      range = range || change.getRange();
      var _range = range,
          startNode = _range.startNode,
          startOffset = _range.startOffset,
          endNode = _range.endNode,
          endOffset = _range.endOffset,
          collapsed = _range.collapsed;
      if (collapsed) return range;
      var startInline = this.closest(startNode); //让其选中节点外的 \u200b 零宽字符

      if (startInline && node.isInline(startInline) && startOffset <= 1) {
        //检测是否处于inline标签内部左侧
        var atBefore = true;
        var childNode = startNode;

        while (childNode && !childNode.equal(startInline)) {
          if (childNode.prev()) {
            atBefore = false;
            break;
          }

          childNode = childNode.parent();
        }

        if (atBefore) {
          var prev = startInline.prev();
          var text = (prev === null || prev === void 0 ? void 0 : prev.text()) || '';

          if (prev && prev.isText() && /\u200B$/g.test(text)) {
            range.setStart(prev, text.length - 1);
          }
        }
      }

      var endInline = this.closest(endNode);

      if (endInline && node.isInline(endInline) && endOffset <= endInline.last().text().length) {
        //检测是否处于inline标签内部右侧
        var atAfter = true;
        var _childNode = startNode;

        while (_childNode && !_childNode.equal(endInline)) {
          if (_childNode.next()) {
            atAfter = false;
            break;
          }

          _childNode = _childNode.parent();
        }

        if (atAfter) {
          var next = endInline.next();

          var _text = (next === null || next === void 0 ? void 0 : next.text()) || '';

          if (next && next.isText() && /^\u200B/g.test(_text)) {
            range.setEnd(next, 1);
          }
        }
      }

      return range;
    }
    /**
     * 解析markdown
     * @param event 事件
     */

  }, {
    key: "triggerMarkdown",
    value: function triggerMarkdown(event) {
      var editor = this.editor;
      if (!isEngine(editor)) return;
      var change = editor.change;
      var range = change.getRange();
      if (!range.collapsed || change.isComposing()) return;
      var startNode = range.startNode,
          startOffset = range.startOffset;
      var node = startNode.type === Node.TEXT_NODE ? startNode : startNode.children().eq(startOffset - 1);
      if (!node) return;
      var text = node.type === Node.TEXT_NODE ? node.text().substr(0, startOffset) : node.text();
      return !Object.keys(editor.plugin.components).some(function (pluginName) {
        var plugin = editor.plugin.components[pluginName];

        if (isInlinePlugin(plugin) && !!plugin.markdown) {
          var reuslt = plugin.triggerMarkdown(event, text, node);
          if (reuslt === false) return true;
        }

        return;
      });
    }
    /**
     * 获取最近的 Inline 节点，找不到返回 node
     */

  }, {
    key: "closest",
    value: function closest(node) {
      var nodeApi = this.editor.node;

      while (node && node.parent() && !nodeApi.isBlock(node)) {
        if (node.isEditable()) break;
        if (nodeApi.isInline(node)) return node;
        var parentNode = node.parent();
        if (!parentNode) break;
        node = parentNode;
      }

      return node;
    }
    /**
     * 获取向上第一个非 Inline 节点
     */

  }, {
    key: "closestNotInline",
    value: function closestNotInline(node) {
      var nodeApi = this.editor.node;

      while (nodeApi.isInline(node) || nodeApi.isMark(node) || node.isText()) {
        if (node.isEditable()) break;
        var parent = node.parent();
        if (!parent) break;
        node = parent;
      }

      return node;
    }
    /**
     * 给当前光标节点添加inline包裹
     * @param inline inline标签
     * @param range 光标，默认获取当前光标
     */

  }, {
    key: "wrap",
    value: function wrap(inline, range) {
      if (!isEngine(this.editor)) return;
      var _this$editor2 = this.editor,
          change = _this$editor2.change,
          mark = _this$editor2.mark,
          node = _this$editor2.node;
      var safeRange = range || change.getSafeRange();
      var doc = getDocument(safeRange.startContainer);

      if (typeof inline === 'string' || isNode(inline)) {
        inline = $(inline, doc);
      } else inline = inline;

      if (!node.isInline(inline)) return;

      if (safeRange.collapsed) {
        this.insert(inline, safeRange);
        if (!range) change.apply(safeRange);
        return;
      }

      mark.split(safeRange);
      this.split(safeRange);
      var commonAncestorNode = safeRange.commonAncestorNode;

      if (commonAncestorNode.type === getWindow().Node.TEXT_NODE || node.isMark(commonAncestorNode)) {
        commonAncestorNode = commonAncestorNode.parent();

        while (node.isMark(commonAncestorNode)) {
          commonAncestorNode = commonAncestorNode.parent();
        }
      } // 插入范围的开始和结束标记


      var selection = safeRange.enlargeToElementNode().createSelection();

      if (!selection.has()) {
        if (!range) change.apply(safeRange);
        return;
      } // 遍历范围内的节点，添加 Inline


      var started = false;
      var inlineClone = node.clone(inline, false);
      commonAncestorNode.traverse(function (child) {
        if (!child.equal(selection.anchor)) {
          if (started) {
            if (child.equal(selection.focus)) {
              started = false;
              return false;
            }

            if (node.isInline(child)) {
              var children = child.children();
              node.unwrap(child);
              child = children;
            }

            if (node.isMark(child) && !child.isCard() || child.isText()) {
              if (node.isEmpty(child)) {
                child.remove();
                return true;
              }

              if (!inlineClone.parent()) {
                child.before(inlineClone);
              }

              inlineClone.append(child);
              return true;
            }

            if (inlineClone[0].childNodes.length !== 0) {
              inlineClone = node.clone(inlineClone, false);
            }

            return;
          }

          return;
        } else {
          started = true;
          return;
        }
      });
      var anchor = selection.anchor,
          focus = selection.focus;
      var anchorParent = anchor === null || anchor === void 0 ? void 0 : anchor.parent();

      if (anchorParent && node.isRootBlock(anchorParent) && !anchor.prev() && !anchor.next()) {
        anchor.after('<br />');
      }

      if (!anchor.equal(focus)) {
        var focusParent = focus === null || focus === void 0 ? void 0 : focus.parent();

        if (focusParent && node.isRootBlock(focusParent) && !focus.prev() && !focus.next()) {
          focus.before('<br />');
        }
      }

      this.repairCursor(inlineClone);
      selection.move();
      safeRange.setStart(inlineClone.first(), 1);
      safeRange.setEnd(inlineClone.last(), inlineClone.last().text().length - 1);
      if (!range) change.apply(safeRange);
    }
    /**
     * 移除inline包裹
     * @param range 光标，默认当前编辑器光标,或者需要移除的inline节点
     */

  }, {
    key: "unwrap",
    value: function unwrap(range) {
      var editor = this.editor;
      if (!isEngine(editor)) return;
      var change = editor.change,
          mark = editor.mark;
      var safeRange = !range || !isRangeInterface(range) ? change.getSafeRange() : range;
      this.repairRange(safeRange);
      mark.split(safeRange);
      var inlineNodes = range && !isRangeInterface(range) ? [range] : this.findInlines(safeRange); // 清除 Inline

      var selection = safeRange.createSelection();
      inlineNodes.forEach(function (node) {
        var _prev, _next, _first, _last;

        var prev = node.prev();
        if (prev && prev.isCursor()) prev = prev.prev();
        var next = node.next();
        if (next && next.isCursor()) next = next.prev();
        var first = node.first();
        if (first && first.isCursor()) first = first.next();
        var prevText = ((_prev = prev) === null || _prev === void 0 ? void 0 : _prev.text()) || '';
        var nextText = ((_next = next) === null || _next === void 0 ? void 0 : _next.text()) || '';
        var firstText = ((_first = first) === null || _first === void 0 ? void 0 : _first.text()) || '';

        if (prev && prev.isText() && /\u200B$/g.test(prevText)) {
          if (/^\u200B$/g.test(prevText)) prev.remove();else prev.text(prevText.substr(0, prevText.length - 1));
        }

        if (next && next.isText() && /^\u200B/g.test(nextText)) {
          if (/^\u200B$/g.test(nextText)) next.remove();else next.text(nextText.substr(1));
        }

        if (first && first.isText() && /^\u200B/g.test(firstText)) {
          if (/^\u200B$/g.test(firstText)) first.remove();else {
            first.get().splitText(1);
            first.remove();
          }
        }

        var last = node.last();
        if (last && last.isCursor()) last = last.prev();
        var lastText = ((_last = last) === null || _last === void 0 ? void 0 : _last.text()) || '';

        if (last && last.isText() && /\u200B$/g.test(lastText)) {
          if (/^\u200B$/g.test(lastText)) last.remove();else last.get().splitText(lastText.length - 1).remove();
        }

        editor.node.unwrap(node);
      });
      selection.move();
      mark.merge(safeRange);
      if (!range) change.apply(safeRange);
    }
    /**
     * 插入inline标签
     * @param inline inline标签
     * @param range 光标
     */

  }, {
    key: "insert",
    value: function insert(inline, range) {
      var _node$insert;

      if (!isEngine(this.editor)) return;
      var _this$editor3 = this.editor,
          change = _this$editor3.change,
          node = _this$editor3.node,
          mark = _this$editor3.mark;
      var safeRange = range || change.getSafeRange();
      var doc = getDocument(safeRange.startContainer);

      if (typeof inline === 'string' || isNode(inline)) {
        inline = $(inline, doc);
      }

      if (!node.isInline(inline)) return; // 范围为折叠状态时先删除内容

      if (!safeRange.collapsed) {
        change.deleteContent(safeRange);
      }

      mark.split(safeRange); // 插入新 Inline

      (_node$insert = node.insert(inline, safeRange)) === null || _node$insert === void 0 ? void 0 : _node$insert.select(inline).collapse(false);

      if (inline.name !== 'br') {
        safeRange.addOrRemoveBr();
      }

      var hasChild = inline.children().length !== 0;
      this.repairCursor(inline); //如果有内容，就让光标选择在节点外的零宽字符前

      if (!inline.isCard()) {
        if (hasChild) {
          var next = inline.next();
          safeRange.setStart(next, 1);
          safeRange.setEnd(next, 1);
        } else {
          //如果没有子节点，就让光标选择在最后的零宽字符前面
          var last = inline.last();
          var text = last.text();
          safeRange.setStart(last, text.length - 1);
          safeRange.setEnd(last, text.length - 1);
        }
      }

      if (!range) change.apply(safeRange);
    }
    /**
     * 去除一个节点下的所有空 Inline callback 可以设置其它条件
     * @param root 节点
     * @param callback 回调
     */

  }, {
    key: "unwrapEmptyInlines",
    value: function unwrapEmptyInlines(root, callback) {
      var node = this.editor.node;
      var children = root.allChildren();
      children.forEach(function (childNode) {
        var child = $(childNode);

        if (node.isEmpty(child) && node.isInline(child) && (!callback || callback(child))) {
          node.unwrap(child);
        }
      });
    }
    /**
     * 在光标重叠位置时分割
     * @param range 光标
     */

  }, {
    key: "splitOnCollapsed",
    value: function splitOnCollapsed(range, keelpNode) {
      var _this2 = this;

      if (!range.collapsed) return; //扩大光标选区

      range.enlargeFromTextNode();
      range.shrinkToElementNode();
      var startNode = range.startNode;
      var startParent = startNode.parent(); //获取卡片

      var node = this.editor.node;
      var card = startNode.isCard() ? startNode : startNode.closest(CARD_SELECTOR);

      if ((card.length === 0 || card.attributes(CARD_TYPE_KEY) !== 'inline') && (node.isInline(startNode) || startParent && node.isInline(startParent))) {
        var _rightNodes$find;

        // 获取上面第一个非inline标签
        var parent = this.closestNotInline(startNode); // 插入范围的开始和结束标记

        var selection = range.createSelection(); // 获取标记左右两侧节点

        var left = selection.getNode(parent, 'left');
        var right = undefined;
        var keelpRoot = undefined;
        var keelpPath = [];

        if (keelpNode) {
          if (isNode(keelpNode)) keelpNode = $(keelpNode); // 获取需要跟踪节点的路径

          var path = keelpNode.getPath(parent.get());
          var cloneParent = parent.clone(true);
          keelpPath = path.slice(1); // 获取需要跟踪节点的root节点

          keelpRoot = $(cloneParent.getChildByPath(path.slice(0, 1)));
          right = selection.getNode(cloneParent, 'right', false);
        } else right = selection.getNode(parent, 'right'); // 删除空标签


        this.unwrapEmptyInlines(left);
        this.unwrapEmptyInlines(right); // 清空原父容器，用新的内容代替

        parent.empty();
        var leftChildren = left.children();
        var leftNodes = leftChildren.toArray();
        parent.append(leftChildren);
        var rightChildren = right.children();
        var rightNodes = rightChildren.toArray(); // 根据跟踪节点的root节点和path获取其在rightNodes中的新节点

        if (keelpRoot) keelpNode = (_rightNodes$find = rightNodes.find(function (node) {
          return node.equal(keelpRoot);
        })) === null || _rightNodes$find === void 0 ? void 0 : _rightNodes$find.getChildByPath(keelpPath);
        parent.append(rightChildren); // 找到卡片，重新设置卡片根节点的引用

        parent.find(CARD_SELECTOR).each(function (cardNode) {
          var cardComponent = _this2.editor.card.find(cardNode);

          if (cardComponent && !cardComponent.root.equal(cardNode)) {
            cardComponent.root[0] = cardNode;
          }
        }); // 重新设置范围
        //移除左右两边的 br 标签

        if (leftNodes.length === 1 && leftNodes[0].name === 'br') {
          leftNodes[0].remove();
          leftNodes.splice(0, 1);
        }

        if (rightNodes.length === 1 && rightNodes[0].name === 'br') {
          rightNodes[0].remove();
          rightNodes.splice(0, 1);
        }

        parent.traverse(function (child) {
          if (node.isInline(child)) {
            _this2.repairCursor(child);
          }
        });
      }

      return keelpNode;
    }
    /**
     * 在光标位置不重合时分割
     * @param range 光标
     * @param removeMark 要移除的空mark节点
     */

  }, {
    key: "splitOnExpanded",
    value: function splitOnExpanded(range) {
      var _this3 = this;

      if (range.collapsed) return;
      range.enlargeToElementNode();
      range.shrinkToElementNode();
      var startNode = range.startNode,
          endNode = range.endNode;
      var cardStart = startNode.isCard() ? startNode : startNode.closest(CARD_SELECTOR);
      var cardEnd = endNode.isCard() ? endNode : endNode.closest(CARD_SELECTOR);

      if (!(cardStart.length > 0 && 'inline' === cardStart.attributes(CARD_TYPE_KEY) || cardEnd.length > 0 && 'inline' === cardEnd.attributes(CARD_TYPE_KEY))) {
        //开始非inline标签父节点
        var startNotInlineParent = this.closestNotInline(startNode); //结束非inine标签父节点

        var endNotInlineParent = this.closestNotInline(endNode);

        if (!startNotInlineParent.equal(endNotInlineParent)) {
          //开始位置
          var startRange = range.cloneRange();
          startRange.collapse(true); //结束位置

          var endRange = range.cloneRange();
          endRange.collapse(false); //如果开始非inline标签父节点包含结束非inline标签父节点，那么分割的时候会清空 结束非inline标签父节点的内容进行重组。结束非inline标签父节点 将无非找到
          //所以需要从被包含的节点开始分割

          var keelpNode = undefined;
          var startOffset = startRange.startOffset;
          var endOffset = endRange.endOffset; //如果开始节点的父节点包含结尾父节点，会将结尾父节点删除重组，导致光标失效，需要先执行开始节点分割，并跟踪结尾节点

          if (startNotInlineParent.contains(endNotInlineParent)) {
            //先分割开始节点，并跟踪结尾节点
            keelpNode = this.splitOnCollapsed(startRange, endRange.endNode);
            range.setStart(startRange.startContainer, startRange.startOffset); //如果有跟踪到，重新设置结尾节点

            if (keelpNode) {
              endRange.setOffset(keelpNode, endOffset, endOffset);
            } //分割结尾节点


            this.splitOnCollapsed(endRange);
            range.setEnd(endRange.startContainer, endRange.startOffset);
          } else {
            //结尾父节点包含开始节点父节点
            //先分割结尾节点，并跟踪开始节点
            keelpNode = this.splitOnCollapsed(endRange, startRange.startNode);
            range.setEnd(endRange.startContainer, endRange.startOffset); //如果有跟踪到，重新设置开始节点

            if (keelpNode) {
              startRange.setOffset(keelpNode, startOffset, startOffset);
            } //分割开始节点


            this.splitOnCollapsed(startRange);
            range.setStart(startRange.startContainer, startRange.startOffset);
          }

          return;
        }

        var node = this.editor.node; // 节点不是Inline，文本节点时判断父节点

        var startParent = startNode.parent();
        var startIsInline = node.isInline(startNode) || startParent && node.isInline(startParent);
        var endParent = endNode.parent();
        var endIsInline = node.isInline(endNode) || endParent && node.isInline(endParent); // 开始节点和结束节点都不是Inline，无需分割

        if (!startIsInline && !endIsInline) {
          return;
        }

        var commonAncestorNode = range.commonAncestorNode;

        if (commonAncestorNode.isText()) {
          commonAncestorNode = commonAncestorNode.parent();
        } // 获取上面第一个非样式标签


        var parent = this.closestNotInline(commonAncestorNode); // 插入范围的开始和结束标记

        var selection = range.createSelection(); // 标记的左边

        var left = selection.getNode(parent, 'left'); // 标记的节点

        var center = selection.getNode(parent); // 标记的右边

        var right = selection.getNode(parent, 'right'); // 删除空标签

        this.unwrapEmptyInlines(left);
        this.unwrapEmptyInlines(right); // 清空原父容器，用新的内容代替

        parent.empty();
        parent.append(left.children());
        var centerChildren = center.children();
        var centerNodes = centerChildren.toArray();
        parent.append(centerChildren);
        parent.append(right.children()); // 找到卡片，重新设置卡片根节点的引用

        parent.find(CARD_SELECTOR).each(function (cardNode) {
          var cardComponent = _this3.editor.card.find(cardNode);

          if (cardComponent && !cardComponent.root.equal(cardNode)) {
            cardComponent.root[0] = cardNode;
          }
        });
        parent.traverse(function (child) {
          if (node.isInline(child)) {
            _this3.repairCursor(child);
          }
        }); // 重新设置范围

        range.setStartBefore(centerNodes[0][0]);
        range.setEndAfter(centerNodes[centerNodes.length - 1][0]);
      }
    }
    /**
     * 分割inline标签
     */

  }, {
    key: "split",
    value: function split(range) {
      if (!isEngine(this.editor)) return;
      var change = this.editor.change;
      var safeRange = range || change.getSafeRange();

      if (safeRange.collapsed) {
        this.splitOnCollapsed(safeRange);
      } else {
        this.splitOnExpanded(safeRange);
      }

      if (!range) change.apply(safeRange);
    }
    /**
     * 获取光标范围内的所有 inline 标签
     * @param range 光标
     */

  }, {
    key: "findInlines",
    value: function findInlines(range) {
      var cloneRange = range.cloneRange();
      var nodeApi = this.editor.node;

      var handleRange = function handleRange(allowBlock, range) {
        var toStart = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
        if (!range.collapsed) return;
        var startNode = range.startNode,
            startOffset = range.startOffset; //没有父节点

        var startParent = startNode.findParent();
        if (!startParent) return; //选择父节点内容

        var cloneRange = range.cloneRange();
        cloneRange.select(startParent, true); //开始位置

        if (toStart) {
          cloneRange.setEnd(startNode, startOffset);
          cloneRange.enlargeFromTextNode();
          cloneRange.enlargeToElementNode(true);
          var startChildren = startNode.children();
          var _endNode = cloneRange.endNode,
              endOffset = cloneRange.endOffset;

          var endChildren = _endNode.children();

          var endOffsetNode = endChildren.eq(endOffset);
          var startOffsetNode = startChildren.eq(startOffset) || startChildren.eq(startOffset - 1);
          if (!allowBlock && _endNode.type === Node.ELEMENT_NODE && endOffsetNode && nodeApi.isBlock(endOffsetNode) && (startNode.type !== Node.ELEMENT_NODE || !!startOffsetNode && !nodeApi.isBlock(startOffsetNode))) return;
          cloneRange.select(startParent, true);
          cloneRange.setStart(_endNode, endOffset);
          cloneRange.shrinkToElementNode();
          cloneRange.shrinkToTextNode();
          range.setStart(cloneRange.startContainer, cloneRange.startOffset);
          range.collapse(true);
        } else {
          cloneRange.setStart(startNode, startOffset);
          cloneRange.enlargeFromTextNode();
          cloneRange.enlargeToElementNode(true);

          var _startChildren = startNode.children();

          var startNodeClone = cloneRange.startNode;
          var startOffsetClone = cloneRange.startOffset;
          var startNodeCloneChildren = startNodeClone.children();

          var _startOffsetNode = startNodeCloneChildren.eq(startOffsetClone);

          var startChildrenOffsetNode = _startChildren.eq(startOffset) || _startChildren.eq(startOffset - 1);

          if (!allowBlock && startNodeClone.type === Node.ELEMENT_NODE && _startOffsetNode && nodeApi.isBlock(_startOffsetNode) && (startNode.type !== Node.ELEMENT_NODE || startChildrenOffsetNode && !nodeApi.isBlock(startChildrenOffsetNode))) return;
          cloneRange.select(startParent, true);
          cloneRange.setEnd(startNodeClone, startOffsetClone);
          cloneRange.shrinkToElementNode();
          cloneRange.shrinkToTextNode();
          range.setEnd(cloneRange.endContainer, cloneRange.endOffset);
          range.collapse(false);
        }
      }; // 左侧不动，只缩小右侧边界
      // <anchor /><strong>foo</strong><focus />bar
      // 改成
      // <anchor /><strong>foo<focus /></strong>bar


      if (!range.collapsed) {
        var leftRange = range.cloneRange();
        var rightRange = range.cloneRange();
        leftRange.collapse(true);
        rightRange.collapse(false);
        handleRange(true, leftRange, true);
        handleRange(true, rightRange);
        cloneRange.setStart(leftRange.startContainer, leftRange.startOffset), cloneRange.setEnd(rightRange.startContainer, rightRange.startOffset);
      }

      handleRange(false, cloneRange);
      var sc = cloneRange.startContainer;
      var so = cloneRange.startOffset;
      var ec = cloneRange.endContainer;
      var eo = cloneRange.endOffset;
      var startNode = sc;
      var endNode = ec;

      if (sc.nodeType === getWindow().Node.ELEMENT_NODE) {
        if (sc.childNodes[so]) {
          startNode = sc.childNodes[so] || sc;
        }
      }

      if (ec.nodeType === getWindow().Node.ELEMENT_NODE) {
        if (eo > 0 && ec.childNodes[eo - 1]) {
          endNode = ec.childNodes[eo - 1] || sc;
        }
      } // 折叠状态时，按右侧位置的方式处理


      if (range.collapsed) {
        startNode = endNode;
      } // 不存在时添加


      var addNode = function addNode(nodes, nodeB) {
        if (!nodes.some(function (nodeA) {
          return nodeA[0] === nodeB[0];
        })) {
          nodes.push(nodeB);
        }
      }; // 向上寻找


      var findNodes = function findNodes(node) {
        var nodes = [];

        while (node) {
          if (node.isEditable()) break;
          if (nodeApi.isInline(node)) nodes.push(node);
          var parent = node.parent();
          if (!parent) break;
          node = parent;
        }

        return nodes;
      };

      var nodes = findNodes($(startNode));

      if (!range.collapsed) {
        findNodes($(endNode)).forEach(function (nodeB) {
          return addNode(nodes, nodeB);
        });

        if (sc !== ec) {
          var isBegin = false;
          var isEnd = false;
          range.commonAncestorNode.traverse(function (child) {
            if (isEnd) return false; //节点不是开始节点

            if (!child.equal(sc)) {
              if (isBegin) {
                //节点是结束节点，标记为结束
                if (child.equal(ec)) {
                  isEnd = true;
                  return false;
                }

                if (nodeApi.isInline(child) && !child.attributes(CARD_KEY) && !child.attributes(CARD_ELEMENT_KEY)) {
                  addNode(nodes, child);
                }
              }
            } else {
              //如果是开始节点，标记为开始
              isBegin = true;
            }

            return;
          });
        }
      }

      return nodes;
    }
    /**
     * 修复inline节点光标占位符
     * @param node inlne 节点
     */

  }, {
    key: "repairCursor",
    value: function repairCursor(node) {
      var _firstChild, _last2;

      var nodeApi = this.editor.node;
      if (isNode(node)) node = $(node);
      if (!nodeApi.isInline(node) || nodeApi.isVoid(node) || node.isCard()) return;
      this.repairBoth(node);
      var firstChild = node.first();
      if ((_firstChild = firstChild) === null || _firstChild === void 0 ? void 0 : _firstChild.isCursor()) firstChild = firstChild.next();

      if (!firstChild || firstChild.type !== Node.TEXT_NODE || !/^\u200B/g.test(firstChild.text())) {
        if (!firstChild) node.append($("\u200B", null));else if (firstChild.isText()) {
          firstChild.text("\u200B" + firstChild.text());
        } else firstChild.before($("\u200B", null));
      }

      var last = node.last();
      if ((_last2 = last) === null || _last2 === void 0 ? void 0 : _last2.isCursor()) last = last.prev();

      if (last && (/^\u200B$/g.test(node.text()) || last.type !== Node.TEXT_NODE || !/\u200B$/g.test(last.text()))) {
        if (last.isText()) {
          last.text(last.text() + "\u200B");
        } else last.after($("\u200B", null));
      }
    }
    /**
     * 修复节点两侧零宽字符占位
     * @param node 节点
     */

  }, {
    key: "repairBoth",
    value: function repairBoth(node) {
      var nodeApi = this.editor.node;
      if (isNode(node)) node = $(node);

      if (node.parent() && !nodeApi.isVoid(node)) {
        var zeroNode = $("\u200B", null);
        var prev = node.prev();
        var prevPrev = prev === null || prev === void 0 ? void 0 : prev.prev();
        var prevText = (prev === null || prev === void 0 ? void 0 : prev.text()) || '';

        if (!prev || !prev.isText() || !/\u200B$/g.test(prevText) || prevPrev && nodeApi.isInline(prevPrev) && !/\u200B\u200B$/g.test(prevText)) {
          if (prev && prev.isText()) {
            prev.text(prevText + "\u200B");
          } else {
            node.before(nodeApi.clone(zeroNode, true));
          }
        } else if (prev && prev.isText() && /\u200B\u200B$/g.test(prevText) && prevPrev && !nodeApi.isInline(prevPrev)) {
          prev.text(prevText.substr(0, prevText.length - 1));
        }

        var next = node.next();
        var nextText = (next === null || next === void 0 ? void 0 : next.text()) || '';
        var nextNext = next === null || next === void 0 ? void 0 : next.next();

        if (!next || !next.isText() || !/^\u200B/g.test(nextText) || nextNext && nodeApi.isInline(nextNext) && !/^\u200B\u200B/g.test(nextText)) {
          if (next && next.isText()) {
            next.text("\u200B" + next.text());
          } else {
            node.after(this.editor.node.clone(zeroNode, true));

            if ((next === null || next === void 0 ? void 0 : next.name) === 'br') {
              next.remove();
            }
          }
        }
      }
    }
  }]);

  return Inline;
}();

var Enter$4 = /*#__PURE__*/function () {
  function Enter(engine) {
    _classCallCheck(this, Enter);

    this.engine = void 0;
    this.engine = engine;
  }

  _createClass(Enter, [{
    key: "trigger",
    value: function trigger(event) {
      var _this$engine = this.engine,
          change = _this$engine.change,
          node = _this$engine.node,
          list = _this$engine.list;
      var range = change.getRange(); // 选区选中最后的节点

      var blockApi = this.engine.block;
      var block = blockApi.closest(range.endNode); // 嵌套 block

      var parent = block.parent();

      if (parent && parent.inEditor() && node.isBlock(parent)) {
        var _range$startContainer;

        if ('li' === parent.name && 'p' === block.name) {
          var _block$first;

          if (1 === block.children().length && 'br' === ((_block$first = block.first()) === null || _block$first === void 0 ? void 0 : _block$first.name)) {
            block.first().remove();
          }

          var selection = range.createSelection();
          change.unwrapNode(block);
          selection.move();
          block = blockApi.closest(range.endNode);
        }

        if (range.collapsed && (range.startContainer.childNodes.length === 1 && 'BR' === ((_range$startContainer = range.startContainer.firstChild) === null || _range$startContainer === void 0 ? void 0 : _range$startContainer.nodeName) || blockApi.isLastOffset(range, 'end') && blockApi.isFirstOffset(range, 'end'))) {
          event.preventDefault();

          if (['li'].indexOf(parent.name) >= 0) {
            blockApi.unwrap('<'.concat(parent.name, ' />'));
            blockApi.setBlocks('<'.concat(parent.name, ' />'));
          } else {
            blockApi.unwrap('<'.concat(parent.name, ' />'));
            blockApi.setBlocks('<p />');
          }

          return false;
        }
      }

      if (node.isBlock(block) && (!parent || !node.isList(parent)) && !block.isCard()) {
        event.preventDefault();
        blockApi.insertOrSplit(range, block);
        return false;
      } // 列表


      if (block.name === 'li') {
        if (node.isCustomize(block)) {
          return;
        }

        event.preventDefault(); // <li>foo<cursor /><li>

        if (blockApi.isLastOffset(range, 'end')) {
          // <li><cursor /><li>
          if (range.collapsed && blockApi.isFirstOffset(range, 'end')) {
            var listRoot = block.closest('ul,ol');
            blockApi.unwrap('<'.concat(listRoot.name, ' />'));
            blockApi.setBlocks('<p />');
          } else {
            var li = $('<li><br /></li>');
            li.attributes(block.attributes());
            blockApi.insertEmptyBlock(range, li);
          }
        } else {
          blockApi.split();
        }

        list.merge();
        range.scrollIntoView();
        return false;
      }

      return true;
    }
  }]);

  return Enter;
}();

var Backspace$6 = /*#__PURE__*/function () {
  function Backspace(engine) {
    _classCallCheck(this, Backspace);

    this.engine = void 0;
    this.engine = engine;
  }

  _createClass(Backspace, [{
    key: "trigger",
    value: function trigger(event) {
      var _this$engine = this.engine,
          change = _this$engine.change,
          node = _this$engine.node,
          block = _this$engine.block,
          card = _this$engine.card;
      var range = change.getRange();
      if (!range.collapsed) return;
      var prevNode = range.getPrevNode(); // 光标前面有空 block，<h1><li><br /></li><cursor /></h1>

      if (prevNode && node.isBlock(prevNode) && node.isEmptyWithTrim(prevNode)) {
        event.preventDefault();
        change.addBrAfterDelete(prevNode);
        return false;
      } // 光标不在段落开始位置时


      var isCard = !!card.closest(range.startNode);

      if (!isCard && !block.isFirstOffset(range, 'start')) {
        var cloneRange = range.cloneRange().shrinkToElementNode().shrinkToTextNode();

        if (cloneRange.startContainer.nodeType === getWindow().Node.TEXT_NODE && function (range) {
          var _range = range,
              commonAncestorContainer = _range.commonAncestorContainer;

          if (range.collapsed && 1 === range.startOffset && range.startContainer === commonAncestorContainer && commonAncestorContainer.nodeType === getWindow().Node.TEXT_NODE) {
            var _commonAncestorContai, _commonAncestorContai2, _commonAncestorContai3;

            range = range.cloneRange();

            if ((((_commonAncestorContai = commonAncestorContainer.parentNode) === null || _commonAncestorContai === void 0 ? void 0 : (_commonAncestorContai2 = _commonAncestorContai.childNodes) === null || _commonAncestorContai2 === void 0 ? void 0 : _commonAncestorContai2.length) || 0) <= 1 && 1 === ((_commonAncestorContai3 = commonAncestorContainer.textContent) === null || _commonAncestorContai3 === void 0 ? void 0 : _commonAncestorContai3.length)) {
              var _range2 = range,
                  startNode = _range2.startNode,
                  startOffset = _range2.startOffset;
              var markNode = startNode.parent(); //开始节点在mark标签内

              if (markNode && node.isMark(markNode) && startOffset > 0) {
                var text = startNode.text();
                var leftText = text.substr(startOffset - 1, 1); //不位于零宽字符后，不处理

                if (/^\u200b$/.test(leftText)) {
                  //选中上一个节点
                  if (startOffset === 1) {
                    var prev = markNode.prev();

                    if (prev && !node.isEmpty(prev)) {
                      var _range$cloneRange$sel = range.cloneRange().select(prev, true).shrinkToTextNode().collapse(false),
                          _startNode = _range$cloneRange$sel.startNode,
                          _startOffset = _range$cloneRange$sel.startOffset;

                      range.setStart(_startNode, _startOffset - 1);
                    }
                  } else {
                    range.setStart(startNode, startOffset - 1);
                  }
                }
              }

              if (range.collapsed) range.select(commonAncestorContainer, true);
              change.deleteContent(range, true);
              change.apply(range);
              return true;
            }
          }

          return false;
        }(cloneRange)) {
          event.preventDefault();
          event['isDelete'] = true;
          change.change();
        }

        return;
      }

      var blockNode = block.closest(range.startNode); // 在正文里

      if (!isCard && node.isRootBlock(blockNode)) {
        event.preventDefault();
        change.mergeAfterDeletePrevNode(blockNode);
        return false;
      }

      return true;
    }
  }]);

  return Backspace;
}();

var Block = /*#__PURE__*/function () {
  function Block(editor) {
    _classCallCheck(this, Block);

    this.editor = void 0;
    this.editor = editor;
  }

  _createClass(Block, [{
    key: "init",
    value: function init() {
      var _this = this;

      var editor = this.editor;

      if (isEngine(editor)) {
        var _typing$getHandleList, _typing$getHandleList2;

        var typing = editor.typing,
            event = editor.event; //绑定回车事件

        var enter = new Enter$4(editor);
        (_typing$getHandleList = typing.getHandleListener('enter', 'keydown')) === null || _typing$getHandleList === void 0 ? void 0 : _typing$getHandleList.on(function (event) {
          return enter.trigger(event);
        }); //删除事件

        var backspace = new Backspace$6(editor);
        (_typing$getHandleList2 = typing.getHandleListener('backspace', 'keydown')) === null || _typing$getHandleList2 === void 0 ? void 0 : _typing$getHandleList2.on(function (event) {
          return backspace.trigger(event);
        });
        event.on('keydown:space', function (event) {
          return _this.triggerMarkdown(event);
        });
      }
    }
    /**
     * 解析markdown
     * @param event 事件
     */

  }, {
    key: "triggerMarkdown",
    value: function triggerMarkdown(event) {
      var editor = this.editor;
      if (!isEngine(editor)) return;
      var change = editor.change,
          block = editor.block;
      var range = change.getRange();
      if (!range.collapsed || change.isComposing()) return;
      var startNode = range.startNode,
          startOffset = range.startOffset;
      var node = startNode.type === Node.TEXT_NODE ? startNode : startNode.children().eq(startOffset - 1);
      if (!node) return;
      var blockNode = block.closest(node);
      if (!editor.node.isRootBlock(blockNode)) return;
      var text = block.getLeftText(blockNode);
      return !Object.keys(editor.plugin.components).some(function (pluginName) {
        var plugin = editor.plugin.components[pluginName];

        if (isBlockPlugin(plugin) && !!plugin.markdown) {
          var reuslt = plugin.markdown(event, text, blockNode, node);
          if (reuslt === false) return true;
        }

        return;
      });
    }
    /**
     * 根据节点查找block插件实例
     * @param node 节点
     */

  }, {
    key: "findPlugin",
    value: function findPlugin(block) {
      var _this$editor = this.editor,
          node = _this$editor.node,
          schema = _this$editor.schema,
          plugin = _this$editor.plugin;
      if (!node.isBlock(block)) return;
      var result = undefined;
      Object.keys(plugin.components).some(function (pluginName) {
        var blockPlugin = plugin.components[pluginName];

        if (isBlockPlugin(blockPlugin) && (!blockPlugin.tagName || typeof blockPlugin.tagName === 'string' ? block.name === blockPlugin.tagName : blockPlugin.tagName.indexOf(block.name) > -1)) {
          var schemaRule = blockPlugin.schema();
          if ((blockPlugin.attributes || blockPlugin.style) && !(Array.isArray(schemaRule) ? schemaRule.find(function (rule) {
            return schema.checkNode(block, rule.attributes);
          }) : schema.checkNode(block, schemaRule.attributes))) return;
          result = blockPlugin;
          return true;
        }

        return;
      });
      return result;
    }
    /**
     * 查找Block节点的一级节点。如 div -> H2 返回 H2节点
     * @param parentNode 父节点
     * @param childNode 子节点
     */

  }, {
    key: "findTop",
    value: function findTop(parentNode, childNode) {
      var _this$editor2 = this.editor,
          schema = _this$editor2.schema,
          node = _this$editor2.node,
          list = _this$editor2.list;
      var topParentName = schema.closest(parentNode.name);
      var topChildName = schema.closest(childNode.name); //如果父节点没有级别或者子节点没有级别就返回子节点

      if (topParentName === parent.name || topChildName === childNode.name) return childNode; //如果父节点的级别大于子节点的级别就返回父节点

      if (schema.isAllowIn(parentNode.name, childNode.name)) return parentNode; //如果父节点是 ul、ol 这样的List列表，并且子节点也是这样的列表，设置ident

      if (node.isList(parentNode) && node.isList(childNode)) {
        var childIndent = parseInt(childNode.attributes(list.INDENT_KEY), 10) || 0;
        var parentIndent = parseInt(parentNode.attributes(list.INDENT_KEY), 10) || 0;
        childNode.attributes(list.INDENT_KEY, parentIndent ? parentIndent + 1 : childIndent + 1);
      } //默认返回子节点


      return childNode;
    }
    /**
     * 获取最近的block节点，找不到返回 node
     * @param node 节点
     */

  }, {
    key: "closest",
    value: function closest(node) {
      var originNode = node;

      while (node) {
        if (node.isEditable() || this.editor.node.isBlock(node)) {
          return node;
        }

        var parentNode = node.parent();
        if (!parentNode) break;
        node = parentNode;
      }

      return originNode;
    }
    /**
     * 在光标位置包裹一个block节点
     * @param block 节点
     * @param range 光标
     */

  }, {
    key: "wrap",
    value: function wrap(block, range) {
      var _this2 = this,
          _blocks$;

      if (!isEngine(this.editor)) return;
      var _this$editor3 = this.editor,
          change = _this$editor3.change,
          node = _this$editor3.node,
          schema = _this$editor3.schema,
          list = _this$editor3.list,
          mark = _this$editor3.mark;
      var safeRange = range || change.getSafeRange();
      var doc = getDocument(safeRange.startContainer);

      if (typeof block === 'string' || isNode(block)) {
        block = $(block, doc);
      } else block = block;

      if (!node.isBlock(block)) return;
      var blocks = this.getBlocks(safeRange);
      var targetPlugin = this.findPlugin(block); //一样的block插件不嵌套

      blocks = blocks.map(function (blockNode) {
        var _blockNode;

        if (!blockNode || blockNode.isCard()) return null;
        var wrapBlock = block;
        var blockParent = (_blockNode = blockNode) === null || _blockNode === void 0 ? void 0 : _blockNode.parent();

        while (blockParent && !blockParent.isEditable()) {
          blockNode = blockParent;

          var _parent2 = blockParent.parent();

          if (_parent2 && node.isBlock(_parent2)) {
            blockParent = _parent2;
          } else break;
        } //|| blockParent && !blockParent.equal(blockNode) && !blockParent.isRoot() && node.isBlock(blockParent) && !schema.isAllowIn(wrapBlock.name, blockParent.name)


        if (!schema.isAllowIn(wrapBlock.name, blockNode.name)) {
          //一样的插件，返回子级
          if (_this2.findPlugin(blockNode) === targetPlugin) {
            return blockNode.children();
          }

          return null;
        }

        return blockNode;
      }).filter(function (block) {
        return block !== null;
      }); // 不在段落内

      if (blocks.length === 0) {
        var root = this.closest(safeRange.startNode);
        if (root.isCard() || root.isEditable() || !schema.isAllowIn(block.name, root.name)) return;

        var _selection = safeRange.createSelection();

        root.children().each(function (node) {
          block.append(node);
        });
        root.append(block);

        _selection.move();

        return;
      }

      var selection = safeRange.createSelection();
      (_blocks$ = blocks[0]) === null || _blocks$ === void 0 ? void 0 : _blocks$.before(block);
      blocks.forEach(function (child) {
        if (child) {
          //先移除不能放入块级节点的mark标签
          if (targetPlugin) {
            child.allChildren().forEach(function (child) {
              var markNode = $(child);

              if (node.isMark(markNode)) {
                var _targetPlugin$disable;

                var markPlugin = mark.findPlugin(markNode);
                if (!markPlugin) return;

                if ((_targetPlugin$disable = targetPlugin.disableMark) === null || _targetPlugin$disable === void 0 ? void 0 : _targetPlugin$disable.indexOf(markPlugin.constructor.pluginName)) {
                  node.unwrap(markNode);
                }
              }
            });
          }

          block.append(child);
        }
      });
      selection.move();
      this.merge(safeRange);
      list.merge(undefined, safeRange);
      if (!range) change.apply(safeRange);
    }
    /**
     * 移除光标所在block节点包裹
     * @param block 节点
     * @param range 光标
     */

  }, {
    key: "unwrap",
    value: function unwrap(block, range) {
      if (!isEngine(this.editor)) return;
      var _this$editor4 = this.editor,
          change = _this$editor4.change,
          node = _this$editor4.node;
      var safeRange = range || change.getSafeRange();
      var doc = getDocument(safeRange.startContainer);

      if (typeof block === 'string' || isNode(block)) {
        block = $(block, doc);
      } else block = block;

      if (!node.isBlock(block)) return;
      var blocks = this.getSiblings(safeRange, block);

      if (blocks.length === 0) {
        return;
      }

      var firstNodeParent = blocks[0].node.parent();

      if (!(firstNodeParent === null || firstNodeParent === void 0 ? void 0 : firstNodeParent.inEditor())) {
        return;
      }

      var hasLeft = blocks.some(function (item) {
        return item.position === 'left';
      });
      var hasRight = blocks.some(function (item) {
        return item.position === 'right';
      });
      var leftParent = undefined;

      if (hasLeft) {
        var _parent3 = firstNodeParent;
        leftParent = node.clone(_parent3, false);

        _parent3.before(leftParent);
      }

      var rightParent = undefined;

      if (hasRight) {
        var _parent = blocks[blocks.length - 1].node.parent();

        if (_parent) {
          rightParent = node.clone(_parent, false);
          _parent === null || _parent === void 0 ? void 0 : _parent.after(rightParent);
        }
      } // 插入范围的开始和结束标记


      var selection = safeRange.createSelection();
      var nodeApi = node;
      blocks.forEach(function (item) {
        var status = item.position,
            node = item.node,
            parent = node.parent();

        if (status === 'left') {
          var _leftParent;

          (_leftParent = leftParent) === null || _leftParent === void 0 ? void 0 : _leftParent.append(node);
        }

        if (status === 'center') {
          var _block;

          if ((parent === null || parent === void 0 ? void 0 : parent.name) === ((_block = block) === null || _block === void 0 ? void 0 : _block.name) && (parent === null || parent === void 0 ? void 0 : parent.inEditor())) {
            nodeApi.unwrap(parent);
          }
        }

        if (status === 'right') {
          var _rightParent;

          (_rightParent = rightParent) === null || _rightParent === void 0 ? void 0 : _rightParent.append(node);
        }
      }); // 有序列表被从中间拆开后，剩余的两个部分的需要保持序号连续

      if (leftParent && leftParent.name === 'ol' && rightParent && rightParent.name === 'ol') {
        rightParent.attributes('start', (parseInt(leftParent.attributes('start'), 10) || 1) + leftParent.find('li').length);
      }

      selection.move();
      if (!range) change.apply(safeRange);
    }
    /**
     * 获取节点相对于光标开始位置、结束位置下的兄弟节点集合
     * @param range 光标
     * @param block 节点
     */

  }, {
    key: "getSiblings",
    value: function getSiblings(range, block) {
      var _this3 = this;

      var blocks = [];
      var nodeApi = this.editor.node;
      if (!nodeApi.isBlock(block)) return blocks;

      var getTargetBlock = function getTargetBlock(node, tagName) {
        var block = _this3.closest(node);

        while (block) {
          var _parent4 = block.parent();

          if (!_parent4) break;
          if (!block.inEditor()) break;
          if (block.text().trim() !== _parent4.text().trim()) break;
          if (_parent4.name === tagName) break;
          block = _parent4;
        }

        return block;
      };

      var startBlock = getTargetBlock(range.startNode, block.name);
      var endBlock = getTargetBlock(range.endNode, block.name);
      var parentBlock = startBlock.parent();
      var position = 'left';
      var node = parentBlock === null || parentBlock === void 0 ? void 0 : parentBlock.first();

      while (node) {
        node = $(node);
        if (!nodeApi.isBlock(node)) return blocks; // 超过编辑区域

        if (!node.inEditor()) return blocks;

        if (node[0] === startBlock[0]) {
          position = 'center';
        }

        blocks.push({
          position: position,
          node: node
        });

        if (node[0] === endBlock[0]) {
          position = 'right';
        }

        node = node.next();
      }

      return blocks;
    }
    /**
     * 分割当前光标选中的block节点
     * @param range 光标
     */

  }, {
    key: "split",
    value: function split(range) {
      var _this4 = this,
          _sideBlock$first;

      if (!isEngine(this.editor)) return;
      var _this$editor5 = this.editor,
          change = _this$editor5.change,
          mark = _this$editor5.mark;
      var safeRange = range || change.getSafeRange(); // 范围为展开状态时先删除内容

      if (!safeRange.collapsed) {
        change.deleteContent(safeRange);
      } // 获取上面第一个 Block


      var block = this.closest(safeRange.startNode); // 获取的 block 超出编辑范围

      if (!block.isEditable() && !block.inEditor()) {
        return;
      }

      if (block.isEditable()) {
        // <p>wo</p><cursor /><p>other</p>
        // to
        // <p>wo</p><p><cursor />other</p>
        var sc = safeRange.startContainer.childNodes[safeRange.startOffset];

        if (sc) {
          safeRange.select(sc, true).shrinkToElementNode().collapse(true);
        }

        if (!range) change.apply(safeRange);
        return;
      }

      var cloneRange = safeRange.cloneRange();
      cloneRange.shrinkToElementNode().shrinkToTextNode().collapse(true);
      var activeMarks = mark.findMarks(cloneRange);
      var sideBlock = this.createSide({
        block: block[0],
        range: safeRange,
        isLeft: false,
        keepID: true
      });
      var nodeApi = this.editor.node;
      sideBlock.traverse(function (node) {
        if (!nodeApi.isVoid(node) && (nodeApi.isInline(node) || nodeApi.isMark(node)) && nodeApi.isEmpty(node)) {
          node.remove();
        }
      }, true);

      var isEmptyElement = function isEmptyElement(node) {
        return nodeApi.isBlock(node) && (node.childNodes.length === 0 || node.innerText === '');
      };

      if (isEmptyElement(block[0]) && !isEmptyElement(sideBlock[0])) {
        this.generateRandomID(block, true);
      } else {
        this.generateRandomID(sideBlock, true);
      }

      block.after(sideBlock); // Chrome 不能选中 <p></p>，里面必须要有节点，插入 BR 之后输入文字自动消失

      if (nodeApi.isEmpty(block)) {
        nodeApi.html(block, nodeApi.getBatchAppendHTML(activeMarks, '<br />'));
      }

      if (nodeApi.isEmpty(sideBlock)) {
        nodeApi.html(sideBlock, nodeApi.getBatchAppendHTML(activeMarks, '<br />'));
      }

      block.children().each(function (child) {
        if (nodeApi.isInline(child)) {
          _this4.editor.inline.repairCursor(child);
        }
      });
      sideBlock.children().each(function (child) {
        if (nodeApi.isInline(child)) {
          _this4.editor.inline.repairCursor(child);
        }
      }); // 重新设置当前选中范围

      safeRange.select(sideBlock, true).shrinkToElementNode();

      if (sideBlock.children().length === 1 && ((_sideBlock$first = sideBlock.first()) === null || _sideBlock$first === void 0 ? void 0 : _sideBlock$first.name) === 'br') {
        safeRange.collapse(false);
      } else {
        safeRange.collapse(true);
      }

      if (!range) change.apply(safeRange);
    }
    /**
     * 在当前光标位置插入block节点
     * @param block 节点
     * @param range 光标
     * @param splitNode 分割节点，默认为光标开始位置的block节点
     */

  }, {
    key: "insert",
    value: function insert(block, range, splitNode) {
      var _container$first;

      if (!isEngine(this.editor)) return;
      var _this$editor6 = this.editor,
          change = _this$editor6.change,
          node = _this$editor6.node,
          list = _this$editor6.list;
      var safeRange = range || change.getSafeRange();
      var doc = getDocument(safeRange.startContainer);

      if (typeof block === 'string' || isNode(block)) {
        block = $(block, doc);
      } else block = block;

      if (!node.isBlock(block)) return; // 范围为折叠状态时先删除内容

      if (!safeRange.collapsed) {
        change.deleteContent(safeRange);
      } // 获取上面第一个 Block


      var container = this.closest(safeRange.startNode); // 超出编辑范围

      if (!container.isEditable() && !container.inEditor()) {
        if (!range) change.apply(safeRange);
        return;
      } // 当前选择范围在段落外面


      if (container.isEditable()) {
        node.insert(block, safeRange);
        safeRange.collapse(false);
        if (!range) change.apply(safeRange);
        return;
      } // <p><cursor /><br /></p>
      // to
      // <p><br /><cursor /></p>


      if (container.children().length === 1 && ((_container$first = container.first()) === null || _container$first === void 0 ? void 0 : _container$first.name) === 'br') {
        safeRange.select(container, true).collapse(false);
      } // 插入范围的开始和结束标记


      var selection = safeRange.enlargeToElementNode().createSelection();

      if (!selection.has()) {
        if (!range) change.apply(safeRange);
        return;
      }

      container = splitNode ? splitNode(container) : container; // 切割 Block

      var leftNodes = selection.getNode(container, 'left');
      leftNodes.allChildren().forEach(function (child) {
        var leftNode = $(child);

        if (node.isBlock(leftNode) && (node.isEmpty(leftNode) || list.isEmptyItem(leftNode))) {
          leftNode.remove();
        }
      });
      var rightNodes = selection.getNode(container, 'right', true, function (child) {
        if (child.isCard()) {
          var _parent5 = child.parent();

          if (_parent5 && node.isCustomize(_parent5)) return false;
        }

        return true;
      });
      rightNodes.allChildren().forEach(function (child) {
        var rightNode = $(child);

        if (node.isBlock(rightNode) && (node.isEmpty(rightNode) || list.isEmptyItem(rightNode))) {
          rightNode.remove();
        } else if (node.isList(rightNode)) {
          list.addBr(rightNode);
        }
      });
      if (rightNodes.length > 0 && !node.isEmpty(rightNodes) && !list.isEmptyItem(rightNodes)) container.after(rightNodes);
      if (leftNodes.length > 0 && !node.isEmpty(leftNodes) && !list.isEmptyItem(leftNodes)) container.after(leftNodes);
      if (!node.isEmpty(container) && !list.isEmptyItem(container)) container.remove(); // 移除原 Block

      var last = node.isEmpty(leftNodes) || list.isEmptyItem(leftNodes) ? container : leftNodes.eq(leftNodes.length - 1);

      if (last) {
        safeRange.setStartAfter(last);
        safeRange.collapse(true);
      } // 插入新 Block


      node.insert(block, safeRange);
      if (!range) change.apply(safeRange);
    }
    /**
     * 设置当前光标所在的所有block节点为新的节点或设置新属性
     * @param block 需要设置的节点或者节点属性
     * @param range 光标
     */

  }, {
    key: "setBlocks",
    value: function setBlocks(block, range) {
      var _this5 = this;

      if (!isEngine(this.editor)) return;
      var _this$editor7 = this.editor,
          node = _this$editor7.node,
          schema = _this$editor7.schema,
          mark = _this$editor7.mark;
      var change = this.editor.change;
      var safeRange = range || change.getSafeRange();
      var doc = getDocument(safeRange.startContainer);
      var targetNode = null;
      var attributes = {};

      if (typeof block === 'string') {
        targetNode = $(block, doc);
        attributes = targetNode.attributes();
        attributes.style = targetNode.css();
      } else {
        attributes = block;
      }

      var blocks = this.getBlocks(safeRange); // 编辑器根节点，无段落

      var startNode = safeRange.startNode;

      if (startNode.isEditable() && blocks.length === 0) {
        if (startNode.isCard() || startNode.isEditable()) return;
        var newBlock = targetNode || $('<p></p>');
        if (!schema.isAllowIn(newBlock.name, startNode.name)) return;
        node.setAttributes(newBlock, attributes);

        var _selection2 = safeRange.createSelection();

        startNode.children().each(function (node) {
          newBlock.append(node);
        });
        startNode.append(newBlock);

        _selection2.move();

        if (!range) change.apply(safeRange);
        return;
      }

      var targetPlugin = targetNode ? this.findPlugin(targetNode) : undefined;
      var selection = safeRange.createSelection();
      blocks.forEach(function (child) {
        // Card 不做处理
        if (child.attributes(CARD_KEY)) {
          return;
        } // 相同标签，或者只传入样式属性


        if (!targetNode || _this5.findPlugin(child) === targetPlugin && child.name === targetNode.name) {
          node.setAttributes(child, attributes);
          return;
        } //如果要包裹的节点可以放入到当前节点中，就不操作


        if (targetNode.name !== 'p' && schema.isAllowIn(child.name, targetNode.name)) {
          return;
        } //先移除不能放入块级节点的mark标签


        if (targetPlugin) {
          child.allChildren().forEach(function (child) {
            var markNode = $(child);

            if (node.isMark(markNode)) {
              var _targetPlugin$disable2;

              var markPlugin = mark.findPlugin(markNode);
              if (!markPlugin) return;

              if ((_targetPlugin$disable2 = targetPlugin.disableMark) === null || _targetPlugin$disable2 === void 0 ? void 0 : _targetPlugin$disable2.indexOf(markPlugin.constructor.pluginName)) {
                node.unwrap(markNode);
              }
            }
          });
        }

        var newNode = node.replace(child, targetNode);
        var parent = newNode.parent();

        if (parent && !parent.isEditable() && !schema.isAllowIn(parent.name, newNode.name)) {
          node.unwrap(parent);
        }
      });
      selection.move();
      if (!range) change.apply(safeRange);
    }
    /**
     * 合并当前光标位置相邻的block
     * @param range 光标
     */

  }, {
    key: "merge",
    value: function merge(range) {
      if (!isEngine(this.editor)) return;
      var _this$editor8 = this.editor,
          change = _this$editor8.change,
          schema = _this$editor8.schema;
      var safeRange = range || change.getSafeRange();
      var blocks = this.getBlocks(safeRange);
      if (0 === blocks.length) return;
      var root = blocks[0].closest(ROOT_SELECTOR);
      var tags = schema.getCanMergeTags();
      if (tags.length === 0) return;
      var block = root.find(tags.join(','));

      if (block.length > 0) {
        var selection = safeRange.createSelection();
        var nextNode = block.next();

        while (nextNode && tags.indexOf(nextNode.name) > 0) {
          var prevNode = nextNode.prev();
          var nextAttributes = nextNode.attributes();
          var prevAttributes = prevNode === null || prevNode === void 0 ? void 0 : prevNode.attributes();

          if (nextNode.name === (prevNode === null || prevNode === void 0 ? void 0 : prevNode.name) && nextAttributes['class'] === (prevAttributes ? prevAttributes['class'] : undefined) && Object.keys(nextAttributes).join(',') === Object.keys(prevAttributes || {}).join(',')) {
            this.editor.node.merge(prevNode, nextNode);
          }

          nextNode = nextNode.next();
        }

        selection.move();
      }

      if (!range) change.apply(safeRange);
    }
    /**
     * 获取对范围有效果的所有 Block
     */

  }, {
    key: "findBlocks",
    value: function findBlocks(range) {
      var _this6 = this;

      range = range.cloneRange();
      range.shrinkToElementNode();
      var sc = range.startContainer;
      var so = range.startOffset;
      var ec = range.endContainer;
      var eo = range.endOffset;
      var startNode = sc;
      var endNode = ec;

      if (sc.nodeType === getWindow().Node.ELEMENT_NODE) {
        if (sc.childNodes[so]) {
          startNode = sc.childNodes[so] || sc;
        }
      }

      if (ec.nodeType === getWindow().Node.ELEMENT_NODE) {
        if (eo > 0 && ec.childNodes[eo - 1]) {
          endNode = ec.childNodes[eo - 1] || sc;
        }
      } // 折叠状态时，按右侧位置的方式处理


      if (range.collapsed) {
        startNode = endNode;
      } // 不存在时添加


      var addNode = function addNode(nodes, nodeB, preppend) {
        if (!nodes.some(function (nodeA) {
          return nodeA[0] === nodeB[0];
        })) {
          if (preppend) {
            nodes.unshift(nodeB);
          } else {
            nodes.push(nodeB);
          }
        }
      }; // 向上寻找


      var findNodes = function findNodes(node) {
        var nodes = [];

        while (node) {
          if (node.isEditable()) {
            break;
          }

          if (_this6.editor.node.isBlock(node)) {
            nodes.push(node);
          }

          var _parent6 = node.parent();

          if (!_parent6) break;
          node = _parent6;
        }

        return nodes;
      };

      var nodes = this.getBlocks(range); // rang头部应该往数组头部插入节点

      findNodes($(startNode)).forEach(function (node) {
        return addNode(nodes, node, true);
      });

      if (!range.collapsed) {
        findNodes($(endNode)).forEach(function (node) {
          return addNode(nodes, node);
        });
      }

      return nodes;
    }
    /**
     * 判断范围的 {Edge}Offset 是否在 Block 的开始位置
     * @param range 光标
     * @param edge start ｜ end
     */

  }, {
    key: "isFirstOffset",
    value: function isFirstOffset(range, edge) {
      var _range = range,
          startNode = _range.startNode,
          endNode = _range.endNode,
          startOffset = _range.startOffset,
          endOffset = _range.endOffset;
      var container = edge === 'start' ? startNode : endNode;
      var offset = edge === 'start' ? startOffset : endOffset;
      range = range.cloneRange();
      var block = this.closest(container);
      range.select(block, true);
      range.setEnd(container[0], offset);
      var fragment = range.cloneContents();

      if (!fragment.firstChild) {
        return true;
      }

      var node = this.editor.node;

      if (fragment.childNodes.length === 1 && $(fragment.firstChild).name === 'br') {
        return true;
      }

      var emptyNode = $('<div />');
      emptyNode.append(fragment);
      return node.isEmpty(emptyNode);
    }
    /**
     * 判断范围的 {Edge}Offset 是否在 Block 的最后位置
     * @param range 光标
     * @param edge start ｜ end
     */

  }, {
    key: "isLastOffset",
    value: function isLastOffset(range, edge) {
      var _range2 = range,
          startNode = _range2.startNode,
          endNode = _range2.endNode,
          startOffset = _range2.startOffset,
          endOffset = _range2.endOffset;
      var container = edge === 'start' ? startNode : endNode;
      var offset = edge === 'start' ? startOffset : endOffset;
      range = range.cloneRange();
      var block = this.closest(container);
      range.select(block, true);
      range.setStart(container, offset);
      var fragment = range.cloneContents();

      if (!fragment.firstChild) {
        return true;
      }

      var node = this.editor.node;
      var emptyNode = $('<div />');
      emptyNode.append(fragment);
      return 0 >= emptyNode.find('br').length && node.isEmpty(emptyNode);
    }
    /**
     * 获取范围内的所有 Block
     * @param range  光标s
     */

  }, {
    key: "getBlocks",
    value: function getBlocks(range) {
      var _this7 = this;

      range = range.cloneRange();
      range.shrinkToElementNode();
      range.shrinkToTextNode();
      var node = this.editor.node;
      var startBlock = this.closest(range.startNode);

      if (range.startNode.isRoot()) {
        startBlock = $(range.getStartOffsetNode());
      }

      var endBlock = this.closest(range.endNode);

      if (range.endNode.isRoot()) {
        endBlock = $(range.getEndOffsetNode());
      }

      var closest = this.closest(range.commonAncestorNode);
      var blocks = [];
      var started = false;
      closest.traverse(function (node) {
        var child = $(node);

        if (child.equal(startBlock)) {
          started = true;
        }

        if (started && _this7.editor.node.isBlock(child) && !child.isCard() && child.inEditor()) {
          blocks.push(child);
        }

        if (child.equal(endBlock)) {
          started = false;
          return false;
        }

        return;
      }); // 未选中文本时忽略该 Block
      // 示例：<h3><anchor />word</h3><p><focus />another</p>

      if (blocks.length > 1 && this.isFirstOffset(range, 'end') && !node.isEmpty(endBlock)) {
        blocks.pop();
      }

      return blocks;
    }
    /**
     * 生成 cursor 左侧或右侧的节点，放在一个和父节点一样的容器里
     * isLeft = true：左侧
     * isLeft = false：右侧
     * @param param0
     */

  }, {
    key: "createSide",
    value: function createSide(_ref) {
      var block = _ref.block,
          range = _ref.range,
          isLeft = _ref.isLeft,
          _ref$clone = _ref.clone,
          clone = _ref$clone === void 0 ? false : _ref$clone,
          _ref$keepID = _ref.keepID,
          keepID = _ref$keepID === void 0 ? false : _ref$keepID;
      if (isNode(block)) block = $(block);
      var newRange = Range.create(this.editor, block.document);

      if (isLeft) {
        newRange.select(block, true);
        newRange.setEnd(range.startContainer, range.startOffset);
      } else {
        newRange.select(block, true);
        newRange.setStart(range.endContainer, range.endOffset);
      }

      var fragement = clone ? newRange.cloneContents() : newRange.extractContents();
      var dupBlock = keepID ? block.clone(false) : this.editor.node.clone(block, false);
      dupBlock.append(fragement);

      if (clone) {
        dupBlock.find(CARD_SELECTOR).each(function (card) {
          var domCard = $(card);
          var cardName = domCard.attributes(CARD_KEY);
          domCard.attributes(READY_CARD_KEY, cardName);
          domCard.removeAttributes(CARD_KEY);
        });
      }

      return dupBlock;
    }
    /**
     * 获取 Block 左侧文本
     * @param block 节点
     */

  }, {
    key: "getLeftText",
    value: function getLeftText(block, range) {
      if (!isEngine(this.editor)) return '';
      range = range || this.editor.change.getRange();
      var leftBlock = this.createSide({
        block: block,
        range: range,
        isLeft: true,
        clone: true
      });
      return leftBlock.text().trim().replace(/\u200B/g, '');
    }
    /**
     * 删除 Block 左侧文本
     * @param block 节点
     */

  }, {
    key: "removeLeftText",
    value: function removeLeftText(block, range) {
      if (!isEngine(this.editor)) return;
      range = range || this.editor.change.getRange();
      if (isNode(block)) block = $(block);
      range.createSelection();
      var cursor = block.find(CURSOR_SELECTOR);
      var isRemove = false; // 删除左侧文本节点

      block.traverse(function (node) {
        var child = $(node);

        if (child.equal(cursor)) {
          cursor.remove();
          isRemove = true;
          return;
        }

        if (isRemove && child.isText()) {
          child.remove();
        }
      }, false);
    }
    /**
     * 整理块级节点
     * @param domNode 节点
     * @param root 根节点
     */

  }, {
    key: "flatten",
    value: function flatten(domNode, root) {
      if (!isEngine(this.editor)) return;
      var _this$editor9 = this.editor,
          schema = _this$editor9.schema,
          node = _this$editor9.node;
      var mergeTags = schema.getCanMergeTags(); //获取父级节点

      var parentNode = domNode[0].parentNode;
      var rootElement = root.isFragment ? root[0].parentNode : root[0]; //在根节点内循环

      while (parentNode !== rootElement) {
        var domParentNode = $(parentNode || []); //如果是卡片节点，就在父节点前面插入

        if (domNode.isCard()) domParentNode.before(domNode);else if ( //如果是li标签，并且父级是 ol、ul 列表标签
        node.isList(domParentNode) && 'li' === domNode.name || //如果是父级可合并标签，并且当前节点是根block节点，并且不是 父节点一样的block节点
        mergeTags.indexOf(domParentNode.name) > -1 && node.isBlock(domNode) && domParentNode.name !== domNode.name) {
          //复制节点
          var cloneNode = node.clone(domParentNode, false); //追加到复制的节点

          cloneNode.append(domNode); //设置新的节点

          domNode = cloneNode; //将新的节点插入到父节点之前

          domParentNode.before(domNode);
        } else {
          domNode = node.replace(domNode, node.clone(this.findTop(domParentNode, domNode), false));
          domParentNode.before(domNode);
        } //如果没有子节点就移除

        if (!domParentNode.first()) domParentNode.remove(); //设置新的父节点

        parentNode = domNode[0].parentNode;
      }
    }
    /**
     * 根据规则获取需要为节点创建 data-id 的标签名称集合
     * @returns
     */

  }, {
    key: "getMarkIdTags",
    value: function getMarkIdTags() {
      var names = [];
      this.editor.schema.data.blocks.forEach(function (schema) {
        if (names.indexOf(schema.name) < 0) {
          names.push(schema.name);
        }
      });
      return names;
    }
    /**
     * 给节点创建data-id
     * @param node 节点
     * @returns
     */

  }, {
    key: "createDataID",
    value: function createDataID(node) {
      if (isNode(node)) node = $(node);
      var id = getHashId(node);
      node.attributes('data-id', id);
      return id;
    }
    /**
     * 获取或产生节点的data-id
     * @param root 根节点
     * @param node 节点
     * @returns
     */

  }, {
    key: "generateDataID",
    value: function generateDataID(root, node) {
      var nodeName = node.nodeName;
      var id = node.getAttribute('data-id');
      if (id) return id;
      var nodes = root.querySelectorAll(nodeName);

      for (var i = 0; i < nodes.length; i++) {
        if (nodes[i] === node) return this.createDataID(node);
      }

      return null;
    }
    /**
     * 在根节点内为需要创建data-id的子节点创建data-id
     * @param root 根节点
     */

  }, {
    key: "generateDataIDForDescendant",
    value: function generateDataIDForDescendant(root) {
      var _this8 = this;

      var tags = this.getMarkIdTags().join(',');
      var nodes = root.querySelectorAll(tags);
      nodes.forEach(function (child) {
        var node = $(child);
        if (!node.attributes('data-id') && !node.isCard() && node.attributes(DATA_ELEMENT) !== UI) _this8.createDataID(node);
      });
    }
    /**
     * 为节点创建一个随机data-id
     * @param node 节点
     * @param isCreate 如果有，是否需要重新创建
     * @returns
     */

  }, {
    key: "generateRandomID",
    value: function generateRandomID(node) {
      var isCreate = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      if (isNode(node)) node = $(node);
      if (node.isCard() || node.attributes(DATA_ELEMENT) === UI) return '';

      if (!isCreate) {
        var _id = node.attributes('data-id');

        if (_id) return _id;
      }

      var id = getHashId(node);
      node.attributes('data-id', id);
      return id;
    }
    /**
     * 在根节点内为需要创建data-id的子节点创建随机data-id
     * @param node 节点
     * @param isCreate 如果有，是否需要重新创建
     */

  }, {
    key: "generateRandomIDForDescendant",
    value: function generateRandomIDForDescendant(root) {
      var _this9 = this;

      var isCreate = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

      if (root.nodeType === getWindow().Node.ELEMENT_NODE || root.nodeType === getWindow().Node.DOCUMENT_FRAGMENT_NODE) {
        this.getMarkIdTags().forEach(function (nodeName) {
          var nodes = root.querySelectorAll(nodeName);

          for (var i = 0; i < nodes.length; i++) {
            var node = $(nodes[i]);
            if (node.isCard() || node.attributes(DATA_ELEMENT) === UI) continue;

            _this9.generateRandomID(node, isCreate);
          }
        });
      }
    }
    /**
     * 判断一个节点是否需要创建data-id
     * @param name 节点名称
     * @returns
     */

  }, {
    key: "needMarkDataID",
    value: function needMarkDataID(name) {
      return !!this.getMarkIdTags()[name.toLowerCase()];
    }
    /**
     * br 换行改成段落
     * @param block 节点
     */

  }, {
    key: "brToBlock",
    value: function brToBlock(block) {
      var _block$first, _block$last;

      // 没有子节点
      if (!block.first()) {
        return;
      } // 只有一个节点


      if (block.children().length === 1) {
        var _node = block.first(); //\n换成 br


        if (_node && _node.isText() && /^\n+$/g.test(_node.text())) {
          this.editor.node.replace(_node, $('<br />'));
        }

        return;
      }

      if ('li' === block.name) return; // 只有一个节点（有光标标记节点）

      if (block.children().length === 2 && ((_block$first = block.first()) === null || _block$first === void 0 ? void 0 : _block$first.attributes(DATA_ELEMENT)) === CURSOR || ((_block$last = block.last()) === null || _block$last === void 0 ? void 0 : _block$last.attributes(DATA_ELEMENT)) === CURSOR) {
        return;
      }

      var container;
      var prevContainer;
      var node = block.first();

      while (node) {
        var next = node.next();

        if (!container || node.name === 'br') {
          prevContainer = container;
          container = this.editor.node.clone(block, false);
          block.before(container);
        }

        if (node.name !== 'br') {
          container.append(node);
        }

        if ((node.name === 'br' || !next) && prevContainer && !prevContainer.first()) {
          prevContainer.append($('<br />'));
        }

        node = next;
      }

      if (container && !container.first()) {
        container.remove();
      }

      block.remove();
    }
    /**
     * 插入一个空的block节点
     * @param range 光标所在位置
     * @param block 节点
     * @returns
     */

  }, {
    key: "insertEmptyBlock",
    value: function insertEmptyBlock(range, block) {
      var editor = this.editor;
      if (!isEngine(editor)) return;
      var change = editor.change;
      var blocks = change.blocks,
          marks = change.marks;
      var nodeApi = editor.node;
      this.insert(block);

      if (blocks[0]) {
        var styles = blocks[0].css();
        block.css(styles);
      }

      var node = block.find('br');
      marks.forEach(function (mark) {
        // 回车后，默认是否复制makr样式
        var plugin = editor.mark.findPlugin(mark);
        mark = nodeApi.clone(mark); //插件判断

        if ((plugin === null || plugin === void 0 ? void 0 : plugin.copyOnEnter) !== false && (plugin === null || plugin === void 0 ? void 0 : plugin.followStyle) !== false) {
          mark = nodeApi.clone(mark);
          node.before(mark);
          mark.append(node);
          node = mark;
        }
      });
      node = block.find('br');
      var parent = node.parent();

      if (parent && nodeApi.isMark(parent)) {
        node = nodeApi.replace(node, $("\u200B", null));
      }

      range.select(node);
      range.collapse(false);
      range.scrollIntoView();
      change.select(range);
    }
    /**
     * 在光标位置插入或分割节点
     * @param range 光标所在位置
     * @param block 节点
     */

  }, {
    key: "insertOrSplit",
    value: function insertOrSplit(range, block) {
      var _block$last2, _block$first2;

      var cloneRange = range.cloneRange();
      cloneRange.enlargeFromTextNode();

      if (this.isLastOffset(range, 'end') || cloneRange.endNode.type === getWindow().Node.ELEMENT_NODE && block.children().length > 0 && cloneRange.endContainer.childNodes[cloneRange.endOffset] === ((_block$last2 = block.last()) === null || _block$last2 === void 0 ? void 0 : _block$last2.get()) && 'br' === ((_block$first2 = block.first()) === null || _block$first2 === void 0 ? void 0 : _block$first2.name)) {
        var _block$get;

        if (block.name === 'p' && ((_block$get = block.get()) === null || _block$get === void 0 ? void 0 : _block$get.className)) {
          var _block$get2;

          this.insertEmptyBlock(range, $("<p class=\"".concat((_block$get2 = block.get()) === null || _block$get2 === void 0 ? void 0 : _block$get2.className, "\"><br /></p>")));
        } else {
          this.insertEmptyBlock(range, $("<p><br /></p>"));
        }
      } else {
        this.split();
      }
    }
  }]);

  return Block;
}();

var buildParams = function buildParams(prefix, data, traditional, add) {
  var name = undefined;
  var i = undefined;
  var rbracket = /\[\]$/;

  if (Array.isArray(data)) {
    // Serialize array item.
    for (i = 0; i < data.length; i++) {
      var value = data[i];

      if (traditional || rbracket.test(prefix)) {
        // Treat each array item as a scalar.
        add(prefix, value);
      } else {
        buildParams(prefix + '[' + ((typeof value === 'undefined' ? 'undefined' : _typeof(value)) === 'object' ? i : '') + ']', value, traditional, add);
      }
    }
  } else if (data.toString() === '[object Object]') {
    // Serialize object item.
    for (name in data) {
      if (data.hasOwnProperty(name)) {
        buildParams(prefix + '[' + name + ']', data[name], traditional, add);
      }
    }
  } else {
    // Serialize scalar item.
    add(prefix, data);
  }
};
/**
 * URL 追加
 * @param url url
 * @param text 要追加的文本
 * @returns
 */

var urlAppend = function urlAppend(url, text) {
  return url + (/\?/.test(url) ? '&' : '?') + text;
};
var toQueryString = function toQueryString(data) {
  var traditional = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
  var prefix = undefined;
  var values = [];

  var add = function add(key, value) {
    // If value is a function, invoke it and return its value
    if (typeof value === 'function') {
      value = value();
    } else if (value === null || value === undefined) {
      value = '';
    }

    values[values.length] = encodeURIComponent(key) + '=' + encodeURIComponent(value);
  }; // If an array was passed in, assume that it is an array of form elements.


  if (Array.isArray(data)) {
    for (var i = 0; i < data.length; i++) {
      add(data[i].name, data[i].value);
    }
  } else {
    // If traditional, encode the "old" way (the way 1.3.2 or older
    // did it), otherwise encode params recursively.
    for (prefix in data) {
      if (data.hasOwnProperty(prefix)) {
        buildParams(prefix, data[prefix], traditional, add);
      }
    }
  } // spaces should be + according to spec


  return values.join('&').replace(/%20/g, '+');
};
var isFormData = function isFormData(data) {
  return typeof FormData !== 'undefined' && data instanceof FormData;
};

var HTTP_REG = /^http/;
var PROTOCO_REG = /(^\w+):\/\//;
var TWO_HUNDO = /^(20\d|1223)$/; // http://stackoverflow.com/questions/10046972/msie-returns-status-code-of-1223-for-ajax-request

var READY_STATE = 'readyState';
var CONTENT_TYPE = 'Content-Type';
var REQUESTED_WITH = 'X-Requested-With';
var XML_HTTP_REQUEST = 'XMLHttpRequest';
var X_DOMAIN_REQUEST = 'XDomainRequest';

var globalSetup = {
  traditional: false,
  contentType: 'application/x-www-form-urlencoded',
  requestedWith: XML_HTTP_REQUEST,
  accept: {
    '*': 'text/javascript, text/html, application/xml, text/xml, */*',
    xml: 'application/xml, text/xml',
    html: 'text/html',
    text: 'text/plain',
    json: 'application/json, text/javascript',
    js: 'application/javascript, text/javascript'
  },
  dataFilter: function dataFilter(data) {
    return data;
  }
};

var Ajax = /*#__PURE__*/function () {
  /**
   * 设置全局选项
   * @param options 选项
   */
  function Ajax(options) {
    var _this$options$doc;

    _classCallCheck(this, Ajax);

    this.options = void 0;
    this.headNode = void 0;
    this.request = void 0;
    this.isAborted = false;
    this.isTimeout = false;
    this.timeout = void 0;
    this.callbackData = void 0;
    this.callbackPrefix = 'request_' + new Date();
    this.uuid = 0;
    this.promise = void 0;
    this.__resolve = void 0;
    this.__reject = void 0;

    if (typeof options === 'string') {
      options = {
        url: options
      };
    }

    var _options = options,
        url = _options.url;

    if (startsWith(url, '//')) {
      url = window.location.protocol + url;
    }

    this.options = _objectSpread(_objectSpread(_objectSpread({}, globalSetup), options), {}, {
      url: url,
      context: options.context || getWindow(),
      doc: options.doc || getDocument(),
      jsonpCallback: options.jsonpCallback || 'callback',
      method: options.method || 'GET'
    });
    this.headNode = (_this$options$doc = this.options.doc) === null || _this$options$doc === void 0 ? void 0 : _this$options$doc.getElementsByTagName('head')[0];
    this.initPromise();
    this.init();
  }

  _createClass(Ajax, [{
    key: "initPromise",
    value: function initPromise() {
      var _this = this;

      this.promise = new Promise(function (resolve, reject) {
        _this.__resolve = resolve;
        _this.__reject = reject;
      }).catch(function () {});
    }
  }, {
    key: "init",
    value: function init() {
      var _this2 = this;

      var timedOut = function timedOut() {
        var _this2$request;

        _this2.isTimeout = true;
        (_this2$request = _this2.request) === null || _this2$request === void 0 ? void 0 : _this2$request.abort();
      };

      if (this.timeout) {
        clearTimeout(this.timeout);
      }

      this.timeout = undefined;

      if (this.options.timeout) {
        this.timeout = setTimeout(timedOut, this.options.timeout);
      }

      var error = function error(errorMsg, request) {
        _this2.triggerError(errorMsg, request);
      };

      var success = function success(request) {
        _this2.triggerSuccess(request);
      };

      this.request = this.getRequest(success, error);
    }
  }, {
    key: "defaultXHR",
    value: function defaultXHR() {
      var _this$options = this.options,
          context = _this$options.context,
          crossOrigin = _this$options.crossOrigin;
      if (!context) return; // is it x-domain

      if (crossOrigin === true) {
        var xhrInstance = context[XML_HTTP_REQUEST] ? new context[XML_HTTP_REQUEST]() : null;

        if (xhrInstance && 'withCredentials' in xhrInstance) {
          return xhrInstance;
        }

        if (context[X_DOMAIN_REQUEST]) {
          return new context[X_DOMAIN_REQUEST]();
        }

        throw new Error('Browser does not support cross-origin requests');
      } else if (context[XML_HTTP_REQUEST]) {
        return new context[XML_HTTP_REQUEST]();
      } else {
        return new context.ActiveXObject('Microsoft.XMLHTTP');
      }
    }
  }, {
    key: "succeed",
    value: function succeed() {
      var _this$request, _this$request$status, _this$request2;

      var _this$options2 = this.options,
          url = _this$options2.url,
          context = _this$options2.context;
      var protocol = PROTOCO_REG.exec(url);
      var protocolValue = protocol ? protocol[1] : '';

      if (!protocolValue) {
        protocolValue = (context === null || context === void 0 ? void 0 : context.location.protocol) || '';
      }

      return HTTP_REG.test(protocolValue) ? TWO_HUNDO.test(((_this$request = this.request) === null || _this$request === void 0 ? void 0 : (_this$request$status = _this$request.status) === null || _this$request$status === void 0 ? void 0 : _this$request$status.toString()) || '') : !!((_this$request2 = this.request) === null || _this$request2 === void 0 ? void 0 : _this$request2.response);
    }
  }, {
    key: "noop",
    value: function noop() {}
  }, {
    key: "handleReadyState",
    value: function handleReadyState(success, error) {
      // use _aborted to mitigate against IE err c00c023f
      // (can't read props on aborted request objects)
      if (this.isAborted) {
        return error('Request is aborted', this.request);
      }

      if (this.isTimeout) {
        return error('Request is aborted: timeout', this.request);
      }

      if (this.request && this.request[READY_STATE] === 4) {
        this.request.onreadystatechange = this.noop;

        if (this.succeed()) {
          success(this.request);
        } else {
          error(this.request.statusText, this.request);
        }
      }
    }
  }, {
    key: "setHeaders",
    value: function setHeaders(request) {
      var headers = this.options.headers || {};
      headers.Accept = headers.Accept || globalSetup.accept[this.options.type || '*']; // breaks cross-origin requests with legacy browsers

      if (!this.options.crossOrigin && !headers[REQUESTED_WITH]) {
        headers[REQUESTED_WITH] = globalSetup.requestedWith;
      }

      if (!headers[CONTENT_TYPE] && !isFormData(this.options.data)) {
        headers[CONTENT_TYPE] = this.options.contentType || globalSetup.contentType;
      }

      Object.keys(headers).forEach(function (name) {
        request.setRequestHeader(name, headers[name]);
      });
    }
  }, {
    key: "setCredentials",
    value: function setCredentials(request) {
      if (typeof this.options.withCredentials !== 'undefined' && typeof request.withCredentials !== 'undefined') {
        request.withCredentials = !!this.options.withCredentials;
      }
    }
  }, {
    key: "generalCallback",
    value: function generalCallback(data) {
      this.callbackData = data;
    }
  }, {
    key: "getCallbackPrefix",
    value: function getCallbackPrefix(id) {
      return this.callbackPrefix + '_' + id;
    }
  }, {
    key: "handleJsonp",
    value: function handleJsonp(url, success, error) {
      var _this3 = this,
          _this$headNode;

      var _this$options3 = this.options,
          jsonpCallback = _this$options3.jsonpCallback,
          jsonpCallbackName = _this$options3.jsonpCallbackName,
          doc = _this$options3.doc,
          context = _this$options3.context;
      if (!doc || !context) return;
      var requestId = this.uuid++;
      var cbkey = jsonpCallback || 'callback'; // the 'callback' key

      var cbval = jsonpCallbackName || this.getCallbackPrefix(requestId);
      var cbreg = new RegExp('((^|\\?|&)' + cbkey + ')=([^&]+)');
      var match = url.match(cbreg);
      var script = doc.createElement('script');
      var loaded = 0;
      var isIE10 = navigator.userAgent.indexOf('MSIE 10.0') !== -1;

      if (match) {
        if (match[3] === '?') {
          url = url.replace(cbreg, '$1=' + cbval); // wildcard callback func name
        } else {
          cbval = match[3]; // provided callback func name
        }
      } else {
        url = urlAppend(url, cbkey + '=' + cbval); // no callback details, add 'em
      }

      context[cbval] = this.generalCallback;
      script.type = 'text/javascript';
      script.src = url;
      script.async = true;

      if (typeof script['onreadystatechange'] !== 'undefined' && !isIE10) {
        // need this for IE due to out-of-order onreadystatechange(), binding script
        // execution to an event listener gives us control over when the script
        // is executed. See http://jaubourg.net/2010/07/loading-script-as-onclick-handler-of.html
        script.htmlFor = script.id = '_request_' + requestId;
      }

      script.onload = script['onreadystatechange'] = function () {
        var _this3$headNode;

        if (script[READY_STATE] && script[READY_STATE] !== 'complete' && script[READY_STATE] !== 'loaded' || loaded) {
          return false;
        }

        script.onload = script['onreadystatechange'] = null;

        if (script.onclick) {
          script.onclick();
        } // Call the user callback with the last value stored and clean up values and scripts.


        success(_this3.callbackData);
        _this3.callbackData = undefined;
        (_this3$headNode = _this3.headNode) === null || _this3$headNode === void 0 ? void 0 : _this3$headNode.removeChild(script);
        loaded = 1;
        return true;
      }; // Add the script to the DOM head


      (_this$headNode = this.headNode) === null || _this$headNode === void 0 ? void 0 : _this$headNode.appendChild(script); // Enable JSONP timeout

      return {
        abort: function abort() {
          var _this3$headNode2;

          script.onload = script['onreadystatechange'] = null;
          error('Request is aborted: timeout');
          _this3.callbackData = undefined;
          (_this3$headNode2 = _this3.headNode) === null || _this3$headNode2 === void 0 ? void 0 : _this3$headNode2.removeChild(script);
          loaded = 1;
        }
      };
    }
  }, {
    key: "getRequest",
    value: function getRequest(success, error) {
      var _this$options$method,
          _this$options$content,
          _this4 = this;

      var method = ((_this$options$method = this.options.method) === null || _this$options$method === void 0 ? void 0 : _this$options$method.toUpperCase()) || 'GET'; // convert non-string objects to query-string form unless o.processData is false

      var _this$options4 = this.options,
          processData = _this$options4.processData,
          traditional = _this$options4.traditional,
          type = _this$options4.type,
          context = _this$options4.context,
          xhr = _this$options4.xhr,
          async = _this$options4.async,
          before = _this$options4.before;
      if (!context) return;
      var _this$options5 = this.options,
          data = _this$options5.data,
          url = _this$options5.url;

      if ((((_this$options$content = this.options.contentType) === null || _this$options$content === void 0 ? void 0 : _this$options$content.indexOf('json')) || -1) > -1 && _typeof(data) === 'object') {
        data = JSON.stringify(data);
      }

      data = processData !== false && data && typeof data !== 'string' && !isFormData(data) ? toQueryString(data, traditional || globalSetup.traditional) : data || null;
      var http = undefined;
      var sendWait = false; // if we're working on a GET request and we have data then we should append
      // query string to end of URL and not post data

      if ((type === 'jsonp' || method === 'GET') && data) {
        url = urlAppend(url, data);
        data = null;
      }

      if (type === 'jsonp') {
        return this.handleJsonp(url, success, error);
      } // get the xhr from the factory if passed
      // if the factory returns null, fall-back to ours


      http = (typeof xhr === 'function' ? xhr(this.options) : xhr) || this.defaultXHR();
      if (!http) return;
      http.open(method, url, async === false ? false : true);
      this.setHeaders(http);
      this.setCredentials(http);

      if (context[X_DOMAIN_REQUEST] && http instanceof context[X_DOMAIN_REQUEST]) {
        http.onload = success;

        http.onerror = function () {
          error('http error', http);
        }; // NOTE: see
        // http://social.msdn.microsoft.com/Forums/en-US/iewebdevelopment/thread/30ef3add-767c-4436-b8a9-f1ca19b4812e


        http.onprogress = this.noop;
        sendWait = true;
      } else {
        http.onreadystatechange = function () {
          _this4.handleReadyState(success, error);
        };
      }

      if (before) {
        before(http);
      }

      if (sendWait) {
        setTimeout(function () {
          var _http;

          (_http = http) === null || _http === void 0 ? void 0 : _http.send(data);
        }, 200);
      } else {
        http.send(data);
      }

      return http;
    }
  }, {
    key: "getType",
    value: function getType(type) {
      // json, javascript, text/plain, text/html, xml
      if (!type) {
        return undefined;
      }

      if (type.match('json')) {
        return 'json';
      }

      if (type.match('javascript')) {
        return 'js';
      }

      if (type.match('text')) {
        return 'html';
      }

      if (type.match('xml')) {
        return 'xml';
      }

      return undefined;
    }
  }, {
    key: "triggerSuccess",
    value: function triggerSuccess(request) {
      var _this$options6 = this.options,
          dataFilter = _this$options6.dataFilter,
          context = _this$options6.context,
          success = _this$options6.success;
      if (!context) return;
      var type = this.options.type; // use global data filter on response text

      var data = (dataFilter || globalSetup.dataFilter)(request.responseText, type);

      if (!type) {
        type = request && this.getType(request.getResponseHeader('Content-Type'));
      } // resp can be undefined in IE


      var response = type !== 'jsonp' ? this.request : request;

      try {
        response['responseText'] = data;
      } catch (e) {// can't assign this in IE<=8, just ignore
      }

      if (data) {
        switch (type) {
          case 'json':
            try {
              response = context.JSON.parse(data);
            } catch (err) {
              return this.triggerError('Could not parse JSON in response', response);
            }

            break;

          case 'html':
            response = data;
            break;

          case 'xml':
            response = response.responseXML && response.responseXML.parseError && response.responseXML.parseError.errorCode && response.responseXML.parseError.reason ? null : response.responseXML;
            break;
        }
      }

      if (success) {
        success(response);
      }

      this.triggerComplete(response);
      if (this.__resolve) this.__resolve(response);
    }
  }, {
    key: "triggerError",
    value: function triggerError(errorMsg, request) {
      var error = this.options.error;
      var e = new Error(errorMsg);
      e['xhr'] = request;

      if (error) {
        error(e);
      }

      this.triggerComplete(e);
      if (this.__reject) this.__reject(e);
    }
  }, {
    key: "triggerComplete",
    value: function triggerComplete(request) {
      var complete = this.options.complete;

      if (this.timeout) {
        clearTimeout(this.timeout);
      }

      this.timeout = undefined;

      if (complete) {
        complete(request);
      }
    }
  }, {
    key: "retry",
    value: function retry() {
      this.initPromise();
      this.init();
    }
  }, {
    key: "then",
    value: function then(success, fail) {
      var _this$promise;

      return (_this$promise = this.promise) === null || _this$promise === void 0 ? void 0 : _this$promise.then(success, fail);
    }
  }, {
    key: "always",
    value: function always(fn) {
      var _this$promise2;

      return (_this$promise2 = this.promise) === null || _this$promise2 === void 0 ? void 0 : _this$promise2.then(fn, fn);
    }
  }, {
    key: "fail",
    value: function fail(fn) {
      var _this$promise3;

      return (_this$promise3 = this.promise) === null || _this$promise3 === void 0 ? void 0 : _this$promise3.then(undefined, fn);
    }
  }, {
    key: "catch",
    value: function _catch(fn) {
      return this.fail(fn);
    }
  }]);

  return Ajax;
}();

Ajax.setup = function (options) {
  Object.keys(options).forEach(function (key) {
    if (globalSetup[key]) globalSetup[key] = options[key];
  });
};

var mime = {
  'image/jpeg': ['jpeg', 'jpg', 'jpe'],
  'image/png': ['png'],
  'image/gif': ['gif'],
  'image/vnd.wap.wbmp': ['wbmp'],
  'image/tiff': ['tiff', 'tiff'],
  'image/vnd.adobe.photoshop': ['psd'],
  'image/svg+xml': ['svg', 'svgz'],
  'text/jsx': ['jsx'],
  'text/less': ['less'],
  'text/css': ['css'],
  'text/x-sass': ['sass'],
  'text/x-scss': ['scss'],
  'text/csv': ['cvs'],
  'text/xml': ['xml'],
  'text/x-vcard': ['vcf'],
  'text/x-vcalendar': ['vcs'],
  'text/markdown': ['md'],
  'text/plain': ['txt'],
  'text/richtext': ['rtx'],
  'text/rtf': ['rtf'],
  'text/html': ['html', 'htm', 'shtml'],
  'text/jade': ['jade'],
  'text/javascript': ['js'],
  'text/yaml': ['yaml', 'yml'],
  'audio/mp3': ['mp3'],
  'audio/mp4': ['mp4', 'mp4a'],
  'video/mp4': ['mp4'],
  'audio/mpeg': ['mpeg', 'mp2', 'mp3'],
  'audio/ogg': ['oga'],
  'audio/wav': ['wav'],
  'audio/wave': ['wav'],
  'audio/webm': ['weba'],
  'video/x-msvideo': ['avi'],
  'video/quicktime': ['mov', 'qt'],
  'video/ogg': ['ogv'],
  'video/x-sgi-movie': ['movie'],
  'application/rss+xml': ['rss'],
  'application/json': ['json'],
  'application/zip': ['zip'],
  'application/gzip': ['gzip'],
  'application/pdf': ['pdf'],
  'application/postscript': ['ai', 'eps', 'ps'],
  'application/msword': ['doc', 'dot'],
  'application/vnd.openxmlformats-officedocument.wordprocessingml.document': ['docx'],
  'application/vnd.openxmlformats-officedocument.wordprocessingml.template': ['dotx'],
  'application/vnd.ms-excel': ['xls', 'xlm', 'xla', 'xlc', 'xlt', 'xlw'],
  'application/vnd.ms-excel.sheet.macroenabled.12': ['xlsm'],
  'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet': ['xlsx'],
  'application/vnd.openxmlformats-officedocument.spreadsheetml.template': ['xltx'],
  'application/vnd.ms-powerpoint': ['ppt', 'pps', 'pot'],
  'application/vnd.openxmlformats-officedocument.presentationml.presentation': ['pptx'],
  'application/vnd.openxmlformats-officedocument.presentationml.slide': ['sldx'],
  'application/vnd.openxmlformats-officedocument.presentationml.slideshow': ['ppsx'],
  'application/vnd.openxmlformats-officedocument.presentationml.template': ['potx'],
  'application/vnd.ms-fontobject': ['eot'],
  'application/vnd.android.package-archive': ['apk'],
  'application/x-apple-diskimage': ['dmg'],
  'application/x-iwork-keynote-sffkey': ['key'],
  'application/x-iwork-pages-sffpages': ['pages'],
  'application/x-iwork-keynote-sffnumbers': ['numbers'],
  'application/x-iwork-numbers-sffnumbers': ['numbers']
};

/**
 * 获取文件扩展名
 * @param file 文件 或 文件名
 * @returns
 */

var getExtensionName = function getExtensionName(file) {
  if (typeof file === 'string') {
    return file.split('.').pop();
  }

  var ext = mime[file.type] && mime[file.type][0];

  if (!ext && file['name']) {
    ext = file['name'].split('.').pop();
  }

  return ext;
};
/**
 * 获取文件大小
 * @param size
 * @param base
 * @returns
 */

var getFileSize = function getFileSize(size) {
  var base = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : isWindows ? 2 : 10;

  //1M以下
  if (size < 1048576) {
    return filesize(size, {
      base: base,
      exponent: 1,
      round: 0
    });
  }

  return filesize(size, {
    base: base,
    exponent: 2,
    round: 1
  });
};

var Uploader = /*#__PURE__*/function () {
  function Uploader(options) {
    _classCallCheck(this, Uploader);

    this.options = void 0;
    this.uploadingFiles = [];
    this.options = options;
  }

  _createClass(Uploader, [{
    key: "createUid",
    value: function createUid(text) {
      return Date.now() + '-' + text;
    }
  }, {
    key: "request",
    value: function () {
      var _request = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2(files) {
        var _this = this;

        return _regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                files.forEach( /*#__PURE__*/function () {
                  var _ref = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(file, index) {
                    return _regeneratorRuntime.wrap(function _callee$(_context) {
                      while (1) {
                        switch (_context.prev = _context.next) {
                          case 0:
                            if (!file.uid) file.uid = _this.createUid(index);
                            _context.next = 3;
                            return _this.handleBefore(file, files);

                          case 3:
                            _context.t0 = _context.sent;

                            if (!(_context.t0 === false)) {
                              _context.next = 6;
                              break;
                            }

                            files.splice(index, 1);

                          case 6:
                          case "end":
                            return _context.stop();
                        }
                      }
                    }, _callee);
                  }));

                  return function (_x2, _x3) {
                    return _ref.apply(this, arguments);
                  };
                }());
                this.upload(files);

              case 2:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function request(_x) {
        return _request.apply(this, arguments);
      }

      return request;
    }()
  }, {
    key: "upload",
    value: function () {
      var _upload = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee4(files) {
        var _this2 = this;

        return _regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                files.forEach( /*#__PURE__*/function () {
                  var _ref2 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee3(file) {
                    var formData, _this2$options, url, data, onUploading, onSuccess, onError;

                    return _regeneratorRuntime.wrap(function _callee3$(_context3) {
                      while (1) {
                        switch (_context3.prev = _context3.next) {
                          case 0:
                            formData = new FormData();
                            formData.append('file', file, file.name);

                            if (file.data) {
                              Object.keys(file.data).forEach(function (key) {
                                formData.append(key, file.data[key]);
                              });
                            }

                            _this2$options = _this2.options, url = _this2$options.url, data = _this2$options.data, onUploading = _this2$options.onUploading, onSuccess = _this2$options.onSuccess, onError = _this2$options.onError;

                            if (data) {
                              Object.keys(data).forEach(function (key) {
                                formData.append(key, data[key]);
                              });
                            }

                            _context3.next = 7;
                            return new Ajax({
                              xhr: function xhr() {
                                var xhr = new window.XMLHttpRequest();
                                xhr.upload.addEventListener('progress', function (evt) {
                                  if (evt.lengthComputable) {
                                    if (onUploading) onUploading(file, {
                                      percent: parseInt((evt.loaded / evt.total * 100).toString(), 10)
                                    });
                                  }
                                }, false);
                                return xhr;
                              },
                              url: url,
                              data: formData,
                              contentType: _this2.options.contentType,
                              type: _this2.options.type || 'json',
                              success: function success(response) {
                                if (onSuccess) onSuccess(response, file);
                              },
                              error: function error(err) {
                                if (onError) onError(err, file);
                              },
                              method: 'POST',
                              processData: true
                            });

                          case 7:
                          case "end":
                            return _context3.stop();
                        }
                      }
                    }, _callee3);
                  }));

                  return function (_x5) {
                    return _ref2.apply(this, arguments);
                  };
                }());

              case 1:
              case "end":
                return _context4.stop();
            }
          }
        }, _callee4);
      }));

      function upload(_x4) {
        return _upload.apply(this, arguments);
      }

      return upload;
    }()
  }, {
    key: "handleBefore",
    value: function handleBefore(file, files) {
      var _this3 = this;

      var type = file.type,
          uid = file.uid,
          name = file.name,
          size = file.size;
      var ext = getExtensionName(file);
      var onBefore = this.options.onBefore;

      if (onBefore && onBefore(file) === false) {
        return false;
      }

      return new Promise(function (resolve, reject) {
        var fileReader = new FileReader();
        fileReader.addEventListener('load', function () {
          _this3.uploadingFiles[uid] = {
            uid: uid,
            src: fileReader.result,
            name: name,
            size: size,
            type: type,
            ext: ext
          }; //全部图片读取完成后再插入编辑器

          if (files.every(function (file) {
            return !!_this3.uploadingFiles[file.uid];
          })) {
            files.forEach(function (file) {
              if (_this3.options.onReady) {
                _this3.options.onReady(_this3.uploadingFiles[file.uid], file);
              }
            });
          }

          resolve(true);
        }, false);
        fileReader.addEventListener('error', function () {
          reject(false);
        });
        fileReader.readAsDataURL(file);
      });
    }
  }]);

  return Uploader;
}();

var Request = /*#__PURE__*/function () {
  function Request() {
    _classCallCheck(this, Request);
  }

  _createClass(Request, [{
    key: "ajax",
    value: function ajax(options) {
      return new Ajax(options);
    }
  }, {
    key: "upload",
    value: function upload(options, files) {
      return new Uploader(options).request(files);
    }
  }, {
    key: "getFiles",
    value: function getFiles(options) {
      var _ref = options || {},
          event = _ref.event,
          accept = _ref.accept,
          multiple = _ref.multiple;

      accept = accept || '*';
      multiple = _typeof(multiple) === undefined ? 100 : multiple;
      var input = document.createElement('input');
      input.type = 'file';
      input.accept = accept;
      input.style.display = 'none';
      input.multiple = multiple !== false;

      var remove = function remove() {
        input.remove();
        document.removeEventListener('mousedown', remove);
      };

      return new Promise(function (resolve) {
        var change = function change() {
          var files = [];

          for (var i = 0; i < (((_input$files = input.files) === null || _input$files === void 0 ? void 0 : _input$files.length) || 0); i++) {
            var _input$files;

            if (typeof multiple === 'number' && i > multiple) break;
            files.push(input.files[i]);
          }

          input.removeEventListener('change', change);
          remove();
          resolve(files);
        };

        input.addEventListener('change', change);
        document.body.appendChild(input);

        if (!event) {
          event = document.createEvent('MouseEvents');
          event.initEvent('click', true, true);
        }

        try {
          if (!!input.dispatchEvent) {
            input.dispatchEvent(event);
          } else if (!!input['fireEvent']) {
            input['fireEvent'](event);
          } else throw '';

          document.addEventListener('mousedown', remove);
        } catch (error) {
          input.removeEventListener('change', change);
          remove();
          resolve([]);
        }
      });
    }
  }]);

  return Request;
}();

var css_248z$6 = "@font-face {\r\n    font-family: 'data-icon';  /* Project id 1456030 */\r\n    src: url('//at.alicdn.com/t/font_1456030_n0z6ova55ec.woff2?t=1623060824976') format('woff2'),\r\n         url('//at.alicdn.com/t/font_1456030_n0z6ova55ec.woff?t=1623060824976') format('woff'),\r\n         url('//at.alicdn.com/t/font_1456030_n0z6ova55ec.ttf?t=1623060824976') format('truetype');\r\n  }\r\n  \r\n  .data-icon {\r\n    font-family: \"data-icon\" !important;\r\n    font-size: 16px;\r\n    font-style: normal;\r\n    -webkit-font-smoothing: antialiased;\r\n    -moz-osx-font-smoothing: grayscale;\r\n  }\r\n  \r\n  .data-icon-text:before {\r\n    content: \"\\e600\";\r\n  }\r\n\r\n  .data-icon-comment:before {\r\n    content: \"\\e73e\";\r\n  }\r\n  \r\n  .data-icon-huanyuan:before {\r\n    content: \"\\e60a\";\r\n  }\r\n  \r\n  .data-icon-label:before {\r\n    content: \"\\e714\";\r\n  }\r\n  \r\n  .data-icon-video:before {\r\n    content: \"\\e741\";\r\n  }\r\n  \r\n  .data-icon-table:before {\r\n    content: \"\\e6a8\";\r\n  }\r\n  \r\n  .data-icon-h:before {\r\n    content: \"\\e7a0\";\r\n  }\r\n  \r\n  .data-icon-collapse-subtree:before {\r\n    content: \"\\e754\";\r\n  }\r\n  \r\n  .data-icon-expand-subtree:before {\r\n    content: \"\\e792\";\r\n  }\r\n  \r\n  .data-icon-sub-node:before {\r\n    content: \"\\e78e\";\r\n  }\r\n  \r\n  .data-icon-sister-node:before {\r\n    content: \"\\e784\";\r\n  }\r\n  \r\n  .data-icon-sup:before {\r\n    content: \"\\e790\";\r\n  }\r\n  \r\n  .data-icon-sub:before {\r\n    content: \"\\e75a\";\r\n  }\r\n  \r\n  .data-icon-maximize:before {\r\n    content: \"\\e752\";\r\n  }\r\n  \r\n  .data-icon-codeblock:before {\r\n    content: \"\\e709\";\r\n  }\r\n  \r\n  .data-icon-emoji:before {\r\n    content: \"\\e73a\";\r\n  }\r\n  \r\n  .data-icon-h4:before {\r\n    content: \"\\e759\";\r\n  }\r\n  \r\n  .data-icon-h1:before {\r\n    content: \"\\e75b\";\r\n  }\r\n  \r\n  .data-icon-h5:before {\r\n    content: \"\\e75c\";\r\n  }\r\n  \r\n  .data-icon-h2:before {\r\n    content: \"\\e75d\";\r\n  }\r\n  \r\n  .data-icon-h3:before {\r\n    content: \"\\e760\";\r\n  }\r\n  \r\n  .data-icon-h6:before {\r\n    content: \"\\e761\";\r\n  }\r\n  \r\n  .data-icon-liuchengtu:before {\r\n    content: \"\\e61c\";\r\n  }\r\n  \r\n  .data-icon-website:before {\r\n    content: \"\\e694\";\r\n  }\r\n  \r\n  .data-icon-preferences:before {\r\n    content: \"\\e788\";\r\n  }\r\n  \r\n  .data-icon-hr:before {\r\n    content: \"\\e76a\";\r\n  }\r\n  \r\n  .data-icon-task-list:before {\r\n    content: \"\\e79f\";\r\n  }\r\n  \r\n  .data-icon-unordered-list:before {\r\n    content: \"\\e777\";\r\n  }\r\n  \r\n  .data-icon-ordered-list:before {\r\n    content: \"\\e795\";\r\n  }\r\n  \r\n  .data-icon-arrow-left:before {\r\n    content: \"\\e748\";\r\n  }\r\n  \r\n  .data-icon-arrow-up:before {\r\n    content: \"\\e769\";\r\n  }\r\n  \r\n  .data-icon-arrow-right:before {\r\n    content: \"\\e779\";\r\n  }\r\n  \r\n  .data-icon-arrow-down:before {\r\n    content: \"\\e79a\";\r\n  }\r\n  \r\n  .data-icon-moremark:before {\r\n    content: \"\\e772\";\r\n  }\r\n  \r\n  .data-icon-clean:before {\r\n    content: \"\\e74d\";\r\n  }\r\n  \r\n  .data-icon-paintformat:before {\r\n    content: \"\\e756\";\r\n  }\r\n  \r\n  .data-icon-lock:before {\r\n    content: \"\\e768\";\r\n  }\r\n  \r\n  .data-icon-loading:before {\r\n    content: \"\\e76b\";\r\n  }\r\n  \r\n  .data-icon-unlock:before {\r\n    content: \"\\e796\";\r\n  }\r\n  \r\n  .data-icon-collapse:before {\r\n    content: \"\\e79e\";\r\n  }\r\n  \r\n  .data-icon-align-bottom:before {\r\n    content: \"\\e72b\";\r\n  }\r\n  \r\n  .data-icon-attachment:before {\r\n    content: \"\\e72c\";\r\n  }\r\n  \r\n  .data-icon-bold:before {\r\n    content: \"\\e72d\";\r\n  }\r\n  \r\n  .data-icon-border-color:before {\r\n    content: \"\\e72e\";\r\n  }\r\n  \r\n  .data-icon-border-all:before {\r\n    content: \"\\e72f\";\r\n  }\r\n  \r\n  .data-icon-border-inner:before {\r\n    content: \"\\e730\";\r\n  }\r\n  \r\n  .data-icon-border-left:before {\r\n    content: \"\\e731\";\r\n  }\r\n  \r\n  .data-icon-border-bottom:before {\r\n    content: \"\\e732\";\r\n  }\r\n  \r\n  .data-icon-border-none:before {\r\n    content: \"\\e733\";\r\n  }\r\n  \r\n  .data-icon-box:before {\r\n    content: \"\\e734\";\r\n  }\r\n  \r\n  .data-icon-border-outer:before {\r\n    content: \"\\e735\";\r\n  }\r\n  \r\n  .data-icon-border-right:before {\r\n    content: \"\\e736\";\r\n  }\r\n  \r\n  .data-icon-clear:before {\r\n    content: \"\\e737\";\r\n  }\r\n  \r\n  .data-icon-close:before {\r\n    content: \"\\e738\";\r\n  }\r\n  \r\n  .data-icon-code-example:before {\r\n    content: \"\\e739\";\r\n  }\r\n  \r\n  .data-icon-clip:before {\r\n    content: \"\\e73b\";\r\n  }\r\n  \r\n  .data-icon-border-up:before {\r\n    content: \"\\e73c\";\r\n  }\r\n  \r\n  .data-icon-code:before {\r\n    content: \"\\e73d\";\r\n  }\r\n  \r\n  .data-icon-command:before {\r\n    content: \"\\e73f\";\r\n  }\r\n  \r\n  .data-icon-compact-display:before {\r\n    content: \"\\e740\";\r\n  }\r\n  \r\n  .data-icon-copy:before {\r\n    content: \"\\e742\";\r\n  }\r\n  \r\n  .data-icon-download:before {\r\n    content: \"\\e743\";\r\n  }\r\n  \r\n  .data-icon-deletecolumn:before {\r\n    content: \"\\e744\";\r\n  }\r\n  \r\n  .data-icon-cut:before {\r\n    content: \"\\e745\";\r\n  }\r\n  \r\n  .data-icon-decreasedecimalplace:before {\r\n    content: \"\\e746\";\r\n  }\r\n  \r\n  .data-icon-drag:before {\r\n    content: \"\\e747\";\r\n  }\r\n  \r\n  .data-icon-delete:before {\r\n    content: \"\\e749\";\r\n  }\r\n  \r\n  .data-icon-drag-circle:before {\r\n    content: \"\\e74a\";\r\n  }\r\n  \r\n  .data-icon-deleterow:before {\r\n    content: \"\\e74b\";\r\n  }\r\n  \r\n  .data-icon-edit:before {\r\n    content: \"\\e74c\";\r\n  }\r\n  \r\n  .data-icon-filter:before {\r\n    content: \"\\e74e\";\r\n  }\r\n  \r\n  .data-icon-embedded-preview:before {\r\n    content: \"\\e74f\";\r\n  }\r\n  \r\n  .data-icon-error:before {\r\n    content: \"\\e750\";\r\n  }\r\n  \r\n  .data-icon-freezerowcoloum:before {\r\n    content: \"\\e751\";\r\n  }\r\n  \r\n  .data-icon-freezefirstrow:before {\r\n    content: \"\\e753\";\r\n  }\r\n  \r\n  .data-icon-freezzecolumn:before {\r\n    content: \"\\e755\";\r\n  }\r\n  \r\n  .data-icon-border-style:before {\r\n    content: \"\\e757\";\r\n  }\r\n  \r\n  .data-icon-gotolink:before {\r\n    content: \"\\e758\";\r\n  }\r\n  \r\n  .data-icon-increasedecimalplace:before {\r\n    content: \"\\e75e\";\r\n  }\r\n  \r\n  .data-icon-insertrowbelow:before {\r\n    content: \"\\e75f\";\r\n  }\r\n  \r\n  .data-icon-image:before {\r\n    content: \"\\e762\";\r\n  }\r\n  \r\n  .data-icon-italic:before {\r\n    content: \"\\e763\";\r\n  }\r\n  \r\n  .data-icon-indent:before {\r\n    content: \"\\e764\";\r\n  }\r\n  \r\n  .data-icon-insertrowabove:before {\r\n    content: \"\\e765\";\r\n  }\r\n  \r\n  .data-icon-insertrowright:before {\r\n    content: \"\\e766\";\r\n  }\r\n  \r\n  .data-icon-left-circle-fill:before {\r\n    content: \"\\e767\";\r\n  }\r\n  \r\n  .data-icon-link:before {\r\n    content: \"\\e76c\";\r\n  }\r\n  \r\n  .data-icon-keyboard:before {\r\n    content: \"\\e76d\";\r\n  }\r\n  \r\n  .data-icon-more:before {\r\n    content: \"\\e76e\";\r\n  }\r\n  \r\n  .data-icon-merge-cells:before {\r\n    content: \"\\e76f\";\r\n  }\r\n  \r\n  .data-icon-outdent:before {\r\n    content: \"\\e770\";\r\n  }\r\n  \r\n  .data-icon-mention:before {\r\n    content: \"\\e771\";\r\n  }\r\n  \r\n  .data-icon-plus:before {\r\n    content: \"\\e773\";\r\n  }\r\n  \r\n  .data-icon-minus-circle-o:before {\r\n    content: \"\\e774\";\r\n  }\r\n  \r\n  .data-icon-highlight:before {\r\n    content: \"\\e775\";\r\n  }\r\n  \r\n  .data-icon-paste:before {\r\n    content: \"\\e776\";\r\n  }\r\n  \r\n  .data-icon-insertrowleft:before {\r\n    content: \"\\e778\";\r\n  }\r\n  \r\n  .data-icon-quote:before {\r\n    content: \"\\e77a\";\r\n  }\r\n  \r\n  .data-icon-plus-circle-o:before {\r\n    content: \"\\e77b\";\r\n  }\r\n  \r\n  .data-icon-right-circle-fill:before {\r\n    content: \"\\e77c\";\r\n  }\r\n  \r\n  .data-icon-question-circle-o:before {\r\n    content: \"\\e77d\";\r\n  }\r\n  \r\n  .data-icon-preview:before {\r\n    content: \"\\e77e\";\r\n  }\r\n  \r\n  .data-icon-reload:before {\r\n    content: \"\\e77f\";\r\n  }\r\n  \r\n  .data-icon-rotate-left:before {\r\n    content: \"\\e780\";\r\n  }\r\n  \r\n  .data-icon-math:before {\r\n    content: \"\\e781\";\r\n  }\r\n  \r\n  .data-icon-overflow:before {\r\n    content: \"\\e782\";\r\n  }\r\n  \r\n  .data-icon-redo:before {\r\n    content: \"\\e783\";\r\n  }\r\n  \r\n  .data-icon-searchreplace:before {\r\n    content: \"\\e785\";\r\n  }\r\n  \r\n  .data-icon-save:before {\r\n    content: \"\\e786\";\r\n  }\r\n  \r\n  .data-icon-singleselect:before {\r\n    content: \"\\e787\";\r\n  }\r\n  \r\n  .data-icon-rotate-right:before {\r\n    content: \"\\e789\";\r\n  }\r\n  \r\n  .data-icon-sort-ascending:before {\r\n    content: \"\\e78a\";\r\n  }\r\n  \r\n  .data-icon-sort-descending:before {\r\n    content: \"\\e78b\";\r\n  }\r\n  \r\n  .data-icon-toc:before {\r\n    content: \"\\e78c\";\r\n  }\r\n  \r\n  .data-icon-solit-cells:before {\r\n    content: \"\\e78d\";\r\n  }\r\n  \r\n  .data-icon-translate:before {\r\n    content: \"\\e78f\";\r\n  }\r\n  \r\n  .data-icon-successful:before {\r\n    content: \"\\e791\";\r\n  }\r\n  \r\n  .data-icon-strikethrough:before {\r\n    content: \"\\e793\";\r\n  }\r\n  \r\n  .data-icon-undo:before {\r\n    content: \"\\e794\";\r\n  }\r\n  \r\n  .data-icon-underline:before {\r\n    content: \"\\e797\";\r\n  }\r\n  \r\n  .data-icon-unlink:before {\r\n    content: \"\\e798\";\r\n  }\r\n  \r\n  .data-icon-wrap:before {\r\n    content: \"\\e799\";\r\n  }\r\n  \r\n  .data-icon-upload:before {\r\n    content: \"\\e79b\";\r\n  }\r\n  \r\n  .data-icon-zoom-out:before {\r\n    content: \"\\e79c\";\r\n  }\r\n  \r\n  .data-icon-zoom-in:before {\r\n    content: \"\\e79d\";\r\n  }\r\n  \r\n  .data-icon-align-center:before {\r\n    content: \"\\e725\";\r\n  }\r\n  \r\n  .data-icon-align-justify:before {\r\n    content: \"\\e726\";\r\n  }\r\n  \r\n  .data-icon-align-left:before {\r\n    content: \"\\e727\";\r\n  }\r\n  \r\n  .data-icon-align-top:before {\r\n    content: \"\\e728\";\r\n  }\r\n  \r\n  .data-icon-align-right:before {\r\n    content: \"\\e729\";\r\n  }\r\n  \r\n  .data-icon-align-middle:before {\r\n    content: \"\\e72a\";\r\n  }\r\n\r\n.data-anticon {\r\n    display: inline-block;\r\n    font-style: normal;\r\n    vertical-align: -0.125em;\r\n    text-align: center;\r\n    text-transform: none;\r\n    line-height: 0;\r\n    text-rendering: optimizeLegibility;\r\n    -webkit-font-smoothing: antialiased;\r\n}\r\n\r\n.data-anticon svg {\r\n    display: inline-block;\r\n}\r\n\r\n@-webkit-keyframes loadingCircle {\r\n    100% {\r\n      transform: rotate(360deg);\r\n    }\r\n}\r\n@keyframes loadingCircle {\r\n    100% {\r\n      transform: rotate(360deg);\r\n    }\r\n}\r\n\r\n.data-anticon .data-anticon-spin {\r\n    display: inline-block;\r\n    -webkit-animation: loadingCircle 1s infinite linear;\r\n    animation: loadingCircle 1s infinite linear;\r\n}\r\n\r\n.data-anticon > * {\r\n    line-height: 1;\r\n}\r\n\r\n*, *::before, *::after {\r\n    box-sizing: border-box;\r\n}\r\n\r\n.am-engine {\r\n    position: relative;\r\n    background-color: #FFFFFF;\r\n}\r\n\r\n.am-engine ::-moz-selection {\r\n    background: rgba(180, 213, 254, 0.5) !important;\r\n    color: inherit!important;\r\n}\r\n\r\n.am-engine ::selection {\r\n    background: rgba(180, 213, 254, 0.5) !important;\r\n    color: inherit!important;\r\n}\r\n\r\n.am-engine , .am-engine-view {\r\n    font-family: 'Chinese Quote', 'Segoe UI', Roboto, 'PingFang SC', 'Hiragino Sans GB', 'Microsoft YaHei', 'Helvetica Neue', Helvetica, Arial, sans-serif, 'Apple Color Emoji';\r\n\tword-wrap: break-word;\r\n\toutline-style: none;\r\n\twhite-space: pre-wrap;\r\n    -webkit-user-select: auto;\r\n       -moz-user-select: auto;\r\n        -ms-user-select: auto;\r\n            user-select: auto;\r\n    font-size: 14px;\r\n    line-height: 1.74;\r\n    color: #262626;\r\n    letter-spacing: .05em;\r\n    /* pt / px 换算表：https://websemantics.uk/articles/font-size-conversion/ */\r\n}\r\n\r\n.am-engine > *:first-child,.am-engine-view > *:first-child {\r\n\tmargin-top: 0 !important;\r\n}\r\n\r\n.am-engine p , .am-engine-view p {\r\n    white-space: normal;\r\n    margin: 0;\r\n    min-height: 24px;\r\n}\r\n\r\n.am-engine [contenteditable=\"true\"],.am-engine-view [contenteditable=\"true\"] {\r\n\toutline-style: none;\r\n}\r\n\r\n.am-engine .selection-transparent::-moz-selection,.am-engine-view .selection-transparent::-moz-selection {\r\n\tbackground: transparent;\r\n}\r\n\r\n.am-engine .selection-transparent::selection,.am-engine-view .selection-transparent::selection {\r\n\tbackground: transparent;\r\n}\r\n/*---------------------------卡片 begin-----------------------*/\r\n\r\n.am-engine [data-card-type], .am-engine-view [data-card-type] {\r\n    white-space: normal;\r\n}\r\n.am-engine span[data-card-type=\"inline\"],.am-engine-view span[data-card-type=\"inline\"] {\r\n\tdisplay: inline-block;\r\n\ttext-indent: 0;\r\n\tvertical-align: baseline;\r\n\twhite-space: initial;\r\n}\r\n\r\n.am-engine span[data-card-type=\"inline\"] span[data-card-element],.am-engine-view span[data-card-type=\"inline\"] span[data-card-element] {\r\n\tdisplay: inline-block;\r\n}\r\n\r\n.am-engine span[data-card-type=\"inline\"] span[data-card-element=\"center\"],.am-engine-view span[data-card-type=\"inline\"] span[data-card-element=\"center\"] {\r\n\tvertical-align: bottom;\r\n}\r\n\r\n.am-engine span[data-card-type=\"inline\"] span[data-card-element=\"left\"],.am-engine-view span[data-card-type=\"inline\"] span[data-card-element=\"left\"],.am-engine span[data-card-type=\"inline\"] span[data-card-element=\"right\"],.am-engine-view span[data-card-type=\"inline\"] span[data-card-element=\"right\"] {\r\n\tmin-width: 1px;\r\n\ttext-align: left;\r\n\t-webkit-user-select: text;\r\n\t-moz-user-select: text;\r\n\t -ms-user-select: text;\r\n\t     user-select: text;\r\n}\r\n\r\n.am-engine div[data-card-type=\"block\"],.am-engine-view div[data-card-type=\"block\"],.am-engine span[data-card-type=\"inline\"].data-card-block,.am-engine-view span[data-card-type=\"inline\"].data-card-block {\r\n\tdisplay: block;\r\n}\r\n\r\n.am-engine div[data-card-type=\"block\"] > div[data-card-element=\"body\"] > span[data-card-element=\"left\"],.am-engine-view div[data-card-type=\"block\"] > div[data-card-element=\"body\"] > span[data-card-element=\"left\"],.am-engine span[data-card-type=\"inline\"].data-card-block > div[data-card-element=\"body\"] > span[data-card-element=\"left\"],.am-engine-view span[data-card-type=\"inline\"].data-card-block > div[data-card-element=\"body\"] > span[data-card-element=\"left\"],.am-engine div[data-card-type=\"block\"] > div[data-card-element=\"body\"] > span[data-card-element=\"right\"],.am-engine-view div[data-card-type=\"block\"] > div[data-card-element=\"body\"] > span[data-card-element=\"right\"],.am-engine span[data-card-type=\"inline\"].data-card-block > div[data-card-element=\"body\"] > span[data-card-element=\"right\"],.am-engine-view span[data-card-type=\"inline\"].data-card-block > div[data-card-element=\"body\"] > span[data-card-element=\"right\"] {\r\n\tbottom: 0;\r\n\tposition: absolute;\r\n\twidth: 2px;\r\n\toverflow: hidden;\r\n\toutline: none;\r\n\ttext-align: left;\r\n\ttext-indent: 0;\r\n\t-webkit-box-flex: 0;\r\n\tflex: 0 0 auto;\r\n\t-webkit-user-select: text;\r\n\t-moz-user-select: text;\r\n\t -ms-user-select: text;\r\n\t     user-select: text;\r\n}\r\n\r\n.am-engine div[data-card-type=\"block\"] > div[data-card-element=\"body\"] > span[data-card-element=\"left\"],.am-engine-view div[data-card-type=\"block\"] > div[data-card-element=\"body\"] > span[data-card-element=\"left\"],.am-engine span[data-card-type=\"inline\"].data-card-block > div[data-card-element=\"body\"] > span[data-card-element=\"left\"],.am-engine-view span[data-card-type=\"inline\"].data-card-block > div[data-card-element=\"body\"] > span[data-card-element=\"left\"] {\r\n\tleft: -2px;\r\n\ttext-align: left;\r\n}\r\n\r\n.am-engine div[data-card-type=\"block\"] > div[data-card-element=\"body\"] > span[data-card-element=\"right\"],.am-engine-view div[data-card-type=\"block\"] > div[data-card-element=\"body\"] > span[data-card-element=\"right\"],.am-engine span[data-card-type=\"inline\"].data-card-block > div[data-card-element=\"body\"] > span[data-card-element=\"right\"],.am-engine-view span[data-card-type=\"inline\"].data-card-block > div[data-card-element=\"body\"] > span[data-card-element=\"right\"] {\r\n\tright: -2px;\r\n\ttext-align: right;\r\n}\r\n\r\n.am-engine span[data-card-element=\"body\"],.am-engine-view span[data-card-element=\"body\"],.am-engine div[data-card-element=\"body\"],.am-engine-view div[data-card-element=\"body\"] {\r\n\tposition: relative;\r\n}\r\n\r\n.am-engine span[data-card-element=\"body\"] [data-card-element=\"center\"],.am-engine-view span[data-card-element=\"body\"] [data-card-element=\"center\"],.am-engine div[data-card-element=\"body\"] [data-card-element=\"center\"],.am-engine-view div[data-card-element=\"body\"] [data-card-element=\"center\"] {\r\n\t-webkit-user-select: text;\r\n\t-moz-user-select: text;\r\n\t -ms-user-select: text;\r\n\t     user-select: text;\r\n}\r\n.am-engine span[data-card-element=\"body\"] [data-element=\"editable\"],.am-engine div[data-card-element=\"body\"] [data-element=\"editable\"]\r\n{\r\n    cursor: text;\r\n}\r\n/*---------------------------卡片 end-----------------------*/";
styleInject(css_248z$6);

var Engine = /*#__PURE__*/function () {
  function Engine(selector, options) {
    var _this = this;

    _classCallCheck(this, Engine);

    this._readonly = false;
    this._container = void 0;
    this.kind = 'engine';
    this.options = {
      lang: 'zh-cn',
      plugins: [],
      cards: [],
      config: {}
    };
    this.language = void 0;
    this.root = void 0;
    this.change = void 0;
    this.card = void 0;
    this.plugin = void 0;
    this.node = void 0;
    this.list = void 0;
    this.mark = void 0;
    this.inline = void 0;
    this.block = void 0;
    this.event = void 0;
    this.typing = void 0;
    this.ot = void 0;
    this.schema = void 0;
    this.conversion = void 0;
    this.history = void 0;
    this.scrollNode = void 0;
    this.command = void 0;
    this.hotkey = void 0;
    this.clipboard = void 0;
    this.request = void 0;
    this.options = _objectSpread(_objectSpread({}, this.options), options);
    this.language = new Language(this.options.lang || 'zh-cn', language);
    this.event = new Event();
    this.command = new Command(this);
    this.schema = new Schema();
    this.schema.add(defualtSchema);
    this.conversion = new Conversion(this);
    defaultConversion.forEach(function (rule) {
      return _this.conversion.add(rule.from, rule.to);
    });
    this.history = new HistoryModel(this);
    this.card = new CardModel(this);
    this.clipboard = new Clipboard(this);
    this.request = new Request();
    this.plugin = new PluginModel(this);
    this.node = new NodeModel(this);
    this.list = new List(this);
    this.mark = new Mark(this);
    this.inline = new Inline(this);
    this.block = new Block(this);
    this._container = new Container(selector, {
      engine: this,
      lang: this.options.lang,
      className: this.options.className,
      tabIndex: this.options.tabIndex
    });
    this.root = $(this.options.root || this.container.parent() || getDocument().body);

    this._container.init();

    this.change = new ChangeModel(this, {
      onChange: function onChange(value, trigger) {
        return _this.trigger('change', value, trigger);
      },
      onSelect: function onSelect() {
        return _this.trigger('select');
      },
      onSetValue: function onSetValue() {
        return _this.trigger('afterSetValue');
      }
    });
    this.typing = new Typing(this);
    this.mark.init();
    this.inline.init();
    this.block.init();
    this.list.init();
    this.hotkey = new Hotkey(this);
    this.scrollNode = this.options.scrollNode ? $(this.options.scrollNode) : null;
    this.card.init(this.options.cards || []);
    this.plugin.init(this.options.plugins || [], this.options.config || {});
    this.ot = new OTModel(this);
  }

  _createClass(Engine, [{
    key: "container",
    get: function get() {
      return this._container.getNode();
    }
  }, {
    key: "readonly",
    get: function get() {
      return this._readonly;
    },
    set: function set(readonly) {
      if (this.readonly === readonly) return;

      if (readonly) {
        this.hotkey.disable();

        this._container.setReadonly(true);
      } else {
        this.hotkey.enable();

        this._container.setReadonly(false);
      }

      this._readonly = readonly; //广播readonly事件

      this.trigger('readonly', readonly);
    }
  }, {
    key: "isSub",
    value: function isSub() {
      return this.container.closest(CARD_SELECTOR).length > 0;
    }
  }, {
    key: "isFocus",
    value: function isFocus() {
      return this._container.isFocus();
    }
  }, {
    key: "focus",
    value: function focus() {
      this.change.focus();
    }
  }, {
    key: "on",
    value: function on(eventType, listener, rewrite) {
      this.event.on(eventType, listener, rewrite);
      return this;
    }
  }, {
    key: "off",
    value: function off(eventType, listener) {
      this.event.off(eventType, listener);
      return this;
    }
  }, {
    key: "trigger",
    value: function trigger(eventType) {
      var _this$event;

      for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        args[_key - 1] = arguments[_key];
      }

      return (_this$event = this.event).trigger.apply(_this$event, [eventType].concat(args));
    }
  }, {
    key: "getValue",
    value: function getValue() {
      var ignoreCursor = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
      var value = this.change.getValue({});
      return ignoreCursor ? Selection.removeTags(value) : value;
    }
  }, {
    key: "getValueAsync",
    value: function () {
      var _getValueAsync = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2() {
        var _this2 = this;

        var ignoreCursor,
            _args2 = arguments;
        return _regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                ignoreCursor = _args2.length > 0 && _args2[0] !== undefined ? _args2[0] : false;
                return _context2.abrupt("return", new Promise(function (resolve) {
                  Object.keys(_this2.plugin.components).forEach( /*#__PURE__*/function () {
                    var _ref = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(pluginName) {
                      var plugin;
                      return _regeneratorRuntime.wrap(function _callee$(_context) {
                        while (1) {
                          switch (_context.prev = _context.next) {
                            case 0:
                              plugin = _this2.plugin.components[pluginName];

                              if (!plugin.waiting) {
                                _context.next = 4;
                                break;
                              }

                              _context.next = 4;
                              return plugin.waiting();

                            case 4:
                            case "end":
                              return _context.stop();
                          }
                        }
                      }, _callee);
                    }));

                    return function (_x) {
                      return _ref.apply(this, arguments);
                    };
                  }());
                  resolve(_this2.getValue(ignoreCursor));
                }));

              case 2:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2);
      }));

      function getValueAsync() {
        return _getValueAsync.apply(this, arguments);
      }

      return getValueAsync;
    }()
  }, {
    key: "getHtml",
    value: function getHtml() {
      var node = $(this.container[0].cloneNode(true));
      node.removeAttributes('contenteditable');
      node.removeAttributes('tabindex');
      node.removeAttributes('autocorrect');
      node.removeAttributes('autocomplete');
      node.removeAttributes('spellcheck');
      node.removeAttributes('data-gramm');
      node.removeAttributes('role');
      return new Parser(node, this).toHTML().html;
    }
  }, {
    key: "setValue",
    value: function setValue(value) {
      this.readonly = false;
      value = this.trigger('beforeSetValue', value) || value;
      this.change.setValue(value);
      this.normalizeTree();
      return this;
    }
  }, {
    key: "setJsonValue",
    value: function setJsonValue(value) {
      var _dom$get;

      var dom = $(toDOM(value));
      var attributes = (_dom$get = dom.get()) === null || _dom$get === void 0 ? void 0 : _dom$get.attributes;

      for (var i = 0; attributes && i < attributes.length; i++) {
        var _ref2 = attributes.item(i) || {},
            nodeName = _ref2.nodeName,
            nodeValue = _ref2.nodeValue;

        if (/^data-selection-/.test(nodeName || '') && nodeValue !== 'null') {
          this.container.attributes(nodeName, nodeValue);
        }
      }

      var html = this.node.html(dom);
      this.change.setValue(html);
      var range = this.change.getRange();
      range.shrinkToElementNode();
      this.change.select(range);
      this.normalizeTree();
      return this;
    }
  }, {
    key: "destroy",
    value: function destroy() {
      this._container.destroy();

      this.change.destroy();
      this.hotkey.destroy();
      this.card.gc();

      if (this.ot) {
        this.ot.destroy();
      }
    }
  }, {
    key: "normalizeTree",
    value: function normalizeTree() {
      var _selection$anchor,
          _selection$focus,
          _this3 = this;

      var block = $('<p />');
      var range = this.change.getRange();
      var selection = range.createSelection();
      var anchorNext, focusPrev, anchorParent, focusParent;

      if (selection.anchor && selection.focus) {
        anchorNext = selection.anchor.next();
        focusPrev = selection.focus.prev();
        anchorParent = selection.anchor.parent();
        focusParent = selection.focus.parent();
      }

      if (anchorNext) (_selection$anchor = selection.anchor) === null || _selection$anchor === void 0 ? void 0 : _selection$anchor.remove();
      if (focusPrev) (_selection$focus = selection.focus) === null || _selection$focus === void 0 ? void 0 : _selection$focus.remove(); // 保证所有行内元素都在段落内

      this.container.children().each(function (child) {
        var node = $(child);

        if (_this3.node.isBlock(node)) {
          if (block.children().length > 0) {
            node.before(block);
          }

          block = $('<p />');
        } else {
          block.append(node);
        }
      });

      if (block.children().length > 0) {
        this.container.append(block);
      }

      if (anchorNext && selection.anchor && selection.anchor.length > 0) anchorNext.before(selection.anchor);
      if (focusPrev && selection.focus && selection.focus.length > 0) focusPrev.after(selection.focus);

      if (selection.has() && anchorNext && focusPrev) {
        var _anchorParent, _focusParent;

        if ((_anchorParent = anchorParent) === null || _anchorParent === void 0 ? void 0 : _anchorParent.isEditable()) {
          block.append(selection.anchor);
        }

        if ((_focusParent = focusParent) === null || _focusParent === void 0 ? void 0 : _focusParent.isEditable()) {
          block.append(selection.focus);
        }

        if (block.children().length > 0) this.container.append(block);
      } // 处理空段落


      this.container.children().each(function (child) {
        var node = $(child);

        _this3.node.removeMinusStyle(node, 'text-indent');

        if (_this3.node.isRootBlock(node)) {
          var childrenLength = node.children().length;

          if (childrenLength === 0) {
            node.remove();
          } else {
            var _child = node.first();

            if (childrenLength === 1 && (_child === null || _child === void 0 ? void 0 : _child.name) === 'span' && [CURSOR, ANCHOR, FOCUS].indexOf(_child.attributes(DATA_ELEMENT)) >= 0) {
              node.prepend($('<br />'));
            }
          }
        }
      });
      selection.move();
    }
  }, {
    key: "messageSuccess",
    value: function messageSuccess(message) {
      console.log("success:".concat(message));
    }
  }, {
    key: "messageError",
    value: function messageError(error) {
      console.log("error:".concat(error));
    }
  }, {
    key: "messageConfirm",
    value: function messageConfirm(message) {
      console.log("confirm:".concat(message));
      return Promise.reject(false);
    }
  }]);

  return Engine;
}();

var css_248z$7 = ".card-maximize-header {\r\n    position: fixed !important;\r\n    top: 0;\r\n    right: 0;\r\n    left: 0;\r\n    z-index: 1032;\r\n    height: 56px;\r\n    background: #fff;\r\n    border-bottom: 1px solid #e8e8e8;\r\n    width: 100%;\r\n}\r\n\r\n.card-maximize-header .header-crumb {\r\n    float: left;\r\n    line-height: 32px;\r\n    display: flex;\r\n    height: 100%;\r\n    align-items: center;\r\n    -webkit-box-align: center;\r\n    -ms-flex-align: center;\r\n}\r\n\r\n.card-maximize-header .header-crumb a\r\n{\r\n    color: #595959;\r\n    font-size: 14px;\r\n    cursor: pointer;\r\n}\r\n\r\n.card-maximize-header .header-crumb .split {\r\n    display: inline-block;\r\n    vertical-align: middle;\r\n    padding: 0 15px;\r\n    font-size: 20px;\r\n    padding-right: 8px;\r\n    font-weight: 200;\r\n    margin: 0 8px;\r\n}\r\n\r\n.card-maximize-header .header-crumb .split + a {\r\n    display: inline-block;\r\n    vertical-align: middle;\r\n}\r\n\r\n.card-maximize-header .header-crumb .split + a:hover {\r\n\tcolor: #8C8C8C;\r\n}\r\n\r\n.data-card-block-max [data-card-element=\"center\"] {\r\n    top: 96px;\r\n    background: #fafafa;\r\n    position: fixed!important;\r\n    right: 0;\r\n    bottom: 0;\r\n    left: 0;\r\n    z-index: 1032;\r\n    overflow: auto;\r\n    padding: 20px;\r\n}\r\n\r\n.am-engine-mobile .data-card-block-max [data-card-element=\"center\"] {\r\n    top:56px;\r\n}";
styleInject(css_248z$7);

var Maximize = /*#__PURE__*/function () {
  function Maximize(editor, card) {
    _classCallCheck(this, Maximize);

    this.card = void 0;
    this.node = void 0;
    this.editor = void 0;
    this.editor = editor;
    this.card = card;
  }

  _createClass(Maximize, [{
    key: "restore",
    value: function restore() {
      this.card.root.removeClass('data-card-block-max');

      if (this.node) {
        this.node.remove();
        this.node = undefined;
      }

      var editor = this.editor;

      if (!this.card.readonly && isEngine(editor)) {
        editor.trigger('card:minimize', this.card);
        editor.history.reset();
      }
    }
  }, {
    key: "maximize",
    value: function maximize() {
      var _this = this;

      if (this.node) return;
      var editor = this.editor;
      var language = editor.language;
      var lang = language.get('maximize', 'back').toString();
      var node = $("<div class=\"card-maximize-header\" data-transient=\"true\">\n            <div class=\"header-crumb\">\n                <a class=\"split\">\n                    <span class=\"data-icon data-icon-arrow-left\"></span>\n                </a>\n                <a>".concat(lang, "</a>\n            </div>\n        </div>"));
      node.on('click', function (event) {
        event.stopPropagation();
      });
      this.card.root.addClass('data-card-block-max');
      var crumnNode = node.find('.header-crumb');
      crumnNode.on('click', function () {
        _this.restore();
      });
      var body = this.card.findByKey('body');
      body.prepend(node);

      if (!this.card.readonly && isEngine(editor)) {
        editor.trigger('card:maximize', this.card);
        editor.history.reset();
      }

      this.node = node;
    }
  }]);

  return Maximize;
}();

var css_248z$8 = ".data-card-resize {\r\n    position: absolute;\r\n    bottom: -3px;\r\n    right: 0px;\r\n    left: 0px;\r\n    margin: 0 auto;\r\n    z-index: 2;\r\n    display: flex;\r\n    align-items: center;\r\n    justify-content: center;\r\n}\r\n\r\n.data-card-resize-btn{\r\n    background: #d9dbdd;\r\n    border-radius: 3px;\r\n    height: 6px;\r\n    padding: 0 50px;\r\n    display: block;\r\n}\r\n\r\n.data-card-resize-btn svg {\r\n    display: block;\r\n    color: #999;\r\n}\r\n\r\n.data-card-resize-btn:hover {\r\n    cursor: row-resize;\r\n}";
styleInject(css_248z$8);

var Resize = /*#__PURE__*/function () {
  function Resize(editor, card) {
    var _this = this;

    _classCallCheck(this, Resize);

    this.editor = void 0;
    this.card = void 0;
    this.point = void 0;
    this.options = {};
    this.component = void 0;

    this.touchStart = function (event) {
      event.preventDefault();
      event.cancelBubble = true;
      _this.point = {
        x: event.targetTouches[0].clientX,
        y: event.targetTouches[0].clientY
      };
      var dragStart = _this.options.dragStart;
      if (dragStart) dragStart(_this.point);
    };

    this.dragStart = function (event) {
      event.preventDefault();
      event.stopPropagation();
      event.cancelBubble = true;
      _this.point = {
        x: event.clientX,
        y: event.clientY
      };
      var dragStart = _this.options.dragStart;
      if (dragStart) dragStart(_this.point);
    };

    this.dragMove = function (event) {
      if (_this.point) {
        var dragMove = _this.options.dragMove;
        if (dragMove) dragMove(event.clientY - _this.point.y);
      }
    };

    this.touchMove = function (event) {
      event.preventDefault();

      if (_this.point) {
        var dragMove = _this.options.dragMove;
        if (dragMove) dragMove(event.targetTouches[0].clientY - _this.point.y);
      }
    };

    this.dragEnd = function (event) {
      _this.point = undefined;
      var dragEnd = _this.options.dragEnd;
      if (dragEnd) dragEnd();
    };

    this.editor = editor;
    this.card = card;
  }

  _createClass(Resize, [{
    key: "create",
    value: function create(options) {
      this.options = options;
      this.component = $('<div class="data-card-resize" draggable="true"><span class="data-card-resize-btn"><svg viewBox="0 0 3413 1024" version="1.1" xmlns="http://www.w3.org/2000/svg" fill="currentColor" width="8px" height="6px"><path d="M341.333333 341.333333h2730.666667a170.666667 170.666667 0 0 0 0-341.333333H341.333333a170.666667 170.666667 0 1 0 0 341.333333zM341.333333 1024h2730.666667a170.666667 170.666667 0 0 0 0-341.333333H341.333333a170.666667 170.666667 0 0 0 0 341.333333z"></path></svg></span></div>');

      if (isMobile) {
        this.component.on('touchstart', this.touchStart);
        this.component.on('touchmove', this.touchMove);
        this.component.on('touchend', this.dragEnd);
        this.component.on('touchcancel', this.dragEnd);
      } else {
        this.component.on('dragstart', this.dragStart);
        document.addEventListener('mousemove', this.dragMove);
        document.addEventListener('mouseup', this.dragEnd);
      }

      this.component.on('click', function (event) {
        event.stopPropagation();
      });
    }
  }, {
    key: "render",
    value: function render() {
      var _this2 = this;

      var container = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.card.root;
      var minHeight = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 80;
      var start = false;
      var height = 0,
          moveHeight = 0;
      this.create({
        dragStart: function dragStart() {
          height = container.height();
          start = true;
        },
        dragMove: function dragMove(y) {
          if (start) {
            moveHeight = height + y;
            moveHeight = moveHeight < minHeight ? minHeight : moveHeight;
            container.css('height', "".concat(moveHeight, "px"));
          }
        },
        dragEnd: function dragEnd() {
          if (start) {
            _this2.card.setValue({
              height: container.height()
            });

            start = false;
          }
        }
      });
      if (!this.component) return;
      this.component.hide();
      container.append(this.component);
    }
  }, {
    key: "show",
    value: function show() {
      var _this$component;

      (_this$component = this.component) === null || _this$component === void 0 ? void 0 : _this$component.show();
    }
  }, {
    key: "hide",
    value: function hide() {
      var _this$component2;

      (_this$component2 = this.component) === null || _this$component2 === void 0 ? void 0 : _this$component2.hide();
    }
  }, {
    key: "destroy",
    value: function destroy() {
      if (isMobile) {
        if (!this.component) return;
        this.component.off('touchstart', this.touchStart);
        this.component.off('touchmove', this.touchMove);
        this.component.off('touchend', this.dragEnd);
        this.component.off('touchcancel', this.dragEnd);
      } else {
        if (this.component) {
          this.component.off('dragstart', this.dragStart);
        }

        document.removeEventListener('mousemove', this.dragMove);
        document.removeEventListener('mouseup', this.dragEnd);
      }
    }
  }]);

  return Resize;
}();

var css_248z$9 = ".data-card-dnd {\r\n    position: absolute;\r\n    top: 0;\r\n    left: -21px;\r\n    right: auto;\r\n    bottom: auto;\r\n    width: 18px;\r\n    height: 24px;\r\n    line-height: 24px;\r\n    font-size: 14px;\r\n    font-weight: normal;\r\n    display: none;\r\n    opacity: 0;\r\n    z-index: 125;\r\n    transition: all 0.3s ease-in-out;\r\n    background: rgba(255, 255, 255, 0.9);\r\n}\r\n\r\n.data-card-dnd-active {\r\n    display: block;\r\n    opacity: 1;\r\n    cursor: pointer;\r\n}\r\n\r\n.data-card-dnd:hover {\r\n\tbackground: #f4f4f4;\r\n\tcolor: #595959;\r\n}\r\n\r\n.data-card-dnd-trigger {\r\n    width: 18px;\r\n    height: 24px;\r\n    text-align: center;\r\n    color: #BFBFBF;\r\n    font-size: 16px;\r\n    border-radius: 2px 2px;\r\n    cursor: move;\r\n    cursor: grab;\r\n    cursor: -moz-grab;\r\n    cursor: -webkit-grab;\r\n}\r\n\r\n.am-engine-view .data-toolbar-active {\r\n\tmin-width: 200px;\r\n}\r\n\r\n.data-card-toolbar.data-toolbar-block {\r\n\ttop: -48px;\r\n\tbottom: auto;\r\n\tdisplay: none;\r\n}\r\n\r\n.data-card-toolbar.data-card-toolbar-active {\r\n\tdisplay: block;\r\n}\r\n\r\n.data-card-toolbar.data-toolbar-active {\r\n\tdisplay: block;\r\n}";
styleInject(css_248z$9);

var isCardToolbarItemOptions = function isCardToolbarItemOptions(item) {
  return ['button', 'input', 'dropdown', 'node'].indexOf(item.type) === -1;
};

var CardToolbar = /*#__PURE__*/function () {
  function CardToolbar(editor, card) {
    _classCallCheck(this, CardToolbar);

    this.card = void 0;
    this.toolbar = void 0;
    this.editor = void 0;
    this.editor = editor;
    this.card = card;
  }

  _createClass(CardToolbar, [{
    key: "getContainer",
    value: function getContainer() {
      var _this$toolbar;

      return (_this$toolbar = this.toolbar) === null || _this$toolbar === void 0 ? void 0 : _this$toolbar.root;
    }
  }, {
    key: "getDefaultItem",
    value: function getDefaultItem(item) {
      var _this = this;

      var editor = this.editor;
      var language = editor.language,
          clipboard = editor.clipboard,
          card = editor.card;

      switch (item.type) {
        case 'separator':
          return {
            type: 'node',
            node: item.node || $("<span class=\"data-toolbar-item-split\"></span>")
          };

        case 'copy':
          return {
            type: 'button',
            content: item.content || "<span class=\"data-icon data-icon-copy\"></span>",
            title: item.title || language.get('copy', 'title').toString(),
            onClick: function onClick() {
              var result = clipboard.copy(_this.card.root[0], true);
              if (result) editor.messageSuccess(language.get('copy', 'success').toString());else editor.messageError(language.get('copy', 'error').toString());
            }
          };

        case 'delete':
          return {
            type: 'button',
            content: item.content || "<span class=\"data-icon data-icon-delete\"></span>",
            title: item.title || language.get('delete', 'title').toString(),
            onClick: function onClick() {
              card.remove(_this.card.root);
            }
          };

        case 'maximize':
          return {
            type: 'button',
            content: item.content || "<span class=\"data-icon data-icon-maximize\"></span>",
            title: item.title || language.get('maximize', 'title').toString(),
            onClick: function onClick() {
              _this.card.maximize();
            }
          };

        case 'more':
          return {
            type: 'dropdown',
            content: item.content || "<span class=\"data-icon data-icon-more\"></span>",
            title: item.title || language.get('more', 'title').toString(),
            items: item.items
          };
      }

      return;
    }
  }, {
    key: "create",
    value: function create() {
      var _this2 = this;

      if (this.card.toolbar) {
        //获取客户端配置
        var config = this.card.toolbar(); //获取渲染节点

        var _this$editor = this.editor,
            root = _this$editor.root,
            language = _this$editor.language;
        this.hide();
        var items = [];
        config.forEach(function (item) {
          //默认项
          if (isCardToolbarItemOptions(item)) {
            switch (item.type) {
              case 'dnd':
                if (isMobile) return;

                var dndNode = _this2.createDnd(item.content || '<span class="data-icon data-icon-drag"></span>', item.title || language.get('dnd', 'title').toString());

                root.append(dndNode);
                break;

              default:
                var resultItem = _this2.getDefaultItem(item);

                if (resultItem) items.push(resultItem);
            }
          } else {
            items.push(item);
          }
        });

        if (items.length > 0) {
          var toolbar = new Toolbar({
            items: items
          });
          toolbar.root.addClass('data-card-toolbar'); //渲染工具栏

          toolbar.render(root);
          toolbar.hide();
          this.toolbar = toolbar;
        }
      }
    }
  }, {
    key: "hide",
    value: function hide() {
      var root = this.editor.root;
      root.find('.data-card-dnd').remove();
      this.hideCardToolbar();
    }
  }, {
    key: "show",
    value: function show(event) {
      this.showCardToolbar(event);
    }
  }, {
    key: "hideCardToolbar",
    value: function hideCardToolbar() {
      var _this$toolbar2;

      (_this$toolbar2 = this.toolbar) === null || _this$toolbar2 === void 0 ? void 0 : _this$toolbar2.destroy();
    }
  }, {
    key: "showCardToolbar",
    value: function showCardToolbar(event) {
      this.create();
      var container = this.getContainer();

      if (container && container.length > 0) {
        var element = container.get();
        element.style.left = '0px';

        if (event) {
          var clientX = event.clientX;
          var groupElement = container.first();
          var cardRect = this.card.root.get().getBoundingClientRect();

          if (groupElement && clientX >= cardRect.left && clientX <= cardRect.right) {
            var groupRect = groupElement.get().getBoundingClientRect();
            var space = cardRect.width - groupRect.width;

            if (space > 0) {
              var left = clientX - cardRect.width - groupRect.width / 2;
              element.style.left = Math.max(Math.min(left, space), 0) + 'px';
            }
          }
        } else {
          var _cardRect = this.card.root.getBoundingClientRect() || {
            left: 0,
            top: 0
          };

          var root = this.editor.root;
          var rootRect = root.getBoundingClientRect() || {
            left: 0,
            top: 0
          };
          var top = _cardRect.top - rootRect.top;

          var _left = _cardRect.left - rootRect.left;

          element.style.top = "".concat(top - 48, "px");
          element.style.left = "".concat(_left, "px");
          var dnd = root.find('.data-card-dnd');
          var dndElement = dnd.get();

          if (dndElement) {
            dndElement.style.top = "".concat(top, "px");
            dndElement.style.left = "".concat(_left - 21, "px");
            dnd.addClass('data-card-dnd-active');
          }
        }

        container.addClass('data-toolbar-active');
        container.attributes('toolbar-trigger-key', this.card.constructor.cardName);
        if (this.toolbar) this.toolbar.show();
      }
    }
  }, {
    key: "createDnd",
    value: function createDnd(content, title) {
      var _this3 = this;

      var dndNode = $("<div ".concat(DATA_ELEMENT, "=\"").concat(UI, "\" class=\"data-card-dnd\" draggable=\"true\" dnd-trigger-key=\"").concat(this.card.constructor.cardName, "\" drag-card-trigger=\"").concat(this.card.id, "\" contenteditable=\"false\">\n                <div class=\"data-card-dnd-trigger\">\n                    ").concat(content, "\n                </div>\n            </div>"));
      dndNode.on('mouseenter', function () {
        Tooltip.show(dndNode, title);
      });
      dndNode.on('mouseleave', function () {
        Tooltip.hide();
      });
      dndNode.on('mousedown', function (e) {
        e.stopPropagation();
        Tooltip.hide();

        _this3.hideCardToolbar();
      });
      dndNode.on('mouseup', function () {
        _this3.showCardToolbar();
      });
      return dndNode;
    }
  }]);

  return CardToolbar;
}();

var CardEntry = /*#__PURE__*/function () {
  function CardEntry(_ref) {
    var editor = _ref.editor,
        value = _ref.value,
        root = _ref.root;

    _classCallCheck(this, CardEntry);

    this.editor = void 0;
    this.root = void 0;
    this.toolbarModel = void 0;
    this.resizeModel = void 0;
    this.activatedByOther = false;
    this.selectedByOther = false;
    this.contenteditable = [];
    this.defaultMaximize = void 0;
    this.isMaximize = false;
    this.resize = void 0;
    this.editor = editor;
    var type = this.constructor.cardType;
    var tagName = type === 'inline' ? 'span' : 'div';
    this.root = root ? root : $('<'.concat(tagName, ' />'));
    if (typeof value === 'string') value = decodeCardValue(value);
    value = value || {};
    value['id'] = this.getId(value['id']);
    this.setValue(value);
    if (this.toolbar) this.toolbarModel = new CardToolbar(this.editor, this);
    this.defaultMaximize = new Maximize(this.editor, this);
  }

  _createClass(CardEntry, [{
    key: "isEditable",
    get:
    /**
     * 可编辑的节点
     */
    function get() {
      return this.contenteditable.length > 0;
    }
  }, {
    key: "readonly",
    get: function get() {
      return !isEngine(this.editor);
    }
  }, {
    key: "activated",
    get: function get() {
      return this.root.hasClass('card-activated');
    }
  }, {
    key: "setActivated",
    value: function setActivated(activated) {
      activated ? this.root.addClass('card-activated') : this.root.removeClass('card-activated');
    }
  }, {
    key: "selected",
    get: function get() {
      return this.root.hasClass('card-selected');
    }
  }, {
    key: "setSelected",
    value: function setSelected(selected) {
      selected ? this.root.addClass('card-selected') : this.root.removeClass('card-selected');
    }
  }, {
    key: "id",
    get: function get() {
      var value = this.getValue();
      return _typeof(value) === 'object' ? value.id : '';
    }
  }, {
    key: "name",
    get: function get() {
      return this.root.attributes(CARD_KEY);
    }
  }, {
    key: "getId",
    value: function getId(curId) {
      var idCache = [];
      this.editor.card.each(function (card) {
        idCache.push(card.id);
      });
      if (curId && idCache.indexOf(curId) < 0) return curId;
      var id = random();

      while (idCache.indexOf(id) >= 0) {
        id = random();
      }

      return id;
    } // 设置 DOM 属性里的数据

  }, {
    key: "setValue",
    value: function setValue(value) {
      if (value == null) {
        return;
      }

      var currentValue = this.getValue();
      if (!!(currentValue === null || currentValue === void 0 ? void 0 : currentValue.id)) delete value['id'];
      value = _objectSpread(_objectSpread({}, this.getValue()), value);
      this.root.attributes(CARD_VALUE_KEY, encodeCardValue(value));
    } // 获取 DOM 属性里的数据

  }, {
    key: "getValue",
    value: function getValue() {
      var value = this.root.attributes(CARD_VALUE_KEY);
      if (!value) return;
      return decodeCardValue(value);
    }
    /**
     * 获取Card内的 DOM 节点
     * @param selector
     */

  }, {
    key: "find",
    value: function find(selector) {
      var _this = this;

      var card = this.editor.card;
      var nodes = this.root.find(selector);
      var children = [];
      nodes.each(function (item) {
        var cardComponent = card.find(item);

        if (cardComponent && cardComponent.root.equal(_this.root)) {
          children.push(item);
        }
      });
      return $(children);
    }
  }, {
    key: "findByKey",
    value: function findByKey(key) {
      return this.find('['.concat(CARD_ELEMENT_KEY, '=').concat(key, ']'));
    }
  }, {
    key: "activate",
    value: function activate(activated) {
      if (activated) {
        if (!this.activated) {
          this.setActivated(activated);
          this.onActivate(activated);
        }
      } else if (this.activated) {
        this.setActivated(activated);
        this.onActivate(false);
      }
    }
  }, {
    key: "select",
    value: function select(selected) {
      if (this.readonly || this.activatedByOther) {
        return;
      }

      if (selected) {
        if (!this.selected) {
          this.setSelected(selected);
          this.onSelect(selected);
        }
      } else if (this.selected) {
        this.setSelected(selected);
        this.onSelect(false);
      }
    }
  }, {
    key: "getCenter",
    value: function getCenter() {
      return this.find(CARD_CENTER_SELECTOR);
    }
  }, {
    key: "isCenter",
    value: function isCenter(node) {
      var center = node.closest(CARD_CENTER_SELECTOR);
      return center.length > 0 && center.equal(this.find(CARD_CENTER_SELECTOR));
    }
  }, {
    key: "isCursor",
    value: function isCursor(node) {
      return this.isLeftCursor(node) || this.isRightCursor(node);
    }
  }, {
    key: "isLeftCursor",
    value: function isLeftCursor(node) {
      var cursor = node.closest(CARD_LEFT_SELECTOR);
      return cursor.length > 0 && cursor.equal(this.find(CARD_LEFT_SELECTOR));
    }
  }, {
    key: "isRightCursor",
    value: function isRightCursor(node) {
      var cursor = node.closest(CARD_RIGHT_SELECTOR);
      return cursor.length > 0 && cursor.equal(this.find(CARD_RIGHT_SELECTOR));
    }
  }, {
    key: "focus",
    value: function focus(range, toStart) {
      var cardLeft = this.findByKey('left');
      var cardRight = this.findByKey('right');

      if (cardLeft.length === 0 || cardRight.length === 0) {
        return;
      }

      range.select(toStart ? cardLeft : cardRight, true);
      range.collapse(false);
      if (this.onFocus) this.onFocus();
    } // 焦点移动到上一个 Block

  }, {
    key: "focusPrevBlock",
    value: function focusPrevBlock(range, hasModify) {
      if (!isEngine(this.editor)) throw 'Engine not initialized';
      var prevBlock;

      if (this.constructor.cardType === 'inline') {
        var block = this.editor.block.closest(this.root);

        if (block.isEditable()) {
          prevBlock = this.root.prevElement();
        } else {
          prevBlock = block.prevElement();
        }
      } else {
        prevBlock = this.root.prevElement();
      }

      if (hasModify) {
        if (!prevBlock || prevBlock.attributes(CARD_KEY)) {
          var _block = $('<p><br /></p>');

          this.root.before(_block);
          range.select(_block, true);
          range.collapse(false);
          return;
        }
      } else {
        if (!prevBlock) {
          return;
        }

        if (prevBlock.attributes(CARD_KEY)) {
          var _this$editor$card$fin;

          (_this$editor$card$fin = this.editor.card.find(prevBlock)) === null || _this$editor$card$fin === void 0 ? void 0 : _this$editor$card$fin.focus(range, false);
          return;
        }
      }

      range.select(prevBlock, true).shrinkToElementNode().collapse(false);
    } // 焦点移动到下一个 Block

  }, {
    key: "focusNextBlock",
    value: function focusNextBlock(range, hasModify) {
      if (!isEngine(this.editor)) throw 'Engine not initialized';
      var nextBlock;

      if (this.constructor.cardType === 'inline') {
        var block = this.editor.block.closest(this.root);

        if (block.isEditable()) {
          nextBlock = this.root.nextElement();
        } else {
          nextBlock = block.nextElement();
        }
      } else {
        nextBlock = this.root.nextElement();
      }

      if (hasModify) {
        if (!nextBlock || nextBlock.attributes(CARD_KEY)) {
          var _block = $('<p><br /></p>');

          this.root.after(_block);
          range.select(_block, true);
          range.collapse(false);
          return;
        }
      } else {
        if (!nextBlock) {
          return;
        }

        if (nextBlock.attributes(CARD_KEY)) {
          var _this$editor$card$fin2;

          (_this$editor$card$fin2 = this.editor.card.find(nextBlock)) === null || _this$editor$card$fin2 === void 0 ? void 0 : _this$editor$card$fin2.focus(range, false);
          return;
        }
      }

      range.select(nextBlock, true).shrinkToElementNode().collapse(true);
    }
  }, {
    key: "maximize",
    value: function maximize() {
      this.isMaximize = true;
      this.defaultMaximize.maximize();
    }
  }, {
    key: "minimize",
    value: function minimize() {
      this.isMaximize = false;
      this.defaultMaximize.restore();
    }
    /**
     * 是否可改变卡片大小，或者传入渲染节点
     */

  }, {
    key: "onSelect",
    value: function onSelect(selected) {
      var selectedClass = "data-card-".concat(this.constructor.selectStyleType, "-selected");
      var center = this.getCenter();
      if (selected) center.addClass(selectedClass);else center.removeClass(selectedClass);
    }
  }, {
    key: "onSelectByOther",
    value: function onSelectByOther(selected, value) {
      var center = this.getCenter();

      if (this.constructor.selectStyleType === 'background') {
        center.css('background-color', selected ? value.rgb : '');
      } else {
        center.css('outline', selected ? '2px solid ' + value.color : '');
      }
    }
  }, {
    key: "onActivate",
    value: function onActivate(activated) {
      var _this$resizeModel, _this$resizeModel2;

      if (!this.resize) return;
      if (activated) (_this$resizeModel = this.resizeModel) === null || _this$resizeModel === void 0 ? void 0 : _this$resizeModel.show();else (_this$resizeModel2 = this.resizeModel) === null || _this$resizeModel2 === void 0 ? void 0 : _this$resizeModel2.hide();
    }
  }, {
    key: "onActivateByOther",
    value: function onActivateByOther(activated, value) {
      this.onSelectByOther(activated, value);
    }
  }, {
    key: "destroy",
    value: function destroy() {
      var _this$toolbarModel;

      (_this$toolbarModel = this.toolbarModel) === null || _this$toolbarModel === void 0 ? void 0 : _this$toolbarModel.hide();
    }
  }, {
    key: "didRender",
    value: function didRender() {
      if (this.resize) {
        var container = typeof this.resize === 'function' ? this.resize() : this.findByKey('body');
        this.resizeModel = new Resize(this.editor, this);
        this.resizeModel.render(container);
      }

      if (this.contenteditable.length > 0) {
        this.editor.block.generateDataIDForDescendant(this.getCenter().get());
      }
    }
  }]);

  return CardEntry;
}();

CardEntry.cardName = void 0;
CardEntry.cardType = void 0;
CardEntry.autoActivate = void 0;
CardEntry.autoSelected = true;
CardEntry.singleSelectable = void 0;
CardEntry.collab = true;
CardEntry.focus = void 0;
CardEntry.selectStyleType = 'border';
CardEntry.toolbarFollowMouse = false;

var View = /*#__PURE__*/function () {
  function View(selector, options) {
    _classCallCheck(this, View);

    this.options = {
      lang: 'zh-cn',
      plugins: [],
      cards: []
    };
    this.kind = 'view';
    this.root = void 0;
    this.language = void 0;
    this.container = void 0;
    this.card = void 0;
    this.plugin = void 0;
    this.node = void 0;
    this.list = void 0;
    this.mark = void 0;
    this.inline = void 0;
    this.block = void 0;
    this.clipboard = void 0;
    this.event = void 0;
    this.schema = void 0;
    this.conversion = void 0;
    this.command = void 0;
    this.request = void 0;
    this.options = _objectSpread(_objectSpread({}, this.options), options);
    this.language = new Language(this.options.lang || 'zh-cn', language);
    this.event = new Event();
    this.command = new Command(this);
    this.schema = new Schema();
    this.schema.add(defualtSchema);
    this.conversion = new Conversion(this);
    this.card = new CardModel(this);
    this.clipboard = new Clipboard(this);
    this.plugin = new PluginModel(this);
    this.node = new NodeModel(this);
    this.list = new List(this);
    this.mark = new Mark(this);
    this.inline = new Inline(this);
    this.block = new Block(this);
    this.clipboard = new Clipboard(this);
    this.request = new Request();
    this.container = $(selector);
    this.root = $(this.options.root || this.container.parent() || document.body);
    this.container.addClass('am-engine-view');
    this.mark.init();
    this.inline.init();
    this.block.init();
    this.list.init();
    this.card.init(this.options.cards || []);
    this.plugin.init(this.options.plugins || [], this.options.config || {});
  }

  _createClass(View, [{
    key: "on",
    value: function on(eventType, listener, rewrite) {
      this.event.on(eventType, listener, rewrite);
      return this;
    }
  }, {
    key: "off",
    value: function off(eventType, listener) {
      this.event.off(eventType, listener);
      return this;
    }
  }, {
    key: "trigger",
    value: function trigger(eventType) {
      var _this$event;

      for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        args[_key - 1] = arguments[_key];
      }

      return (_this$event = this.event).trigger.apply(_this$event, [eventType].concat(args));
    }
  }, {
    key: "render",
    value: function render(content) {
      var trigger = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
      var parser = new Parser(content, this);
      var value = parser.toValue(this.schema, this.conversion, false, true);
      this.container.html(value);
      this.card.render();
      if (trigger) this.trigger('render', this.container);
    }
  }, {
    key: "messageSuccess",
    value: function messageSuccess(message) {
      console.log("success:".concat(message));
    }
  }, {
    key: "messageError",
    value: function messageError(error) {
      console.log("error:".concat(error));
    }
  }, {
    key: "messageConfirm",
    value: function messageConfirm(message) {
      console.log("confirm:".concat(message));
      return Promise.reject(false);
    }
  }]);

  return View;
}();

var css_248z$a = ".data-scrollable::-webkit-scrollbar {\r\n    display: none;\r\n  }\r\n  .data-scrollable.scroll-x {\r\n    padding-bottom: 10px;\r\n    overflow-x: hidden;\r\n  }\r\n  .data-scrollable.scroll-x:hover {\r\n    overflow-x: auto;\r\n  }\r\n  .data-scrollable.scroll-y {\r\n    padding-right: 10px;\r\n    overflow-y: hidden;\r\n  }\r\n  .data-scrollable.scroll-y:hover {\r\n    overflow-y: auto;\r\n  }\r\n  .data-scrollable:hover .data-scrollbar {\r\n    display: block;\r\n  }\r\n  .data-scrollable.scrolling {\r\n    -webkit-user-select: none;\r\n       -moz-user-select: none;\r\n        -ms-user-select: none;\r\n            user-select: none;\r\n  }\r\n  .data-scrollable.scrolling .data-scrollbar {\r\n    display: block;\r\n  }\r\n  .data-scrollable .data-scrollbar {\r\n    display: none;\r\n    position: absolute;\r\n    cursor: default;\r\n    transition: opacity 0.3s ease-in-out;\r\n  }\r\n  .data-scrollable .data-scrollbar .data-scrollbar-trigger {\r\n    position: absolute;\r\n    background: #c1c1c1;\r\n    border-radius: 10px;\r\n    cursor: pointer;\r\n  }\r\n  .data-scrollable .data-scrollbar .data-scrollbar-trigger:hover {\r\n    background: #888;\r\n  }\r\n  .data-scrollable .data-scrollbar.data-scrollbar-x {\r\n    width: 100%;\r\n    height: 8px;\r\n    bottom: 0px;\r\n  }\r\n  .data-scrollable .data-scrollbar.data-scrollbar-x .data-scrollbar-trigger {\r\n    height: 8px;\r\n    min-width: 60px;\r\n  }\r\n  .data-scrollable .data-scrollbar.data-scrollbar-y {\r\n    top: 0;\r\n    width: 8px;\r\n    height: 100%;\r\n    right: 0px;\r\n  }\r\n  .data-scrollable .data-scrollbar.data-scrollbar-y .data-scrollbar-trigger {\r\n    width: 8px;\r\n    min-height: 60px;\r\n  }\r\n  .data-scrollable .scrollbar-shadow-left {\r\n    position: absolute;\r\n    z-index: 10;\r\n    left: 0;\r\n    top: 0;\r\n    bottom: 0;\r\n    width: 4px;\r\n    opacity: 0.8;\r\n    background: linear-gradient(270deg, rgba(99, 114, 130, 0) 0, rgba(99, 114, 130, 0.16));\r\n    background: -webkit-linear-gradient(right, rgba(99, 114, 130, 0), rgba(99, 114, 130, 0.16));\r\n    pointer-events: none;\r\n  }\r\n  .data-scrollable .scrollbar-shadow-right {\r\n    position: absolute;\r\n    z-index: 10;\r\n    left: 0;\r\n    top: 0;\r\n    bottom: 0;\r\n    width: 4px;\r\n    opacity: 0.8;\r\n    background: linear-gradient(90deg, rgba(99, 114, 130, 0) 0, rgba(99, 114, 130, 0.16));\r\n    background: -webkit-linear-gradient(left, rgba(99, 114, 130, 0), rgba(99, 114, 130, 0.16));\r\n    pointer-events: none;\r\n  }";
styleInject(css_248z$a);

var Scrollbar = /*#__PURE__*/function (_EventEmitter) {
  _inherits(Scrollbar, _EventEmitter);

  var _super = _createSuper(Scrollbar);

  /**
   * @param {nativeNode} container 需要添加滚动条的容器元素
   * @param {boolean} x 横向滚动条
   * @param {boolean} y 竖向滚动条
   * @param {boolean} needShadow 是否显示阴影
   */
  function Scrollbar(container) {
    var _this;

    var x = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
    var y = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
    var shadow = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : true;

    _classCallCheck(this, Scrollbar);

    _this = _super.call(this);
    _this.container = void 0;
    _this.x = void 0;
    _this.y = void 0;
    _this.shadow = void 0;
    _this.scrollBarX = void 0;
    _this.slideX = void 0;
    _this.sildeXDragging = void 0;
    _this.scrollBarY = void 0;
    _this.slideY = void 0;
    _this.slideYDragging = void 0;
    _this.shadowLeft = void 0;
    _this.shadowRight = void 0;
    _this.oWidth = 0;
    _this.oHeight = 0;
    _this.sWidth = 0;
    _this.sHeight = 0;
    _this.xWidth = 0;
    _this.yHeight = 0;
    _this.maxScrollLeft = 0;
    _this.shadowTimer = void 0;

    _this.scrollX = function (event) {
      if (_this.sildeXDragging) {
        var _this$slideX;

        var _this$sildeXDragging = _this.sildeXDragging,
            point = _this$sildeXDragging.point,
            position = _this$sildeXDragging.position;
        var left = position + (event.clientX - point);
        left = Math.max(0, Math.min(left, _this.oWidth - _this.xWidth));
        (_this$slideX = _this.slideX) === null || _this$slideX === void 0 ? void 0 : _this$slideX.css('left', left + 'px');
        var min = left / (_this.oWidth - _this.xWidth);
        min = Math.min(1, min);
        _this.container.get().scrollLeft = (_this.sWidth - _this.oWidth) * min;
      }
    };

    _this.scrollY = function (event) {
      if (_this.slideYDragging) {
        var _this$slideY;

        var _this$slideYDragging = _this.slideYDragging,
            point = _this$slideYDragging.point,
            position = _this$slideYDragging.position;
        var top = position + (event.clientY - point);
        top = Math.max(0, Math.min(top, _this.oHeight - _this.yHeight));
        (_this$slideY = _this.slideY) === null || _this$slideY === void 0 ? void 0 : _this$slideY.css('top', top + 'px');
        var min = top / (_this.oHeight - _this.yHeight);
        min = Math.min(1, min);
        _this.container.get().scrollTop = (_this.sHeight - _this.oHeight) * min;
      }
    };

    _this.scrollXEnd = function () {
      _this.sildeXDragging = undefined;
      document.body.removeEventListener('mousemove', _this.scrollX);
      document.body.removeEventListener('mouseup', _this.scrollXEnd);

      _this.container.removeClass('scrolling');
    };

    _this.scrollYEnd = function () {
      _this.slideYDragging = undefined;
      document.body.removeEventListener('mousemove', _this.scrollY);
      document.body.removeEventListener('mouseup', _this.scrollYEnd);

      _this.container.removeClass('scrolling');
    };

    _this.bindXScrollEvent = function () {
      if (_this.x) {
        var _this$slideX2;

        (_this$slideX2 = _this.slideX) === null || _this$slideX2 === void 0 ? void 0 : _this$slideX2.on('mousedown', function (event) {
          var _this$slideX3;

          _this.container.addClass('scrolling');

          _this.sildeXDragging = {
            point: event.clientX,
            position: parseInt(((_this$slideX3 = _this.slideX) === null || _this$slideX3 === void 0 ? void 0 : _this$slideX3.css('left')) || '0')
          };
          document.body.addEventListener('mousemove', _this.scrollX);
          document.body.addEventListener('mouseup', _this.scrollXEnd);
        });
      }
    };

    _this.bindYScrollEvent = function () {
      if (_this.y) {
        var _this$slideY2;

        (_this$slideY2 = _this.slideY) === null || _this$slideY2 === void 0 ? void 0 : _this$slideY2.on('mousedown', function (event) {
          var _this$slideY3;

          _this.container.addClass('scrolling');

          _this.sildeXDragging = {
            point: event.clientY,
            position: parseInt(((_this$slideY3 = _this.slideY) === null || _this$slideY3 === void 0 ? void 0 : _this$slideY3.css('top')) || '0')
          };
          document.body.addEventListener('mousemove', _this.scrollY);
          document.body.addEventListener('mouseup', _this.scrollYEnd);
        });
      }
    };

    _this.reRenderShadow = function (width) {
      if (_this.shadow) {
        var _this$shadowLeft, _this$shadowRight, _this$shadowLeft2, _this$shadowRight2;

        (_this$shadowLeft = _this.shadowLeft) === null || _this$shadowLeft === void 0 ? void 0 : _this$shadowLeft.css('left', width + 'px');
        (_this$shadowRight = _this.shadowRight) === null || _this$shadowRight === void 0 ? void 0 : _this$shadowRight.css('left', width + _this.oWidth - 4 + 'px');
        (_this$shadowLeft2 = _this.shadowLeft) === null || _this$shadowLeft2 === void 0 ? void 0 : _this$shadowLeft2.css('display', 'none');
        (_this$shadowRight2 = _this.shadowRight) === null || _this$shadowRight2 === void 0 ? void 0 : _this$shadowRight2.css('display', 'none');
        if (_this.shadowTimer) clearTimeout(_this.shadowTimer);
        _this.shadowTimer = setTimeout(function () {
          if (0 !== width) {
            var _this$shadowLeft3;

            (_this$shadowLeft3 = _this.shadowLeft) === null || _this$shadowLeft3 === void 0 ? void 0 : _this$shadowLeft3.css('display', 'block');
          }

          if (width !== _this.maxScrollLeft) {
            var _this$shadowRight3;

            (_this$shadowRight3 = _this.shadowRight) === null || _this$shadowRight3 === void 0 ? void 0 : _this$shadowRight3.css('display', 'block');
          }
        }, 100);
      }
    };

    _this.reRenderX = function (left) {
      if (_this.x) {
        var _this$scrollBarX, _this$slideX4;

        (_this$scrollBarX = _this.scrollBarX) === null || _this$scrollBarX === void 0 ? void 0 : _this$scrollBarX.css('left', left + 'px');
        var min = left / (_this.sWidth - _this.oWidth);
        min = Math.min(1, min);
        (_this$slideX4 = _this.slideX) === null || _this$slideX4 === void 0 ? void 0 : _this$slideX4.css('left', (_this.oWidth - _this.xWidth) * min + 'px');

        _this.reRenderShadow(left);
      }
    };

    _this.reRenderY = function (top) {
      if (_this.y) {
        var _this$scrollBarY, _this$slideY4;

        (_this$scrollBarY = _this.scrollBarY) === null || _this$scrollBarY === void 0 ? void 0 : _this$scrollBarY.css('top', top + 'px');
        var min = top / (_this.sHeight - _this.oHeight);
        min = Math.min(1, min);
        (_this$slideY4 = _this.slideY) === null || _this$slideY4 === void 0 ? void 0 : _this$slideY4.css('top', (_this.oHeight - _this.yHeight) * min + 'px');
      }
    };

    _this.container = isNode(container) ? $(container) : container;
    _this.x = x;
    _this.y = y;
    _this.shadow = shadow;

    _this.init();

    return _this;
  }

  _createClass(Scrollbar, [{
    key: "init",
    value: function init() {
      var children = this.container.children();
      var hasScrollbar = false;
      children.each(function (child) {
        if ($(child).hasClass('data-scrollbar')) {
          hasScrollbar = true;
        }
      });

      if (!hasScrollbar) {
        this.container.css('position', 'relative');
        this.container.addClass('data-scrollable');

        if (this.x) {
          this.scrollBarX = $("<div ".concat(DATA_ELEMENT, "=\"").concat(UI, "\" class=\"data-scrollbar data-scrollbar-x \"><div class=\"data-scrollbar-trigger\"></div></div>"));
          this.slideX = this.scrollBarX.find('.data-scrollbar-trigger');
          this.container.append(this.scrollBarX);
          this.container.addClass('scroll-x');
        }

        if (this.y) {
          this.scrollBarY = $("<div ".concat(DATA_ELEMENT, "=\"").concat(UI, "\" class=\"data-scrollbar data-scrollbar-y \"><div class=\"data-scrollbar-trigger\"></div></div>"));
          this.slideY = this.scrollBarY.find('.data-scrollbar-trigger');
          this.container.append(this.scrollBarY);
          this.container.addClass('scroll-y');
        }

        if (this.shadow) {
          this.shadowLeft = $("<div ".concat(DATA_ELEMENT, "=\"").concat(UI, "\" class=\"scrollbar-shadow-left\"></div>"));
          this.shadowRight = $("<div ".concat(DATA_ELEMENT, "=\"").concat(UI, "\" class=\"scrollbar-shadow-right\"></div>"));
          this.container.append(this.shadowLeft);
          this.container.append(this.shadowRight);
        }

        this.refresh();
        this.bindEvents();
      }
    }
  }, {
    key: "refresh",
    value: function refresh() {
      var element = this.container.get();

      if (element) {
        var offsetWidth = element.offsetWidth,
            offsetHeight = element.offsetHeight,
            scrollWidth = element.scrollWidth,
            scrollHeight = element.scrollHeight,
            scrollLeft = element.scrollLeft,
            scrollTop = element.scrollTop;
        this.oWidth = offsetWidth;
        this.oHeight = offsetHeight;
        this.sWidth = scrollWidth;
        this.sHeight = scrollHeight;
        this.xWidth = Math.floor(offsetWidth * offsetWidth / scrollWidth);
        this.yHeight = Math.floor(offsetHeight * offsetHeight / scrollHeight);
        this.maxScrollLeft = scrollWidth - offsetWidth;

        if (this.x) {
          var _this$slideX5, _this$slideX6;

          (_this$slideX5 = this.slideX) === null || _this$slideX5 === void 0 ? void 0 : _this$slideX5.css('width', this.xWidth + 'px');
          var display = this.oWidth === this.sWidth ? 'none' : 'block';
          (_this$slideX6 = this.slideX) === null || _this$slideX6 === void 0 ? void 0 : _this$slideX6.css('display', display);
          this.emit('display', display);
        }

        if (this.y) {
          var _this$slideY5, _this$slideY6;

          (_this$slideY5 = this.slideY) === null || _this$slideY5 === void 0 ? void 0 : _this$slideY5.css('height', this.yHeight + 'px');

          var _display = this.oHeight === this.sHeight ? 'none' : 'block';

          (_this$slideY6 = this.slideY) === null || _this$slideY6 === void 0 ? void 0 : _this$slideY6.css('display', _display);
          this.emit('display', _display);
        }

        this.reRenderX(scrollLeft);
        this.reRenderY(scrollTop);
      }
    }
  }, {
    key: "bindEvents",
    value: function bindEvents() {
      var _this2 = this;

      this.container.on('scroll', function (event) {
        var target = event.target;
        var scrollTop = target.scrollTop,
            scrollLeft = target.scrollLeft;

        _this2.reRenderX(scrollLeft);

        _this2.reRenderY(scrollTop);
      });
      this.bindXScrollEvent();
      this.bindYScrollEvent();
    }
  }]);

  return Scrollbar;
}(EventEmitter2);

export default Engine;
export { $, ANCHOR, ANCHOR_SELECTOR, ActiveTrigger, Ajax, BlockEntry as BlockPlugin, CARD_ASYNC_RENDER, CARD_CENTER_SELECTOR, CARD_ELEMENT_KEY, CARD_KEY, CARD_LEFT_SELECTOR, CARD_RIGHT_SELECTOR, CARD_SELECTOR, CARD_TAG, CARD_TYPE_KEY, CARD_VALUE_KEY, CURSOR, CURSOR_SELECTOR, CamelCaseType, CardEntry as Card, CardType, defaultConversion as Conversion, DATA_ELEMENT, DATA_TRANSIENT_ATTRIBUTES, DATA_TRANSIENT_ELEMENT, EDITABLE, EDITABLE_SELECTOR, ElementPluginEntry as ElementPlugin, FOCUS, FOCUS_SELECTOR, InlineEntry as InlinePlugin, JSONML, ListEntry as ListPlugin, MarkEntry as MarkPlugin, Parser, PluginEntry as Plugin, READY_CARD_KEY, READY_CARD_SELECTOR, ROOT, ROOT_SELECTOR, Range, Request, defualtSchema as Schema, Scrollbar, Selection, TinyCanvas, Toolbar, Tooltip, UI, UI_SELECTOR, Uploader, View, addUnit, combinTextNode, decodeCardValue, encodeCardValue, escape, escapeDots, formatEngineValue, formatHotkey, getAttrMap, getComputedStyle, getDocument, getExtensionName, getFileSize, getHashId, getListStyle, getParentInRoot, getStyleMap, getTextNodes, getWindow, inside, isAndroid, isBlockPlugin, isChrome, isEdge, isEngine, isFirefox, isInlinePlugin, isIos, isMacos, isMarkPlugin, isMobile, isNode, isNodeEntry, isNodeList, isRange, isRangeInterface, isSafari, isSchemaRule, isSelection, isServer, isUrl, isWindows, random, removeUnit, sanitizeUrl, toCamelCase, toHex, transformCustomTags, unescape, unescapeDots, validUrl };
